@@ -59,7 +59,7 @@ static cmd_options cmd_break_options[] =
 
 cmd_break::cmd_break()
   : command("break", "br")
-{ 
+{
   brief_doc = string("Set a break point");
 
   long_doc = string ("The 'break' command can be used to examine or set breakpoints.\n"
@@ -106,7 +106,7 @@ cmd_break::cmd_break()
                      "\tbreak c 1000000    # break on the one million'th cycle\n"
                      );
 
-  op = cmd_break_options; 
+  op = cmd_break_options;
 }
 
 
@@ -115,7 +115,8 @@ void cmd_break::list(guint64 value)
   if(value == CMDBREAK_BAD_BREAK_NUMBER)
     get_bp().dump();
   else
-    get_bp().dump1((unsigned int)value);
+    if(!get_bp().dump1((unsigned int)value))
+	printf("break not found at given break point number %u (0x%x)\n", (unsigned int)value, (unsigned int)value);
 }
 
 const char *TOO_FEW_ARGS="missing register or location\n";
@@ -201,7 +202,7 @@ unsigned int cmd_break::set_break(cmd_options *co, ExprList_t *pEL, bool bLog)
 
   // See if the expression supports break points. If it does, the break points
   // will get set and the expressions deleted.
-  int bpn = pFirst ? pFirst->set_break(MapBreakActions(co->value), 
+  int bpn = pFirst ? pFirst->set_break(MapBreakActions(co->value),
 				       (bLog ? gpsimObject::eActionLog : gpsimObject::eActionHalt),
 				       pSecond) : -1;
   if (bpn == -1 && co->value!=CYCLE)
@@ -210,7 +211,8 @@ unsigned int cmd_break::set_break(cmd_options *co, ExprList_t *pEL, bool bLog)
 
   if (bpn<0) {
 
-    // We failed to set a break point from the first expression. 
+
+    // We failed to set a break point from the first expression.
     // It may be that we have a type of break point that is not supported
     // by the expression code.
 
@@ -243,14 +245,15 @@ unsigned int cmd_break::set_break(cmd_options *co, ExprList_t *pEL, bool bLog)
 
 }
 
+
 //------------------------------------------------------------------------
-// set_break(cmd_options *co, 
+// set_break(cmd_options *co,
 //           Expression *pExpr1,
 //           Expression *pExpr2)
 //
 // Given two expressions, this function will call the set
 
-unsigned int cmd_break::set_break(cmd_options *co, 
+unsigned int cmd_break::set_break(cmd_options *co,
 				  Expression *pExpr1,
 				  Expression *pExpr2,
 				  bool bLog)
@@ -268,7 +271,7 @@ unsigned int cmd_break::set_break(cmd_options *co,
 
   // See if the expression supports break points. If it does, the break points
   // will get set and the expressions deleted.
-  int i = pExpr1 ? pExpr1->set_break(MapBreakActions(co->value), 
+  int i = pExpr1 ? pExpr1->set_break(MapBreakActions(co->value),
 				     (bLog ? gpsimObject::eActionLog : gpsimObject::eActionHalt), pExpr2) : -1;
   if (i>=0) {
     get_bp().dump1(i);
@@ -352,7 +355,7 @@ unsigned int cmd_break::set_break(int bit_flag, bool bLog)
 
   return b;
 }
- // attribute breakpoints 	 
+ // attribute breakpoints
 unsigned int cmd_break::set_break(gpsimObject *v)
 {
   if (v)
