@@ -29,16 +29,15 @@ Boston, MA 02111-1307, USA.  */
 
 static cmd_options cmd_help_options[] =
 {
-  0,0,0
+  { 0, 0, 0 }
 };
 
 cmd_help help;
 
 
-cmd_help::cmd_help(void)
-{ 
-  name = "help";
-
+cmd_help::cmd_help()
+  : command("help",0)
+{
   brief_doc = string("Type help \"command\" for more help on a command");
 
   long_doc = string ("\n\tgpsim is a software simulator for the Microchip PIC microcontrollers\n\
@@ -59,18 +58,18 @@ void cmd_help::help(void)
   for(int i=0; i<number_of_commands; i++)
     {
       command * pCmd = command_list[i];
-      cout << pCmd->name;
-      int l = 16 - strlen(pCmd->name);
-      if(pCmd->abbreviation != 0) {
-        cout << ":" << pCmd->abbreviation;
-        l -= strlen(pCmd->abbreviation) + 1;
+      cout << pCmd->name();
+      int l = 16 - strlen(pCmd->name());
+      if(pCmd->abbreviation() != 0) {
+        cout << ":" << pCmd->abbreviation();
+        l -= strlen(pCmd->abbreviation()) + 1;
       }
 
       for(int k=0; k<l; k++)
         cout << ' ';
 
       cout << pCmd->brief_doc << '\n';
-	
+
     }
 }
 
@@ -79,14 +78,14 @@ void cmd_help::help(const char *cmd)
 {
   command * pCmd = search_commands(cmd);
   if(pCmd != 0) {
-	  cout << pCmd->long_doc << '\n';
-	  return;
-	}
+          cout << pCmd->long_doc << '\n';
+          return;
+        }
   cout << cmd << " is not a valid gpsim command. Try these instead:\n";
   help();
 }
 
-void cmd_help::help(Value *s)
+void cmd_help::help(gpsimObject *s)
 {
   if(s) {
     cout << s->toString() << endl;
