@@ -38,10 +38,9 @@ static cmd_options cmd_run_options[] =
 };
 
 
-cmd_run::cmd_run(void)
+cmd_run::cmd_run()
+  : command("run",0)
 { 
-  name = "run";
-
   brief_doc = string("Initiate the simulation");
 
   long_doc = string ("run\n\
@@ -55,7 +54,7 @@ cmd_run::cmd_run(void)
 void cmd_run::run()
 {
 
-  Integer &verbosity = *get_symbol_table().findInteger("sim.verbosity");
+  Integer &verbosity = *globalSymbolTable().findInteger("sim.verbosity");
 
   get_interface().start_simulation();
 
