@@ -268,7 +268,6 @@ void catch_control_c(int sig)
 
 void initialize_threads(void)
 {
-#if GLIB_MAJOR_VERSION >= 2
   if( !g_thread_supported() )
   {
 #if GLIB_MINOR_VERSION < 32
@@ -278,8 +277,6 @@ void initialize_threads(void)
     gdk_threads_init();
 #endif
   }
-
-#endif
 }
 
 void initialize_signals(void)
@@ -505,7 +502,8 @@ void process_command_file(const char * file_name, bool bCanChangeDirectory)
       strncpy(directory,file_name,dir_path_end-file_name);
       directory[dir_path_end-file_name]=0;
       printf("directory is \"%s\"\n",directory);
-      chdir(directory);
+      if(chdir(directory) < 0)
+          perror(directory);
       file_name=dir_path_end+1;
       printf("file_name is \"%s\"\n",file_name);
   }
@@ -554,10 +552,10 @@ void process_command_file(const char * file_name, bool bCanChangeDirectory)
       cout << file_name;
       cout << endl;
       char cw[_MAX_PATH];
-      GETCWD(cw, _MAX_PATH);
-      cout << "current working directory is ";
-      cout << cw;
-      cout << endl;
+      if (GETCWD(cw, _MAX_PATH) == 0)
+	perror("getcwd()");
+      else
+          cout << "current working directory is " << cw << endl;
 
     }
 
@@ -620,10 +618,12 @@ gpsim_read (char *buf, unsigned max_size)
 
 
   LLInput *d = Stack ? Stack->GetNext() : 0;
+  if (Stack && verbose)
+	Stack->print();
 
   if (!d || !d->data) {
-  if(verbose&4)
-    cout <<"gpsim_read -- no more data\n";
+    if(verbose&4)
+      cout <<"gpsim_read -- no more data\n";
     return 0;
   }
 
@@ -632,7 +632,7 @@ gpsim_read (char *buf, unsigned max_size)
 
   char *cPstr = d->data;
   unsigned int count = strlen(cPstr);
-  count = (count < max_size) ? count : max_size;
+  count = (count < max_size) ? count : max_size-1;
 
   strncpy(buf, cPstr, count);
   buf[count] = 0;
@@ -874,11 +874,7 @@ static int gpsim_rl_getc(FILE *in)
   gchar buf[6];
   gsize bytes_read;
 
-#if GLIB_MAJOR_VERSION >= 2
   g_io_channel_read_chars(channel, buf, 1, &bytes_read, NULL);
-#else
-  g_io_channel_read(channel, buf, 1, &bytes_read);
-#endif
   return buf[0];
   // JRH, 7-1-2005 - I tried this but am not comfortable that it
   // is working as I intended.
@@ -896,7 +892,7 @@ void initialize_readline (void)
 
   const char *prompt = get_interface().bUsingGUI() ? gpsim_prompt : gpsim_cli_prompt;
 
- 
+
 #ifdef HAVE_READLINE
   // Lets us have a gpsim section to .inputrc
   // JRH - not tested
@@ -913,11 +909,6 @@ void initialize_readline (void)
 #endif
 
 #ifdef _WIN32
-#if GLIB_MAJOR_VERSION < 2 || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 6)
-  /* set console to raw mode */
-  win32_set_is_readable(channel);
-#endif
-
   // The channel is not readable if it is a redirected
   // standard input, in which case we will not be getting
   // any keypress events.
@@ -939,13 +930,14 @@ void initialize_readline (void)
 
 #if defined(HAVE_READLINE) && defined(HAVE_PERL)
 // JRH - An experiment
-void EnableKeypressHook(bool bEnable) {
-  if(bEnable) {
+void EnableKeypressHook(bool bEnable)
+{
+  if (bEnable) {
     g_iWatchSourceID = g_io_add_watch (channel, G_IO_IN, keypressed, NULL);
   }
   else {
 //    g_source_remove_by_funcs_user_data(keypressed, NULL);
-    bEnable = g_source_remove(g_iWatchSourceID);
+    g_source_remove(g_iWatchSourceID);
     g_io_channel_unref(channel);
   }
 }
@@ -978,7 +970,7 @@ int CCliCommandHandler::ExecuteScript(list<string *> &script, ISimConsole *out)
   if (verbose & 4)
     cout << "GCLICommandHandler::Execute Script:" << endl;
 
-  if (script.size() == 0)
+  if (script.empty())
     return CMD_ERR_OK;
 
   // We need to execute the script now. There may be other commands
