@@ -21,6 +21,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include <string>
 #include <iostream>
+#include <memory>
 
 #ifndef _WIN32
 #include <pthread.h>
@@ -210,7 +211,7 @@ bool ParseSocketLink(Packet *buffer, SocketLink **);
 class SocketInterface : public Interface
 {
 private:
-  Socket *sock;
+//  Socket *sock;
 
 public:
 
@@ -494,13 +495,11 @@ void Socket::AssignChannel(gboolean (*server_function)(GIOChannel *,GIOCondition
     GIOChannel *channel = g_io_channel_unix_new(my_socket->getSocket());
     GIOCondition condition = (GIOCondition)(G_IO_IN | G_IO_HUP | G_IO_ERR);
 
-#if GLIB_MAJOR_VERSION >= 2
     GError *err = NULL;
 
     g_io_channel_set_encoding (channel, NULL, &err);
 #if !defined _WIN32 || defined _DEBUG
         g_io_channel_set_flags (channel, G_IO_FLAG_SET_MASK, &err);
-#endif
 #endif
 
     g_io_add_watch(channel,
@@ -513,14 +512,12 @@ void Socket::AssignChannel(gboolean (*server_function)(GIOChannel *,GIOCondition
 
 void Socket::Close(SocketBase **sock)
 {
-  if (sock && *sock)
+  if (sock && *sock) {
     (*sock)->Close();
-
-  *sock = 0;
-
+    *sock = 0;
+  }
 }
 
-
 void Socket::Bind()
 {
   if(!my_socket)
@@ -780,7 +777,7 @@ void SocketBase::ParseObject()
     break;
 
   default:
-    printf("Invalid object type: %d\n",ObjectType);
+    printf("Invalid object type: %u\n", ObjectType);
     Send("-");
 
   }
@@ -820,7 +817,7 @@ void SocketBase::Service()
 // Socket Interface
 
 SocketInterface::SocketInterface(Socket *new_socket)
-  : sock(new_socket)
+//  : sock(new_socket)
 {
 }
 void SocketInterface::SimulationHasStopped (gpointer object)
@@ -1034,9 +1031,6 @@ void start_server(void)
 }
 #else   // if USE_THREADS
 
-
-#if GLIB_MAJOR_VERSION >= 2
-
 static void debugPrintChannelStatus(GIOStatus stat)
 {
   switch (stat) {
@@ -1072,7 +1066,6 @@ static void debugPrintCondition(GIOCondition cond)
     std::cout << "  G_IO_NVAL\n";
 }
 #endif
-#endif
 
 /*========================================================================
   server_callback ()
@@ -1101,12 +1094,11 @@ static gboolean server_callback(GIOChannel *channel,
   if(condition & G_IO_HUP) {
     std::cout<< "client has gone away\n";
 
-#if GLIB_MAJOR_VERSION >= 2
     GError *err=NULL;
     GIOStatus stat = g_io_channel_shutdown(channel, TRUE, &err);
     std::cout << "channel status " << hex << stat << "  " ;
     debugPrintChannelStatus(stat);
-#endif
+
     delete s;
 
     return FALSE;
@@ -1118,8 +1110,6 @@ static gboolean server_callback(GIOChannel *channel,
     s->packet->prepare();
     memset(s->packet->rxBuff(), 0, 256);
 
-#if GLIB_MAJOR_VERSION >= 2
-
     GError *err=NULL;
     gsize b;
 
@@ -1138,11 +1128,6 @@ static gboolean server_callback(GIOChannel *channel,
       std::cout << "GError:" << err->message << endl;
     }
 
-
-#else
-    g_io_channel_read(channel, s->packet->rxBuff(), BUFSIZE, &bytes_read);
-#endif
-
     if(bytes_read) {
       if(get_interface().bSimulating()) {
         std::cout << "setting a socket break point because sim is running \n";
@@ -1187,14 +1172,12 @@ static gboolean sink_server_accept(GIOChannel *channel, GIOCondition condition,
   GIOChannel *new_channel = g_io_channel_unix_new(client->getSocket());
   GIOCondition new_condition = (GIOCondition)(G_IO_IN | G_IO_HUP | G_IO_ERR);
 
-#if GLIB_MAJOR_VERSION >= 2
   GError *err = NULL;
 
   g_io_channel_set_encoding (channel, NULL, &err);
 #if !defined _WIN32 || defined _DEBUG
   //stat = g_io_channel_set_flags (channel, G_IO_FLAG_SET_MASK, &err);
   g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, &err);
-#endif
 #endif
 
   g_io_add_watch(new_channel,
@@ -1236,6 +1219,8 @@ static gboolean source_server_accept(GIOChannel *channel, GIOCondition condition
 
   std::cout << " SourceServer serviced client\n";
 
+  // FIXME: SocketBase *client goes out of scope almost certainly leaking resources.
+
   return TRUE;
 }
 
