@@ -34,6 +34,9 @@ Boston, MA 02111-1307, USA.  */
 
 
 #include "hd44780.h"
+
+#include <iostream>
+
 //#define DEBUG
 #if defined(DEBUG)
 #define Dprintf(arg) {printf("%s:%d ",__FILE__,__LINE__); printf arg; }
@@ -187,13 +190,15 @@ HD44780::HD44780()
     m_bCursorBlink(false),// no cursor blink
     m_bCursorOn(false),   // cursor is off
     m_bDataBusPhase(false),
-    m_busyTimer(new HD44780Busy()),
     m_DDRamAdd(0),
     m_CGRamAdd(0),
     m_bInCGRam(false),
     m_CGRamupdate(false)
 {
 
+
+  m_busyTimer = new HD44780Busy();
+
   memset(&m_CGRam[0], 0xff, sizeof(m_CGRam));
   memset(&m_DDRam[0], 0xff, sizeof(m_DDRam)/2);
   memset(&m_DDRam[DDRAM_SIZE/2], 0, sizeof(m_DDRam)/2);
@@ -201,10 +206,13 @@ HD44780::HD44780()
   row_offset[1] = 0x40;
   row_offset[2] = 0x14;
   row_offset[3] = 0x54;
-  
+
 
 }
-
+HD44780::~HD44780()
+{
+    delete m_busyTimer;
+}
 
 //------------------------------------------------------------------------
 void HD44780::setDC(bool newDC)
@@ -249,7 +257,7 @@ void HD44780::setE(bool newE)
       executeCommand();
       break;
     default:
-      Dprintf((" unhandled control state:%d\n",m_controlState));
+      Dprintf((" unhandled control state:%u\n", m_controlState));
     }
   }
 
@@ -351,7 +359,7 @@ void HD44780::executeCommand()
   //
   // Determine the command type
   //
-  Dprintf(("Execute Command:0x%d\n",command));
+  Dprintf(("Execute Command:0x%u\n", command));
   if( (command & LCD_MASK_SET_DDRAM) ==  LCD_CMD_SET_DDRAM) {
     Dprintf(("LCD_CMD_SET_DDRAM\n"));
     writeDDRamAddress(command & DDRAM_MASK);
@@ -533,8 +541,8 @@ void HD44780::debugChipState(const char *pCFrom)
 {
 #ifdef DEBUG
   printf("Chip state from %s\n",pCFrom);
-  printf(" ControlState: %d  dataBus:0x%x phase:%d\n",
-	 m_controlState,m_dataBus,m_bDataBusPhase);
+  printf(" ControlState: %u  dataBus:0x%x phase:%d\n",
+	 m_controlState, m_dataBus, m_bDataBusPhase);
   printf(" Mode: %dbit %dLine Display-%s\n",
 	 (b8BitMode() ? 8 : 4),
 	 (b1LineMode() ? 1 : 2),
