@@ -187,7 +187,6 @@ public:
 #define CURSOR_ON_FLAG          (1<<4)
 #define BLINK_ON_FLAG           (1<<5)
 
-  std::auto_ptr<LCD_Interface> interface;
 
   State current_state, previous_state;
   ControlLineEvent last_event;
@@ -221,7 +220,7 @@ public:
 
   gfloat contrast;  // pixel on/off ratio
 
-  std::auto_ptr<LcdFont> fontP;
+  LcdFont *fontP;
 
   GtkWidget *window;
   GtkWidget *darea;
@@ -291,14 +290,16 @@ public:
   void UpdatePinState(ePins, char);
 protected:
 
-  std::auto_ptr<LCD_InputPin> m_E;
-  std::auto_ptr<LCD_InputPin> m_RW;
-  std::auto_ptr<LCD_InputPin> m_DC;
-  std::auto_ptr<PortRegister> m_dataBus;
+  LCD_InputPin * m_E;
+  LCD_InputPin * m_RW;
+  LCD_InputPin * m_DC;
+  PortRegister *m_dataBus;
+  IO_bi_directional *lcd_bus[8];
 
-  std::auto_ptr<HD44780> m_hd44780;
+  HD44780 *m_hd44780;
   unsigned int m_controlState;
   bool cgram_updated;
+  unsigned int interface_seq_number;
 };
 
 class LcdDisplay20x2 : public LcdDisplay {
