@@ -32,8 +32,6 @@ Boston, MA 02111-1307, USA.  */
 #include "../src/processor.h"
 #include "settings.h"
 
-#define SBAW_NRFILES 100 // Max number of source files
-
 //#define DEBUG
 #if defined(DEBUG)
 #define Dprintf(arg) {printf("%s:%d ",__FILE__,__LINE__); printf arg; }
@@ -41,8 +39,6 @@ Boston, MA 02111-1307, USA.  */
 #define Dprintf(arg) {}
 #endif
 
-#define GTKWAIT { while(gtk_events_pending()) gtk_main_iteration(); }
-
 //------------------------------------------------------------
 //
 // Create structures to generically access the pic-processor
@@ -58,21 +54,18 @@ Boston, MA 02111-1307, USA.  */
 // to reference it. The simulator keeps a linked listed of pointers
 // to all instances of these graphical representations
 
-class CrossReferenceToGUI {
+class CrossReferenceToGUI : public XrefObject
+{
 public:
   gpointer     parent_window;
-  gpointer     data;
 
   CrossReferenceToGUI();
-  virtual ~CrossReferenceToGUI()
-  {
-  }
+  ~CrossReferenceToGUI();
 
   virtual void Update(int new_value) = 0;
   virtual void Remove();
 };
 
-
 #include "gui_object.h"
 #include "gui_processor.h"
 
@@ -81,13 +74,10 @@ class EntryWidget
 {
 public:
   EntryWidget();
-
-  virtual ~EntryWidget()
-  {
-  }
+  virtual ~EntryWidget();
 
   virtual void Update()=0;
-  virtual void Create(bool isEditable=true);
+  virtual void set_editable(bool Editable = true);
   void SetEntryWidth(int string_width);
 
   GtkWidget *entry;
@@ -101,97 +91,29 @@ public:
 
 class LabeledEntry : public EntryWidget {
 public:
-  GtkWidget *label;
-
-  LabeledEntry();
+  LabeledEntry(GtkWidget *box, const char *clabel);
 
   virtual ~LabeledEntry()
   {
   }
 
-  void Create(GtkWidget *box, const char *clabel, int string_width, bool isEditable);
-  void NewLabel(const char *clabel);
   virtual void Update();
   virtual void put_value(unsigned int);
 
+private:
+  GtkWidget *label;
 };
 
 class RegisterLabeledEntry : public LabeledEntry {
 public:
-
-  Register *reg;
-  char pCellFormat[10];
-
-  RegisterLabeledEntry(GtkWidget *,Register *,bool);
-
-  virtual ~RegisterLabeledEntry(){}
+  RegisterLabeledEntry(GtkWidget *, Register *, bool);
+  virtual ~RegisterLabeledEntry();
 
   virtual void put_value(unsigned int);
-  void AssignRegister(Register *new_reg);
   virtual void Update();
-
-};
-
-class CyclesLabeledEntry : public LabeledEntry {
-public:
-
-  CyclesLabeledEntry();
-
-  virtual ~CyclesLabeledEntry()
-  {
-  }
-
-  virtual void Update(void);
-};
-
-
-typedef enum {
-  MENU_TIME_USECONDS,
-  MENU_TIME_MSECONDS,
-  MENU_TIME_SECONDS,
-  MENU_TIME_HHMMSS
-} time_menu_id;
-
-typedef struct _time_menu_item {
-  const char *name;
-  time_menu_id id;
-} time_menu_item;
-
-class TimeLabeledEntry : public LabeledEntry
-{
-public:
-  TimeLabeledEntry();
-
-  virtual ~TimeLabeledEntry()
-  {
-  }
-
-  virtual void Update(void);
-  GtkWidget *build_menu();
-
-  void set_time_format(time_menu_id id)
-  {
-    time_format = id;
-  }
-
-  GtkWidget *menu;
-  time_menu_id time_format;
-
-};
-
-//------------------------------------------------------------------------
-// The KeyEvent class is used for key mapping (e.g. mapping the 's'
-// key to step)
-
-class KeyEvent
-{
-public:
-  virtual ~KeyEvent()
-  {
-  }
-
-  virtual void press(gpointer data = 0) {};
-  virtual void release(gpointer data = 0) {};
+private:
+  Register *reg;
+  char pCellFormat[10];
 };
 
 //
@@ -201,13 +123,7 @@ public:
 extern GtkUIManager *ui;
 extern GUI_Processor *gpGuiProcessor;
 
-void exit_gpsim(void);
-
-void update_menu_item(GUI_Object *_this);
-
-
-void SourceBrowser_update_line(struct cross_reference_to_gui *xref, int new_value);
-
+void exit_gpsim();
 
 // Configuration -- records window states.
 extern Settings *settings;
@@ -220,9 +136,6 @@ int config_remove(const char *module, const char *entry);
 
 gboolean gui_object_configure_event(GtkWidget *widget, GdkEventConfigure *e, GUI_Object *go);
 
-
-void ProfileWindow_notify_start_callback(Profile_Window *pw);
-void ProfileWindow_notify_stop_callback(Profile_Window *pw);
 int gui_get_value(const char *prompt);
 
 #endif // __GUI_H__
