@@ -89,7 +89,12 @@ static void treeselect_module(GtkItem *item, GuiModule *p);
 /*
  board matrix contains information about how a track can be routed.
  */
-static unsigned char board_matrix[XSIZE][YSIZE];
+static unsigned char *board_matrix;
+// mask_matrix is used by trace_two_poins to know where is has been, and
+// how quickly it came here. (depth is stored here if lower)
+static unsigned short *mask_matrix;
+static unsigned int xsize;
+static unsigned int ysize;
 
 
 //========================================================================
@@ -123,23 +128,50 @@ BB_ModuleLabel::~BB_ModuleLabel()
   gtk_widget_destroy(m_label);
 }
 
+static inline unsigned char *board_matrix_pt(unsigned int x, unsigned int y)
+{
+    if (x < xsize && y < ysize)
+    {
+        return board_matrix + y*xsize + x;
+    }
+    else
+    {
+        return  NULL;
+    }
+}
+
+static inline unsigned short *mask_matrix_pt(unsigned int x, unsigned int y)
+{
+    if (x < xsize && y < ysize)
+    {
+        return mask_matrix + (y*xsize + x);
+    }
+    else
+    {
+        return  NULL;
+    }
+}
+
 /* Check the flags in board_matrix to see if we are allowed to
    route horizontally here */
 static inline int allow_horiz(point &p)
 {
-  if(board_matrix[p.x][p.y] & HMASK)
-    return FALSE;
-  return TRUE;
+  unsigned char *pt = board_matrix_pt(p.x, p.y);
+
+  if (pt && ! (*pt & HMASK))
+	return TRUE;
+  return FALSE;
 }
 
 /* Check the flags in board_matrix to see if we are allowed to
    route vertically here */
 static inline int allow_vert(point &p)
 {
-  if(board_matrix[p.x][p.y] & VMASK)
-    return FALSE;
+  unsigned char *pt = board_matrix_pt(p.x, p.y);
 
-  return TRUE;
+  if (pt && ! (*pt & VMASK))
+	return TRUE;
+  return FALSE;
 }
 
 // Find the direction to go to get from s to e if there are no obstacles.
@@ -290,10 +322,6 @@ static void compress_path(path **pat)
 }
 #endif
 
-// mask_matrix is used by trace_two_poins to know where is has been, and
-// how quickly it came here. (depth is stored here if lower)
-static unsigned short mask_matrix[XSIZE][YSIZE];
-
 // maxdepth is shortest path from start to end
 static unsigned short maxdepth;
 
@@ -322,9 +350,9 @@ static int trace_two_points(path **pat,   // Pointer to resulting path
         int x,y;
         // Initialize mask_matrix and maxdepth
         //maxdepth=500;
-        for(x=0;x<XSIZE;x++)
-            for(y=0;y<YSIZE;y++)
-                mask_matrix[x][y]=maxdepth;
+        for(x=0;x<(int)xsize;x++)
+            for(y=0;y<(int)ysize;y++)
+                *mask_matrix_pt(x, y) = maxdepth;
 
         clear_path(pat);
         calls=0;
@@ -337,7 +365,7 @@ static int trace_two_points(path **pat,   // Pointer to resulting path
     ////////////////////////////////////////
     if(depth>maxdepth)
         return FALSE;
-    if(depth>mask_matrix[p.x][p.y])
+    if(depth>*mask_matrix_pt(p.x, p.y))
         return FALSE;
     if(abs(p.x-end.x)+abs(p.y-end.y)+depth>maxdepth)
         return FALSE;
@@ -360,7 +388,7 @@ static int trace_two_points(path **pat,   // Pointer to resulting path
     }
 
     // Store new (closer) depth in mask_matrix.
-    mask_matrix[p.x][p.y]=depth;
+    *mask_matrix_pt(p.x, p.y) = depth;
 
     // Find the general direction we want to go
     dir = calculate_route_direction(p,end);
@@ -587,23 +615,38 @@ void Breadboard_Window::update_board_matrix()
     int x,y, width, height;
     int i;
 
-    // Clear first.
-    for(y=YSIZE-1;y>=0;y--)
+    gtk_window_get_size(GTK_WINDOW(window), &width, &height);
+    if (width/ROUTE_RES > (int)xsize || height/ROUTE_RES > (int)ysize)
     {
-        for(x=0;x<XSIZE;x++)
-            board_matrix[x][y]=0;
+        xsize = width/ROUTE_RES;
+        ysize = height/ROUTE_RES;
+        void *realloc_temp = realloc(board_matrix, xsize * ysize);
+        if (realloc_temp) {
+          board_matrix  = (unsigned char *)realloc_temp;
+        }
+        realloc_temp = realloc(mask_matrix, xsize * ysize * sizeof(unsigned short));
+        if (realloc_temp) {
+          mask_matrix  = (unsigned short *)realloc_temp;
+        }
+    }
+
+    // Clear first.
+    for(y=ysize-1;y>=0;y--)
+    {
+        for(x=0; x < (int)xsize; x++)
+            *board_matrix_pt(x, y) = 0;
     }
 
     // Mark board outline, so we limit traces here
-    for(x=0;x<XSIZE;x++)
+    for(x=0; x < (int)xsize; x++)
     {
-        board_matrix[x][0]=(HMASK|VMASK);
-        board_matrix[x][YSIZE-1]=(HMASK|VMASK);
+        *board_matrix_pt(x, 0) = (HMASK|VMASK);
+        *board_matrix_pt(x, ysize-1) = (HMASK|VMASK);
     }
-    for(y=0;y<YSIZE;y++)
+    for(y=0; y < (int)ysize; y++)
     {
-        board_matrix[0][y]=(HMASK|VMASK);
-        board_matrix[XSIZE-1][y]=(HMASK|VMASK);
+        *board_matrix_pt(0, y) = (HMASK|VMASK);
+        *board_matrix_pt(xsize-1, y) = (HMASK|VMASK);
     }
 
 
@@ -620,13 +663,16 @@ void Breadboard_Window::update_board_matrix()
         height=p->height();
 
         for(y = p->y() - ROUTE_RES;
-            y < p->y() + height + ROUTE_RES && y/ROUTE_RES < YSIZE;
+            y < p->y() + height + ROUTE_RES && y/ROUTE_RES < (int)ysize;
             y += ROUTE_RES)
         {
           for(x = p->x();
-              x < p->x() + width && x/ROUTE_RES<XSIZE;
+              x < p->x() + width && x/ROUTE_RES < (int)xsize;
               x += ROUTE_RES)
-                board_matrix[x/ROUTE_RES][y/ROUTE_RES]=(HMASK|VMASK);
+          {
+               unsigned char *pt = board_matrix_pt(x/ROUTE_RES, y/ROUTE_RES);
+               if(pt) *pt = (HMASK|VMASK);
+          }
         }
 
         // Draw barriers around pins so the tracker can only get in
@@ -643,13 +689,19 @@ void Breadboard_Window::update_board_matrix()
               for(x = gp->x() -  PINLENGTH;
                   x < gp->x() + gp->width();
                   x += ROUTE_RES)
-                board_matrix[x/ROUTE_RES][y/ROUTE_RES]=(HMASK|VMASK);
+              {
+                unsigned char *pt = board_matrix_pt(x/ROUTE_RES, y/ROUTE_RES);
+                if (pt) *pt = (HMASK|VMASK);
+              }
 
               y = gp->y() + gp->height() / 2;
               for(x = gp->x() -  PINLENGTH;
                   x < gp->x() + gp->width();
                   x += ROUTE_RES)
-                board_matrix[x/ROUTE_RES][y/ROUTE_RES]=(HMASK|VMASK);
+              {
+                unsigned char *pt = board_matrix_pt(x/ROUTE_RES, y/ROUTE_RES);
+                if (pt) *pt = (HMASK|VMASK);
+              }
               break;
 
             case RIGHT:
@@ -657,12 +709,18 @@ void Breadboard_Window::update_board_matrix()
               for(x = gp->x() - PINLENGTH;
                   x < gp->x() + gp->width();
                   x += ROUTE_RES)
-                board_matrix[x/ROUTE_RES][y/ROUTE_RES]=(HMASK|VMASK);
+	      {
+                unsigned char *pt = board_matrix_pt(x/ROUTE_RES, y/ROUTE_RES);
+                if (pt) *pt = (HMASK|VMASK);
+               }
               y = gp->y() + gp->height() / 2;
               for(x = gp->x() - PINLENGTH;
                   x < gp->x() + gp->width();
                   x += ROUTE_RES)
-                board_matrix[x/ROUTE_RES][y/ROUTE_RES]=(HMASK|VMASK);
+	      {
+                unsigned char *pt = board_matrix_pt(x/ROUTE_RES, y/ROUTE_RES);
+                if (pt) *pt = (HMASK|VMASK);
+              }
               break;
             default:
               assert(0);
@@ -688,10 +746,11 @@ static void add_path_to_matrix(path *pat)
     }
     while(pat!=0)
     {
-        if(pat->dir==R_LEFT || pat->dir==R_RIGHT)
-            board_matrix[x][y]|=HMASK;
-        if(pat->dir==R_DOWN || pat->dir==R_UP)
-            board_matrix[x][y]|=VMASK;
+        unsigned char *pt = board_matrix_pt(x, y);
+        if(pt && (pat->dir==R_LEFT || pat->dir==R_RIGHT))
+            *pt |= HMASK;
+        if(pt && (pat->dir==R_DOWN || pat->dir==R_UP))
+            *pt |= VMASK;
         while(x!=pat->p.x || y!=pat->p.y)
         {
             if(x<pat->p.x)
@@ -702,10 +761,11 @@ static void add_path_to_matrix(path *pat)
                 y++;
             if(y>pat->p.y)
                 y--;
-            if(pat->dir==R_LEFT || pat->dir==R_RIGHT)
-                board_matrix[x][y]|=HMASK;
-            if(pat->dir==R_DOWN || pat->dir==R_UP)
-                board_matrix[x][y]|=VMASK;
+	    pt = board_matrix_pt(x, y);
+            if(pt && (pat->dir==R_LEFT || pat->dir==R_RIGHT))
+                *pt |= HMASK;
+            if(pt && (pat->dir==R_DOWN || pat->dir==R_UP))
+                *pt |= VMASK;
         }
 
         pat = pat->next;
@@ -842,13 +902,15 @@ static void path_copy_and_cat(path **pat, path **source)
 
 
 
-/*
- Trace a node, and add result to nodepath_list
- */
-static void trace_node(struct gui_node *gn)
+//
+// Trace a node, and add result to nodepath_list
+// return true if OK, false if it could not find a route
+//
+
+static bool trace_node(struct gui_node *gn)
 {
   Breadboard_Window *bbw;
-  int didnt_work=0;
+  bool did_work = true;
 
   point start = {-1, -1}, end;
 
@@ -895,10 +957,10 @@ static void trace_node(struct gui_node *gn)
 //          printf("Trying maxdepth %d\n",maxdepth);
       trace_two_points(&shortest_path[i][j], start, end,0,R_UP);
       if (shortest_path[i][j] == 0) {
-        printf("\n### Couldn't trace from pin %s to pin %s!\n",
-          pi->getIOpin()->name().c_str(),
-          pj->getIOpin()->name().c_str());
-        didnt_work=1;
+        //printf("\n### Couldn't trace from pin %s to pin %s!\n",
+        //  pi->getIOpin()->name().c_str(),
+        //  pj->getIOpin()->name().c_str());
+        did_work = false;
       }
       pathlen[i][j] = maxdepth;
 
@@ -907,14 +969,14 @@ static void trace_node(struct gui_node *gn)
     }
   }
 
-  if (didnt_work) {
-    printf("\n###### Couldn't trace node %s!\n",gn->node->name().c_str());
+  if (!did_work) {
+    //printf("\n###### Couldn't trace node %s!\n",gn->node->name().c_str());
     for (int i = 0; i < (int)pinlist.size(); i++)
       for (int j = i + 1; j < (int)pinlist.size(); j++)
         clear_path(&shortest_path[i][j]);
     delete[] permutations;
     delete[] shortest_permutation;
-    return;
+    return false;
   }
 
     // Find the combination that produces the shortest node.
@@ -964,6 +1026,7 @@ static void trace_node(struct gui_node *gn)
 
     nodepath_list.push_back(nodepath);
   }
+  return true;
 }
 
 
@@ -1022,6 +1085,31 @@ static void treeselect_stimulus(GtkItem *item, GuiPin *pin)
     pin->bbw()->selected_pin = pin;
 }
 
+static stimulus  *stim;
+static string module_name;
+static string full_name;
+static void dumpStimulus(const SymbolEntry_t &sym)
+{
+  stimulus *ps = dynamic_cast<stimulus *>(sym.second);
+
+  if (ps == stim) { 
+    full_name = module_name + "." + ps->name();
+  }
+}
+
+static void scanModules(const SymbolTableEntry_t &st)
+{
+  module_name = st.first;
+  (st.second)->ForEachSymbolTable(dumpStimulus);
+}
+
+static const char * stim_full_name(stimulus *stimulus)
+{
+    stim = stimulus;
+    globalSymbolTable().SymbolTable::ForEachModule(scanModules);
+    return full_name.c_str();
+}
+
 static void treeselect_node(GtkItem *item, struct gui_node *gui_node)
 {
     stimulus *stimulus;
@@ -1058,7 +1146,7 @@ static void treeselect_node(GtkItem *item, struct gui_node *gui_node)
             GtkTreeIter iter;
 
             gtk_list_store_append(list_store, &iter);
-            gtk_list_store_set(list_store, &iter, 0, stimulus->name().c_str(), 1, stimulus, -1);
+            gtk_list_store_set(list_store, &iter, 0, stim_full_name(stimulus), 1, stimulus, -1);
 
             stimulus = stimulus->next;
         }
@@ -1072,16 +1160,16 @@ static void treeselect_cb(GtkTreeSelection *selection,
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
-    
+
   gtk_tree_selection_get_selected(selection, &model, &iter);
-    
+
   if (!iter.stamp)
     return;
-    
+
   GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
-    
+
   gchar *spath = gtk_tree_path_to_string(path);
-    
+
   if (spath[0] == '0') {
     struct gui_node *gn;
     gtk_tree_model_get (model, &iter, 1, &gn, -1);
@@ -1117,7 +1205,7 @@ static void settings_clist_cb(GtkTreeSelection *selection,
         char val[256];
         GtkTreeIter iter;
         GtkTreeModel *model;
-        
+
         gtk_tree_selection_get_selected (selection, &model, &iter);
         if (!iter.stamp) // check if iter is valid
             return;
@@ -1370,6 +1458,7 @@ static GuiModule *find_closest_module(Breadboard_Window *bbw, int x, int y)
 // FIXME
 static GuiModule *dragged_module=0;
 static int dragging=0;
+static int all_trace = 0;
 static int grab_next_module=0;
 
 void grab_module(GuiModule *p)
@@ -1389,11 +1478,12 @@ void grab_module(GuiModule *p)
     gtk_widget_set_app_paintable(p->bbw()->layout, FALSE);
 }
 
-static void pointer_cb(GtkWidget *w,
+static void trace_all(GtkWidget *button, Breadboard_Window *bbw);
+void Breadboard_Window::pointer_cb(GtkWidget *w,
                        GdkEventButton *event,
                        Breadboard_Window *bbw)
 {
-    static int x,y;
+    int x,y;
 
     x = (int) (event->x);
     y = (int) (event->y);
@@ -1446,7 +1536,8 @@ static void pointer_cb(GtkWidget *w,
             bbw->update_board_matrix();
             dragging = 0;
             gtk_widget_set_app_paintable(bbw->layout, TRUE);
-            bbw->update_board_matrix();
+	    if (all_trace)
+                 trace_all(w, bbw);
             UpdateModuleFrame(dragged_module, bbw);
         }
         break;
@@ -1577,14 +1668,14 @@ static void select_node_ok_cb(GtkTreeView *tree_view,
     Stimulus_Node* snode;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    
+
     model = gtk_tree_view_get_model (tree_view);
-        
+
     gtk_tree_model_get_iter (model, &iter, path);
     gtk_tree_model_get (model, &iter, 1, &snode, -1);
-    
+
     g_object_set_data ((GObject*)dialog, "snode", snode);
-    
+
     gtk_dialog_response(dialog, GTK_RESPONSE_ACCEPT);
 }
 
@@ -1601,21 +1692,25 @@ static void copy_tree_to_clist(GtkTreeModel *model, GtkListStore *list_store)
     struct gui_node *gn;
     GtkTreeIter node_iter, iter, new_iter;
 
-    gtk_tree_model_get_iter_first (model, &node_iter);
-    gtk_tree_model_iter_n_children (model, &node_iter);
-    gtk_tree_model_iter_children (model, &iter, &node_iter);
-    
-    do
+    if (gtk_tree_model_get_iter_first (model, &node_iter))
     {
-        gtk_tree_model_get (model, &iter, 1, &gn, -1);
-   
-        gtk_list_store_append(list_store, &new_iter);
-        gtk_list_store_set(list_store, &new_iter,
-          0, gn->node->name().c_str(), 1, (gpointer) gn->node, -1);
-        
-        if (!iter.stamp)
-            break;
-    } while (gtk_tree_model_iter_next (model, &iter));
+        if (gtk_tree_model_iter_n_children (model, &node_iter) > 0)
+        {
+            gtk_tree_model_iter_children (model, &iter, &node_iter);
+
+            do
+            {
+                gtk_tree_model_get (model, &iter, 1, &gn, -1);
+
+                gtk_list_store_append(list_store, &new_iter);
+                gtk_list_store_set(list_store, &new_iter,
+                  0, gn->node->name().c_str(), 1, (gpointer) gn->node, -1);
+
+                if (!iter.stamp)
+                    break;
+            } while (gtk_tree_model_iter_next (model, &iter));
+	}
+    }
 }
 
 static Stimulus_Node *select_node_dialog(Breadboard_Window *bbw)
@@ -1678,7 +1773,7 @@ static std::string select_module_dialog(Breadboard_Window *bbw)
     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   const gchar *module_clist_titles[] = {"Name1","Name2", "Library"};
- 
+
 #ifdef OLD_MODULE_LIBRARY
   int n_columns = 2;
   GtkListStore *list_store = gtk_list_store_new(n_columns + 1,
@@ -1762,12 +1857,12 @@ static std::string select_module_dialog(Breadboard_Window *bbw)
         ++mti)
     {
           const char *text[3];
-          
+
           Module_Types * (*get_mod_list)(void) = mti->second->mod_list();
           Module_Types *pLibModList = get_mod_list();
 
           text[2] = mti->second->user_name().c_str();
-          
+
           if(pLibModList)
               for(Module_Types *pModTypes = pLibModList;  pModTypes->names[0]; pModTypes++) {
                 GtkTreeIter iter;
@@ -1955,12 +2050,12 @@ static void remove_module(GtkWidget *button, Breadboard_Window *bbw)
     GtkTreeModel *model;
     GtkTreeSelection *selection;
     GtkTreeIter iter;
-    
+
     selection = gtk_tree_view_get_selection ((GtkTreeView*) bbw->tree);
     gtk_tree_selection_get_selected (selection, &model, &iter);
     gtk_tree_store_set ((GtkTreeStore*) model, &iter, 1, NULL, -1);
     gtk_tree_store_remove ((GtkTreeStore*) model, &iter);
-    
+
     // Remove from local list of modules
     std::vector<GuiModule *>::iterator mi =
       std::find(bbw->modules.begin(), bbw->modules.end(), bbw->selected_module);
@@ -1980,16 +2075,16 @@ static void remove_node(GtkWidget *button, Breadboard_Window *bbw)
     GtkTreeModel *model;
     GtkTreeSelection *selection;
     struct gui_node *gn;
-    
+
     selection = gtk_tree_view_get_selection ((GtkTreeView*) bbw->tree);
     gtk_tree_selection_get_selected (selection, &model, &iter);
     gtk_tree_model_get (model, &iter, 1, &gn, -1);
-    
+
     gtk_tree_store_remove ((GtkTreeStore*) model, &iter);
-    
+
     g_object_set_data (G_OBJECT (bbw->tree), gn->node->name().c_str(), NULL);
     delete gn;
-    
+
     gtk_widget_hide(bbw->node_frame);
     gtk_widget_hide(bbw->stimulus_frame);
     gtk_widget_hide(bbw->module_frame);
@@ -2001,7 +2096,7 @@ static void remove_node_stimulus(GtkWidget *button, Breadboard_Window *bbw)
     GtkTreeSelection *selection;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    
+
     selection = gtk_tree_view_get_selection ((GtkTreeView*) bbw->node_clist);
     gtk_tree_selection_get_selected (selection, &model, &iter);
     gtk_tree_model_get (model, &iter, 1, &s, -1);
@@ -2025,7 +2120,18 @@ static char *gui_get_filename(const char *filename)
     NULL);
 
   gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(dialog), TRUE);
-  gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), filename);
+
+  if (filename)
+  {
+     gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), filename);
+  }
+  else
+  {
+      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), ".");
+      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), "netlist.stc");
+  }
+
+
 
   char *file = NULL;
   if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
@@ -2057,14 +2163,15 @@ static void OneAttribute(const SymbolEntry_t &sym)
       }
 }
 
-
+static char *stc_file = 0;
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 static void save_stc(GtkWidget *button, Breadboard_Window *bbw)
 {
     Module *m;
 
-    char *filename = gui_get_filename("netlist.stc");
+    char *filename = gui_get_filename(stc_file);
+
     if(!filename)
         return;
     if ((fo = fopen(filename, "w")) == NULL)
@@ -2073,6 +2180,8 @@ static void save_stc(GtkWidget *button, Breadboard_Window *bbw)
         g_free(filename);
         return;
     }
+    if (stc_file) free(stc_file);
+    stc_file = strdup(filename);
     g_free(filename);
 
     fprintf(fo, "\n# This file was written by gpsim.\n");
@@ -2139,14 +2248,6 @@ static void save_stc(GtkWidget *button, Breadboard_Window *bbw)
                 m->type(),
                 m->name().c_str());
       }
-/*
-      else
-      {
-        fprintf(fo, "processor %s %s\n",
-        m->type(),
-        m->name().c_str());
-      }
-*/
 
       mod_name = m->name().c_str();
       st->ForEachSymbolTable(OneAttribute);
@@ -2169,7 +2270,7 @@ static void save_stc(GtkWidget *button, Breadboard_Window *bbw)
             fprintf(fo, "attach %s",node->name().c_str());
 
             for (stimulus = node->stimuli; stimulus; stimulus = stimulus->next)
-                fprintf(fo, " %s",stimulus->name().c_str());
+                fprintf(fo, " %s",stim_full_name(stimulus));
         }
         fprintf(fo, "\n\n");
     }
@@ -2184,15 +2285,16 @@ static void save_stc(GtkWidget *button, Breadboard_Window *bbw)
 
 static void clear_traces(GtkWidget *button, Breadboard_Window *bbw)
 {
+  all_trace = 0;
   bbw->update_board_matrix();
 }
-
 static void trace_all(GtkWidget *button, Breadboard_Window *bbw)
 {
     struct gui_node *gn;
     GtkTreeModel *model;
     GtkTreeIter p_iter, c_iter;
-    
+    bool did_work = true;
+
     bbw->update_board_matrix();
 
     if ((model = gtk_tree_view_get_model ((GtkTreeView*) bbw->tree)) == NULL)
@@ -2201,15 +2303,21 @@ static void trace_all(GtkWidget *button, Breadboard_Window *bbw)
 	return;
     if (!gtk_tree_model_iter_children (model, &c_iter, &p_iter))
 	return;
-    
+
     do
     {
         gtk_tree_model_get (model, &c_iter, 1, &gn, -1);
-        trace_node(gn);
-    } while (gtk_tree_model_iter_next (model, &c_iter));    
+        if (!trace_node(gn))
+          did_work = false;
+    } while (gtk_tree_model_iter_next (model, &c_iter));
 
     bbw->draw_nodes();
+    if (!did_work)
+      gtk_label_set_text(GTK_LABEL(bbw->status_line), "Can not trace all nodes");
+    else
+      gtk_label_set_text(GTK_LABEL(bbw->status_line), "");
 
+    all_trace = 1;
     if (verbose)
         puts("Trace all is done.");
 }
@@ -2756,7 +2864,7 @@ void GuiModule::Build()
 
   GtkTreeIter module_titer, pin_titer;
   GtkTreeStore *tree_store;
-  
+
   g_object_get (m_bbw->tree, "model", &tree_store, NULL);
   gtk_tree_store_append (tree_store, &module_titer, NULL);
   gtk_tree_store_set (tree_store, &module_titer,
@@ -2831,7 +2939,7 @@ void GuiModule::Build()
   // Create xref
   cross_reference = new BreadBoardXREF();
   cross_reference->parent_window = (gpointer) m_bbw;
-  cross_reference->data = (gpointer) 0;
+  cross_reference->data =  0;
   m_module->xref->_add(cross_reference);
 
 
@@ -3025,7 +3133,7 @@ void Breadboard_Window::NodeConfigurationChanged(Stimulus_Node *node)
 
   struct gui_node *gn = (struct gui_node*) g_object_get_data (G_OBJECT (tree), node->name().c_str());
   GtkTreeStore *tree_store;
-  
+
   g_object_get (tree, "model", &tree_store, NULL);
 
   if (!gn) {
@@ -3047,42 +3155,30 @@ void Breadboard_Window::NodeConfigurationChanged(Stimulus_Node *node)
   }
 }
 
-static GtkWidget *bb_vbox(GtkWidget *window, const char *name)
+static GtkWidget *bb_vbox(GtkWidget *window)
 {
   GtkWidget *vbox = gtk_vbox_new (FALSE, 0);
-  g_object_ref(vbox);
-  g_object_set_data_full (G_OBJECT (window), name, vbox,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (vbox);
 
   return vbox;
 }
 
-static GtkWidget *bb_hbox(GtkWidget *window, const char *name)
+static GtkWidget *bb_hbox(GtkWidget *window)
 {
   GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
-  g_object_ref(hbox);
-  g_object_set_data_full (G_OBJECT (window), name, hbox,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (hbox);
 
   return hbox;
 }
 
-GtkWidget* Breadboard_Window::add_button(const char *label, const char *name,
-                                         GCallback f, GtkWidget *box)
+GtkWidget* Breadboard_Window::add_button(const char *label, GCallback f,
+  GtkWidget *box)
 {
-
   GtkWidget *button = gtk_button_new_with_label (label);
-  g_object_ref(button);
-  g_object_set_data_full (G_OBJECT (window), name, button,
-                            (GDestroyNotify) g_object_unref);
+
   gtk_widget_show (button);
   gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
-  g_signal_connect(button,
-                     "clicked",
-                     G_CALLBACK(f),
-                     this);
+  g_signal_connect(button, "clicked", G_CALLBACK(f), this);
 
   return button;
 }
@@ -3116,20 +3212,15 @@ void Breadboard_Window::Build(void)
   GtkWidget *hbox9;
   GtkWidget *hbox14;
   GtkWidget *scrolledwindow5;
-  
+
   GtkCellRenderer *renderer;
   GtkListStore *list_store;
   GtkTreeStore *tree_store;
   GtkTreeSelection *selection;
 
-  GdkColormap *colormap = gdk_colormap_get_system();
-
   gdk_color_parse("red",&high_output_color);
   gdk_color_parse("green",&low_output_color);
 
-  gdk_colormap_alloc_color(colormap, &high_output_color,FALSE,TRUE);
-  gdk_colormap_alloc_color(colormap, &low_output_color,FALSE,TRUE);
-
   //
   // Top level window
   //
@@ -3138,29 +3229,36 @@ void Breadboard_Window::Build(void)
   g_object_set_data (G_OBJECT (window), "window", window);
   gtk_window_set_title (GTK_WINDOW (window), "Breadboard");
 
+  //
+  // Status line
+  //
+
+  GtkWidget *base_box = gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(window), base_box);
+  status_line = gtk_label_new("");
+  gtk_box_pack_end(GTK_BOX(base_box), status_line, FALSE, FALSE, 0);
+  gtk_widget_show_all(base_box);
+
   //
   // Horizontal pane
   //
 
   hpaned1 = gtk_hpaned_new ();
-  g_object_ref(hpaned1);
-  g_object_set_data_full (G_OBJECT (window), "hpaned1", hpaned1,
-                            (GDestroyNotify) g_object_unref);
+
   gtk_widget_show (hpaned1);
-  gtk_container_add (GTK_CONTAINER (window), hpaned1);
+  gtk_box_pack_end(GTK_BOX(base_box), hpaned1, TRUE, TRUE, 0);
+  //gtk_container_add (GTK_CONTAINER (window), hpaned1);
   gtk_paned_set_position (GTK_PANED (hpaned1), 196);
 
   // vbox9 holds the left pane.
-  vbox9 = bb_vbox(window, "vbox9");
+  vbox9 = bb_vbox(window);
   gtk_paned_pack1 (GTK_PANED (hpaned1), vbox9, FALSE, TRUE);
 
-  vbox13 = bb_vbox(window, "vbox13");
+  vbox13 = bb_vbox(window);
   gtk_box_pack_start (GTK_BOX (vbox9), vbox13, TRUE, TRUE, 2);
 
   scrolledwindow4 = gtk_scrolled_window_new (0, 0);
-  g_object_ref(scrolledwindow4);
-  g_object_set_data_full (G_OBJECT (window), "scrolledwindow4", scrolledwindow4,
-                            (GDestroyNotify) g_object_unref);
+
   gtk_widget_show (scrolledwindow4);
   gtk_box_pack_start (GTK_BOX (vbox13), scrolledwindow4, TRUE, TRUE, 0);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4),
@@ -3181,53 +3279,40 @@ void Breadboard_Window::Build(void)
   g_signal_connect (gtk_tree_view_get_selection ((GtkTreeView*) tree1),
                     "changed", (GCallback) treeselect_cb,
                     NULL);
-  g_object_ref(tree1);
-  g_object_set_data_full (G_OBJECT (window), "tree1", tree1,
-                            (GDestroyNotify) g_object_unref);
+
   gtk_widget_show (tree1);
   gtk_container_add (GTK_CONTAINER (scrolledwindow4), tree1);
 
-  hbox12 = bb_hbox(window, "hbox12");
+  hbox12 = bb_hbox(window);
   gtk_box_pack_start (GTK_BOX (vbox13), hbox12, FALSE, FALSE, 0);
 
-  add_button("Add node","button5", G_CALLBACK(add_new_snode), hbox12);
-  add_button("Add module","button6", G_CALLBACK(add_module), hbox12);
-  add_button("Add library","button7", G_CALLBACK(add_library), hbox12);
+  add_button("Add node", G_CALLBACK(add_new_snode), hbox12);
+  add_button("Add module", G_CALLBACK(add_module), hbox12);
+  add_button("Add library", G_CALLBACK(add_library), hbox12);
 
-  hbox15 = bb_hbox(window, "hbox15");
+
+
+  hbox15 = bb_hbox(window);
   gtk_box_pack_start (GTK_BOX (vbox13), hbox15, FALSE, FALSE, 0);
 
-  add_button("Trace all","button25", G_CALLBACK(trace_all), hbox15);
-  add_button("Clear traces","button26", G_CALLBACK(clear_traces), hbox15);
+  add_button("Trace all", G_CALLBACK(trace_all), hbox15);
+  add_button("Clear traces", G_CALLBACK(clear_traces), hbox15);
 
 
 
   node_frame = gtk_frame_new ("Node connections");
-  g_object_ref(node_frame);
-  g_object_set_data_full (G_OBJECT (window), "node_frame", node_frame,
-                            (GDestroyNotify) g_object_unref);
-  //  gtk_widget_show (node_frame);
   gtk_box_pack_start (GTK_BOX (vbox9), node_frame, TRUE, TRUE, 0);
 
   vbox11 = gtk_vbox_new (FALSE, 0);
-  g_object_ref(vbox11);
-  g_object_set_data_full (G_OBJECT (window), "vbox11", vbox11,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (vbox11);
   gtk_container_add (GTK_CONTAINER (node_frame), vbox11);
 
   scrolledwindow2 = gtk_scrolled_window_new (0, 0);
-  g_object_ref(scrolledwindow2);
-  g_object_set_data_full (G_OBJECT (window), "scrolledwindow2", scrolledwindow2,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (scrolledwindow2);
   gtk_box_pack_start (GTK_BOX (vbox11), scrolledwindow2, TRUE, TRUE, 0);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   viewport7 = gtk_viewport_new (0, 0);
-  g_object_ref(viewport7);
-  g_object_set_data_full (G_OBJECT (window), "viewport7", viewport7,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (viewport7);
   gtk_container_add (GTK_CONTAINER (scrolledwindow2), viewport7);
 
@@ -3245,23 +3330,16 @@ void Breadboard_Window::Build(void)
   gtk_widget_show (node_clist);
   gtk_container_add (GTK_CONTAINER (viewport7), node_clist);
 
-  hbox10 = bb_hbox(window, "hbox10");
+  hbox10 = bb_hbox(window);
   gtk_box_pack_start (GTK_BOX (vbox11), hbox10, FALSE, FALSE, 0);
 
-  add_button("Remove stimulus","rsb", G_CALLBACK(remove_node_stimulus), hbox10);
-  add_button("Remove node","rnb", G_CALLBACK(remove_node), hbox10);
+  add_button("Remove stimulus", G_CALLBACK(remove_node_stimulus), hbox10);
+  add_button("Remove node", G_CALLBACK(remove_node), hbox10);
 
   stimulus_frame = gtk_frame_new ("Stimulus settings");
-  g_object_ref(stimulus_frame);
-  g_object_set_data_full (G_OBJECT (window), "stimulus_frame", stimulus_frame,
-                            (GDestroyNotify) g_object_unref);
-  //  gtk_widget_show (stimulus_frame);
   gtk_box_pack_start (GTK_BOX (vbox9), stimulus_frame, FALSE, FALSE, 0);
 
   vbox14 = gtk_vbox_new (FALSE, 0);
-  g_object_ref(vbox14);
-  g_object_set_data_full (G_OBJECT (window), "vbox14", vbox14,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (vbox14);
   gtk_container_add (GTK_CONTAINER (stimulus_frame), vbox14);
 
@@ -3270,40 +3348,27 @@ void Breadboard_Window::Build(void)
   gtk_widget_show(stimulus_settings_label);
   gtk_box_pack_start(GTK_BOX(vbox14), stimulus_settings_label, FALSE,FALSE,0);
 
-  hbox13 = bb_hbox(window, "hbox13");
+  hbox13 = bb_hbox(window);
   gtk_box_pack_start (GTK_BOX (vbox14), hbox13, FALSE, FALSE, 0);
 
-  add_button("Connect stimulus to node","sanb", G_CALLBACK(stimulus_add_node), hbox13);
+  add_button("Connect stimulus to node", G_CALLBACK(stimulus_add_node), hbox13);
 
 
 
 
   module_frame = gtk_frame_new ("Module settings");
-  g_object_ref (module_frame);
-  g_object_set_data_full (G_OBJECT (window), "module_frame", module_frame,
-                            (GDestroyNotify) g_object_unref);
-  //  gtk_widget_show (module_frame);
   gtk_box_pack_start (GTK_BOX (vbox9), module_frame, TRUE, TRUE, 0);
 
   vbox10 = gtk_vbox_new (FALSE, 0);
-  g_object_ref(vbox10);
-  g_object_set_data_full (G_OBJECT (window), "vbox10", vbox10,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (vbox10);
   gtk_container_add (GTK_CONTAINER (module_frame), vbox10);
 
   scrolledwindow1 = gtk_scrolled_window_new (0, 0);
-  g_object_ref(scrolledwindow1);
-  g_object_set_data_full (G_OBJECT (window), "scrolledwindow1", scrolledwindow1,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (scrolledwindow1);
   gtk_box_pack_start (GTK_BOX (vbox10), scrolledwindow1, TRUE, TRUE, 0);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
   viewport6 = gtk_viewport_new (0, 0);
-  g_object_ref(viewport6);
-  g_object_set_data_full (G_OBJECT (window), "viewport6", viewport6,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (viewport6);
   gtk_container_add (GTK_CONTAINER (scrolledwindow1), viewport6);
 
@@ -3315,23 +3380,16 @@ void Breadboard_Window::Build(void)
                                               "text", 0, NULL);
   g_object_set(attribute_clist, "headers-visible", FALSE, NULL);
 
-
-  g_object_ref (attribute_clist);
-  g_object_set_data_full (G_OBJECT (window), "attribute_clist", attribute_clist,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (attribute_clist);
   gtk_container_add (GTK_CONTAINER (viewport6), attribute_clist);
   selection = gtk_tree_view_get_selection ((GtkTreeView*) attribute_clist);
   g_signal_connect (selection,
                     "changed", (GCallback) settings_clist_cb,
                     this);
-  hbox9 = bb_hbox(window, "hbox9");
+  hbox9 = bb_hbox(window);
   gtk_box_pack_start (GTK_BOX (vbox10), hbox9, FALSE, FALSE, 0);
 
   attribute_entry = gtk_entry_new ();
-  g_object_ref(attribute_entry);
-  g_object_set_data_full (G_OBJECT (window), "attribute_entry", attribute_entry,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (attribute_entry);
   gtk_box_pack_start (GTK_BOX (hbox9), attribute_entry, FALSE, FALSE, 0);
   g_signal_connect(attribute_entry,
@@ -3339,18 +3397,15 @@ void Breadboard_Window::Build(void)
                   G_CALLBACK(settings_set_cb),
                   this);
 
-  add_button("Set","attribute_button", G_CALLBACK(settings_set_cb), hbox9);
+  add_button("Set", G_CALLBACK(settings_set_cb), hbox9);
 
-  hbox14 = bb_hbox(window, "hbox14");
+  hbox14 = bb_hbox(window);
   gtk_box_pack_start (GTK_BOX (vbox10), hbox14, FALSE, FALSE, 0);
 
-  add_button("Remove module","remove_module_button", G_CALLBACK(remove_module), hbox14);
-  add_button("Save Configuration ...","save_stc_button", G_CALLBACK(save_stc), vbox9);
+  add_button("Remove module", G_CALLBACK(remove_module), hbox14);
+  add_button("Save Configuration ...", G_CALLBACK(save_stc), vbox9);
 
   scrolledwindow5 = gtk_scrolled_window_new (0, 0);
-  g_object_ref(scrolledwindow5);
-  g_object_set_data_full (G_OBJECT (window), "scrolledwindow5", scrolledwindow5,
-                            (GDestroyNotify) g_object_unref);
   gtk_widget_show (scrolledwindow5);
   gtk_paned_pack2 (GTK_PANED (hpaned1), scrolledwindow5, TRUE, TRUE);
 
@@ -3358,9 +3413,6 @@ void Breadboard_Window::Build(void)
   hadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolledwindow5));
 
   layout = gtk_layout_new (hadj, vadj);
-  g_object_ref(layout);
-  g_object_set_data_full (G_OBJECT (window), "layout", layout,
-                            (GDestroyNotify) g_object_unref);
   gtk_container_add (GTK_CONTAINER (scrolledwindow5), layout);
   gtk_layout_set_size (GTK_LAYOUT (layout), LAYOUTSIZE_X, LAYOUTSIZE_Y);
 
@@ -3382,7 +3434,14 @@ void Breadboard_Window::Build(void)
 
   gtk_widget_set_app_paintable(layout, TRUE);
   gtk_widget_show (layout);
+  unsigned int rrx, rry;
 
+  gtk_layout_get_size(GTK_LAYOUT(layout), &rrx, &rry);
+
+  xsize = ((width < LAYOUTSIZE_X)?LAYOUTSIZE_X:width)/ROUTE_RES;
+  ysize = ((height < LAYOUTSIZE_Y)?LAYOUTSIZE_Y:height)/ROUTE_RES;
+  board_matrix = (unsigned char *)malloc(xsize*ysize);
+  mask_matrix = (unsigned short *)realloc(NULL, xsize*ysize*sizeof(unsigned short));
   gtk_window_set_default_size(GTK_WINDOW(window), width,height);
   gtk_window_move(GTK_WINDOW(window), x, y);
   gtk_window_set_wmclass(GTK_WINDOW(window),name(),"Gpsim");
@@ -3399,12 +3458,12 @@ void Breadboard_Window::Build(void)
   pinnamefont = pango_font_description_from_string("Courier Bold 8");
 
   cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(window));
-  PangoLayout *layout = pango_cairo_create_layout(cr);
-  pango_layout_set_font_description(layout, pinnamefont);
-  pango_layout_set_text(layout, "9y", -1);
-  pango_layout_get_size(layout, &pinnameheight, NULL);
+  PangoLayout *pang_layout = pango_cairo_create_layout(cr);
+  pango_layout_set_font_description(pang_layout, pinnamefont);
+  pango_layout_set_text(pang_layout, "9y", -1);
+  pango_layout_get_size(pang_layout, &pinnameheight, NULL);
   pinnameheight /= PANGO_SCALE;
-  g_object_unref(layout);
+  g_object_unref(pang_layout);
   cairo_destroy(cr);
 
   if(pinspacing<pinnameheight)
@@ -3473,6 +3532,16 @@ const char *Breadboard_Window::name()
   return "pinout";
 }
 
+Breadboard_Window::~Breadboard_Window()
+{
+   if (mask_matrix)
+       free(mask_matrix);
+   if (board_matrix)
+	free(board_matrix);
+   mask_matrix = 0;
+   board_matrix = 0;
+}
+
 Breadboard_Window::Breadboard_Window(GUI_Processor *_gp)
   : pinstatefont(0), pinnamefont(0), node_clist(0),
     stimulus_settings_label(0), stimulus_add_node_button(0),
@@ -3480,6 +3549,8 @@ Breadboard_Window::Breadboard_Window(GUI_Processor *_gp)
     selected_pin(0), selected_node(0), selected_module(0)
 {
   menu = "/menu/Windows/Breadboard";
+  mask_matrix = 0;
+  board_matrix = 0;
 
   gp = _gp;
 
