@@ -23,10 +23,10 @@ Boston, MA 02111-1307, USA.  */
 #ifdef HAVE_GUI
 
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
 #include <glib.h>
 
 #include <cstring>
+#include <memory>
 #include <string>
 
 #include "gui.h"
@@ -47,7 +47,7 @@ Boston, MA 02111-1307, USA.  */
 
 #include "../cli/input.h"  // for gpsim_open()
 #include "../src/gpsim_interface.h"
-
+#include "../src/processor.h"
 
 GtkUIManager *ui;
 
@@ -61,16 +61,16 @@ do_quit_app(GtkWidget *widget)
 
 
 //========================================================================
+static const gchar * const authors[] = {
+  "Scott Dattalo - <scott@dattalo.com>",
+  "Ralf Forsberg - <rfg@home.se>",
+  "Borut Ra" "\xc5\xbe" "em - <borut.razem@gmail.com>",
+  NULL
+};
+
 static void
 about_cb(GtkAction *action, gpointer user_data)
 {
-  static const gchar *authors[] = {
-    "Scott Dattalo - <scott@dattalo.com>",
-    "Ralf Forsberg - <rfg@home.se>",
-    "Borut Ra" "\xc5\xbe" "em - <borut.razem@gmail.com>",
-    NULL
-  };
-
   gtk_show_about_dialog(NULL,
     "authors", authors,
     "comments", "A simulator for Microchip PIC microcontrollers.",
@@ -96,18 +96,19 @@ class ColorButton
 {
 public:
   ColorButton (GtkWidget *pParent,
-               TextStyle *pStyle,
+               GtkTextTag *pStyle,
                const char *label,
                SourceBrowserPreferences *
                );
+  ~ColorButton();
+
+  void cancel();
+private:
   static void setColor_cb(GtkColorButton *widget,
                           ColorButton    *This);
-  void apply();
-  void cancel();
-  TextStyle *m_pStyle;
-private:
-  SourceBrowserPreferences *m_prefs;
-  const char *m_label;
+
+  GtkTextTag *m_pStyle;
+  GdkColor *old_color;
 };
 
 //========================================================================
@@ -186,7 +187,8 @@ private:
 class SourceBrowserPreferences : public SourceWindow
 {
 public:
-  SourceBrowserPreferences(GtkWidget *pParent);
+  explicit SourceBrowserPreferences(GtkWidget *pParent);
+  ~SourceBrowserPreferences();
 
   void apply();
   void cancel();
@@ -203,25 +205,25 @@ public:
   const char *getFont();
 private:
 
-  ColorButton *m_LabelColor;
-  ColorButton *m_MnemonicColor;
-  ColorButton *m_SymbolColor;
-  ColorButton *m_CommentColor;
-  ColorButton *m_ConstantColor;
+  ColorButton * m_LabelColor;
+  ColorButton * m_MnemonicColor;
+  ColorButton * m_SymbolColor;
+  ColorButton * m_CommentColor;
+  ColorButton * m_ConstantColor;
 
-  MarginButton *m_LineNumbers;
-  MarginButton *m_Addresses;
-  MarginButton *m_Opcodes;
+  MarginButton * m_LineNumbers;
+  MarginButton * m_Addresses;
+  MarginButton * m_Opcodes;
 
   int m_currentTabPosition;
   int m_originalTabPosition;
-  TabButton    *m_Up;
-  TabButton    *m_Left;
-  TabButton    *m_Down;
-  TabButton    *m_Right;
-  TabButton    *m_None;
+  TabButton *m_Up;
+  TabButton *m_Left;
+  TabButton *m_Down;
+  TabButton *m_Right;
+  TabButton *m_None;
 
-  FontSelection *m_FontSelector;
+  FontSelection  *m_FontSelector;
 };
 
 //------------------------------------------------------------------------
@@ -235,6 +237,8 @@ public:
 
 
 private:
+  gpsimGuiPreferences(const gpsimGuiPreferences&);  // Non-copiable
+
   SourceBrowserPreferences *m_SourceBrowser;
 
   static void response_cb(GtkDialog *dialog, gint response_id,
@@ -263,17 +267,18 @@ void gpsimGuiPreferences::response_cb(GtkDialog *dialog, gint response_id,
 
 //------------------------------------------------------------------------
 // ColorButton Constructor
-ColorButton::ColorButton(GtkWidget *pParent, TextStyle *pStyle,
-                         const char *colorName,SourceBrowserPreferences *prefs)
-  : m_pStyle(pStyle),m_prefs(prefs), m_label(colorName)
+ColorButton::ColorButton(GtkWidget *pParent, GtkTextTag *pStyle,
+                         const char *colorName, SourceBrowserPreferences *prefs)
+  : m_pStyle(pStyle)
 {
   GtkWidget *hbox        = gtk_hbox_new(0,0);
   gtk_box_pack_start (GTK_BOX (pParent), hbox, FALSE, TRUE, 0);
 
-  GtkWidget *colorButton = gtk_color_button_new_with_color (pStyle->mFG.CurrentColor());
-  gtk_color_button_set_title (GTK_COLOR_BUTTON(colorButton), colorName);
-  gtk_box_pack_start (GTK_BOX(hbox),colorButton,FALSE, FALSE, 0);
-  gtk_widget_show(colorButton);
+  g_object_get(m_pStyle, "foreground-gdk", &old_color, NULL);
+
+  GtkWidget *colorButton = gtk_color_button_new_with_color(old_color);
+  gtk_color_button_set_title(GTK_COLOR_BUTTON(colorButton), colorName);
+  gtk_box_pack_start(GTK_BOX(hbox), colorButton, FALSE, FALSE, 0);
 
   g_signal_connect (colorButton,
                       "color-set",
@@ -282,9 +287,13 @@ ColorButton::ColorButton(GtkWidget *pParent, TextStyle *pStyle,
 
   GtkWidget *label       = gtk_label_new(colorName);
   gtk_box_pack_start (GTK_BOX(hbox),label,FALSE,FALSE, 10);
-  gtk_widget_show (label);
 
-  gtk_widget_show (hbox);
+  gtk_widget_show_all(hbox);
+}
+
+ColorButton::~ColorButton()
+{
+  gdk_color_free(old_color);
 }
 
 //------------------------------------------------------------------------
@@ -293,16 +302,12 @@ void ColorButton::setColor_cb(GtkColorButton *widget,
 {
   GdkColor newColor;
   gtk_color_button_get_color (widget, &newColor);
-  This->m_pStyle->setFG(&newColor);
+  g_object_set(This->m_pStyle, "foreground-gdk", &newColor, NULL);
 }
 
-void ColorButton::apply()
-{
-  m_pStyle->apply();
-}
 void ColorButton::cancel()
 {
-  m_pStyle->revert();
+  g_object_set(m_pStyle, "foreground-gdk", old_color, NULL);
 }
 
 //------------------------------------------------------------------------
@@ -460,25 +465,26 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
     GtkWidget *colorVbox = gtk_vbox_new(0,0);
     gtk_container_add (GTK_CONTAINER (colorFrame), colorVbox);
 
+    GtkTextTagTable *tag_table = m_pParent->getTagTable();
+
     m_LabelColor    = new ColorButton(colorVbox,
-                                      m_pParent->mLabel,
-                                      "Label", this);
+			gtk_text_tag_table_lookup(tag_table, "Label"),
+      			"Label", this);
     m_MnemonicColor = new ColorButton(colorVbox,
-                                      m_pParent->mMnemonic,
-                                      "Mnemonic", this);
+			gtk_text_tag_table_lookup(tag_table, "Mnemonic"),
+      			"Mnemonic", this);
     m_SymbolColor   = new ColorButton(colorVbox,
-                                      m_pParent->mSymbol,
-                                      "Symbols", this);
+			gtk_text_tag_table_lookup(tag_table, "Symbols"),
+      			"Symbols", this);
     m_ConstantColor = new ColorButton(colorVbox,
-                                      m_pParent->mConstant,
-                                      "Constants", this);
+			gtk_text_tag_table_lookup(tag_table, "Constants"),
+      			"Constants", this);
     m_CommentColor  = new ColorButton(colorVbox,
-                                      m_pParent->mComment,
-                                      "Comments", this);
+			gtk_text_tag_table_lookup(tag_table, "Comments"),
+      			"Comments", this);
 
     // Font selector
-    //preferences_AddFontSelect(GTK_WIDGET(vbox), "Font Selector", "font");
-    m_FontSelector = new FontSelection(vbox,this);
+    m_FontSelector = new FontSelection(vbox, this);
 
     label = gtk_label_new("Font");
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,label);
@@ -503,14 +509,18 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
     gtk_container_add (GTK_CONTAINER (tabFrame), tabVbox);
 
     m_Up    = new TabButton(tabVbox, radioUp, GTK_POS_TOP, this);
-    m_Left  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"left"),
-                           GTK_POS_LEFT, this);
-    m_Down  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"down"),
-                           GTK_POS_BOTTOM, this);
-    m_Right = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"right"),
-                           GTK_POS_RIGHT, this);
-    m_None  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"none"),
-                           -1, this);
+    m_Left  = new TabButton(tabVbox,
+		gtk_radio_button_new_with_label_from_widget (rb,"left"),
+      		GTK_POS_LEFT, this);
+    m_Down  = new TabButton(tabVbox,
+		gtk_radio_button_new_with_label_from_widget (rb,"down"),
+      		GTK_POS_BOTTOM, this);
+    m_Right = new TabButton(tabVbox,
+		gtk_radio_button_new_with_label_from_widget (rb,"right"),
+      		GTK_POS_RIGHT, this);
+    m_None  = new TabButton(tabVbox,
+		gtk_radio_button_new_with_label_from_widget (rb,"none"),
+      		-1, this);
 
 
     // Source browser margin
@@ -520,11 +530,11 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
     gtk_container_add (GTK_CONTAINER (marginFrame), marginVbox);
 
     m_LineNumbers = new MarginButton(marginVbox, "Line Numbers",
-                                     MarginButton::eLineNumbers, this);
+      			MarginButton::eLineNumbers, this);
     m_Addresses   = new MarginButton(marginVbox, "Addresses",
-                                     MarginButton::eAddresses, this);
+      			MarginButton::eAddresses, this);
     m_Opcodes     = new MarginButton(marginVbox, "Opcodes",
-                                     MarginButton::eOpcodes, this);
+			MarginButton::eOpcodes, this);
 
     label = gtk_label_new("Margins");
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook),hbox,label);
@@ -536,7 +546,7 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
 
 
     GtkWidget *frame = gtk_frame_new ("Sample");
-    gtk_box_pack_start (GTK_BOX (pParent), frame, FALSE, TRUE, 0);
+    gtk_box_pack_start (GTK_BOX (pParent), frame, TRUE, TRUE, 0);
 
     m_Notebook = gtk_notebook_new();
     //m_currentTabPosition = m_pParent->getTabPosition();
@@ -547,11 +557,11 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
 
     bIsBuilt = true;
 
-    int id = AddPage (pBuffer, "file1.asm");
+    AddPage (pBuffer, "file1.asm");
 
-    pages[id]->m_pBuffer->parseLine( "        MOVLW   0x34       ; Comment",1);
-    pages[id]->m_pBuffer->parseLine( "; Comment only",1);
-    pages[id]->m_pBuffer->parseLine( "Label:  ADDWF  Variable,F  ; Comment",1);
+    pBuffer->parseLine( "        MOVLW   0x34       ; Comment\n",1);
+    pBuffer->parseLine( "; Comment only\n",1);
+    pBuffer->parseLine( "Label:  ADDWF  Variable,F  ; Comment\n",1);
 
     gtk_widget_show_all(frame);
 
@@ -564,6 +574,24 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
   gtk_widget_show_all(notebook);
 
 }
+SourceBrowserPreferences::~SourceBrowserPreferences()
+{
+  delete m_Left;
+  delete m_Down;
+  delete m_Right;
+  delete m_None;
+  delete m_Up;
+  delete  m_LabelColor;
+  delete  m_MnemonicColor;
+  delete  m_SymbolColor;
+  delete  m_CommentColor;
+  delete  m_ConstantColor;
+
+  delete  m_LineNumbers;
+  delete  m_Addresses;
+  delete  m_Opcodes;
+  delete  m_FontSelector;
+}
 
 void SourceBrowserPreferences::setTabPosition(int tabPosition)
 {
@@ -590,12 +618,6 @@ const char *SourceBrowserPreferences::getFont()
 
 void SourceBrowserPreferences::apply()
 {
-  m_LabelColor->apply();
-  m_MnemonicColor->apply();
-  m_SymbolColor->apply();
-  m_ConstantColor->apply();
-  m_CommentColor->apply();
-
   m_pParent->setTabPosition(m_currentTabPosition);
 }
 
@@ -662,7 +684,41 @@ gpsimGuiPreferences::~gpsimGuiPreferences()
 
 //========================================================================
 
-extern int gui_message(const char *message);
+void gui_message(const char *message);
+
+static void update_preview(GtkFileChooser *file_chooser, gpointer data)
+{
+  gchar *file = gtk_file_chooser_get_preview_filename(file_chooser);
+  gboolean show = FALSE;
+  if (file) {
+    size_t len = strlen(file);
+    if (len >= 4) {
+      gchar *p = file + len - 4;
+      if (!strcmp(p, ".hex") || !strcmp(p, ".HEX")) {
+        show = TRUE;
+      }
+    }
+  }
+  g_free(file);
+  gtk_file_chooser_set_preview_widget_active(file_chooser, show);
+}
+
+static void file_selection_changed(GtkTreeSelection *sel, gpointer data)
+{
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  gchar *processor;
+  gchar **d = (gchar **)data;
+
+  if (gtk_tree_selection_get_selected(sel, &model, &iter)) {
+    gtk_tree_model_get (model, &iter, 0, &processor, -1);
+    gchar *p = *d;
+    if (p) {
+      g_free(p);
+    }
+    *d = processor;
+  }
+}
 
 static void
 fileopen_dialog(GtkAction *action, gpointer user_data)
@@ -674,9 +730,75 @@ fileopen_dialog(GtkAction *action, gpointer user_data)
     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
     NULL);
 
+  GtkFileFilter *filter = gtk_file_filter_new();
+  gtk_file_filter_set_name(filter, "Gpsim");
+  gtk_file_filter_add_pattern(filter, "*.cod");
+  gtk_file_filter_add_pattern(filter, "*.COD");
+  gtk_file_filter_add_pattern(filter, "*.stc");
+  gtk_file_filter_add_pattern(filter, "*.STC");
+  gtk_file_filter_add_pattern(filter, "*.hex");
+  gtk_file_filter_add_pattern(filter, "*.HEX");
+
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+
+  filter = gtk_file_filter_new();
+  gtk_file_filter_add_pattern(filter, "*");
+  gtk_file_filter_set_name(filter, "All files");
+
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+
+  GtkListStore *pic_list = gtk_list_store_new(1, G_TYPE_STRING);
+
+  ProcessorConstructorList::iterator processor_iterator;
+  ProcessorConstructorList *pl = ProcessorConstructor::GetList();
+  for (processor_iterator = pl->begin(); processor_iterator != pl->end();
+    ++processor_iterator) {
+    ProcessorConstructor *p = *processor_iterator;
+    GtkTreeIter iter;
+    gtk_list_store_append(pic_list, &iter);
+    gtk_list_store_set(pic_list, &iter, 0, p->names[1], -1);
+  }
+
+  GtkWidget *scroll_window = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_window),
+    GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+
+  GtkWidget *tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pic_list));
+
+  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), TRUE);
+  GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+  GtkTreeViewColumn *column
+    = gtk_tree_view_column_new_with_attributes("Processor", renderer, "text",
+    0, NULL);
+  gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
+
+  gtk_tree_view_set_enable_search(GTK_TREE_VIEW(tree), TRUE);
+  GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
+
+  gtk_container_add(GTK_CONTAINER(scroll_window), tree);
+
+  gtk_widget_show_all(scroll_window);
+
+  gchar *processor = NULL;
+
+  gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(dialog), scroll_window);
+  gtk_file_chooser_set_use_preview_label(GTK_FILE_CHOOSER(dialog), FALSE);
+  g_signal_connect(selection, "changed", G_CALLBACK(file_selection_changed), (gpointer)&processor);
+  g_signal_connect(dialog, "update-preview", G_CALLBACK(update_preview), NULL);
+
   if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
     char *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
-    if (!gpsim_open(gpGuiProcessor->cpu, filename, 0, 0)) {
+    gchar *use_processor = NULL;
+    if (filename) {
+      size_t len = strlen(filename);
+      if (len >= 4) {
+        gchar *p = filename + len - 4;
+        if (!strcmp(p, ".hex") || !strcmp(p, ".HEX")) {
+          use_processor = processor;
+        }
+      }
+    }
+    if (!gpsim_open(gpGuiProcessor->cpu, filename, use_processor, 0)) {
       gchar *msg = g_strdup_printf(
         "Open failed. Could not open \"%s\"", filename);
       gui_message(msg);
@@ -685,6 +807,9 @@ fileopen_dialog(GtkAction *action, gpointer user_data)
     g_free(filename);
   }
 
+  g_free(processor);
+  g_object_unref(pic_list);
+  gtk_widget_destroy(tree);
   gtk_widget_destroy(dialog);
 }
 
@@ -786,40 +911,19 @@ int gui_animate_delay; // in milliseconds
 
 class UpdateRateMenuItem {
 public:
-
-  char id;
-  int menu_index;
-  bool bRealTime;
-  bool bWithGui;
-  bool bAnimate;
-
-  int update_rate;
-
   UpdateRateMenuItem(GtkWidget *,char, const char *, int update_rate=0, bool _bRealTime=false, bool _bWithGui=false);
 
   void Select();
-  static GtkWidget *menu;
-  static int seq_no;
+
+private:
+  int update_rate;
+public:
+  char id;
+private:
+  bool bAnimate;
+  bool bRealTime;
+  bool bWithGui;
 };
-//========================================================================
-// UpdateRateMenuItem members
-
-GtkWidget * UpdateRateMenuItem::menu = 0;
-int UpdateRateMenuItem::seq_no=0;
-
-map<guint, UpdateRateMenuItem*> UpdateRateMenuItemMap;
-map<guint, UpdateRateMenuItem*> UpdateRateMenuItemIndexed;
-
-
-static void
-gui_update_cb(GtkWidget *widget, gpointer data)
-{
-  GtkComboBox *combo_box = GTK_COMBO_BOX(widget);
-  gint index = combo_box ? gtk_combo_box_get_active(combo_box) : 0;
-
-  UpdateRateMenuItem *umi = UpdateRateMenuItemIndexed[index];
-  umi->Select();
-}
 
 UpdateRateMenuItem::UpdateRateMenuItem(GtkWidget *parent,
                                        char _id,
@@ -827,26 +931,15 @@ UpdateRateMenuItem::UpdateRateMenuItem(GtkWidget *parent,
                                        int _update_rate,
                                        bool _bRealTime,
                                        bool _bWithGui)
-  : id(_id), bRealTime(_bRealTime), bWithGui(_bWithGui), update_rate(_update_rate)
+  : update_rate(_update_rate), id(_id), bRealTime(_bRealTime), bWithGui(_bWithGui)
 {
-
-  if(update_rate <0) {
+  if (update_rate < 0) {
     bAnimate = true;
     update_rate = -update_rate;
   } else
     bAnimate = false;
 
-  if(!menu)
-    menu = gtk_menu_new();
-
   gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(parent), label);
-
-  menu_index = seq_no;
-  seq_no++;
-
-  UpdateRateMenuItemMap[id] = this;
-  UpdateRateMenuItemIndexed[menu_index] = this;
-
 }
 
 void UpdateRateMenuItem::Select()
@@ -1077,7 +1170,7 @@ void TimeFormatter::AddToMenu(GtkWidget *menu,
 
 void TimeWidget::Create(GtkWidget *container)
 {
-  EntryWidget::Create(false);
+  set_editable(false);
 
   gtk_container_add(GTK_CONTAINER(container),entry);
 
@@ -1091,8 +1184,8 @@ void TimeWidget::Create(GtkWidget *container)
   new TimeMilliSeconds(this,menu);
   new TimeSeconds(this,menu);
   new TimeHHMMSS(this,menu);
-  NewFormat(new TimeCyclesHex(this,menu));
-  new TimeCyclesDec(this,menu);
+  new TimeCyclesHex(this,menu);
+  NewFormat(new TimeCyclesDec(this,menu));
 
   // Associate a callback with the user button-click actions
   g_signal_connect(entry,
@@ -1206,50 +1299,39 @@ GtkWidget *dispatcher_window = 0;
 class MainWindow
 {
 public:
-
-  TimeWidget   *timeW;
-
   void Update();
-  void Create();
 
   MainWindow();
+  ~MainWindow();
+
+private:
+  static void gui_update_cb(GtkWidget *widget, gpointer data);
+
+  TimeWidget timeW;
+  std::vector<UpdateRateMenuItem> rate_menu_items;
 };
 
-MainWindow TheWindow;
+MainWindow * TheWindow;
 
-MainWindow::MainWindow()
-  : timeW(0)
+MainWindow::~MainWindow()
 {
 }
 
 void MainWindow::Update()
 {
-  if(timeW)
-   timeW->Update();
+   timeW.Update();
 }
 
-
-//========================================================================
-//========================================================================
-
-void dispatch_Update()
+void MainWindow::gui_update_cb(GtkWidget *widget, gpointer data)
 {
-  if(!dispatcher_window)
-    return;
+  gint index = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
+  MainWindow *main_window = static_cast<MainWindow *>(data);
 
-  if(gpGuiProcessor && gpGuiProcessor->cpu) {
-
-    TheWindow.Update();
-
-  }
+  main_window->rate_menu_items[index].Select();
 }
 
-void MainWindow::Create ()
+MainWindow::MainWindow()
 {
-
-  if (dispatcher_window)
-    return;
-
   GtkWidget *box1;
 
   GtkWidget *buttonbox;
@@ -1355,7 +1437,6 @@ void MainWindow::Create ()
     {
       SimulationMode='4';
     }
-  //  set_simulation_mode(SimulationMode);
 
   //
   // Gui Update Rate
@@ -1364,26 +1445,35 @@ void MainWindow::Create ()
   update_rate_menu = gtk_combo_box_text_new();
   gtk_container_add(GTK_CONTAINER(frame),update_rate_menu);
 
-  new UpdateRateMenuItem(update_rate_menu,'5',"Without gui (fastest simulation)",0);
-  new UpdateRateMenuItem(update_rate_menu,'4',"2000000 cycles/gui update",2000000);
-  new UpdateRateMenuItem(update_rate_menu,'3',"100000 cycles/gui update",100000);
-  new UpdateRateMenuItem(update_rate_menu,'2',"1000 cycles/gui update",1000);
-  new UpdateRateMenuItem(update_rate_menu,'1',"Update gui every cycle",1);
-  new UpdateRateMenuItem(update_rate_menu,'b',"100ms animate",-100);
-  new UpdateRateMenuItem(update_rate_menu,'c',"300ms animate",-300);
-  new UpdateRateMenuItem(update_rate_menu,'d',"700ms animate",-700);
-  new UpdateRateMenuItem(update_rate_menu,'r',"Realtime without gui",0,true);
-  new UpdateRateMenuItem(update_rate_menu,'R',"Realtime with gui",0,true,true);
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'5',"Without gui (fastest simulation)",0));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'4',"2000000 cycles/gui update",2000000));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'3',"100000 cycles/gui update",100000));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'2',"1000 cycles/gui update",1000));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'1',"Update gui every cycle",1));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'b',"100ms animate",-100));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'c',"300ms animate",-300));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'d',"700ms animate",-700));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'r',"Realtime without gui",0,true));
+  rate_menu_items.push_back(
+    UpdateRateMenuItem(update_rate_menu,'R',"Realtime with gui",0,true,true));
 
-  UpdateRateMenuItem *umi = UpdateRateMenuItemMap[SimulationMode];
+  for (size_t i = 0 ; i < rate_menu_items.size(); ++i) {
+    if (rate_menu_items[i].id == SimulationMode) {
+      rate_menu_items[i].Select();
+      gtk_combo_box_set_active(GTK_COMBO_BOX(update_rate_menu), i);
+    }
+  }
 
-  umi->Select();
-
-  gtk_combo_box_set_active(GTK_COMBO_BOX(update_rate_menu), umi->menu_index);
-
-  g_signal_connect(update_rate_menu, "changed",
-                     G_CALLBACK(gui_update_cb),
-                     (gpointer)update_rate_menu);
+  g_signal_connect(update_rate_menu, "changed", G_CALLBACK(gui_update_cb), this);
 
   gtk_box_pack_start (GTK_BOX (buttonbox), frame, FALSE, FALSE, 5);
 
@@ -1394,9 +1484,8 @@ void MainWindow::Create ()
   frame = gtk_frame_new("Simulation Time");
   gtk_box_pack_start (GTK_BOX (buttonbox), frame, FALSE, FALSE, 5);
 
-  timeW = new TimeWidget();
-  timeW->Create(frame);
-  timeW->Update();
+  timeW.Create(frame);
+  timeW.Update();
 
   //gtk_box_pack_start (GTK_BOX (buttonbox), frame, TRUE, TRUE, 5);
 
@@ -1408,15 +1497,21 @@ void MainWindow::Create ()
 
   gtk_box_pack_start (GTK_BOX (box1), button, FALSE, TRUE, 5);
   gtk_widget_show_all (dispatcher_window);
-
-
 }
 
 //========================================================================
 
 void create_dispatcher ()
 {
-  TheWindow.Create();
+  if (!TheWindow)
+    TheWindow = new MainWindow;
+}
+
+void dispatch_Update()
+{
+  if (TheWindow && gpGuiProcessor && gpGuiProcessor->cpu) {
+    TheWindow->Update();
+  }
 }
 
 #endif // HAVE_GUI
