@@ -22,9 +22,6 @@ Boston, MA 02111-1307, USA.  */
 #ifndef __GUI_OBJECT_H__
 #define __GUI_OBJECT_H__
 
-//#include "gui.h"
-
-
 //
 // Forward reference
 //
@@ -45,8 +42,6 @@ class GUI_Object {
   GUI_Processor *gp;
 
   GtkWidget *window;
-  enum window_category wc;
-  enum window_types wt;
 
   const char *menu;
 
@@ -64,28 +59,27 @@ class GUI_Object {
 #define VIEW_SHOW 1
 #define VIEW_TOGGLE 2
 
-  GUI_Object(void);
-  virtual ~GUI_Object(void);
-  virtual void ChangeView(int view_state);
+  GUI_Object();
+  virtual ~GUI_Object();
+  virtual void ChangeView(gboolean view_state);
 
-  int get_config(void);
-  void check(void);
-  int set_default_config(void);
-  virtual int set_config(void);
-  char *name(void);
-  void set_name(const char * new_name);
-  void set_name(string &new_name);
-  virtual void Build(void);
-  virtual int Create(GUI_Processor *_gp);
-  virtual void UpdateMenuItem(void);
-  virtual void Update(void);
+  int get_config();
+  void check();
+  int set_default_config();
+  virtual int set_config();
+
+  virtual void Build() = 0;
+  virtual void UpdateMenuItem();
+  virtual void Update() = 0;
   virtual void NewProcessor(GUI_Processor *_gp)
     {
       gp = _gp;
     }
- private:
-  string name_str;
 
+protected:
+  virtual const char *name() = 0;
+
+private:
 };
 
 
