@@ -24,6 +24,8 @@ Boston, MA 02111-1307, USA.  */
 
 #include "gui_object.h"
 
+#include <gtk/gtk.h>
+
 class GUI_Processor;
 
 //
@@ -42,35 +44,24 @@ struct cycle_histogram_counter{
 
 class Profile_Window : public GUI_Object
 {
- public:
-
+public:
   int program;    // if non-zero window has program
 
   GtkListStore *profile_list;
   GtkWidget *profile_tree;
 
-  GtkListStore *profile_range_list;
-  GtkWidget *profile_range_tree;
-
   GtkListStore *profile_register_list;
   GtkWidget *profile_register_tree;
 
   GtkListStore *profile_exestats_list;
   GtkWidget *profile_exestats_tree;
 
-//  GList *profile_range_list;
-
   GtkWidget *notebook;
-  gint range_current_row;
-  GtkWidget *exestats_popup_menu;
-  GtkWidget *range_popup_menu;
-  GtkWidget *plot_popup_menu;
-  GtkWidget *plot_canvas;
 
   // List of cycle_count structs
   GList *histogram_profile_list;
 
-  Profile_Window(GUI_Processor *gp);
+  explicit Profile_Window(GUI_Processor *gp);
   virtual void Build();
   virtual void Update();
   virtual void NewProcessor(GUI_Processor *gp);
@@ -82,7 +73,5 @@ protected:
   virtual const char *name();
 };
 
-
-
 #endif // __GUI_PROFILE_H__
 
