@@ -20,18 +20,12 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-
 #include "../config.h"
 #ifdef HAVE_GUI
 
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
-#include <string.h>
 
 #include "../src/interface.h"
 #include "../src/trace.h"
@@ -40,10 +34,11 @@ Boston, MA 02111-1307, USA.  */
 #include "gui.h"
 #include "preferences.h"
 #include "gui_register.h"
-#include "gui_statusbar.h"
 #include "gui_regwin.h"
 #include "gui_watch.h"
 
+#include <algorithm>
+#include <stdio.h>
 
 #define TRACE_FILE_FORMAT_ASCII 0
 #define TRACE_FILE_FORMAT_LXT 1
@@ -70,63 +65,12 @@ typedef enum {
     MENU_REGWIN_REFRESH,
 } menu_id;
 
-
-/////////////////////
-/////
-/////  Experimental code to test an object oriented way of implementing menus
-/////
-/////////////////////
-class MenuItem {
-private:
-  char *name;
-  menu_id id;
-
-public:
-  virtual ~MenuItem()
-  {
-  }
-
-  virtual void execute()=0;
-};
-
-class RegWindowMenuItem : public MenuItem {
-public:
-  virtual ~RegWindowMenuItem()
-  {
-  }
-
-private:
- Register_Window *rw;
-};
-
-class Menu_BreakClear : public RegWindowMenuItem {
-public:
-  virtual ~Menu_BreakClear()
-  {
-  }
-
-  virtual void execute() {
-    printf("BreakClear");
-  }
-} BreakClear;
-
-MenuItem *__menu_items[] = {
-  &BreakClear
-};
-
-/////////////////////
-/////
-/////  End of experimental code
-/////
-/////////////////////
-
-
 typedef struct _menu_item {
     const char *name;
-    menu_id id;
+    const menu_id id;
 } menu_item;
 
-static menu_item menu_items[] = {
+static const menu_item menu_items[] = {
     {"Clear breakpoints", MENU_BREAK_CLEAR},
     {"Set break on read", MENU_BREAK_READ},
     {"Set break on write", MENU_BREAK_WRITE},
@@ -143,9 +87,6 @@ static menu_item menu_items[] = {
     {"Settings...", MENU_SETTINGS}
 };
 
-// Used only in popup menus
-Register_Window *popup_rw;
-
 //========================================================================
 //
 
@@ -206,36 +147,32 @@ RegisterValue GUIRegister::getRV()
   return RegisterValue(0,0);
 }
 
-char * GUIRegister::getValueAsString(char *str, int len, char *pFormat,
-                                     RegisterValue value)
+char * GUIRegister::getValueAsString(char *str, int len, RegisterValue value)
 {
-
-  if(!str || !len)
+  if (!str || len <= 0)
     return 0;
 
-  if(bIsValid()) {
+  if (bIsValid()) {
 
-    char hex2ascii[] = "0123456789ABCDEF";
-    int i;
-    int min = (len < register_size*2) ? len : register_size*2;
+    const char hex2ascii[] = "0123456789ABCDEF";
+    const int min = (len < register_size * 2) ? len : register_size * 2;
 
-    if(value.data == INVALID_VALUE)
+    if (value.data == INVALID_VALUE)
       value.init = 0xfffffff;
 
-    for(i=0; i < min; i++) {
-      if(value.init & 0x0f)
-        str[min-i-1] = '?';
+    for (int i = 0; i < min; i++) {
+      if (value.init & 0x0f)
+        str[min - i - 1] = '?';
       else
-        str[min-i-1] = hex2ascii[value.data & 0x0f];
+        str[min - i - 1] = hex2ascii[value.data & 0x0f];
       value >>= 4;
     }
     str[min] = 0;
 
   } else
-    *str = 0;
+    *str = '\0';
 
   return str;
-
 }
 
 bool GUIRegister::hasChanged(RegisterValue &current_value) const
@@ -271,37 +208,24 @@ bool GUIRegister::hasBreak()
   return false;
 }
 
-char *GUIRegister::name()
+std::string GUIRegister::name()
 {
-
-  static char buffer[128];
-
   Register *reg = get_register();
   if (!reg) {
-    sprintf(buffer,"NULL");
-    return buffer;
+    return "NULL";
   }
 
   //register_symbol * pRegSym = get_symbol_table().findRegisterSymbol(reg->address);
 
-  if(!reg || reg->isa()==Register::INVALID_REGISTER)
-    return 0;
+  if (!reg || reg->isa() == Register::INVALID_REGISTER)
+    return "";
 
-  // FIXME -- potential buffer overflow...
-  if(bIsAliased)
-    sprintf(buffer,"alias (%s)", reg->name().c_str());
+  std::string buffer;
+  if (bIsAliased)
+    buffer = "alias (" + reg->name() + ")";
   else
-    sprintf(buffer,"%s", reg->name().c_str());
+    buffer = reg->name();
 
-  /*  else {
-    if(pRegSym == 0) {
-      strcpy(buffer,reg->name().c_str());
-    }
-    else {
-      strcpy(buffer,pRegSym->name().c_str());
-    }
-  }
-  */
   return buffer;
 }
 
@@ -320,9 +244,9 @@ bool GUIRegister::bIsSFR()
 }
 
 GUIRegister::GUIRegister()
+  : rma(0), address(0), row(0), col(0), register_size(0),
+    bUpdateFull(false), bIsAliased(false), xref(0)
 {
-  rma = 0;
-  xref = 0;
 }
 
 GUIRegister::~GUIRegister()
@@ -420,6 +344,61 @@ gtk_sheet_REALLY_set_row_height(GtkSheet *sheet, gint row, gint height)
   sheet->row[row].height = height;
 }
 
+// Sanatize the numeric text input
+// Only checks that the input is less that 0xffff
+
+static void sanatize_numeric(GtkEditable *editable, gchar *new_text,
+  gint new_text_length, gint *position, gpointer user_data)
+{
+  bool ok = true;
+  char *current_text = gtk_editable_get_chars(editable, 0, -1);
+  std::string text(current_text);
+  text.insert(*position, new_text);
+  g_free(current_text);
+
+  if (text != "0x" && text != "0X") {
+    char *end_char;
+    unsigned long value = strtoul(text.c_str(), &end_char, 0);
+    if (value > 0xffff || *end_char != '\0')
+      ok = false;
+  }
+
+  if (ok) {
+    g_signal_handlers_block_by_func(G_OBJECT(editable),
+      gpointer(sanatize_numeric), user_data);
+    gtk_editable_insert_text(editable, new_text, new_text_length, position);
+    g_signal_handlers_unblock_by_func(G_OBJECT(editable),
+      gpointer(sanatize_numeric), user_data);
+  }
+
+  g_signal_stop_emission_by_name(G_OBJECT(editable), "insert-text");
+}
+
+static void sanatize_numeric_0x(GtkEditable *editable, gchar *new_text,
+  gint new_text_length, gint *position, gpointer user_data)
+{
+  bool ok = true;
+  char *current_text = gtk_editable_get_chars(editable, 0, -1);
+  std::string text(current_text);
+  text.insert(*position, new_text);
+  g_free(current_text);
+
+  char *end_char;
+  unsigned long value = strtoul(text.c_str(), &end_char, 16);
+  if (value > 0xffff || *end_char != '\0')
+    ok = false;
+
+  if (ok) {
+    g_signal_handlers_block_by_func(G_OBJECT(editable),
+      gpointer(sanatize_numeric_0x), user_data);
+    gtk_editable_insert_text(editable, new_text, new_text_length, position);
+    g_signal_handlers_unblock_by_func(G_OBJECT(editable),
+      gpointer(sanatize_numeric_0x), user_data);
+  }
+
+  g_signal_stop_emission_by_name(G_OBJECT(editable), "insert-text");
+}
+
 //========================================================================
 // get_value
 // used for reading a value from user when break on value is requested
@@ -444,6 +423,7 @@ int gui_get_value(const char *prompt)
   label = gtk_label_new(prompt);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   GtkWidget *entry = gtk_entry_new();
+  g_signal_connect(entry, "insert-text", G_CALLBACK(sanatize_numeric), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
 
   gtk_widget_show_all(dialog);
@@ -453,27 +433,22 @@ int gui_get_value(const char *prompt)
     return -1;
   }
 
-  const gchar *entry_text;
-  char *end;
-  int value;
+  const gchar *entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
 
-  entry_text = gtk_entry_get_text(GTK_ENTRY(entry));
-  value = strtoul(entry_text, &end, 0);
-
-  gtk_widget_destroy(dialog);
-
-  if (*entry_text != '\0' && *end == '\0')
-    return value;
-  else
+  if (*entry_text == '\0') {
+    gtk_widget_destroy(dialog);
     return -1;
+  }
+
+  int value = strtoul(entry_text, NULL, 0);
+  gtk_widget_destroy(dialog);
+  return value;
 }
 
 // used for reading a value from user when break on value is requested
 void gui_get_2values(const char *prompt1, int *value1, const char *prompt2, int *value2)
 {
-  GtkWidget *dialog;
-
-  dialog = gtk_dialog_new_with_buttons("enter values",
+  GtkWidget *dialog = gtk_dialog_new_with_buttons("enter values",
     NULL,
     GTK_DIALOG_MODAL,
     "_Cancel", GTK_RESPONSE_CANCEL,
@@ -491,6 +466,7 @@ void gui_get_2values(const char *prompt1, int *value1, const char *prompt2, int
   label = gtk_label_new(prompt1);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   entry1 = gtk_entry_new();
+  g_signal_connect(entry1, "insert-text", G_CALLBACK(sanatize_numeric), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), entry1, FALSE, FALSE, 0);
 
   hbox = gtk_hbox_new(FALSE, 6);
@@ -498,6 +474,7 @@ void gui_get_2values(const char *prompt1, int *value1, const char *prompt2, int
   label = gtk_label_new(prompt2);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   entry2 = gtk_entry_new();
+  g_signal_connect(entry2, "insert-text", G_CALLBACK(sanatize_numeric), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), entry2, FALSE, FALSE, 0);
 
   gtk_widget_show_all(dialog);
@@ -509,27 +486,25 @@ void gui_get_2values(const char *prompt1, int *value1, const char *prompt2, int
     return;
   }
 
-  char *end;
   const gchar *entry_text;
-  int value;
 
   entry_text = gtk_entry_get_text(GTK_ENTRY(entry1));
-  value = strtoul(entry_text, &end, 0);
-  if (*entry_text == '\0' || *end != '\0') {
+  if (*entry_text == '\0') {
     *value1 = -1;
     *value2 = -1;
     gtk_widget_destroy(dialog);
+    return;
   }
-  *value1 = value;
+  *value1 = strtoul(entry_text, NULL, 0);
 
   entry_text = gtk_entry_get_text(GTK_ENTRY(entry2));
-  value = strtoul(entry_text, &end, 0);
-  if (*entry_text == '\0' || *end != '\0') {
+  if (*entry_text == '\0') {
     *value1 = -1;
     *value2 = -1;
     gtk_widget_destroy(dialog);
+    return;
   }
-  *value2 = value;
+  *value2 = strtoul(entry_text, NULL, 0);
   gtk_widget_destroy(dialog);
 }
 
@@ -582,11 +557,10 @@ static char *gui_get_log_settings(int *mode)
 
 // called when user has selected a menu item
 static void
-popup_activated(GtkWidget *widget, gpointer data)
+popup_activated(GtkWidget *widget, Register_Window *rw)
 {
   GtkSheet *sheet;
 
-  menu_item *item;
   int i,j;
   GtkSheetRange range;
   unsigned int address;
@@ -596,80 +570,74 @@ popup_activated(GtkWidget *widget, gpointer data)
 
   Dprintf((" popup_activated\n"));
 
-  if(widget==0 || data==0)
-    {
-      printf("Warning popup_activated(%p,%p)\n",widget,data);
-      return;
-    }
-  if(!popup_rw || !popup_rw->gp || !popup_rw->gp->cpu) {
+  if (!rw->gp || !rw->gp->cpu) {
     printf(" no cpu\n");
     return;
   }
 
-  item = (menu_item *)data;
-  sheet=GTK_SHEET(popup_rw->register_sheet);
+  sheet = GTK_SHEET(rw->register_sheet);
   range = sheet->range;
+  gsize item = GPOINTER_TO_SIZE(g_object_get_data(G_OBJECT(widget), "item"));
 
+  for (j = range.row0; j <= range.rowi; j++)
+    for (i = range.col0; i <= range.coli; i++) {
 
-  for(j=range.row0;j<=range.rowi;j++)
-    for(i=range.col0;i<=range.coli;i++) {
-
-      address=popup_rw->row_to_address[j]+i;
-      switch(item->id) {
+      address = rw->row_to_address[j] + i;
+      switch (item) {
       case MENU_BREAK_READ:
-        get_bp().set_read_break(popup_rw->gp->cpu, address);
+        get_bp().set_read_break(rw->gp->cpu, address);
         break;
 
       case MENU_BREAK_WRITE:
-        get_bp().set_write_break(popup_rw->gp->cpu, address);
+        get_bp().set_write_break(rw->gp->cpu, address);
         break;
       case MENU_BREAK_ON_CHANGE:
-        get_bp().set_change_break(popup_rw->gp->cpu, address);
+        get_bp().set_change_break(rw->gp->cpu, address);
         break;
       case MENU_BREAK_READ_VALUE:
         value = gui_get_value("value to read for breakpoint:");
         if(value<0)
           break; // Cancel
-        get_bp().set_read_value_break(popup_rw->gp->cpu,address,value);
+        get_bp().set_read_value_break(rw->gp->cpu,address,value);
         break;
       case MENU_BREAK_WRITE_VALUE:
         value = gui_get_value("value to write for breakpoint:");
         if(value<0)
           break; // Cancel
-        get_bp().set_write_value_break(popup_rw->gp->cpu,address,value);
+        get_bp().set_write_value_break(rw->gp->cpu,address,value);
         break;
       case MENU_BREAK_CLEAR:
-        get_bp().clear_all_register(popup_rw->gp->cpu,address);
+        get_bp().clear_all_register(rw->gp->cpu,address);
         break;
       case MENU_ADD_WATCH:
-        popup_rw->gp->watch_window->Add(popup_rw->type, popup_rw->registers->Get(address));
+        rw->gp->watch_window->Add(rw->type, rw->registers->Get(address));
         break;
       case MENU_LOG_READ:
         GetTraceLog().enable_logging();
         // FIXME the register type is ignored here (and in all other cases
         // where we're logging -- it's assumed that the register address is
         // for ram, even if in fact the user requests eeprom.
-        get_bp().set_notify_read(popup_rw->gp->cpu,address);
+        get_bp().set_notify_read(rw->gp->cpu,address);
         break;
       case MENU_LOG_WRITE:
-        get_bp().set_notify_write(popup_rw->gp->cpu,address);
+        get_bp().set_notify_write(rw->gp->cpu,address);
         break;
       case MENU_LOG_READ_VALUE:
         gui_get_2values("Value that the read must match for logging it:", &value,
                         "Bitmask that specifies the bits to bother about:", &mask);
         if(value<0)
           break; // Cancel
-        get_bp().set_notify_read_value(popup_rw->gp->cpu,address, value, mask);
+        get_bp().set_notify_read_value(rw->gp->cpu,address, value, mask);
         break;
       case MENU_LOG_WRITE_VALUE:
         gui_get_2values("Value that the write must match for logging it:", &value,
                         "Bitmask that specifies the bits to bother about:", &mask);
         if(value<0)
           break; // Cancel
-        get_bp().set_notify_write_value(popup_rw->gp->cpu,address, value, mask);
+        get_bp().set_notify_write_value(rw->gp->cpu,address, value, mask);
         break;
       case MENU_SETTINGS:
-        popup_rw->SettingsDialog();
+        rw->SettingsDialog();
         return;
         break;
       case MENU_LOG_SETTINGS:
@@ -681,7 +649,7 @@ popup_activated(GtkWidget *widget, gpointer data)
         break;
 
       case MENU_REGWIN_REFRESH:
-        popup_rw->Update();
+        rw->Update();
         return;
         break;
       default:
@@ -693,98 +661,67 @@ popup_activated(GtkWidget *widget, gpointer data)
 }
 
 
-static GtkWidget *
-build_menu(Register_Window *rw)
+GtkWidget *Register_Window::build_menu()
 {
-  GtkWidget *menu;
-  GtkWidget *item;
-//  GtkAccelGroup *accel_group;
-  int i;
+  GtkWidget *menu = gtk_menu_new();
 
+  for (gsize i = 0; i < G_N_ELEMENTS(menu_items); i++) {
+    GtkWidget *item = gtk_menu_item_new_with_label(menu_items[i].name);
 
-  if(rw==0)
-  {
-      printf("Warning build_menu(%p)\n",rw);
-      return 0;
-  }
+    g_signal_connect(item, "activate", G_CALLBACK (popup_activated), this);
+    g_object_set_data(G_OBJECT(item), "item",
+      GSIZE_TO_POINTER(menu_items[i].id));
 
-  menu=gtk_menu_new();
-
-  for (i=0; i < (int)(sizeof(menu_items)/sizeof(menu_items[0])) ; i++){
-      item=gtk_menu_item_new_with_label(menu_items[i].name);
-
-      g_signal_connect(item, "activate",
-                         G_CALLBACK (popup_activated),
-                         &menu_items[i]);
-
-      if(rw->type == REGISTER_EEPROM
-         && menu_items[i].id!=MENU_ADD_WATCH
-         &&menu_items[i].id!=MENU_SETTINGS)
-      {
-        gtk_widget_set_sensitive(item, FALSE);
-      }
-      gtk_widget_show(item);
-      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    if (type == REGISTER_EEPROM
+       && menu_items[i].id!=MENU_ADD_WATCH
+       &&menu_items[i].id!=MENU_SETTINGS)
+    {
+      gtk_widget_set_sensitive(item, FALSE);
+    }
+    gtk_widget_show(item);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   }
 
   return menu;
 }
 
 // button press handler
-static gint
-do_popup(GtkWidget *widget, GdkEventButton *event, Register_Window *rw)
+void Register_Window::do_popup(GtkWidget *widget, GdkEventButton *event)
 {
+  int button, event_time;
 
-  GtkWidget *popup;
-  //    GdkModifierType mods;
+  if (event) {
+    button = event->button;
+    event_time = event->time;
+  } else {
+    button = 0;
+    event_time = gtk_get_current_event_time();
+  }
+  gtk_menu_popup(GTK_MENU(popup_menu), 0, 0, 0, 0,
+    button, event_time);
+}
 
-  popup=rw->popup_menu;
+// button press signal handler (static)
 
-  if(widget==0 || event==0 || rw==0)
-    {
-      printf("Warning do_popup(%p,%p,%p)\n",widget,event,rw);
-      return 0;
-    }
+gboolean Register_Window::button_press(GtkWidget *widget,
+  GdkEventButton *event, Register_Window *rw)
+{
+  if (event->button == 3 && event->type == GDK_BUTTON_PRESS) {
+    rw->do_popup(widget, event);
+    return TRUE;
+  }
 
-  GTK_SHEET(widget);
-
-  if( (event->type == GDK_BUTTON_PRESS) &&  (event->button == 3) )
-    {
-
-      popup_rw = rw;
-
-      gtk_menu_popup(GTK_MENU(popup), 0, 0, 0, 0,
-                     3, event->time);
-    }
   return FALSE;
 }
 
-// The following routine will convert the first number it finds in
-// a string to an unsigned long integer. All of the hard work is done
-// in the library function strtoul (string to unsigned long).
+// Popup menu keyboard signal handler (static)
 
-static unsigned long get_number_in_string(const char *number_string)
+gboolean
+Register_Window::popup_menu_handler(GtkWidget *widget,
+  Register_Window *rw)
 {
-  unsigned long retval = 0;
-  char *bad_position;
-  int current_base = 16;
-
-  if(number_string==0)
-  {
-      printf("Warning get_number_in_string(%p)\n",number_string);
-      errno = EINVAL;
-      return (unsigned long)-1;
-  }
-
-
-  errno = 0;
-
-  retval = strtoul(number_string, &bad_position, current_base);
-
-  if( strlen(bad_position) )
-    errno = EINVAL;  /* string contains an invalid number */
-
-  return(retval);
+  rw->do_popup(widget, NULL);
+  return TRUE;
 }
 
 // when a new cell is selected, we write changes in
@@ -824,25 +761,15 @@ set_cell(GtkWidget *widget, int row, int col, Register_Window *rw)
 
   text = gtk_entry_get_text(GTK_ENTRY(sheet_entry));
 
-  errno = 0;
-  if(text!=0 && strlen(text)>0)
-    n = get_number_in_string(text);
-  else
-    errno = ERANGE;
+  n = strtoul(text, NULL, 16);
 
-  if(errno != 0)
-    {
-      n = reg->get_value();
-      reg->put_shadow(RegisterValue(INVALID_VALUE,INVALID_VALUE));
-    }
-
-  // n is the value in the sheet cell
-
-  if(errno != EINVAL && n != (int) reg->get_shadow().data)
-    {
-      reg->put_value(n & gpGuiProcessor->cpu->register_mask());
-      rw->UpdateASCII(row);
-    }
+  if (*text == '\0') {
+    n = reg->get_value();
+    reg->put_shadow(RegisterValue(INVALID_VALUE,INVALID_VALUE));
+  } else if (n != (int) reg->get_shadow().data) {
+    reg->put_value(n & gpGuiProcessor->cpu->register_mask());
+    rw->UpdateASCII(row);
+  }
 
 }
 
@@ -902,9 +829,9 @@ void Register_Window::UpdateLabel()
 
         GUIRegister *reg = getRegister(row,col);
 
-        const char *n = reg ? reg->name() : "INVALID_REGISTER";
+        std::string n = reg ? reg->name() : "INVALID_REGISTER";
 
-        gtk_label_set_text(GTK_LABEL(location), n);
+        gtk_label_set_text(GTK_LABEL(location), n.c_str());
       }
     }
   }
@@ -919,22 +846,19 @@ void Register_Window::UpdateEntry()
 {
   gint row, col;
 
-  const char *text;
-  GtkWidget * sheet_entry;
-
-  if(register_sheet != 0) {
-    sheet_entry = gtk_sheet_get_entry(register_sheet);
+  if (register_sheet != 0) {
+    GtkWidget *sheet_entry = gtk_sheet_get_entry(register_sheet);
     gtk_sheet_get_active_cell(register_sheet, &row, &col);
 
-    if(row_to_address[row] < 0)
+    if (row_to_address[row] < 0)
       return;
 
     GUIRegister *reg = getRegister(row,col);
 
     if(reg && reg->bIsValid() )
     {
-      if((text=gtk_entry_get_text (GTK_ENTRY(sheet_entry))))
-          gtk_entry_set_text(GTK_ENTRY(entry), text);
+      const char *text = gtk_entry_get_text(GTK_ENTRY(sheet_entry));
+      gtk_entry_set_text(GTK_ENTRY(entry), text);
     }
   }
 }
@@ -988,7 +912,7 @@ void Register_Window::UpdateStyle()
 //------------------------------------------------------------------------
 int Register_Window::LoadStyles()
 {
-  normalfont = pango_font_description_from_string(normalfont_string);
+  normalfont = pango_font_description_from_string(normalfont_string.c_str());
 
   if(!normalfont)
   {
@@ -1033,7 +957,7 @@ int Register_Window::SettingsDialog()
   GtkWidget *label = gtk_label_new("Normal font:");
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
-  GtkWidget *font_btn = gtk_font_button_new_with_font(normalfont_string);
+  GtkWidget *font_btn = gtk_font_button_new_with_font(normalfont_string.c_str());
   gtk_box_pack_start(GTK_BOX(hbox), font_btn, FALSE, FALSE, 0);
 
   gtk_widget_show_all(dialog);
@@ -1049,8 +973,8 @@ int Register_Window::SettingsDialog()
 
   if (font) {
     pango_font_description_free(font);
-    strcpy(normalfont_string, font_name);
-    config_set_string(name(), "normalfont", normalfont_string);
+    normalfont_string = font_name;
+    config_set_string(name(), "normalfont", normalfont_string.c_str());
 
     gtk_sheet_freeze(register_sheet);
     UpdateStyle();
@@ -1163,10 +1087,6 @@ show_sheet_entry(GtkWidget *widget, Register_Window *rw)
 {
   Dprintf((" show_sheet_entry\n"));
 
-  const char *text;
-  GtkSheet *sheet;
-  GtkEntry *sheet_entry;
-
   int row,col;
 
   if(!widget || !rw)
@@ -1177,18 +1097,19 @@ show_sheet_entry(GtkWidget *widget, Register_Window *rw)
 
   if(!gtk_widget_has_focus(widget)) return;
 
-  sheet=GTK_SHEET(rw->register_sheet);
-  sheet_entry = GTK_ENTRY(gtk_sheet_get_entry(sheet));
+  GtkSheet *sheet = GTK_SHEET(rw->register_sheet);
+  GtkEntry *sheet_entry = GTK_ENTRY(gtk_sheet_get_entry(sheet));
 
   //row=sheet->active_cell.row; col=sheet->active_cell.col;
   gtk_sheet_get_active_cell(sheet, &row, &col);
 
   GUIRegister *reg = rw->getRegister(row,col);
 
-  if(reg && reg->bIsValid() )
-    if((text=gtk_entry_get_text (GTK_ENTRY(rw->entry))) && sheet_entry)
+  if (reg && reg->bIsValid()) {
+    const char *text = gtk_entry_get_text(GTK_ENTRY(rw->entry));
+    if (sheet_entry)
       gtk_entry_set_text(sheet_entry, text);
-
+  }
 }
 
 /* when we have new data in the entry above the sheet, we
@@ -1378,7 +1299,8 @@ build_entry_bar(GtkWidget *main_vbox, Register_Window *rw)
   gtk_widget_set_can_default(rw->location, TRUE);
   gtk_widget_show(rw->location);
 
-  rw->entry=gtk_entry_new();
+  rw->entry = gtk_entry_new();
+  g_signal_connect(rw->entry, "insert-text", G_CALLBACK(sanatize_numeric_0x), NULL);
   gtk_box_pack_start(GTK_BOX(status_box), rw->entry,
                      TRUE, TRUE, 0);
   gtk_widget_show(rw->entry);
@@ -1442,7 +1364,7 @@ gboolean Register_Window::UpdateRegisterCell(int reg_number)
   if((unsigned int)reg_number >= guiReg->rma->get_size())
     return 0;
 
-  
+
   range.row0=guiReg->row;
   range.rowi=guiReg->row;
   range.col0=guiReg->col;
@@ -1469,7 +1391,7 @@ gboolean Register_Window::UpdateRegisterCell(int reg_number)
 
     if(guiReg->row<=register_sheet->maxrow) {
 
-      guiReg->getValueAsString(name,sizeof(name),pCellFormat, new_value);
+      guiReg->getValueAsString(name, sizeof(name), new_value);
 
       gtk_sheet_set_cell(GTK_SHEET(register_sheet),
                          guiReg->row,
@@ -1479,7 +1401,7 @@ gboolean Register_Window::UpdateRegisterCell(int reg_number)
     // else the register is invalid and out of the register sheet
 
 
-    //if(new_value != last_value) 
+    //if(new_value != last_value)
     if(guiReg->hasChanged(new_value)) {
 
       guiReg->put_shadow(new_value);
@@ -1510,13 +1432,12 @@ gboolean Register_Window::UpdateRegisterCell(int reg_number)
     if(new_value.data==INVALID_VALUE) {
 
       guiReg->put_shadow(RegisterValue(INVALID_VALUE,INVALID_VALUE));
-      sprintf (name, "??");
+      g_snprintf(name, sizeof(name), "??");
     } else {
 
       // the register has changed since last update
       guiReg->put_shadow(new_value);
-      guiReg->getValueAsString(name,sizeof(name),pCellFormat, new_value);
-      //sprintf (name, pCellFormat, new_value.data);
+      guiReg->getValueAsString(name, sizeof(name), new_value);
     }
 
     gtk_sheet_set_cell(GTK_SHEET(register_sheet),
@@ -1602,23 +1523,18 @@ void Register_Window::Update()
 //------------------------------------------------------------------------
 void Register_Window::SetRegisterSize()
 {
-  if(gp && gp->cpu)
+  if (gp && gp->cpu)
     register_size = gp->cpu->register_size();
   else
     register_size = 1;
 
   chars_per_column = 1 + 2*register_size;
 
-  sprintf(pCellFormat,"%%0%dx",register_size*2);
-
   if(register_sheet) {
-
-    int i;
-    char buffer[10];
-
     // Column labels
-    for(i=0; i<register_sheet->maxcol; i++){
-      sprintf(buffer,"%02x",i);
+    for (int i = 0; i < register_sheet->maxcol; i++) {
+      char buffer[10];
+      g_snprintf(buffer, sizeof(buffer), "%02x", i);
       gtk_sheet_column_button_add_label(register_sheet, i, buffer);
       gtk_sheet_set_column_title(register_sheet, i, buffer);
       gtk_sheet_set_column_width (register_sheet, i, column_width(i));
@@ -1626,10 +1542,10 @@ void Register_Window::SetRegisterSize()
 
 
     // ASCII column
-    i = REGISTERS_PER_ROW;
-    sprintf(buffer,"ASCII");
-    gtk_sheet_column_button_add_label(register_sheet, i, buffer);
-    gtk_sheet_set_column_title(register_sheet, i, buffer);
+    int i = REGISTERS_PER_ROW;
+    const char *ascii = "ASCII";
+    gtk_sheet_column_button_add_label(register_sheet, i, ascii);
+    gtk_sheet_set_column_title(register_sheet, i, ascii);
 
     gtk_sheet_set_column_width (register_sheet, i, column_width(i));
 
@@ -1740,7 +1656,7 @@ void Register_Window::NewProcessor(GUI_Processor *_gp)
 
       cross_reference = new RegisterWindowXREF();
       cross_reference->parent_window = (gpointer) this;
-      cross_reference->data = (gpointer) pGReg;
+      cross_reference->data = (gpsimObject *) pGReg;
       pGReg->Assign_xref(cross_reference);
 
       if(!row_created)
@@ -1752,7 +1668,9 @@ void Register_Window::NewProcessor(GUI_Processor *_gp)
           gtk_sheet_REALLY_set_row_height (register_sheet, j, row_height(0));
         }
 
-        sprintf(row_label,"%x0",reg_number/REGISTERS_PER_ROW);
+        g_snprintf(row_label, sizeof(row_label),
+          "%x0", reg_number / REGISTERS_PER_ROW);
+
         gtk_sheet_row_button_add_label(register_sheet, j, row_label);
         gtk_sheet_set_row_title(register_sheet, j, row_label);
 
@@ -1835,7 +1753,7 @@ void Register_Window::Build()
 
   char *fontstring;
 
-  if(window!=0) {
+  if (window) {
     gtk_widget_destroy(window);
   }
 
@@ -1850,11 +1768,6 @@ void Register_Window::Build()
   {
     register_sheet=GTK_SHEET(gtk_sheet_new(1,MAXCOLS,"gpsim Register Viewer [RAM]"));
     gtk_window_set_title(GTK_WINDOW(window), "register viewer [RAM]");
-    // Add a status bar
-
-    RAM_RegisterWindow *rrw = dynamic_cast<RAM_RegisterWindow *>(this);
-    if(rrw && rrw->sbw)
-      rrw->sbw->Create(main_vbox);
   }
   else
   {
@@ -1864,7 +1777,7 @@ void Register_Window::Build()
   //gtk_sheet_hide_column_titles(register_sheet);
   //gtk_sheet_hide_row_titles(register_sheet);
   /* create popupmenu */
-  popup_menu=build_menu(this);
+  popup_menu=build_menu();
 
   build_entry_bar(main_vbox,this);
 
@@ -1874,16 +1787,16 @@ void Register_Window::Build()
 
   /**************************** load fonts *********************************/
 #define DEFAULT_NORMALFONT "Monospace 10"
-  strcpy(normalfont_string,DEFAULT_NORMALFONT);
+  normalfont_string = DEFAULT_NORMALFONT;
   if(config_get_string(name(),"normalfont",&fontstring))
-      strcpy(normalfont_string,fontstring);
+      normalfont_string = fontstring;
 
   while(!LoadStyles())
   {
     if(gui_question("Some fonts did not load.","Open font dialog","Try defaults")==FALSE)
       {
-      strcpy(normalfont_string,DEFAULT_NORMALFONT);
-      config_set_string(name(),"normalfont",normalfont_string);
+      normalfont_string = DEFAULT_NORMALFONT;
+      config_set_string(name(),"normalfont", normalfont_string.c_str());
       }
       else
       {
@@ -1909,6 +1822,9 @@ void Register_Window::Build()
 
   gtk_box_pack_start(GTK_BOX(main_vbox), scrolled_window, TRUE, TRUE, 0);
 
+  // RP - I think this is wrong. The sheet's entry widget seems to get
+  // replaced every time a new cell is selected, so this signal can't
+  // work. Should it be hooked to the "changed" signal of the whole sheet?
   g_signal_connect(gtk_sheet_get_entry(GTK_SHEET(register_sheet)),
                      "changed", G_CALLBACK(show_entry), this);
 
@@ -1938,10 +1854,10 @@ void Register_Window::Build()
                      G_CALLBACK(move_handler),
                      this);
 
-  g_signal_connect(register_sheet,
-                     "button_press_event",
-                     G_CALLBACK(do_popup),
-                     this);
+  g_signal_connect(register_sheet, "button_press_event",
+    G_CALLBACK(button_press), this);
+  g_signal_connect(register_sheet, "popup-menu", G_CALLBACK(popup_menu_handler),
+    this);
 
   g_signal_connect(register_sheet,
                      "set_cell",
@@ -1958,8 +1874,6 @@ void Register_Window::Build()
 
   gtk_widget_grab_default(location);
 
-  //  GTKWAIT;
-
   bIsBuilt = true;
 
   NewProcessor(gp);
@@ -1969,28 +1883,18 @@ void Register_Window::Build()
 }
 
 //------------------------------------------------------------------------
-Register_Window::Register_Window()
+
+Register_Window::Register_Window(GUI_Processor *_gp, REGISTER_TYPE _type)
+  : normalfont(0), current_line_number_style(0), breakpoint_line_number_style(0),
+    type(_type),
+    registers(0), register_sheet(NULL), rma(0), entry(0), location(0),
+    popup_menu(0), registers_loaded(0), register_size(0),
+    char_width(0), char_height(0), chars_per_column(3)
+
 {
-  register_sheet = NULL;
-  printf("WARNING: calling default constructor: %s\n",__FUNCTION__);
-
-}
-
-Register_Window::Register_Window(GUI_Processor *_gp)
-{
-  int i;
-
   gp = _gp;
-  char_width = 0;
-  chars_per_column = 3; // assume byte-sized registers
-  register_sheet = NULL;
 
-  registers_loaded=0;
-
-  registers = 0;
-
-  for(i=0;i<MAX_REGISTERS/REGISTERS_PER_ROW;i++)
-    row_to_address[i]=-1;
+  std::fill_n(row_to_address, MAX_ROWS, -1);
 }
 
 const char *RAM_RegisterWindow::name()
@@ -1999,13 +1903,9 @@ const char *RAM_RegisterWindow::name()
 }
 
 RAM_RegisterWindow::RAM_RegisterWindow(GUI_Processor *_gp) :
-  Register_Window(_gp)
+  Register_Window(_gp, REGISTER_RAM)
 {
   menu = "/menu/Windows/Ram";
-  type = REGISTER_RAM;
-
-  // Add a status bar
-  sbw = new StatusBar_Window();
 
   get_config();
 
@@ -2016,7 +1916,6 @@ RAM_RegisterWindow::RAM_RegisterWindow(GUI_Processor *_gp) :
 
 void RAM_RegisterWindow::NewProcessor(GUI_Processor *_gp)
 {
-
   if(!_gp || !_gp->cpu)
     return;
 
@@ -2025,16 +1924,11 @@ void RAM_RegisterWindow::NewProcessor(GUI_Processor *_gp)
   Dprintf((" RAM_RegisterWindow::NewProcessor rma=%p\n",rma));
 
   Register_Window::NewProcessor(_gp);
-  if(sbw)
-    sbw->NewProcessor(_gp, rma);
-
 }
 
 void RAM_RegisterWindow::Update()
 {
   Register_Window::Update();
-  if(sbw)
-    sbw->Update();
 }
 
 const char *EEPROM_RegisterWindow::name()
@@ -2043,10 +1937,9 @@ const char *EEPROM_RegisterWindow::name()
 }
 
 EEPROM_RegisterWindow::EEPROM_RegisterWindow(GUI_Processor *_gp) :
-  Register_Window(_gp)
+  Register_Window(_gp, REGISTER_EEPROM)
 {
   menu = "/menu/Windows/EEPROM";
-  type = REGISTER_EEPROM;
 
   get_config();
 
@@ -2057,7 +1950,6 @@ EEPROM_RegisterWindow::EEPROM_RegisterWindow(GUI_Processor *_gp) :
 
 void EEPROM_RegisterWindow::NewProcessor(GUI_Processor *_gp)
 {
-
   if(!_gp || !_gp->cpu)
     return;
 
