@@ -25,17 +25,20 @@ Boston, MA 02111-1307, USA.  */
 
 #include "gui_object.h"
 #include "gui_register.h"
-#include <gtkextra/gtkextra.h>
+#include "gtkextra/gtksheet.h"
+
+#include <string>
 
-class StatusBar_Window;
 class GUI_Processor;
 
 //======================================================================
-// The register window 
+// The register window
 //
 #define REGISTERS_PER_ROW    16
 #define MAX_ROWS ((MAX_REGISTERS)/(REGISTERS_PER_ROW))
 
+// Base class for RAM_RegisterWindow and EEPROM_RegisterWindow
+
 class Register_Window : public GUI_Object
 {
  public:
@@ -44,7 +47,7 @@ class Register_Window : public GUI_Object
   // first cell in the given row.
   int row_to_address[MAX_ROWS];
 
-  char normalfont_string[256];
+  std::string normalfont_string;
   PangoFontDescription *normalfont;
 
   GtkStyle *current_line_number_style;
@@ -55,7 +58,7 @@ class Register_Window : public GUI_Object
   GtkSheet *register_sheet;
 
   RegisterMemoryAccess *rma;  // Apointer to the Processor's rma or ema.
-    
+
   GtkWidget *entry;
   GtkWidget *location;
   GtkWidget *popup_menu;
@@ -82,13 +85,18 @@ class Register_Window : public GUI_Object
   int column_width(int col);
   int row_height(int row);
 
-  Register_Window(GUI_Processor *gp);
-  Register_Window();
+protected:
+  Register_Window(GUI_Processor *gp, REGISTER_TYPE type);
 
  private:
+  GtkWidget *build_menu();
+  void do_popup(GtkWidget *widget, GdkEventButton *event);
+  static gboolean button_press(GtkWidget *widget, GdkEventButton *event,
+    Register_Window *rw);
+  static gboolean popup_menu_handler(GtkWidget *widget, Register_Window *rw);
+
   // Formatting
   int register_size;    // The size (in bytes) of a single register
-  char pCellFormat[12]; // format string
   int char_width;       // nominal character width.
   int char_height;      // nominal character height
   int chars_per_column; // width of 1 column
@@ -99,12 +107,10 @@ class Register_Window : public GUI_Object
 class RAM_RegisterWindow : public Register_Window
 {
  public:
-  RAM_RegisterWindow(GUI_Processor *gp);
+  explicit RAM_RegisterWindow(GUI_Processor *gp);
   virtual void NewProcessor(GUI_Processor *gp);
   virtual void Update();
 
-  StatusBar_Window *sbw;
-
 protected:
   virtual const char *name();
 };
@@ -112,7 +118,7 @@ protected:
 class EEPROM_RegisterWindow : public Register_Window
 {
  public:
-  EEPROM_RegisterWindow(GUI_Processor *gp);
+  explicit EEPROM_RegisterWindow(GUI_Processor *gp);
   virtual void NewProcessor(GUI_Processor *gp);
 
 protected:
