@@ -101,7 +101,7 @@ private:
 class ZoomAttribute : public Integer
 {
 public:
-  ZoomAttribute(Scope_Window *);
+  explicit ZoomAttribute(Scope_Window *);
   virtual void set(gint64 i);
 
 private:
@@ -114,7 +114,7 @@ private:
 class PanAttribute : public Integer
 {
 public:
-  PanAttribute(Scope_Window *);
+  explicit PanAttribute(Scope_Window *);
   virtual void set(gint64 i);
 
 private:
@@ -220,7 +220,7 @@ protected:
 class WaveformSink : public SignalSink
 {
 public:
-  WaveformSink(Waveform *pParent);
+  explicit WaveformSink(Waveform *pParent);
   virtual void setSinkState(char);
   virtual void release() {}
 
@@ -246,68 +246,6 @@ protected:
   WaveBase *m_selectedWave;
 };
 
-//========================================================================
-class ZoomInEvent : public KeyEvent
-{
-public:
-  virtual ~ZoomInEvent()
-  {
-  }
-
-  virtual void press(gpointer data)
-  {
-    Scope_Window *sw = static_cast<Scope_Window *>(data);
-    if (sw)
-      sw->zoom(2);
-  }
-};
-//========================================================================
-class ZoomOutEvent : public KeyEvent
-{
-public:
-  virtual ~ZoomOutEvent()
-  {
-  }
-
-  virtual void press(gpointer data)
-  {
-    Scope_Window *sw = static_cast<Scope_Window *>(data);
-    if (sw)
-      sw->zoom(-2);
-  }
-};
-
-//========================================================================
-class PanLeftEvent : public KeyEvent
-{
-public:
-  virtual ~PanLeftEvent()
-  {
-  }
-
-  virtual void press(gpointer data)
-  {
-    Scope_Window *sw = static_cast<Scope_Window *>(data);
-    if (sw)
-      sw->pan(-( (gint64) sw->getSpan()/4));
-  }
-};
-//========================================================================
-class PanRightEvent : public KeyEvent
-{
-public:
-  virtual ~PanRightEvent()
-  {
-  }
-
-  virtual void press(gpointer data)
-  {
-    Scope_Window *sw = static_cast<Scope_Window *>(data);
-    if (sw)
-      sw->pan( (gint64) sw->getSpan()/4);
-  }
-};
-
 //========================================================================
 
 gboolean Scope_Window::key_press(GtkWidget *widget, GdkEventKey *key,
@@ -315,14 +253,24 @@ gboolean Scope_Window::key_press(GtkWidget *widget, GdkEventKey *key,
 {
   Dprintf (("press 0x%x\n", key->keyval));
 
-  KeyEvent *pKE = sw->KeyMap[key->keyval];
-  if(pKE)
-    {
-      pKE->press(sw);
-      return TRUE;
-    }
+  switch (key->keyval) {
+  case 'z':
+    sw->zoom(2);
+    break;
+  case 'Z':
+    sw->zoom(-2);
+    break;
+  case 'l':
+    sw->pan(-( (gint64) sw->getSpan()/4));
+    break;
+  case 'r':
+    sw->pan( (gint64) sw->getSpan()/4);
+    break;
+  default:
+    return FALSE;
+  }
 
-  return FALSE;
+  return TRUE;
 }
 
 //========================================================================
@@ -989,7 +937,7 @@ gboolean Scope_Window::signal_expose(GtkWidget *widget,
   for (; i != sw->signals.end(); ++i) {
     (*i)->draw(cr);
   }
-  
+
   double xpos = sw->mapTimeToPixel(sw->m_Markers[eLeftButton]->getVal()
     + sw->waveXoffset());
   cairo_move_to(cr, xpos, 0.0);
@@ -997,7 +945,7 @@ gboolean Scope_Window::signal_expose(GtkWidget *widget,
   cairo_stroke(cr);
 
   cairo_destroy(cr);
- 
+
   return TRUE;
 }
 
@@ -1110,12 +1058,6 @@ void Scope_Window::Build()
   g_signal_connect(signalDrawingArea, "expose-event",
     G_CALLBACK(signal_name_expose), this);
 
-  KeyMap['z'] = new ZoomInEvent();
-  KeyMap['Z'] = new ZoomOutEvent();
-  KeyMap['l'] = new PanLeftEvent();
-  KeyMap['r'] = new PanRightEvent();
-
-
   /* Add a signal handler for key press events. This will capture
    * key commands for single stepping, running, etc.
    */
