@@ -19,87 +19,17 @@ along with gpsim; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
-#include <cstdio>
 #include <typeinfo>
 
 #include "../config.h"
 #ifdef HAVE_GUI
 
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 
-#include "gui.h"
 #include "gui_src.h"
 
-class StepEvent : public KeyEvent
-{
-public:
-  void press(gpointer data)
-  {
-    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
-    if(sbw && sbw->pma)
-        sbw->pma->step(1);
-  }
-  void release(gpointer data) {}
-};
-
-class StepOverEvent : public KeyEvent
-{
-public:
-  void press(gpointer data)
-  {
-    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
-    if(sbw && sbw->pma)
-      // Step Over Next instruction, or hll statement
-      sbw->pma->step_over();
-  }
-  void release(gpointer data) {}
-};
-
-class RunEvent : public KeyEvent
-{
-public:
-  void press(gpointer data)
-  {
-    /*
-    SourceBrowser_Window *sbw = (SourceBrowser_Window *) data;
-    if(sbw && sbw->pma)
-      sbw->pma->run();
-    */
-    get_interface().start_simulation();
-  }
-  void release(gpointer data) {}
-};
-
-class StopEvent : public KeyEvent
-{
-public:
-  void press(gpointer data)
-  {
-    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
-    if(sbw && sbw->pma)
-      sbw->pma->stop();
-  }
-  void release(gpointer data) {}
-};
-
-class FinishEvent : public KeyEvent
-{
-public:
-  void press(gpointer data)
-  {
-    SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
-    if(sbw && sbw->pma)
-      sbw->pma->finish();
-  }
-  void release(gpointer data) {}
-};
-
-static map<guint, KeyEvent *> KeyMap;
-
-
 static gint
 key_press(GtkWidget *widget,
           GdkEventKey *key,
@@ -108,19 +38,40 @@ key_press(GtkWidget *widget,
 
   SourceBrowser_Window *sbw = static_cast<SourceBrowser_Window *>(data);
 
-  if(!sbw) return(FALSE);
-  if(!sbw->gp) return(FALSE);
-  if(!sbw->gp->cpu) return(FALSE);
+  if (!sbw) return FALSE;
+  if (!sbw->pma) return FALSE;
+  if (!sbw->gp) return FALSE;
+  if (!sbw->gp->cpu) return FALSE;
+  
+  switch (key->keyval) {
+  case 's':
+  case 'S':
+  case GDK_KEY_F7:
+    sbw->pma->step(1);
+    break;
+  case 'o':
+  case 'O':
+  case 'n':
+  case GDK_KEY_F8:
+    sbw->pma->step_over();
+    break;
+  case 'r':
+  case 'R':
+  case GDK_KEY_F9:
+    get_interface().start_simulation();
+    break;
+  case GDK_KEY_Escape:
+    sbw->pma->stop();
+    break;
+  case 'f':
+  case 'F':
+    sbw->pma->finish();
+    break;
+  default:
+    return FALSE;
+  }
 
-
-  KeyEvent *pKE = KeyMap[key->keyval];
-  if(pKE)
-    {
-      pKE->press(data);
-      return TRUE;
-    }
-
-  return FALSE;
+  return TRUE;
 }
 
 static int delete_event(GtkWidget *widget,
@@ -131,17 +82,6 @@ static int delete_event(GtkWidget *widget,
     return TRUE;
 }
 
-
-void SourceBrowser_Window::SetPC(int address)
-{
-  printf("%s shouldn't be called \n",__FUNCTION__);
-
-}
-void SourceBrowser_Window::UpdateLine(int address)
-{
-  printf("%s shouldn't be called \n",__FUNCTION__);
-
-}
 void SourceBrowser_Window::Update()
 {
   if(!gp || !gp->cpu)
@@ -165,27 +105,6 @@ void SourceBrowser_Window::Create()
                       G_CALLBACK(delete_event),
                       (gpointer) this);
 
-  // FIXME - populate the KeyMap map with source browser functions. This should
-  // FIXME   probably go into some kind of configuration file.
-
-  KeyMap['s'] = new StepEvent();
-  KeyMap['S'] = KeyMap['s'];
-  KeyMap[GDK_KEY_F7] = KeyMap['s'];
-
-  KeyMap['o'] = new StepOverEvent();
-  KeyMap['O'] = KeyMap['o'];
-  KeyMap['n'] = KeyMap['o'];
-  KeyMap[GDK_KEY_F8] = KeyMap['o'];
-
-  KeyMap['r'] = new RunEvent();
-  KeyMap['R'] = KeyMap['r'];
-  KeyMap[GDK_KEY_F9] = KeyMap['r'];
-
-  KeyMap[GDK_KEY_Escape] = new StopEvent();
-
-  KeyMap['f'] = new FinishEvent();
-  KeyMap['F'] = KeyMap['f'];
-
   /* Add a signal handler for key press events. This will capture
    * key commands for single stepping, running, etc.
    */
@@ -204,8 +123,8 @@ void SourceBrowser_Window::Create()
 }
 
 void SourceBrowser_Window::SetTitle() {
-  char *buffer = 0;
-  if (gp->cpu == NULL || pma == NULL) {
+
+  if (!gp->cpu || !pma) {
       return;
   }
 
@@ -225,21 +144,12 @@ void SourceBrowser_Window::SetTitle() {
   else // if (gp->cpu->simulation_mode == eSM_STOPPED)
     sStatus = "Stopped";
 
-  buffer = g_strdup_printf(buffer,
-    "Source Browser: [%s] %s", sStatus, pma->name().c_str());
+  char *buffer = g_strdup_printf("Source Browser: [%s] %s", sStatus, pma->name().c_str());
   sLastPmaName = pma->name();
   gtk_window_set_title (GTK_WINDOW (window), buffer);
   g_free(buffer);
 }
 
-void SourceBrowser_Window::NewProcessor(GUI_Processor *gp)
-{
-  printf("%s shouldn't be called \n",__FUNCTION__);
-}
-void SourceBrowser_Window::SelectAddress(int address)
-{
-  printf("%s shouldn't be called \n",__FUNCTION__);
-}
 void SourceBrowser_Window::SelectAddress(Value *addrSym)
 {
 
