@@ -19,12 +19,13 @@ along with gpsim; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
-#include <cstdio>
-#include <typeinfo>
-
 #include "../config.h"
 #ifdef HAVE_GUI
 
+#include <cstdio>
+#include <typeinfo>
+#include <iostream>
+
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <glib.h>
@@ -201,7 +202,7 @@ static void updateSymbolTables(const SymbolTableEntry_t &st)
   (st.second)->ForEachSymbolTable(updateOneSymbol);
 }
 
-void Symbol_Window::Update(void)
+void Symbol_Window::Update()
 {
   load_symbols=1;
 
@@ -251,12 +252,7 @@ void Symbol_Window::symbol_list_row_selected(GtkTreeSelection *treeselection,
   sw->do_symbol_select(e);
 }
 
-void Symbol_Window::SelectSymbolName(const char *symbol_name)
-{
-  cout << "SelectSymbolName is broken\n";
-}
-
-void Symbol_Window::NewSymbols(void)
+void Symbol_Window::NewSymbols()
 {
   Update();
 }
@@ -297,7 +293,7 @@ Symbol_Window::toggle_registers (GtkToggleButton *button, Symbol_Window *sw)
 // Build
 //
 
-void Symbol_Window::Build(void)
+void Symbol_Window::Build()
 {
   if(bIsBuilt)
     return;
