@@ -31,6 +31,7 @@ License along with this library; if not, see
 
 #include <cstdio>
 #include <cassert>
+#include <iostream>
 
 #include "../config.h"    // get the definition for HAVE_GUI
 #ifdef HAVE_GUI
@@ -71,10 +72,12 @@ void Encoder::create_iopin_map(void)
   //   need to reference these newly created I/O pins (like
   //   below) then we can call the member function 'get_pin'.
 
-  a_pin = new IO_bi_directional((name() + ".a").c_str());
+  a_pin = new IO_bi_directional("a");
+  addSymbol(a_pin);
   assign_pin(1, a_pin);
   package->set_pin_position(1,(float)0.0);
-  b_pin = new IO_bi_directional((name() + ".b").c_str());
+  b_pin = new IO_bi_directional("b");
+  addSymbol(b_pin);
   assign_pin(2, b_pin);
   package->set_pin_position(2,(float)0.9999);
 }
