@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -475,8 +475,6 @@ namespace I2C_Module {
       m_MSBmask(1<<8)
   {
 
-    m_uState = eI2CIdle;
-
     // Transmit and Receive propogation delays
     // The units are simulation cycles.
 
@@ -484,15 +482,12 @@ namespace I2C_Module {
     tClkToSample = 10;
 
 
-    string sName;
 
-    sName = _name;
-    sName += ".scl";
-    m_pSCL = new I2C_SCL_PIN(this, sName.c_str());
+    m_pSCL = new I2C_SCL_PIN(this, "scl");
+    addSymbol(m_pSCL);
 
-    sName = _name;
-    sName += ".sda";
-    m_pSDA = new I2C_SDA_PIN(this, sName.c_str());
+    m_pSDA = new I2C_SDA_PIN(this, "sda");
+    addSymbol(m_pSDA);
 
     mTxByte     = new I2C_TxBuffer(this);
     mTxReady    = new I2C_TxReady(this);
@@ -519,6 +514,26 @@ namespace I2C_Module {
 
   I2CMaster::~I2CMaster()
   {
+    removeSymbol(mTxByte);
+    removeSymbol(mTxReady);
+    removeSymbol(mRxByte);
+    removeSymbol(mRxSequence);
+    removeSymbol(mSend7BitAddress);
+    removeSymbol(mStop);
+    removeSymbol(mAddress);
+    removeSymbol(mDebug);
+    removeSymbol(m_pSCL);
+    removeSymbol(m_pSDA);
+
+
+    delete mTxByte;
+    delete mTxReady;
+    delete mRxByte;
+    delete mRxSequence;
+    delete mSend7BitAddress;
+    delete mStop;
+    delete mAddress;
+    delete mDebug;
   }
 
   void I2CMaster::startIdle()
@@ -646,8 +661,8 @@ namespace I2C_Module {
       m_bitCount--;
       m_nextBit = (m_xfr_data & m_MSBmask) == m_MSBmask;
 
-      Dprintf(("I2CMaster SCL : Rising edge, data in=%d, nextOut=%d bit_count=%d\n",
-               m_xfr_data&1, m_nextBit, m_bitCount));
+      Dprintf(("I2CMaster SCL : Rising edge, data in=%u, nextOut=%d bit_count=%u\n",
+               m_xfr_data & 1, m_nextBit, m_bitCount));
 
       return true;
     }
