@@ -34,6 +34,7 @@ In here you'll find some simple logic devices:
 #define IN_MODULE
 
 #include "../config.h"    // get the definition for HAVE_GUI
+#include <iostream>
 
 #ifdef HAVE_GUI
 #include <gtk/gtk.h>
@@ -212,8 +213,8 @@ void Logic_Input::setDrivenState( bool new_state)
 {
 
   if(verbose)
-    cout << name()<< " setDrivenState= "
-         << (new_state ? "high" : "low") << endl;
+    std::cout << name()<< " setDrivenState= "
+         << (new_state ? "high" : "low") << std::endl;
 
   if(new_state != getDrivenState()) {
 
@@ -227,6 +228,19 @@ void Logic_Input::setDrivenState( bool new_state)
   }
 }
 
+#ifdef RRR
+void Logic_Input::get(char *return_str, int len)
+{
+      if (return_str)
+        strncpy(return_str, IOPIN::getState()?"1": "0", len);
+}
+void Logic_Output::get(char *return_str, int len)
+{
+      if (return_str)
+        strncpy(return_str, IOPIN::getDrivingState()?"1": "0", len);
+}
+#endif
+
 
 ANDGate::ANDGate(const char *name, const char *desc)
   : LogicGate(name, desc)
@@ -269,8 +283,10 @@ LogicGate::LogicGate(const char *name, const char *desc)
 
 LogicGate::~LogicGate()
 {
+#ifdef HAVE_GUI
   if (pixbuf)
     g_object_unref(pixbuf);
+#endif
 }
 
 //--------------------------------------------------------------
@@ -322,7 +338,9 @@ void LogicGate::create_iopin_map()
 
   string outname = name() + ".out";
 
-  pOutputPin = new Logic_Output(this, OUTPUT_BITPOSITION, outname.c_str());
+  //RRRpOutputPin = new Logic_Output(this, OUTPUT_BITPOSITION, outname.c_str());
+  pOutputPin = new Logic_Output(this, OUTPUT_BITPOSITION, "out");
+  addSymbol(pOutputPin);
   pOutputPin->update_direction(1,true);  // make the bidirectional an output
 
   // Position pin on middle right side of package
@@ -339,7 +357,8 @@ void LogicGate::create_iopin_map()
   string inname;
   for(i=j=INPUT_FIRST_BITPOSITION; i<number_of_pins; i++) {
     char pin_number = i-j +'0';
-    inname = name() + ".in" + pin_number;
+    //RRRinname = name() + ".in" + pin_number;
+    inname = (string)"in" + pin_number;
     //p[2] = i-j +'0';
     LIP = new Logic_Input(this, i-INPUT_FIRST_BITPOSITION,inname.c_str());
     pInputPins[i-INPUT_FIRST_BITPOSITION] = LIP;
@@ -348,6 +367,7 @@ void LogicGate::create_iopin_map()
       package->set_pin_position(i+1, 0.5); // Left side of package
     else
       package->set_pin_position(i+1, (float)((i-INPUT_FIRST_BITPOSITION)*0.9999)); // Left side of package
+    addSymbol(LIP);
     assign_pin(i+1, LIP );       //  Pin numbers begin at 1
   }
 
@@ -495,7 +515,7 @@ NOTGate::~NOTGate()
 void NOTGate::update_state()
 {
   if (verbose)
-    cout << name() << " update_state\n";
+    std::cout << name() << " update_state\n";
   pOutputPin->putState((input_state & input_bit_mask) == 0);
 }
 
