@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -35,6 +35,7 @@ License along with this library; if not, see
 #include "../src/packages.h"
 #include "../src/gpsim_time.h"
 
+#include <iostream>
 #include <string>
 
 //#define DEBUG
@@ -63,7 +64,7 @@ TTLbase::~TTLbase()
 //------------------------------------------------------------------------
 // Some edge-sensitive pins
 //
-// 
+//
 
 //------------------------------------------------------------
 // Clock
@@ -173,7 +174,7 @@ void Reset::setDrivenState(bool bNewState)
 //------------------------------------------------------------------------
 // TTL377 - Octal Latch
 //
-// 
+//
 
 Module *TTL377::construct(const char *_new_name)
 {
@@ -195,26 +196,38 @@ TTL377::TTL377(const char *_name)
   m_Q = new IO_bi_directional *[8];
 
   char pName[4];
-  pName[0] = '.';
-  pName[3] = 0;
+  pName[2] = 0;
   int i;
-  string sPinName;
-  for (i=0; i<8; i++) {
-    pName[1] = 'D';
-    pName[2] = '0' + i;
-    sPinName = name() + pName;
-    m_D[i] = new IOPIN(sPinName.c_str());
 
-    pName[1] = 'Q';
-    sPinName = name() + pName;
-    m_Q[i] = new IO_bi_directional(sPinName.c_str());
+  for (i=0; i<8; i++) {
+    pName[0] = 'D';
+    pName[1] = '0' + i;
+    m_D[i] = new IOPIN(pName);
+    addSymbol(m_D[i]);
+
+    pName[0] = 'Q';
+    m_Q[i] = new IO_bi_directional(pName);
+    addSymbol(m_Q[i]);
     m_Q[i]->setDriving(true);
   }
 
-  sPinName = name() + ".E";
-  m_enable = new Enable(sPinName.c_str(),this);
-  sPinName = name() + ".CP";
-  m_clock  = new Clock(sPinName.c_str(),this);
+  m_enable = new Enable("E",this);
+  addSymbol(m_enable);
+  m_clock  = new Clock("CP",this);
+  addSymbol(m_clock);
+}
+
+TTL377::~TTL377()
+{
+    for (int i = 0; i < 8; i++)
+    {
+	removeSymbol(m_D[i]);
+	removeSymbol(m_Q[i]);
+    }
+    delete [] m_D;
+    delete [] m_Q;
+    removeSymbol(m_enable);
+    removeSymbol(m_clock);
 }
 
 void TTL377::setClock(bool bNewClock)
@@ -236,9 +249,9 @@ void TTL377::update_state()
   bool state[8];
   // Copy the inputs to the outputs through an intermediary to simulate
   // the simultaneous action of the real part.
-  for (i=0; i<8; i++) 
+  for (i=0; i<8; i++)
     state[i]=m_D[i]->getDrivenState();
-  for (i=0; i<8; i++) 
+  for (i=0; i<8; i++)
     m_Q[i]->putState(state[i]);
 }
 
@@ -264,14 +277,14 @@ void TTL377::create_iopin_map()
   package->assign_pin(17, m_D[6]);
   package->assign_pin(18, m_D[7]);
   package->assign_pin(19, m_Q[7]);
-  
+
 }
 
 
 //------------------------------------------------------------------------
 // TTL595 - Octal shift register
 //
-// 
+//
 
 Module *TTL595::construct(const char *_new_name)
 {
@@ -291,32 +304,43 @@ TTL595::TTL595(const char *_name)
 
   m_Q = new IO_bi_directional *[8];
 
-  char pName[4];
-  pName[0] = '.';
-  pName[3] = 0;
+  char pName[4] = "Q0";
   int i;
-  string sPinName;
   for (i=0; i<8; i++) {
-    pName[1] = 'Q';
-    pName[2] = '0' + i;
-    sPinName = name() + pName;
-    m_Q[i] = new IO_bi_directional(sPinName.c_str());
+    pName[1] = '0' + i;
+    m_Q[i] = new IO_bi_directional(pName);
+    addSymbol(m_Q[i]);
     m_Q[i]->setDriving(true);
   }
 
-  sPinName = name() + ".Ds";
-  m_Ds = new IOPIN(sPinName.c_str());
-  sPinName = name() + ".Qs";
-  m_Qs = new IO_bi_directional(sPinName.c_str());
+  m_Ds = new IOPIN("Ds");
+  addSymbol(m_Ds);
+  m_Qs = new IO_bi_directional("Qs");
+  addSymbol(m_Qs);
   m_Qs->setDriving(true);
-  sPinName = name() + ".OE";
-  m_enable = new Enable(sPinName.c_str(),this);
-  sPinName = name() + ".SCK";
-  m_clock  = new Clock(sPinName.c_str(),this);
-  sPinName = name() + ".RCK";
-  m_strobe = new Strobe(sPinName.c_str(),this);
-  sPinName = name() + ".MR";
-  m_reset  = new Reset(sPinName.c_str(),this);
+  m_enable = new Enable("OE",this);
+  addSymbol(m_enable);
+  m_clock  = new Clock("SCK",this);
+  addSymbol(m_clock);
+  m_strobe = new Strobe("RCK",this);
+  addSymbol(m_strobe);
+  m_reset  = new Reset("MR",this);
+  addSymbol(m_reset);
+}
+
+TTL595::~TTL595()
+{
+    for(int i = 0; i < 8; i++)
+    {
+        removeSymbol(m_Q[i]);
+    }
+    delete [] m_Q;
+    removeSymbol(m_Ds);
+    removeSymbol(m_Qs);
+    removeSymbol(m_enable);
+    removeSymbol(m_clock);
+    removeSymbol(m_strobe);
+    removeSymbol(m_reset);
 }
 
 void TTL595::setClock(bool bNewClock)
@@ -338,7 +362,7 @@ void TTL595::setClock(bool bNewClock)
 void TTL595::setEnable(bool bNewEnable)
 {
   // This is the output enable pin on this device
-  for (int i=0; i<8; i++) 
+  for (int i=0; i<8; i++)
     m_Q[i]->update_direction(!bNewEnable,true);
 }
 
@@ -400,7 +424,7 @@ void TTL595::create_iopin_map()
 //------------------------------------------------------------------------
 // TTL165 - 8-bit parallel to serial shift register
 //
-// 
+//
 
 Module *TTL165::construct(const char *_new_name)
 {
@@ -415,40 +439,50 @@ Module *TTL165::construct(const char *_new_name)
 
 
 TTL165::TTL165(const char *_name)
-  : TTLbase(_name, "TTL165 - PISO Shift Register"), 
+  : TTLbase(_name, "TTL165 - PISO Shift Register"),
      m_bStrobe(false), sreg(0)
 {
 
   m_D = new IOPIN *[8];
 
-  char pName[4];
-  pName[0] = '.';
-  pName[3] = 0;
+  char pName[4] = "D0";
   int i;
-  string sPinName;
   for (i=0; i<8; i++) {
-    pName[1] = 'D';
-    pName[2] = '0' + i;
-    sPinName = name() + pName;
-    m_D[i] = new IOPIN(sPinName.c_str());
+    pName[1] = '0' + i;
+    m_D[i] = new IOPIN(pName);
+    addSymbol(m_D[i]);
   }
 
-  sPinName = name() + ".Ds";
-  m_Ds = new IOPIN(sPinName.c_str());
-  sPinName = name() + ".Q7";
-  m_Q = new IO_bi_directional(sPinName.c_str());
+  m_Ds = new IOPIN("Ds");
+  addSymbol(m_Ds);
+  m_Q = new IO_bi_directional("Q7");
+  addSymbol(m_Q);
   m_Q->setDriving(true);
-  sPinName = name() + ".nQ7";
-  m_Qbar = new IO_bi_directional(sPinName.c_str());
+  m_Qbar = new IO_bi_directional("nQ7");
+  addSymbol(m_Qbar);
   m_Qbar->setDriving(true);
-  sPinName = name() + ".CE";
-  m_enable = new Enable(sPinName.c_str(),this);
-  sPinName = name() + ".CP";
-  m_clock  = new Clock(sPinName.c_str(),this);
-  sPinName = name() + ".PL";
-  m_strobe = new Strobe(sPinName.c_str(),this);
+  m_enable = new Enable("CE", this);
+  addSymbol(m_enable);
+  m_clock  = new Clock("CP",this);
+  addSymbol(m_clock);
+  m_strobe = new Strobe("PL",this);
+  addSymbol(m_strobe);
 }
 
+TTL165::~TTL165()
+{
+    for(int i = 0; i < 8; i++)
+    {
+	removeSymbol(m_D[i]);
+    }
+    delete [] m_D;
+    removeSymbol(m_Ds);
+    removeSymbol(m_Q);
+    removeSymbol(m_Qbar);
+    removeSymbol(m_enable);
+    removeSymbol(m_clock);
+    removeSymbol(m_strobe);
+}
 
 void TTL165::setClock(bool bNewClock)
 {
