@@ -18,7 +18,7 @@ case "$1" in
 	        txisr_test tmr0_16bit tmr1_16bit tmr3_16bit switch_test p18f 
 		comparator a2d psp ttl ccp ccp_628 wavegen spi i2c port_stim 
 		p12c509 wdt p12f675 p16f676 p16f690 p16f684 p1xf18xx
-		p18f26k22'
+		p18f26k22 p16f91x'
 
         echo ${DIRS}
         for i in ${DIRS} ; do
@@ -213,6 +213,7 @@ ${RT} p16f676 sim_p16f676
 ${RT} p16f676 sim_reset
 ${RT} p16f690 sim_epwm 
 ${RT} p16f690 sim_eusart 
+${RT} p16f690 sim_eusart_690 
 ${RT} p16f690 sim_nwdt_16f631
 ${RT} p16f690 sim_p16f690
 ${RT} p16f690 sim_wdt_16f677
@@ -234,3 +235,6 @@ ${RT}       p1xf18xx sim_p16f1823_i2c_v2
 ${RT}       p1xf18xx sim_p16f1823_spi
 
 ${RT}	p18f26k22 sim
+${RT}	p16f88x sim
+${RT}	p16f91x sim
+grep ERR */*log
