@@ -16,7 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -92,9 +92,10 @@ public:
              " WDTEN=%d - WDT is %s\n"
              " PWRTEN=%d - Power up timer is %s\n",
              i,
-             i&(FOSC0|FOSC1), (i&FOSC0 ? (i&FOSC1 ? "EXTRC":"XT"):(i&FOSC1 ? "INTRC":"LP")),
-             (i&WDTEN?1:0), ((i&WDTEN) ? "enabled" : "disabled"),
-             (i&PWRTEN?1:0), ((i&PWRTEN) ? "disabled" : "enabled"));
+             i & (FOSC0 | FOSC1),
+             ((i & FOSC0) ? ((i & FOSC1) ? "EXTRC":"XT") :((i & FOSC1) ? "INTRC":"LP")),
+             ((i & WDTEN) ? 1 : 0), ((i & WDTEN) ? "enabled" : "disabled"),
+             ((i & PWRTEN) ? 1 : 0), ((i & PWRTEN) ? "disabled" : "enabled"));
 
     return string(buff);
   }
@@ -104,7 +105,10 @@ public:
 
 //-------------------------------------------------------------------
 _14bit_processor::_14bit_processor(const char *_name, const char *_desc)
-  : pic_processor(_name,_desc), intcon(0), has_SSP(false)
+  : pic_processor(_name,_desc), intcon(0),
+    two_speed_clock(false), config_clock_mode(0),
+    has_SSP(false)
+
 {
   pc = new Program_Counter("pc", "Program Counter", this);
   pc->set_trace_command(); //trace.allocateTraceType(new PCTraceType(this,1)));
@@ -203,6 +207,7 @@ bool _14bit_processor::set_config_word(unsigned int address,unsigned int cfg_wor
   if((address == config_word_address()) && config_modes) {
 
     config_word = cfg_word;
+    oscillator_select(cfg_word, false);
 
     if (m_configMemory && m_configMemory->getConfigWord(0))
       m_configMemory->getConfigWord(0)->set((int)cfg_word);
@@ -214,6 +219,13 @@ bool _14bit_processor::set_config_word(unsigned int address,unsigned int cfg_wor
 
 }
 
+// The working version of oscillator_select should be called at a higher level
+// where the IO pins are defined
+//
+void _14bit_processor::oscillator_select(unsigned int mode, bool not_clkout)
+{
+//    printf("Error _14bit_processor::oscillator_select called\n");
+}
 //-------------------------------------------------------------------
 void _14bit_processor::enter_sleep()
 {
@@ -523,9 +535,9 @@ class Config_E : public ConfigWord
 {
 public:
   Config_E(_14bit_e_processor *pCpu, const char *name, unsigned int address, bool EEw=false)
-    : ConfigWord(name, 0x3fff, "Configuration Word", pCpu, address, EEw) 
+    : ConfigWord(name, 0x3fff, "Configuration Word", pCpu, address, EEw)
   {
-    if (m_pCpu) 
+    if (m_pCpu)
     {
 	m_pCpu->set_config_word(address, 0x3fff);
     }
@@ -576,7 +588,7 @@ bool _14bit_e_processor::set_config_word(unsigned int address,unsigned int cfg_w
 	wdt_flag = (cfg_word & (WDTEN0|WDTEN1)) >> 3;
         Dprintf((" cfg_word %x MCLRE %x\n", cfg_word, cfg_word & MCLRE));
         if ((cfg_word & MCLRE) == MCLRE)
-            assignMCLRPin(mclr_pin);   
+            assignMCLRPin(mclr_pin);
         else
             unassignMCLRPin();
 
