@@ -1,7 +1,7 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
    Copyright (C) 2013 Roy R. Rankin
-		 
+
 
 This file is part of the libgpsim library of gpsim
 
@@ -16,7 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -37,6 +37,7 @@ class _14bit_processor;
 
 #include "rcon.h"
 #include "intcon.h"
+#include "pir.h"
 
 class stimulus;  // forward reference
 class IOPIN;
@@ -101,7 +102,7 @@ public:
   unsigned int valid_bits;
   unsigned int register_page_bits;   /* Only used by the 12-bit core to define
                                         the valid paging bits in the FSR. */
-  FSR_12(Processor *, const char *pName, 
+  FSR_12(Processor *, const char *pName,
          unsigned int _register_page_bits, unsigned int _valid_bits);
 
   virtual void put(unsigned int new_value);
@@ -196,7 +197,7 @@ public:
   {
     get_trace().raw(write_trace.get() | value.get());
 
-    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC)) |  
+    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC)) |
 	      ((new_value & 0xff)   ? 0 : STATUS_Z)   |
 	      ((new_value & 0x100)  ? STATUS_C : 0)   |
 	      (((new_value ^ src1 ^ src2)&0x10) ? STATUS_DC : 0));
@@ -208,7 +209,7 @@ public:
 
     get_trace().raw(write_trace.get() | value.get());
 
-    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC)) |  
+    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC)) |
 	      ((new_value & 0xff)   ? 0 : STATUS_Z)   |
 	      ((new_value & 0x100)  ? 0 : STATUS_C)   |
 	      (((new_value ^ src1 ^ src2)&0x10) ? 0 : STATUS_DC));
@@ -223,7 +224,7 @@ public:
     {
         get_trace().raw(write_trace.get() | value.get());
         value.put((value.get() & ~STATUS_PD) | ((new_pd) ? STATUS_PD : 0));
-    }  
+    }
 }
 
   inline unsigned int get_PD()
@@ -265,7 +266,7 @@ public:
   void put_N_Z(unsigned int new_value)
   {
     get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~(STATUS_Z | STATUS_N)) | 
+    value.put((value.get() & ~(STATUS_Z | STATUS_N)) |
 	      ((new_value & 0xff )  ? 0 : STATUS_Z)   |
 	      ((new_value & 0x80) ? STATUS_N : 0));
   }
@@ -273,7 +274,7 @@ public:
   void put_Z_C_N(unsigned int new_value)
   {
     get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~(STATUS_Z | STATUS_C | STATUS_N)) | 
+    value.put((value.get() & ~(STATUS_Z | STATUS_C | STATUS_N)) |
 	      ((new_value & 0xff )  ? 0 : STATUS_Z)   |
 	      ((new_value & 0x100)  ? STATUS_C : 0)   |
 	      ((new_value & 0x80) ? STATUS_N : 0));
@@ -282,18 +283,18 @@ public:
   inline void put_Z_C_DC_OV_N(unsigned int new_value, unsigned int src1, unsigned int src2)
   {
     get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC | STATUS_OV | STATUS_N)) |  
+    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC | STATUS_OV | STATUS_N)) |
 	      ((new_value & 0xff )  ? 0 : STATUS_Z)   |
 	      ((new_value & 0x100)  ? STATUS_C : 0)   |
 	      (((new_value ^ src1 ^ src2)&0x10) ? STATUS_DC : 0) |
-	      ((new_value ^ src1) & 0x80 ? STATUS_OV : 0) |
+	      (((new_value ^ src1) & 0x80) ? STATUS_OV : 0) |
 	      ((new_value & 0x80) ? STATUS_N : 0));
   }
 
   inline void put_Z_C_DC_OV_N_for_sub(unsigned int new_value, unsigned int src1, unsigned int src2)
   {
     get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC | STATUS_OV | STATUS_N)) |  
+    value.put((value.get() & ~ (STATUS_Z | STATUS_C | STATUS_DC | STATUS_OV | STATUS_N)) |
 	      ((new_value & 0xff)   ? 0 : STATUS_Z)   |
 	      ((new_value & 0x100)  ? 0 : STATUS_C)   |
 	      (((new_value ^ src1 ^ src2)&0x10) ? 0 : STATUS_DC) |
@@ -305,7 +306,7 @@ public:
   void put_FSR0_mode(unsigned int new_value)
   {
     get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~(STATUS_FSR0_MODE)) | 
+    value.put((value.get() & ~(STATUS_FSR0_MODE)) |
 	      (new_value & 0x03 ));
   }
 
@@ -318,7 +319,7 @@ public:
   void put_FSR1_mode(unsigned int new_value)
   {
     get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~(STATUS_FSR1_MODE)) | 
+    value.put((value.get() & ~(STATUS_FSR1_MODE)) |
 	      (new_value & 0x03 ));
   }
 
@@ -341,20 +342,20 @@ public:
 class Stack
 {
 public:
-  unsigned int contents[32];       /* the stack array */ 
+  unsigned int contents[32];       /* the stack array */
   int pointer;                     /* the stack pointer */
   unsigned int stack_mask;         /* 1 for 12bit, 7 for 14bit, 31 for 16bit */
   bool stack_warnings_flag;        /* Should over/under flow warnings be printed? */
   bool break_on_overflow;          /* Should over flow cause a break? */
   bool break_on_underflow;         /* Should under flow cause a break? */
 
-  Stack(Processor *);
+  explicit Stack(Processor *);
   virtual ~Stack() {}
   virtual bool push(unsigned int);
   virtual bool stack_overflow();
   virtual bool stack_underflow();
   virtual unsigned int pop();
-  virtual void reset(RESET_TYPE r) {pointer = 0;};  // %%% FIX ME %%% reset may need to change 
+  virtual void reset(RESET_TYPE r) {pointer = 0;};  // %%% FIX ME %%% reset may need to change
   // because I'm not sure how the stack is affected by a reset.
   virtual bool set_break_on_overflow(bool clear_or_set);
   virtual bool set_break_on_underflow(bool clear_or_set);
@@ -393,7 +394,7 @@ public:
   virtual void put_value(unsigned int new_value);
   virtual unsigned int get();
   virtual unsigned int get_value();
-      
+
 };
 
 class TOSH : public sfr_register
@@ -407,7 +408,7 @@ public:
   virtual void put_value(unsigned int new_value);
   virtual unsigned int get();
   virtual unsigned int get_value();
-      
+
 };
 //
 // Stack for enhanced 14 bit porcessors
@@ -419,7 +420,7 @@ public:
   TOSL   tosl;
   TOSH   tosh;
 
-  Stack14E(Processor *);
+  explicit Stack14E(Processor *);
   ~Stack14E();
 
   virtual void reset(RESET_TYPE r);
@@ -466,7 +467,7 @@ public:
 };
 
 //---------------------------------------------------------
-// 
+//
 // Indirect_Addressing
 //
 // This class coordinates the indirect addressing on the 18cxxx
@@ -476,7 +477,7 @@ public:
 // in the Indirect_Addressing class. So accessing these registers
 // is the same as accessing any register: through the core cpu's
 // register memory. The only difference for these registers is that
-// the 
+// the
 
 class Indirect_Addressing14;   // Forward reference
 
@@ -501,7 +502,7 @@ class FSRH14 : public sfr_register
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
-      
+
 protected:
   Indirect_Addressing14  *iam;
 };
@@ -515,7 +516,7 @@ class INDF14 : public sfr_register
   void put_value(unsigned int new_value);
   unsigned int get();
   unsigned int get_value();
-      
+
 protected:
   Indirect_Addressing14  *iam;
 };
@@ -534,7 +535,7 @@ public:
 			       * write operations on the indirect registers
 			       * eg. btg POSTINC1,4 . The post increment must
 			       * occur after the bit is toggled and not during
-			       * the read operation that's determining the 
+			       * the read operation that's determining the
 			       * current state.
 			       */
   int     fsr_delta;          /* If there's a pending update to the fsr register
@@ -542,7 +543,7 @@ public:
 			       * stored here.
 			       */
   guint64 current_cycle;      /* Stores the cpu cycle when the fsr was last
-			       * changed. 
+			       * changed.
 			       */
   FSRL14    fsrl;
   FSRH14    fsrh;
@@ -620,7 +621,7 @@ class PCON : public sfr_register
   enum {
     BOR = 1<<0,   // clear on Brown Out Reset
     POR = 1<<1,    // clear on Power On Reset
-    RI  = 1<<2,	   // clear on Reset instruction 
+    RI  = 1<<2,	   // clear on Reset instruction
     RMCLR = 1<<3,  // clear if hardware MCLR occurs
     SBOREN = 1<<4, //  Software BOR Enable bit
     ULPWUE = 1<<5,  // Ultra Low-Power Wake-up Enable bit
@@ -632,7 +633,7 @@ class PCON : public sfr_register
 
   void put(unsigned int new_value);
 
-  PCON(Processor *, const char *pName, const char *pDesc=0, 
+  PCON(Processor *, const char *pName, const char *pDesc=0,
 			unsigned int bitMask=0x03);
 };
 
@@ -644,7 +645,7 @@ class OSCTUNE : public  sfr_register
   void put(unsigned int new_value);
   virtual void set_osccon(OSCCON *new_osccon) { osccon = new_osccon;}
   unsigned int valid_bits;
-                                                                                
+
   enum {
     TUN0 = 1<<0,
     TUN1 = 1<<1,
@@ -653,10 +654,10 @@ class OSCTUNE : public  sfr_register
     TUN4 = 1<<4,
     TUN5 = 1<<5,
     PLLEN= 1<<6,
-    INTSRC=1<<7 
+    INTSRC=1<<7
   };
   OSCCON *osccon;
-                                                                                
+
   OSCTUNE(Processor *pCpu, const char *pName, const char *pDesc)
     : sfr_register(pCpu,pName,pDesc), valid_bits(6), osccon(0)
   {
@@ -675,27 +676,62 @@ class OSCCAL : public  sfr_register
   void put(unsigned int new_value);
   void set_freq(float base_freq);
   float base_freq;
-                                                                                
+
   OSCCAL(Processor *pCpu, const char *pName, const char *pDesc, unsigned int bitMask)
     : sfr_register(pCpu,pName,pDesc), base_freq(0.)
   {
-      mValidBits=bitMask;  // Can't use initialiser for parent class members 
+      mValidBits=bitMask;  // Can't use initialiser for parent class members
   }
 };
 
 class OSCCON : public  sfr_register,  public TriggerObject
 {
  public:
-  void put(unsigned int new_value);
+  virtual void put(unsigned int new_value);
   virtual void callback();
-  virtual bool set_rc_frequency();
+  virtual bool set_rc_frequency(bool override=false);
   virtual void set_osctune(OSCTUNE *new_osctune) { osctune = new_osctune;}
-  unsigned int valid_bits;
+  virtual void set_config_irc(unsigned int cfg_irc){config_irc = cfg_irc;}
+  virtual void set_config_xosc(unsigned int cfg_xosc){config_xosc = cfg_xosc;}
+  virtual void set_config_ieso(unsigned int cfg_ieso){config_ieso = cfg_ieso;}
+  virtual void reset(RESET_TYPE r);
+  virtual void sleep();
+  virtual void wake();
+  virtual void por_wake();
+  virtual bool internal_RC();
+  virtual void clear_irc_stable_bits() { value.put(value.get() & ~(HTS|LTS));}
+  virtual guint64 irc_por_time(); // time to stable intrc after power on reset
+  virtual guint64 irc_lh_time(); // time to stable intrc after tran low to high range
+  unsigned int write_mask;
+  unsigned int clock_state;
+  guint64      future_cycle;
+  bool         config_irc;     // FOSC bits select internal RC oscillator
+  bool         config_ieso;    //internal/external switchover bit from config word
+  bool         config_xosc;    // FOSC bits select crystal/resonator
+  bool         has_iofs_bit;
+  bool	       is_sleeping;
+  
   OSCTUNE *osctune;
-                                                                                
+
+  enum MODE
+  {
+	UNDEF = 0,
+        EXCSTABLE, 	// external source
+        LFINTOSC,       // Low Freq RC osc
+	MFINTOSC,	// Med Freq rc osc
+        HFINTOSC,       // High Freq RC osc
+        INTOSC,         // IOFS set
+	T1OSC,		// T1 OSC
+	EC,		// external clock, always stable
+        OST,      	// startup
+	PLL = 0x10
+  };   
+
   enum {
     SCS0 = 1<<0,
     SCS1 = 1<<1,
+    LTS  = 1<<1,
+    HTS  = 1<<2,
     IOFS = 1<<2,
     OSTS = 1<<3,
     IRCF0 = 1<<4,
@@ -703,13 +739,91 @@ class OSCCON : public  sfr_register,  public TriggerObject
     IRCF2 = 1<<6,
     IDLEN = 1<<7
   };
-                                                                                
+
   OSCCON(Processor *pCpu, const char *pName, const char *pDesc)
-    : sfr_register(pCpu,pName,pDesc), valid_bits(7), osctune(0)
+    : sfr_register(pCpu,pName,pDesc), write_mask(0x71), 
+	clock_state(OST), future_cycle(0), config_irc(false), config_ieso(true),
+	config_xosc(false), has_iofs_bit(false), is_sleeping(false), osctune(0)
   {
   }
 };
 
+/* OSCCON_1 IOFS bit takes 4 ms to stablize
+ */
+
+class OSCCON_1 : public OSCCON
+{
+
+public:
+
+
+//  virtual void callback();
+//  virtual void put(unsigned int new_value);
+  virtual guint64 irc_por_time(); // time to stable intrc after power on reset
+  virtual guint64 irc_lh_time();
+
+  OSCCON_1(Processor *pCpu, const char *pName, const char *pDesc)
+    : OSCCON(pCpu,pName,pDesc)
+  {
+  }
+};
+
+
+class OSCCON2 : public  sfr_register
+{
+ public:
+  void put(unsigned int new_value);
+  void set_osccon(OSCCON *new_osccon) { osccon = new_osccon;}
+  OSCCON2(Processor *pCpu, const char *pName, const char *pDesc)
+    : sfr_register(pCpu,pName,pDesc) , write_mask(0x1c), osccon(0)
+	{;}
+
+
+  unsigned int write_mask;
+  enum
+  {
+	LFIOFS  = 1<<0,		// LFINTOSC Frequency Stable bit
+	MFIOFS  = 1<<1,		// MFINTOSC Frequency Stable bit
+	PRISD   = 1<<2,		// Primary Oscillator Drive Circuit Shutdown bit
+	SOSCGO  = 1<<3,		// Secondary Oscillator Start Control bit
+	MFIOSEL = 1<<4,		// MFINTOSC Select bit
+	SOSCRUN = 1<<6,		// SOSC Run Status bit
+	PLLRDY  = 1<<7		// PLL Run Status bit
+  };
+
+  OSCCON  *osccon;
+
+};
+
+/* RC clock 16Mhz with pll to 64Mhz
+ */
+class OSCCON_HS : public OSCCON
+{
+ public:
+   virtual bool set_rc_frequency(bool override=false);
+   virtual bool internal_RC();
+   virtual void callback();
+   virtual void por_wake();
+
+   OSCCON_HS(Processor *pCpu, const char *pName, const char *pDesc) :
+       OSCCON(pCpu, pName, pDesc), osccon2(0), minValPLL(5) {}
+
+   OSCCON2  *osccon2;
+
+   enum {
+	SCS0  	= 1<<0,
+	SCS1 	= 1<<1,
+	HFIOFS 	= 1<<2,
+	OSTS	= 1<<3,
+	IRCF0	= 1<<4,
+	IRCF1	= 1<<5,
+	IRCF2	= 1<<6,
+	IDLEN	= 1<<7
+   };
+
+
+   unsigned char minValPLL;
+};
 
 class OSCSTAT : public  sfr_register
 {
@@ -730,37 +844,21 @@ class OSCSTAT : public  sfr_register
   OSCSTAT(Processor *pCpu, const char *pName, const char *pDesc)
     : sfr_register(pCpu,pName,pDesc) {}
 };
-class OSCCON_2 : public  sfr_register,  public TriggerObject
+/*
+ * OSC status in OSCSTAT register
+ */
+class OSCCON_2 : public  OSCCON
 {
  public:
-  void put(unsigned int new_value);
+  virtual void put(unsigned int new_value);
   void put_value(unsigned int new_value);
-  virtual void callback();
-  virtual bool set_rc_frequency();
-  virtual void set_osctune(OSCTUNE *new_osctune) { osctune = new_osctune;}
-  virtual void set_oscstat(OSCSTAT *_oscstat) { oscstat = _oscstat;}
-  virtual void set_config(unsigned int cfg_fosc, bool cfg_ieso);
-  virtual void set_callback();
-  virtual void wake();
-  unsigned int 	valid_bits;
-  unsigned int 	mode;
-  guint64	next_callback;
-  OSCTUNE 	*osctune;
+  virtual void  callback();
+  virtual bool  set_rc_frequency(bool override = false);
+  virtual void  set_oscstat(OSCSTAT *_oscstat) { oscstat = _oscstat;}
+  virtual void  set_callback();
+  virtual void  por_wake();
   OSCSTAT 	*oscstat;
-  unsigned int  config_fosc;	// FOSC bits from config word
-  bool		config_ieso;	//internal/external switchover bit from config word
 
-  enum MODE
-  {
-	LF = 0,
-	MF,
-	HF,
-	T1OSC,
-	EC,	// external clock, always stable
-	OST,	// use on startup timer
-	PLL = 0x10
-  };
-                                                                                
   enum {
     SCS0   = 1<<0,
     SCS1   = 1<<1,
@@ -770,10 +868,11 @@ class OSCCON_2 : public  sfr_register,  public TriggerObject
     IRCF3  = 1<<6,
     SPLLEN = 1<<7
   };
-                                                                                
+
   OSCCON_2(Processor *pCpu, const char *pName, const char *pDesc)
-    : sfr_register(pCpu,pName,pDesc), valid_bits(0xfb), 
-	next_callback(0), osctune(0), oscstat(0)
+    : OSCCON(pCpu,pName,pDesc),
+       oscstat(0)
+
   {
   }
 };
@@ -781,9 +880,9 @@ class OSCCON_2 : public  sfr_register,  public TriggerObject
 class WDTCON : public  sfr_register
 {
  public:
-                                                                                
+
   unsigned int valid_bits;
-                                                                                
+
   enum {
     WDTPS3 = 1<<4,
     WDTPS2 = 1<<3,
@@ -791,14 +890,14 @@ class WDTCON : public  sfr_register
     WDTPS0 = 1<<1,
     SWDTEN = 1<<0
   };
-                                                                                
+
   WDTCON(Processor *pCpu, const char *pName, const char *pDesc, unsigned int bits)
     : sfr_register(pCpu,pName,pDesc), valid_bits(bits)
   {
   }
   virtual void put(unsigned int new_value);
   virtual void reset(RESET_TYPE r);
-                                                                                
+
 };
 
 
@@ -859,7 +958,7 @@ public:
   WPU(Processor *pCpu, const char *pName, const char *pDesc, PicPortRegister* gpio, unsigned int mask=0x37)
     : sfr_register(pCpu,pName,pDesc), wpu_gpio(gpio), wpu_pu(false)
   {
-      mValidBits=mask;  // Can't use initialiser for parent class members 
+      mValidBits=mask;  // Can't use initialiser for parent class members
   }
 };
 
@@ -877,7 +976,7 @@ public:
 	T0XCS	= 1<<0,		// Timer0 External Clock Source Select bit
 	CPSOUT	= 1<<1,		// Capacitive Sensing Oscillator Status bit
 	CPSRNG0	= 1<<2,		// Capacitive Sensing Current Range bits
-	CPSRNG1	= 1<<3,		
+	CPSRNG1	= 1<<3,
 	CPSRM	= 1<<6,		// Capacitive Sensing Reference Mode bit
 	CPSON	= 1<<7		// CPS Module Enable bit
 	};
@@ -999,7 +1098,7 @@ class SR_MODULE: public TriggerObject
 
 public:
 
-    SR_MODULE(Processor *);
+    explicit SR_MODULE(Processor *);
     ~SR_MODULE();
 
     void	update();
@@ -1042,4 +1141,25 @@ protected:
     bool                    m_SRNQsource_active;
 };
 
+class LVDCON_14 : public  sfr_register, public TriggerObject
+{
+public:
+  unsigned int valid_bits;
+
+  enum {
+    LVDL0 = 1<<0,
+    LVDL1 = 1<<1,
+    LVDL2 = 1<<2,
+    LVDEN = 1<<4,
+    IRVST = 1<<5,
+  };
+
+  LVDCON_14(Processor *, const char *pName, const char *pDesc=0);
+  void check_lvd();
+  unsigned int 	write_mask;
+  InterruptSource *IntSrc;
+  void callback();
+  void put(unsigned int new_value);
+  virtual void setIntSrc(InterruptSource *_IntSrc) { IntSrc = _IntSrc;}
+};
 #endif
