@@ -1,6 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
-   Copyright (C) 2006,2009,2010,2013 Roy R Rankin
+   Copyright (C) 2006,2009,2010,2013,2015 Roy R Rankin
 
 This file is part of the libgpsim library of gpsim
 
@@ -15,7 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -121,7 +121,7 @@ void CCPRL::start_pwm_mode()
 }
 void CCPRL::stop_pwm_mode()
 {
-  //cout << "CCPRL: stopping pwm mode\n"; 
+  //cout << "CCPRL: stopping pwm mode\n";
 
   ccprh->pwm_mode = 0;
 }
@@ -183,7 +183,7 @@ unsigned int CCPRH::get()
 }
 
 //--------------------------------------------------
-// 
+//
 //--------------------------------------------------
 class CCPSignalSource : public SignalControl
 {
@@ -263,8 +263,8 @@ CCPCON::CCPCON(Processor *pCpu, const char *pName, const char *pDesc)
     mValidBits = 0x3f;
 }
 
-CCPCON::~CCPCON() 
-{ 
+CCPCON::~CCPCON()
+{
 
 
     for(int i = 0; i<4; i++)
@@ -330,7 +330,7 @@ void CCPCON::setIOPin2(PinModule *p2)
 void CCPCON::setIOpin(PinModule *p1, PinModule *p2, PinModule *p3, PinModule *p4)
 {
   Dprintf(("%s::setIOpin %s\n", name().c_str(), (p1 && &(p1->getPin())) ? p1->getPin().name().c_str():"unknown"));
-  if (!&(p1->getPin()))
+  if (p1 && !&(p1->getPin()))
   {
 	Dprintf(("FIXME %s::setIOpin called where p1 has unassigned pin\n", name().c_str()));
 	return;
@@ -370,7 +370,7 @@ void CCPCON::setIOpin(PinModule *p1, PinModule *p2, PinModule *p3, PinModule *p4
 
 }
 
-void CCPCON::setCrosslinks ( CCPRL *_ccprl, PIR *_pir, unsigned int _mask, 
+void CCPCON::setCrosslinks ( CCPRL *_ccprl, PIR *_pir, unsigned int _mask,
                              TMR2 *_tmr2, ECCPAS *_eccpas )
 {
   ccprl = _ccprl;
@@ -392,7 +392,7 @@ char CCPCON::getState()
 
 void CCPCON::new_edge(unsigned int level)
 {
-  Dprintf(("%s::new_edge() level=%d\n",name().c_str(), level));
+  Dprintf(("%s::new_edge() level=%u\n",name().c_str(), level));
 
   switch(value.get() & (CCPM3 | CCPM2 | CCPM1 | CCPM0))
     {
@@ -550,7 +550,7 @@ void CCPCON::releasePins(int i)
 void CCPCON::pwm_match(int level)
 {
   unsigned int new_value = value.get();
-  Dprintf(("%s::pwm_match() level=%d\n", name().c_str(), level));
+  Dprintf(("%s::pwm_match() level=%d now=0x%" PRINTF_GINT64_MODIFIER "x\n", name().c_str(), level, get_cycles().get()));
 
 
   // if the level is 1, then tmr2 == pr2 and the pwm cycle
@@ -561,7 +561,7 @@ void CCPCON::pwm_match(int level)
       // Auto shutdown comes off at start of PWM if ECCPASE clear
       if (bridge_shutdown && (!eccpas || !(eccpas->get_value() & ECCPAS::ECCPASE)))
       {
-          Dprintf(("bridge_shutdown=%d eccpas=%p ECCPAS=%x\n", bridge_shutdown, eccpas, 
+          Dprintf(("bridge_shutdown=%d eccpas=%p ECCPAS=%x\n", bridge_shutdown, eccpas,
 		eccpas ? eccpas->get_value() & ECCPAS::ECCPASE: 0));
 	  for(int i = 0; i < 4; i++)
 	  {
@@ -587,16 +587,16 @@ void CCPCON::pwm_match(int level)
         m_source[0]->setState(level ? '1' : '0');
         m_PinModule[0]->setSource(m_source[0]);
 	source_active[0] = true;
-    
-    
+
+
         if(level && !ccprl->ccprh->pwm_value)  // if duty cycle == 0 output stays low
             m_source[0]->setState('0');
 
         m_PinModule[0]->updatePinModule();
-    
+
         //cout << "iopin should change\n";
     }
-  }  
+  }
   else	// EPWM
   {
 
@@ -629,28 +629,28 @@ void CCPCON::drive_bridge(int level, int new_value)
 	active_high[2] = true;
 	active_high[3] = true;
     	break;
-    
+
         case PWM1:	// P1A P1C active high P1B P1D active low
 	active_high[0] = true;
 	active_high[1] = false;
 	active_high[2] = true;
 	active_high[3] = false;
     	break;
-    
-        case PWM2: 	// P1A P1C active low P1B P1D active high 
+
+        case PWM2: 	// P1A P1C active low P1B P1D active high
 	active_high[0] = false;
 	active_high[1] = true;
 	active_high[2] = false;
 	active_high[3] = true;
     	break;
-    
+
         case PWM3:	// //P1A, P1C, P1B, P1D active low
 	active_high[0] = false;
 	active_high[1] = false;
 	active_high[2] = false;
 	active_high[3] = false;
     	break;
-    
+
         default:
             cout << "not pwm mode. bug?\n";
     	return;
@@ -681,7 +681,7 @@ void CCPCON::drive_bridge(int level, int new_value)
 		    }
 		}
 		break;
-	
+
 	    case 2:	// Half-Bridge
 		Dprintf(("half-bridge %s\n", name().c_str()));
 		m_PinModule[0]->setSource(m_source[0]);
@@ -702,8 +702,8 @@ void CCPCON::drive_bridge(int level, int new_value)
 		delay_source1 = false;
 		// FIXME need to add deadband
 		// follow level except where duty cycle = 0
-		pwm_width = level ? 
-			ccprl->ccprh->pwm_value : 
+		pwm_width = level ?
+			ccprl->ccprh->pwm_value :
 			((tmr2->pr2->value.get()+1)*4)-ccprl->ccprh->pwm_value;
 		if (!(level^active_high[0]) && ccprl->ccprh->pwm_value)
 		{
@@ -740,7 +740,7 @@ void CCPCON::drive_bridge(int level, int new_value)
     		m_PinModule[0]->updatePinModule();
     		m_PinModule[1]->updatePinModule();
 		break;
-	
+
 	    case 1:	// Full bidge Forward
 		Dprintf(("full-bridge %s, forward\n", name().c_str()));
 		if (m_PinModule[0])
@@ -779,7 +779,7 @@ void CCPCON::drive_bridge(int level, int new_value)
     		    m_PinModule[3]->updatePinModule();
 		}
 		break;
-	
+
 	    case 3:	// Full bridge reverse
 		Dprintf(("full-bridge reverse %s\n", name().c_str()));
 		if (m_PinModule[0])
@@ -823,7 +823,7 @@ void CCPCON::drive_bridge(int level, int new_value)
 		printf("%s::pwm_match impossible ECCP bridge mode\n", name().c_str());
 		break;
        }
-	
+
 }
 //
 // Set PWM bridge into shutdown mode
@@ -879,7 +879,7 @@ void CCPCON::put(unsigned int new_value)
 
   unsigned int old_value =  value.get();
   new_value &= mValidBits;
-  
+
   Dprintf(("%s::put() new_value=0x%x\n",name().c_str(), new_value));
   trace.raw(write_trace.get() | value.get());
 
@@ -887,7 +887,7 @@ void CCPCON::put(unsigned int new_value)
   if (!ccprl || !tmr2)
     return;
 
-  // Return if no change other than possibly the duty cycle 
+  // Return if no change other than possibly the duty cycle
   if (((new_value ^ old_value) & ~(CCPY|CCPX)) == 0)
     return;
 
@@ -1021,19 +1021,49 @@ bool CCPCON::test_compare_mode()
     case PWM1:
     case PWM2:
     case PWM3:
-      return false;  
+      return false;
       break;
 
     case COM_SET_OUT:
     case COM_CLEAR_OUT:
     case COM_INTERRUPT:
     case COM_TRIGGER:
-      return true;  
+      return true;
       break;
   }
   return false;
 }
 
+TRISCCP::TRISCCP(Processor *pCpu, const char *pName, const char *pDesc) :
+	sfr_register(pCpu, pName), first(true)
+{
+}
+void TRISCCP::put(unsigned int new_value)
+{
+  if (first)
+  {
+      first = false;
+      cout << name() << " not implemented, if required, file feature request\n";
+  }
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value);
+}
+
+DATACCP::DATACCP(Processor *pCpu, const char *pName, const char *pDesc) :
+	sfr_register(pCpu, pName), first(true)
+{
+}
+
+void DATACCP::put(unsigned int new_value)
+{
+  if (first)
+  {
+      first = false;
+      cout << name() << " not implemented, if required, file feature request\n";
+  }
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value);
+}
 
 class TMR1_Interface : public Interface
 {
@@ -1060,7 +1090,7 @@ class TMR1_Interface : public Interface
 class TMR1_Freq_Attribute : public Float
 {
 public:
-  TMR1_Freq_Attribute(Processor * _cpu, double freq); 
+  TMR1_Freq_Attribute(Processor * _cpu, double freq, const char *name = "tmr1_freq");
 
   virtual void set(double d);
   double get_freq();
@@ -1069,12 +1099,13 @@ private:
   Processor * cpu;
 };
 
-TMR1_Freq_Attribute::TMR1_Freq_Attribute(Processor * _cpu, double freq)
-  : Float("tmr1_freq",freq, " Tmr1 oscillator frequency."),
+TMR1_Freq_Attribute::TMR1_Freq_Attribute(Processor * _cpu, double freq, const char *name)
+  : Float(name, freq, " Tmr oscillator frequency."),
     cpu(_cpu)
 {
 }
 
+
 double TMR1_Freq_Attribute::get_freq()
 {
 	double d;
@@ -1091,15 +1122,19 @@ void TMR1_Freq_Attribute::set(double d)
 //--------------------------------------------------
 T1CON::T1CON(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    tmrl(0)
+    tmrl(0), cpu(pCpu)
 {
 
-  pCpu->addSymbol(freq_attribute = new TMR1_Freq_Attribute(pCpu, 32768.));
+  char freq_name[] = "tmr1_freq";
+  if (*(pName+1) >= '1' && *(pName+1) <= '9')
+	freq_name[3] = *(pName+1);
+  cpu->addSymbol(freq_attribute = new TMR1_Freq_Attribute(pCpu, 32768., freq_name));
 
 }
 T1CON::~T1CON()
 {
-    delete freq_attribute;
+        cpu->removeSymbol(freq_attribute);
+        delete freq_attribute;
 }
 
 void T1CON::put(unsigned int new_value)
@@ -1135,6 +1170,10 @@ unsigned int T1CON::get_prescale()
 }
 
 
+double T1CON::t1osc()
+{
+    return (value.get() & T1OSCEN) ? freq_attribute->get_freq() : 0.;
+}
 //--------------------------------------------------
 //
 //--------------------------------------------------
@@ -1257,11 +1296,11 @@ void T1GCON::put(unsigned int new_value)
 
 void T1GCON::setGatepin(PinModule *pin)
 {
-    
+
 
     if (pin != gate_pin)
     {
-        if(sink) 
+        if(sink)
 	{
 	    gate_pin->removeSink(sink);
         }
@@ -1317,11 +1356,11 @@ void T1GCON::new_gate(bool state)
 {
     // TMR1 counts when state low (unless t1gpol is set)
     // t1g_in is inverted as per XOR in spec sheet flow chart
-    bool t1g_in = (!get_t1GPOL()) ^ state ; 
+    bool t1g_in = (!get_t1GPOL()) ^ state ;
     bool t1g_val = value.get() & T1GVAL;
     unsigned int reg_value = value.get();
 
-    
+
     if ((t1g_in == last_t1g_in) && (t1g_in == t1g_val)) // no state change, do nothing
     {
   //  	tmrl->IO_gate(t1g_val);
@@ -1363,14 +1402,14 @@ void T1GCON::new_gate(bool state)
 	//t1g_val = t1g_in;
     }
 
-    
+
     if (t1g_val)
     {
 	reg_value |= T1GVAL;
     }
     else
     {
-	if (reg_value & T1GVAL)	// interrupt on T1GVAL negative edge 
+	if (reg_value & T1GVAL)	// interrupt on T1GVAL negative edge
 	{
 	    m_Interrupt->Trigger();
 	}
@@ -1391,7 +1430,6 @@ T1CON_G::T1CON_G(Processor *pCpu, const char *pName, const char *pDesc)
     t1gcon(pCpu, "t1gcon", "TM1 Gate Control Register", this)
 {
 
-  pCpu->addSymbol(freq_attribute = new TMR1_Freq_Attribute(pCpu, 32768.));
   new_name("T1CON");
 
 }
@@ -1436,7 +1474,6 @@ TMRH::TMRH(Processor *pCpu, const char *pName, const char *pDesc)
 {
 
   value.put(0);
-  new_name("TMRH");
 
 }
 
@@ -1452,8 +1489,8 @@ void TMRH::put(unsigned int new_value)
   tmrl->set_ext_scale();
   value.put(new_value & 0xff);
   tmrl->synchronized_cycle = get_cycles().get();
-  tmrl->last_cycle = tmrl->synchronized_cycle 
-	- (gint64)((tmrl->value.get() + (value.get()<<8) 
+  tmrl->last_cycle = tmrl->synchronized_cycle
+	- (gint64)((tmrl->value.get() + (value.get()<<8)
 	* tmrl->prescale * tmrl->ext_scale) +0.5);
 
   if(tmrl->t1con->get_tmr1on())
@@ -1484,7 +1521,7 @@ unsigned int TMRH::get_value()
   tmrl->current_value();
 
   return(value.get());
-  
+
 }
 
 
@@ -1532,7 +1569,7 @@ TMRL::TMRL(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
     tmr1_interface(0),
     m_cState('?'), m_GateState(false), m_compare_GateState(true),
-    m_io_GateState(true), m_bExtClkEnabled(false), 
+    m_io_GateState(true), m_bExtClkEnabled(false),
     m_sleeping(false), m_t1gss(true), m_Interrupt(0)
 {
 
@@ -1560,9 +1597,9 @@ TMRL::~TMRL()
  * instruction cycle rate
  *
  * If tmr1cs = 1 Fosc is 4 x normal speed so reduce ticks by 1/4
- */ 
-void TMRL::set_ext_scale() 
-{ 
+ */
+void TMRL::set_ext_scale()
+{
     current_value();
     if (t1con->get_t1oscen()  && (t1con->get_tmr1cs() == 2)) // external clock
     {
@@ -1672,7 +1709,7 @@ void TMRL::compare_gate(bool state)
   if (!m_t1gss && m_GateState != state)
   {
     m_GateState = state;
-    
+
     Dprintf(("TMRL::compare_gate state %d \n", state));
 
     if (t1con->get_tmr1GE())
@@ -1688,7 +1725,7 @@ void TMRL::IO_gate(bool state)
   if (m_t1gss && (m_GateState != state))
   {
     m_GateState = state;
-    
+
     Dprintf(("TMRL::IO_gate state %d \n", state));
 
     if (t1con->get_tmr1GE())
@@ -1726,7 +1763,7 @@ void TMRL::increment()
   if (!t1con->get_tmr1on())
 	return;
 
-    // If TMRH/TMRL have been manually changed, we'll want to 
+    // If TMRH/TMRL have been manually changed, we'll want to
     // get the up-to-date values;
 
     trace.raw(write_trace.get() | value.get());
@@ -1786,17 +1823,17 @@ void TMRL::on_or_off(int new_state)
 void TMRL::update()
 {
 
-  Dprintf(("TMR1 %s update now=0x%"PRINTF_GINT64_MODIFIER"x\n",name().c_str(), get_cycles().get()));
+  Dprintf(("TMR1 %s update now=0x%" PRINTF_GINT64_MODIFIER "x\n",name().c_str(), get_cycles().get()));
   // if t1con->get_t1GINV() is false, timer can run if m_GateState == 0
-     
+
   bool gate = t1con->get_t1GINV() ? m_GateState : !m_GateState;
   Dprintf(("TMRL::update gate %d GateState %d inv %d get_tmr1on %x tmr1GE %x tmr1cs %x t1oscen %x\n", gate, m_GateState, t1con->get_t1GINV(), t1con->get_tmr1on(), t1con->get_tmr1GE(), t1con->get_tmr1cs(), t1con->get_t1oscen()));
-  /* When tmr1 is on, and t1con->get_tmr1GE() is true, 
+  /* When tmr1 is on, and t1con->get_tmr1GE() is true,
      gate == 1 allows timer to run, gate == 0 stops timer.
      However, if t1con->get_tmr1GE() is false gate has no
      effect on timer running or not.
   */
-  if(t1con->get_tmr1on() && (t1con->get_tmr1GE() ? gate : true)) 
+  if(t1con->get_tmr1on() && (t1con->get_tmr1GE() ? gate : true))
   {
     switch(t1con->get_tmr1cs())
     {
@@ -1814,9 +1851,9 @@ void TMRL::update()
 	/*
 	 external timer1 clock runs off a crystal which is typically
 	 32768 Hz and is independant on the instruction clock, but
-	 gpsim runs on the instruction clock. Ext_scale is the ratio 
+	 gpsim runs on the instruction clock. Ext_scale is the ratio
 	 of these two clocks so the breakpoint can be adjusted to be
-	 triggered at the correct time. 
+	 triggered at the correct time.
 	*/
 	      if(verbose & 0x4)
 		cout << "Tmr1 External clock\n";
@@ -1845,7 +1882,7 @@ void TMRL::update()
      set_ext_scale();
 
 
-      // Note, unlike TMR0, anytime something is written to TMRL, the 
+      // Note, unlike TMR0, anytime something is written to TMRL, the
       // prescaler is unaffected on the P18 processors. However, it is
       // reset on the p16f88 processor, which is how the current code
       // works. This only effects the external drive mode.
@@ -1859,7 +1896,7 @@ void TMRL::update()
       synchronized_cycle = get_cycles().get();
 
 
-      last_cycle = synchronized_cycle  
+      last_cycle = synchronized_cycle
 			- (gint64)(value_16bit *( prescale * ext_scale) + 0.5);
 
 
@@ -1879,7 +1916,7 @@ void TMRL::update()
       if(verbose & 0x4)
         cout << "TMR1 now at " << value_16bit << ", next event at " << break_value << '\n';
 
-      guint64 fc = get_cycles().get() 
+      guint64 fc = get_cycles().get()
 		+ (guint64)((break_value - value_16bit) * prescale * ext_scale);
 
       if(future_cycle)
@@ -1904,7 +1941,7 @@ void TMRL::update()
         current_value();
         get_cycles().clear_break(this);
 	future_cycle = 0;
-      } 
+      }
     }
 }
 
@@ -1919,7 +1956,7 @@ void TMRL::put(unsigned int new_value)
     return;
 
   synchronized_cycle = get_cycles().get();
-  last_cycle = synchronized_cycle - (gint64)(( value.get() 
+  last_cycle = synchronized_cycle - (gint64)(( value.get()
 	+ (tmrh->value.get()<<8)) * prescale * ext_scale + 0.5);
 
   current_value();
@@ -1953,7 +1990,7 @@ unsigned int TMRL::get_value()
 }
 
 //%%%FIXME%%% inline this
-// if break inactive (future_cycle == 0), just read the TMR1H and TMR1L 
+// if break inactive (future_cycle == 0), just read the TMR1H and TMR1L
 // registers otherwise compute what the register should be and then
 // update TMR1H and TMR1L.
 // RP: Using future_cycle here is not strictly right. What we really want is
@@ -1971,8 +2008,13 @@ void TMRL::current_value()
       value_16bit = tmrh->value.get() * 256 + value.get();
   else
   {
-    value_16bit = (guint64)((get_cycles().get() - last_cycle)/ 
-		(prescale* ext_scale)) & 0xffff;
+    value_16bit = (guint64)((get_cycles().get() - last_cycle)/
+		(prescale* ext_scale));
+
+
+    if (value_16bit > 0x10000)
+	cerr << "overflow TMRL " << name() << " " << value_16bit << endl;
+
     value.put(value_16bit & 0xff);
     tmrh->value.put((value_16bit>>8) & 0xff);
   }
@@ -1992,7 +2034,7 @@ unsigned int TMRL::get_low_and_high()
   trace.raw(tmrh->read_trace.get() | tmrh->value.get());
 
   return(value_16bit);
-  
+
 }
 
 // set m_bExtClkEnable is tmr1 is being clocked by an external stimulus
@@ -2062,9 +2104,9 @@ void TMRL::new_clock_source()
 }
 
 //
-// clear_timer - This is called by either the CCP or PWM modules to 
+// clear_timer - This is called by either the CCP or PWM modules to
 // reset the timer to zero. This is rather easy since the current TMR
-// value is always referenced to the cpu cycle counter. 
+// value is always referenced to the cpu cycle counter.
 //
 
 void TMRL::clear_timer()
@@ -2162,6 +2204,7 @@ void TMRL::sleep()
     m_sleeping = true;
     Dprintf(("TMRL::sleep t1sysc %d\n", t1con->get_t1sync()));
     // If tmr1 is running off Fosc/4 or Fosc this assumes Fosc stops during sleep
+
     if (  t1con->get_tmr1on() && t1con->get_tmr1cs() < 2)
     {
       if (future_cycle)
@@ -2169,7 +2212,7 @@ void TMRL::sleep()
         current_value();
         get_cycles().clear_break(this);
 	future_cycle = 0;
-      } 
+      }
     }
 }
 
@@ -2230,7 +2273,7 @@ void T2CON::put(unsigned int new_value)
   value.put(new_value);
 
   if (tmr2) {
-    
+
     tmr2->new_pre_post_scale();
 
     if( diff & TMR2ON)
@@ -2260,8 +2303,13 @@ TMR2::TMR2(Processor *pCpu, const char *pName, const char *pDesc)
   for ( int cc=0; cc<MAX_PWM_CHANS; cc++ )
       ccp[cc] = 0;
 }
+TMR2::~TMR2()
+{
+  if (m_Interrupt)
+    m_Interrupt->release();
+}
 
-void TMR2::callback_print() 
+void TMR2::callback_print()
 {
   cout << "TMR2 " << name() << " CallBack ID " << CallBackID << '\n';
 }
@@ -2334,9 +2382,9 @@ void TMR2::on_or_off(int new_state)
 }
 
 //
-// pwm_dc - 
+// pwm_dc -
+//
 //
-//  
 
 void TMR2::pwm_dc(unsigned int dc, unsigned int ccp_address)
 {
@@ -2393,11 +2441,11 @@ void TMR2::stop_pwm(unsigned int ccp_address)
 }
 
 //
-// update 
+// update
 //  This member function will determine if/when there is a TMR2 break point
 // that needs to be set and will set/move it if so.
 //  There are two different types of break sources:
-//     1) TMR2 matching PR2 
+//     1) TMR2 matching PR2
 //     2) TMR2 matching one of the ccp registers in pwm mode
 //
 
@@ -2437,9 +2485,9 @@ void TMR2::update(int ut)
       {
         if ( pwm_mode & ut & modeMask )
         {
-          /* We are in pwm mode... So let's see what happens first: a pr2 
-	     compare or a duty cycle compare. The duty cycle is 10-bits, 
-	     but the two least significant match against the prescaler 
+          /* We are in pwm mode... So let's see what happens first: a pr2
+	     compare or a duty cycle compare. The duty cycle is 10-bits,
+	     but the two least significant match against the prescaler
 	     rather than TMR2.
 	  */
 
@@ -2447,7 +2495,7 @@ void TMR2::update(int ut)
 	  {
             guint64 nc = last_cycle + ( duty_cycle[cc] * prescale ) / 4;
 
-	    // cout << "TMR2:PWM" << cc+1 << " update at " << hex << nc << 
+	    // cout << "TMR2:PWM" << cc+1 << " update at " << hex << nc <<
             //         ", dc=" << duty_cycle[cc] << "\n";
             if ( nc < fc )      /// @bug not robust against wrap-around of guint64
             {
@@ -2457,15 +2505,16 @@ void TMR2::update(int ut)
             else if ( nc == fc )
             {
               last_update |= modeMask;
-            }  
+            }
 	  }
         }
         modeMask <<= 1;
       }
 
-      if(fc < future_cycle)
+      if(fc < future_cycle && verbose & 0x04)
         cout << "TMR2: update note: new breakpoint=" << hex << fc <<
-           " before old breakpoint " << future_cycle << endl;
+           " before old breakpoint " << future_cycle <<
+	    " now " << get_cycles().get() << endl;
 
       if (fc != future_cycle)
       {
@@ -2476,7 +2525,7 @@ void TMR2::update(int ut)
     }
     else
     {
-      cout << "TMR2 BUG!! tmr2 is on but has no cycle_break set on it\n";
+      cerr << "TMR2 BUG!! tmr2 is on but has no cycle_break set on it\n";
     }
   }
   else
@@ -2514,7 +2563,7 @@ void TMR2::put(unsigned int new_value)
 
       last_cycle = current_cycle - shift;
       unsigned int now = (current_cycle - last_cycle);
-      
+
 //      printf ( "   value now is 0x%X\n", now );
 
       guint64 fc;
@@ -2527,7 +2576,7 @@ void TMR2::put(unsigned int new_value)
 		Assume TMR2 must count up to 0xff, roll-over and then
 		we are back in business. High CCP outputs stay high.
 	   3> TMR2 is now less than PR2 but greater than a CCP duty cycle point.
-		The CCP output stays as the duty cycle comparator does not 
+		The CCP output stays as the duty cycle comparator does not
 		match on this cycle.
       */
 
@@ -2559,9 +2608,9 @@ void TMR2::put(unsigned int new_value)
 
 
 
-     /* 
-	'clear' the post scale counter. (I've actually implemented the 
-	post scale counter as a count-down counter, so 'clearing it' 
+     /*
+	'clear' the post scale counter. (I've actually implemented the
+	post scale counter as a count-down counter, so 'clearing it'
 	means resetting it to the starting point.
      */
       if (t2con)
@@ -2579,7 +2628,7 @@ unsigned int TMR2::get()
 
   trace.raw(read_trace.get() | value.get());
   return(value.get());
-  
+
 }
 
 unsigned int TMR2::get_value()
@@ -2591,7 +2640,7 @@ unsigned int TMR2::get_value()
     }
 
   return(value.get());
-  
+
 }
 void TMR2::new_pre_post_scale()
 {
@@ -2618,16 +2667,6 @@ void TMR2::new_pre_post_scale()
       // which means there's a cycle break point set on TMR2 that needs to
       // be moved to a new cycle.
 
-      // Get the current value of TMR2
-      ///value = (cycles.value - last_cycle)/prescale;
-
-      current_value();
-
-      //cout << "cycles " << cycles.value.lo  << " old prescale " << prescale;
-
-
-      //cout << " prescale " << prescale;
-
 
       if (prescale != old_prescale)	// prescaler value change
       {
@@ -2659,7 +2698,7 @@ void TMR2::new_pre_post_scale()
       {
 	cout << "Warning TMR2 turned on with TMR2 greater than PR2\n";
 	// this will cause TMR2 to wrap
-        future_cycle  = get_cycles().get() + 
+        future_cycle  = get_cycles().get() +
 		(1 + pr2->value.get() + (0x100 -  value.get())) * prescale;
         get_cycles().set_break(future_cycle, this);
       }
@@ -2676,7 +2715,7 @@ void TMR2::new_pre_post_scale()
 
 void TMR2::new_pr2(unsigned int new_value)
 {
-  Dprintf(("TMR2::new_pr2 on=%d\n", t2con->get_tmr2on()));
+  Dprintf(("TMR2::new_pr2 on=%u\n", t2con->get_tmr2on()));
 
   if(t2con->get_tmr2on())
     {
@@ -2730,11 +2769,27 @@ void TMR2::current_value()
 {
   unsigned int tmr2_val = (get_cycles().get() - last_cycle)/ prescale;
 
-  value.put(tmr2_val & 0xff);
-
-  if(tmr2_val > 0x100)	// Can get to 0x100 during transition
+  if (tmr2_val == max_counts())
   {
-   cout << "TMR2 BUG!! value = " << tmr2_val << " which is greater than 0xff\n";
+    // tmr2 is about to roll over. However, the user code
+    // has requested the current value before the callback function
+    // has been invoked. So do callback and return 0.
+     if (future_cycle)
+     {
+        future_cycle = 0;
+        get_cycles().clear_break(this);
+        callback();
+     }
+     tmr2_val = 0;
+  }
+
+  value.put(tmr2_val & (max_counts() - 1));
+
+
+  if(tmr2_val >= max_counts())	// Can get to max_counts  during transition
+  {
+   cerr << "TMR2 BUG!! value = 0x" << tmr2_val << " which is greater than 0x";
+   cerr << max_counts() << endl;
   }
 }
 
@@ -2744,18 +2799,18 @@ void TMR2::current_value()
 
 void TMR2::callback()
 {
-  int cc;  
+  int cc;
 
   //cout<<"TMR2 callback cycle: " << hex << cycles.value << '\n';
 
-  // If tmr2 is still enabled, then set up for the next break. 
+  // If tmr2 is still enabled, then set up for the next break.
   // If tmr2 was disabled then ignore this break point.
   if(t2con->get_tmr2on())
     {
 
       // What caused the callback: PR2 match or duty cyle match ?
 
-      if (last_update & TMR2_WRAP) // TMR2 > PR2 
+      if (last_update & TMR2_WRAP) // TMR2 > PR2
       {
 	last_update &= ~TMR2_WRAP;
 	// This (implicitly) resets the timer to zero:
@@ -2802,7 +2857,7 @@ void TMR2::callback()
 
         for ( cc=0; cc<MAX_PWM_CHANS; cc++ )
         {
-          if ( ccp[cc] && 
+          if ( ccp[cc] &&
 		( ccp[cc]->value.get() & (CCPCON::PWM0 | CCPCON::PWM1 )))
 	  {
              ccp[cc]->pwm_match(1);
@@ -2834,7 +2889,7 @@ void TMR2::callback()
 //------------------------------------------------------------------------
 // TMR2_MODULE
 //
-// 
+//
 
 TMR2_MODULE::TMR2_MODULE()
 {
@@ -2884,7 +2939,7 @@ private:
 //--------------------------------------------------
 ECCPAS::ECCPAS(Processor *pCpu, const char *pName, const char *pDesc)
   : sfr_register(pCpu, pName, pDesc),
-    pwm1con(0), ccp1con(0), 
+    pwm1con(0), ccp1con(0),
     m_PinModule(0)
 {
     trig_state[0] = trig_state[1] = trig_state[2] = false;
