@@ -1,22 +1,22 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 
 #ifndef __16BIT_INSTRUCTIONS_H__
@@ -103,11 +103,11 @@ protected:
 };
 
 //---------------------------------------------------------
-class ADDFSR : public instruction 
+class ADDFSR16 : public instruction 
 {
 
 public:
-  ADDFSR(Processor *new_cpu, unsigned int new_opcode,const char *, unsigned int address);
+  ADDFSR16(Processor *new_cpu, unsigned int new_opcode,const char *, unsigned int address);
   virtual bool isBase() { return true;}
   virtual void execute();
   virtual char *name(char *,int);
@@ -121,8 +121,8 @@ public:
 	return new ADDULNK(new_cpu,new_opcode,"addulnk", address);
     }
     if (new_opcode & 0x100)
-      return new ADDFSR(new_cpu,new_opcode,"subfsr", address);
-    return new ADDFSR(new_cpu,new_opcode,"addfsr", address);
+      return new ADDFSR16(new_cpu,new_opcode,"subfsr", address);
+    return new ADDFSR16(new_cpu,new_opcode,"addfsr", address);
   }
 protected:
   unsigned int m_fsr;
@@ -131,16 +131,16 @@ protected:
 };
 
 //---------------------------------------------------------
-class CALLW : public instruction
+class CALLW16 : public CALLW
 {
 public:
-  CALLW(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
+  CALLW16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) :
+	CALLW(new_cpu, new_opcode, address){};
   virtual bool isBase() { return true;}
   virtual void execute();
-  virtual char *name(char *,int);
   static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
   {
-    return new CALLW(new_cpu,new_opcode,address);
+    return new CALLW16(new_cpu,new_opcode,address);
   }
 };
 //---------------------------------------------------------
@@ -204,14 +204,15 @@ public:
 };
 
 //---------------------------------------------------------
-class ADDWFC : public Register_op
+class ADDWFC16 : public ADDWFC
 {
 public:
 
-  ADDWFC(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
+  ADDWFC16(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+    : ADDWFC(new_cpu,new_opcode,address){};
   virtual void execute();
   static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  {return new ADDWFC(new_cpu,new_opcode,address);}
+  {return new ADDWFC16(new_cpu,new_opcode,address);}
 };
 
 //---------------------------------------------------------
@@ -324,18 +325,70 @@ public:
 };
 
 //---------------------------------------------------------
-class BRA : public instruction
+class BRA16 : public instruction
 {
 public:
   int destination_index;
   unsigned int absolute_destination_index;
 
-  BRA(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
+  BRA16(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
   virtual void execute();
   virtual char *name(char *,int);
   virtual bool isBase() { return true;}
   static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  {return new BRA(new_cpu,new_opcode,address);}
+  {return new BRA16(new_cpu,new_opcode,address);}
+};
+
+//---------------------------------------------------------
+class BSF16 : public BSF
+{
+public:
+  int i;
+
+  BSF16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) 
+    : BSF(new_cpu,new_opcode,address){};
+  virtual void execute();
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  {return new BSF16(new_cpu,new_opcode,address);}
+};
+
+//---------------------------------------------------------
+class BCF16 : public BCF
+{
+public:
+  int i;
+
+  BCF16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) 
+    : BCF(new_cpu,new_opcode,address){};
+  virtual void execute();
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  {return new BCF16(new_cpu,new_opcode,address);}
+};
+
+//---------------------------------------------------------
+class BTFSC16 : public BTFSC
+{
+public:
+  int i;
+
+  BTFSC16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) 
+    : BTFSC(new_cpu,new_opcode,address){};
+  virtual void execute();
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  {return new BTFSC16(new_cpu,new_opcode,address);}
+};
+
+//-----------------------------------------------------------
+class BTFSS16 : public BTFSS
+{
+public:
+  int i;
+
+  BTFSS16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) 
+    : BTFSS(new_cpu,new_opcode,address){};
+  virtual void execute();
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  {return new BTFSS16(new_cpu,new_opcode,address);}
 };
 
 //---------------------------------------------------------
@@ -378,6 +431,18 @@ public:
 };
 
 
+//-----------------------------------------------------------
+class CLRF16 : public CLRF
+{
+public:
+
+  CLRF16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) 
+    : CLRF(new_cpu,new_opcode,address){};
+  virtual void execute();
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  {return new CLRF16(new_cpu,new_opcode,address);}
+};
+
 //---------------------------------------------------------
 class COMF16 : public COMF
 {
@@ -632,13 +697,13 @@ public:
 
 //---------------------------------------------------------
 
-class MOVLB : public Literal_op
+class MOVLB16 : public Literal_op
 {
 public:
-  MOVLB(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
+  MOVLB16(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
   virtual void execute();
   static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  {return new MOVLB(new_cpu,new_opcode,address);}
+  {return new MOVLB16(new_cpu,new_opcode,address);}
 
 };
 
@@ -685,6 +750,7 @@ public:
 
 //---------------------------------------------------------
 
+#ifdef RRR
 class MOVWF16a : public MOVWF
 {
 public:
@@ -697,6 +763,7 @@ public:
 
 };
 
+#endif //RRR
 //---------------------------------------------------------
 
 class MULLW : public Literal_op
@@ -787,19 +854,6 @@ public:
   {return new RCALL(new_cpu,new_opcode,address);}
 };
 
-//---------------------------------------------------------
-class RESET : public instruction
-{
-public:
-
-  RESET(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
-  virtual void execute();
-  virtual bool isBase() { return true;}
-  static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  {return new RESET(new_cpu,new_opcode,address);}
-
-};
-
 
 
 //---------------------------------------------------------
@@ -967,15 +1021,30 @@ public:
 };
 
 //---------------------------------------------------------
-class SUBWFB : public Register_op
+class SUBWFB16 : public SUBWFB
 {
 public:
 
-  SUBWFB(Processor *new_cpu, unsigned int new_opcode, unsigned int address);
+  SUBWFB16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) :
+      SUBWFB(new_cpu,new_opcode,address) { };
   virtual void execute();
 
   static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
-  {return new SUBWFB(new_cpu,new_opcode,address);}
+  {return new SUBWFB16(new_cpu,new_opcode,address);}
+
+};
+
+//---------------------------------------------------------
+class SWAPF16 : public SWAPF
+{
+public:
+
+  SWAPF16(Processor *new_cpu, unsigned int new_opcode, unsigned int address) :
+      SWAPF(new_cpu,new_opcode,address) { };
+  virtual void execute();
+
+  static instruction *construct(Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  {return new SWAPF16(new_cpu,new_opcode,address);}
 
 };
 
