@@ -570,8 +570,7 @@ void Program_Counter16::computed_goto(unsigned int new_address)
   if (value >= memory_size)
 	value -= memory_size;
 
-  // see Update pcl comment in Program_Counter::increment()
-  cpu_pic->pcl->value.put((value<<1) & 0xff);
+  update_pcl();
 
   // The instruction modifying the PCL will also increment the program counter.
   // So, pre-compensate the increment with a decrement:
@@ -581,7 +580,7 @@ void Program_Counter16::computed_goto(unsigned int new_address)
   // the instruction (i.e. via the ::increment() method). The second cycle occurs
   // here:
 
-  mExecute2ndHalf->advance();
+  cpu_pic->mExecute2ndHalf->advance();
 }
 
 //--------------------------------------------------
@@ -621,6 +620,15 @@ unsigned int Program_Counter16::get_value()
   return value << 1;
 }
 
+//--------------------------------------------------
+// update_pcl - Updates the PCL from within the Program_Counter class.
+//   
+ 
+void Program_Counter16::update_pcl()
+{
+  // For 16 bit devices the PCL will be Program_Counter*2
+  cpu_pic->pcl->value.put((value<<1) & 0xff);
+}
 
 //------------------------------------------------
 // TOSU
@@ -1305,89 +1313,6 @@ LVDCON::LVDCON(Processor *pCpu, const char *pName, const char *pDesc)
 }
 
 
-bool OSCCON_HS::set_rc_frequency()
-{
-  double base_frequency = 31.e3;
-  bool config_pplx4 = cpu_pic->get_pplx4_osc();
-  bool osccon_pplx4 = (osctune)?osctune->value.get() & OSCTUNE::PLLEN:0;
-
-  if (!cpu_pic->get_int_osc())
-     return false;
-
-    unsigned int new_IRCF = (value.get() & ( IRCF0 | IRCF1 | IRCF2)) >> 4;
-    switch (new_IRCF)
-    {
-    case 0:
-	base_frequency = 31.e3;
-	break;
-	
-    case 1:
-	base_frequency = 125e3;
-	break;
-	
-    case 2:
-	base_frequency = 250e3;
-	break;
-	
-    case 3:
-	base_frequency = 1e6;
-	break;
-	
-    case 4:
-	base_frequency = 2e6;
-	break;
-	
-    case 5:
-	if (osccon_pplx4 || config_pplx4)
-	{
-	    base_frequency = 16e6;
-	}
-	else
-	{
-	    base_frequency = 4e6;
-	}
-	break;
-	
-    case 6:
-	if (osccon_pplx4 || config_pplx4)
-	{
-	    base_frequency = 32e6;
-	}
-	else
-	{
-	    base_frequency = 8e6;
-	}
-	break;
-	
-    case 7:
-	if (osccon_pplx4 || config_pplx4)
-	{
-	    base_frequency = 64e6;
-	}
-	else
-	{
-	    base_frequency = 16e6;
-	}
-	break;
-   }
-   if (osctune)
-   {
-       int tune;
-       unsigned int osctune_value = osctune->value.get();
-       tune = osctune_value & (OSCTUNE::TUN5-1);
-       tune = (OSCTUNE::TUN5 & osctune_value) ? -tune : tune;
-       base_frequency *= 1. + 0.125 * tune / 31.;
-   }
-   cpu_pic->set_frequency(base_frequency);
-   if ((bool)verbose)
-   {
-	cout << "set_rc_frequency() : osccon=" << hex << value.get();
-	if (osctune)
-	    cout << " osctune=" << osctune->value.get();
-	cout << " new frequency=" << base_frequency << endl;
-   }
-   return true;
-}
 /*******************************************************************
 	HLVDCON - High/Low-Voltage Detect Module
 */
