@@ -1,5 +1,5 @@
 /*
-        
+
    Copyright (C) 1998 T. Scott Dattalo
    Copyright (C) 2006,2010,2013,2015 Roy R. Rankin
 
@@ -16,7 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -189,11 +189,12 @@ CMCON::CMCON(Processor *pCpu, const char *pName, const char *pDesc)
   cm_input[0]=cm_input[1]=cm_input[2]=cm_input[3]=0;
   cm_output[0] = cm_output[1] = 0;
   cm_input_pin[0]=cm_input_pin[1]=cm_input_pin[2]=cm_input_pin[3]=0;
+  cm_an[0] = cm_an[1] = cm_an[2] = cm_an[3] = 0;
   cm_output_pin[0]=cm_output_pin[1]=0;
   cm_source[0]=cm_source[1]=0;
   cm_stimulus[0]=cm_stimulus[1]=cm_stimulus[2]=cm_stimulus[3]=0;
   cm_source_active[0]=cm_source_active[1] = false;
-  
+
 }
 
 CMCON::~CMCON()
@@ -206,7 +207,7 @@ CMCON::~CMCON()
 	if (cm_source[i])
 	{
 	    int cfg = m_configuration_bits[i][mode] & CFG_MASK;
-	    
+
 	    // Our Source active so port still defined if cm_output defined,
 	    // set default source.
 	    if (cfg == i && cm_output[i] && cm_source_active[i])
@@ -214,13 +215,14 @@ CMCON::~CMCON()
 	    delete cm_source[i];
 	}
   }
-  if (cm_stimulus[0]) delete cm_stimulus[0];
-  if (cm_stimulus[1]) delete cm_stimulus[1];
-  if (cm_stimulus[2]) delete cm_stimulus[2];
-  if (cm_stimulus[3]) delete cm_stimulus[3];
-  free(cm_input_pin[0]);  free(cm_input_pin[1]);
-  free(cm_input_pin[2]);  free(cm_input_pin[3]);
-  free(cm_output_pin[0]); free(cm_output_pin[1]);
+  for (int i = 0; i < 4; i++)
+  {
+      if (cm_stimulus[i]) delete cm_stimulus[i];
+      if (cm_input_pin[i]) free(cm_input_pin[i]);
+      if (cm_an[i]) free(cm_an[i]);
+  }
+  if (cm_output_pin[0]) free(cm_output_pin[0]);
+  if (cm_output_pin[1]) free(cm_output_pin[1]);
 }
 
 
@@ -249,22 +251,35 @@ void CMCON::assign_pir_set(PIR_SET *new_pir_set)
 double CMCON::comp_voltage(int ind, int invert)
 {
     double Voltage;
+    const char *name;
+
 
     switch(ind)
     {
+    case V06:
+	Voltage = 0.6;
+	name = "V0.6";
+	break;
+
     case VREF:
         Voltage = _vrcon->get_Vref();
+	name = "Vref";
         break;
 
     case NO_IN:
         Voltage = invert ? cpu->get_Vdd() : 0.;
+	name = "No_IN";
         break;
 
     default:
         Voltage = cm_input[ind]->getPin().get_nodeVoltage();
+	name = cm_input[ind]->getPin().name().c_str();
         break;
     }
-    Dprintf(("CMCON::comp_voltage ind=%d %.2f %s\n", ind, Voltage, (ind==VREF)?"Vref":cm_input[ind]->getPin().name().c_str()));
+    if (name)	// this is just to avoid a compiler warning
+    {
+    Dprintf(("CMCON::comp_voltage ind=%d IN%c %.2f %s\n", ind, invert?'-':'+', Voltage, name));
+    }
     return Voltage;
 }
 /*
@@ -288,6 +303,7 @@ unsigned int CMCON::get()
         int output_bit = (i == 0) ? C1OUT : C2OUT;
         int shift = (cmcon_val & CIS) ? CFG_SHIFT : CFG_SHIFT*3;
 
+
         if ((m_configuration_bits[i][mode] & CFG_MASK) != ZERO)
         {
             Vhigh = comp_voltage(
@@ -302,6 +318,7 @@ unsigned int CMCON::get()
             else
                 out_true = (cmcon_val & invert_bit)?true:false;
 
+
             if (out_true)
                 cmcon_val |= output_bit;
             else
@@ -332,13 +349,13 @@ unsigned int CMCON::get()
 	}
         // Generate interupt ?
         if (pir_set)
-	{
-	    if (diff & C1OUT)
+        {
+    	   if (diff & C1OUT)
                 pir_set->set_c1if();
-
-	    if (diff & C2OUT)
+    
+    	   if (diff & C2OUT)
                 pir_set->set_c2if();
-	}
+        }
    }
    if (m_tmrl)
 	m_tmrl->compare_gate((cmcon_val & C1OUT) == C1OUT);
@@ -446,7 +463,7 @@ void CMCON::put(unsigned int new_value)
 }
 //------------------------------------------------------------------------
 CMCON1::CMCON1(Processor *pCpu, const char *pName, const char *pDesc)
-  : sfr_register(pCpu, pName, pDesc), m_tmrl(0)
+  : sfr_register(pCpu, pName, pDesc), m_tmrl(0), valid_bits(0x3)
 {
 }
 CMCON1::~CMCON1() {}
@@ -461,7 +478,7 @@ void CMCON1::put(unsigned int new_value)
   m_tmrl->set_T1GSS(new_value & T1GSS);
 
   trace.raw(write_trace.get() | value.get());
-  value.put(new_value);
+  value.put(new_value & valid_bits);
 }
 //------------------------------------------------------------------------
 SRCON::SRCON(Processor *pCpu, const char *pName, const char *pDesc)
@@ -484,7 +501,7 @@ void SRCON::put(unsigned int new_value)
 
 }
 
-CMxCON0_base::CMxCON0_base(Processor *pCpu, const char *pName, 
+CMxCON0_base::CMxCON0_base(Processor *pCpu, const char *pName,
 	const char *pDesc, unsigned int _cm, ComparatorModule2 *cmModule)
   : sfr_register(pCpu, pName, pDesc),
     cm_output(0),
@@ -517,7 +534,7 @@ CMxCON0_base::~CMxCON0_base()
 unsigned int CMxCON0_base::get()
 {
     bool output;
-    
+
     if (! is_on())
     {
 	// need to test what happens in a real device RRR
@@ -722,7 +739,7 @@ void VRCON_2::put(unsigned int new_value)
 	    vr_pu->set_Vth(((Processor *)cpu)->get_Vdd());
       else
 	    vr_pu->set_Vth(0.0);
-	
+
       vr_Rhigh = (8 + (16 - (new_value & 0x0f))) * 2000.;
       vr_pu->set_Zth(vr_Rhigh);
       vr_Rlow = (new_value & 0x0f) * 2000.;
@@ -736,7 +753,7 @@ void VRCON_2::put(unsigned int new_value)
 }
 
 
-CMxCON0::CMxCON0(Processor *pCpu, const char *pName, 
+CMxCON0::CMxCON0(Processor *pCpu, const char *pName,
 	const char *pDesc, unsigned int _cm, ComparatorModule2 *cmModule)
   : CMxCON0_base(pCpu, pName, pDesc, _cm, cmModule)
 {
@@ -758,7 +775,7 @@ void CMxCON0::put(unsigned int new_value)
 
   if (diff == 0)
   {
-     get(); 
+     get();
      return;
   }
 
@@ -771,7 +788,7 @@ void CMxCON0::put(unsigned int new_value)
           if ( ! cm_source)
                 cm_source = new CMxSignalSource(cm_output, this);
 
-	  sprintf(name, "c%dout", cm+1);
+	  snprintf(name, sizeof(name), "c%uout", cm + 1);
 	  assert(cm_output);
 	  cm_output->getPin().newGUIname(name);
           cm_output->setSource(cm_source);
@@ -806,7 +823,7 @@ void CMxCON0::set_output(bool output)
 	cmxcon0 |= CxOUT;
     else
 	cmxcon0 &= ~CxOUT;
-	Dprintf(("cm%d POL %d output %d cmxcon0=%x old_out %d\n", cm+1, (bool)(cmxcon0 & CxPOL), output, cmxcon0, old_out));
+	Dprintf(("cm%u POL %d output %d cmxcon0=%x old_out %d\n", cm+1, (bool)(cmxcon0 & CxPOL), output, cmxcon0, old_out));
     value.put(cmxcon0);
     m_cmModule->set_cmout(cm, output);
     if (cmxcon0 & CxOE)
@@ -816,11 +833,11 @@ void CMxCON0::set_output(bool output)
     }
     if (old_out != output) // state change
     {
-	// Positive going edge, set interrupt ? 
+	// Positive going edge, set interrupt ?
 	if (output && (m_cmModule->cmxcon1[cm]->value.get() & CMxCON1::CxINTP))
 	    IntSrc->Trigger();
 
-	// Negative going edge, set interrupt ? 
+	// Negative going edge, set interrupt ?
 	if (!output && (m_cmModule->cmxcon1[cm]->value.get() & CMxCON1::CxINTN))
 	    IntSrc->Trigger();
     }
@@ -841,7 +858,7 @@ void CMxCON0_V2::put(unsigned int new_value)
 
   if (diff == 0)
   {
-     get(); 
+     get();
      return;
   }
   if ((diff & CxON) && !(new_value & CxON)) // turning off
@@ -864,7 +881,7 @@ void CMxCON0_V2::put(unsigned int new_value)
           if ( ! cm_source)
                 cm_source = new CMxSignalSource(cm_output, this);
 
-	  sprintf(name, "c%dout", cm+1);
+	  snprintf(name, sizeof(name), "c%uout", cm + 1);
 	  assert(cm_output);
 	  cm_output->getPin().newGUIname(name);
           cm_output->setSource(cm_source);
@@ -894,7 +911,7 @@ void CMxCON0_V2::set_output(bool output)
 	cmxcon0 &= ~CxOUT;
 	cmxcon1 &= ~((cm==0)? CM2CON1_V2::MC1OUT : CM2CON1_V2::MC2OUT);
     }
-	Dprintf(("cm%d POL %d output %d cmxcon0=%x old_out %d\n", cm+1, (bool)(cmxcon0 & CxPOL), output, cmxcon0, old_out));
+	Dprintf(("cm%u POL %d output %d cmxcon0=%x old_out %d\n", cm+1, (bool)(cmxcon0 & CxPOL), output, cmxcon0, old_out));
     value.put(cmxcon0);
     m_cmModule->cmxcon1[cm]->value.put(cmxcon1);
     m_cmModule->set_cmout(cm, output);
@@ -906,7 +923,7 @@ void CMxCON0_V2::set_output(bool output)
     if (old_out != output) // state change
     {
 	m_cmModule->cmxcon1[cm]->tmr_gate(cm, output);
-	// Positive going edge, set interrupt ? 
+	// Positive going edge, set interrupt ?
 	if (output)
 	    IntSrc->Trigger();
     }
@@ -923,7 +940,7 @@ double CMxCON0_V2::get_hysteresis()
 
     return hyst_volt;
 }
-CMxCON0_V2::CMxCON0_V2(Processor *pCpu, const char *pName, 
+CMxCON0_V2::CMxCON0_V2(Processor *pCpu, const char *pName,
 	const char *pDesc, unsigned int _cm, ComparatorModule2 *cmModule)
   : CMxCON0_base(pCpu, pName, pDesc, _cm, cmModule)
 {
@@ -933,13 +950,13 @@ CMxCON0_V2::~CMxCON0_V2()
 }
 
 
-double CMxCON0_V2::get_Vpos() 
-{ 
-    return m_cmModule->cmxcon1[cm]->get_Vpos(cm, value.get()); 
+double CMxCON0_V2::get_Vpos()
+{
+    return m_cmModule->cmxcon1[cm]->get_Vpos(cm, value.get());
 }
-double CMxCON0_V2::get_Vneg() 
-{ 
-    return m_cmModule->cmxcon1[cm]->get_Vneg(cm, value.get()); 
+double CMxCON0_V2::get_Vneg()
+{
+    return m_cmModule->cmxcon1[cm]->get_Vneg(cm, value.get());
 }
 
 void CM2CON1_V4::put(unsigned int new_value)
@@ -965,12 +982,12 @@ double CM2CON1_V4::get_Vpos(unsigned int cm, unsigned int cmxcon0)
 	{
 	    Voltage = ((Processor *)cpu)->CVREF->get_nodeVoltage();
 	    Dprintf(("%s CVref %.2f\n", __FUNCTION__, Voltage));
-	    
+
         }
 	else
 	{
 	    Voltage = ((Processor *)cpu)->V06REF->get_nodeVoltage();
-	    Dprintf(("%s cm%d V06ref %.2f\n", __FUNCTION__, cm+1, Voltage));
+	    Dprintf(("%s cm%u V06ref %.2f\n", __FUNCTION__, cm+1, Voltage));
 	}
     }
     else		// use CM1IN+ or CM2IN+
@@ -978,14 +995,14 @@ double CM2CON1_V4::get_Vpos(unsigned int cm, unsigned int cmxcon0)
         if (!stimulus_pin[POS])
 	    setPinStimulus(cm_inputPos[cm], POS);
         Voltage =  cm_inputPos[cm]->getPin().get_nodeVoltage();
-        Dprintf(("%s cm%d %s %.2f\n", __FUNCTION__, cm+1, cm_inputPos[cm]->getPin().name().c_str(), Voltage));
+        Dprintf(("%s cm%u %s %.2f\n", __FUNCTION__, cm+1, cm_inputPos[cm]->getPin().name().c_str(), Voltage));
     }
     return Voltage;
 }
   CM2CON1_V4::CM2CON1_V4(Processor *pCpu, const char *pName, const char *pDesc,
                 unsigned int _cm, ComparatorModule2 * cmModule) :
                 CM2CON1_V3(pCpu, pName, pDesc, _cm, cmModule),
-                m_vrcon(0) 
+                m_vrcon(0)
   {
         cm1_cvref = new CM_stimulus((CMCON *)m_cmModule->cmxcon0[0], "cm1_cvref", 0, 1e12);
         cm1_v06ref = new CM_stimulus((CMCON *)m_cmModule->cmxcon0[0], "cm1_v06ref", 0, 1e12);
@@ -1035,12 +1052,12 @@ double CM2CON1_V3::get_Vpos(unsigned int cm, unsigned int cmxcon0)
 	    (cm == 1 && (cmxcon1 & C2RSEL)))
 	{
 	    Voltage =  m_vrcon->get_Vref();
-	    Dprintf(("%s cm%d Vref %.2f\n", __FUNCTION__, cm+1, Voltage));
+	    Dprintf(("%s cm%u Vref %.2f\n", __FUNCTION__, cm+1, Voltage));
 	}
-	else 
+	else
 	{
 	    Voltage =  0.6;
-	    Dprintf(("%s cm%d Absref %.2f\n", __FUNCTION__, cm+1, Voltage));
+	    Dprintf(("%s cm%u Absref %.2f\n", __FUNCTION__, cm+1, Voltage));
 	}
     }
     else		// use CM1IN+ or CM2IN+
@@ -1048,7 +1065,7 @@ double CM2CON1_V3::get_Vpos(unsigned int cm, unsigned int cmxcon0)
         if (stimulus_pin[POS] != cm_inputPos[cm])
 	    setPinStimulus(cm_inputPos[cm], POS);
         Voltage =  cm_inputPos[cm]->getPin().get_nodeVoltage();
-        Dprintf(("%s cm%d %s %.2f\n", __FUNCTION__, cm+1, cm_inputPos[cm]->getPin().name().c_str(), Voltage));
+        Dprintf(("%s cm%u %s %.2f\n", __FUNCTION__, cm+1, cm_inputPos[cm]->getPin().name().c_str(), Voltage));
     }
     return Voltage;
 }
@@ -1058,7 +1075,7 @@ double CM2CON1_V3::get_Vneg(unsigned int cm, unsigned int cmxcon0)
     unsigned int cxNchan = cmxcon0 & (CMxCON0_V2::CxCH0 | CMxCON0_V2::CxCH1);
     if (stimulus_pin[NEG] != cm_inputNeg[cxNchan])
 	setPinStimulus(cm_inputNeg[cxNchan], NEG);
-    Dprintf(("%s cm%d pin %d %s %.2f\n", __FUNCTION__, cm+1, cxNchan, cm_inputNeg[cxNchan]->getPin().name().c_str(), cm_inputNeg[cxNchan]->getPin().get_nodeVoltage()));
+    Dprintf(("%s cm%u pin %u %s %.2f\n", __FUNCTION__, cm+1, cxNchan, cm_inputNeg[cxNchan]->getPin().name().c_str(), cm_inputNeg[cxNchan]->getPin().get_nodeVoltage()));
     return cm_inputNeg[cxNchan]->getPin().get_nodeVoltage();
 }
 //*************************************************************
@@ -1080,11 +1097,11 @@ void CM2CON1_V3::tmr_gate(unsigned int cm, bool output)
 {
     if (cm == 1 && m_cmModule->tmr1l[0]) //CM2
     {
-	Dprintf(("CM2CON1_V3::tmr_gate cm%d output=%d\n", cm+1, output));
+	Dprintf(("CM2CON1_V3::tmr_gate cm%u output=%d\n", cm+1, output));
 	m_cmModule->tmr1l[0]->compare_gate(output);
     }
 }
-CM2CON1_V2::CM2CON1_V2(Processor *pCpu, const char *pName, 
+CM2CON1_V2::CM2CON1_V2(Processor *pCpu, const char *pName,
         const char *pDesc, ComparatorModule2 * cmModule):
 	CMxCON1_base(pCpu, pName, pDesc, 0, cmModule),
         ctmu_stim(0), ctmu_attached(false)
@@ -1095,8 +1112,10 @@ CM2CON1_V2::CM2CON1_V2(Processor *pCpu, const char *pName,
 
      ctmu_stimulus_pin = 0;
 }
-CM2CON1_V2::~CM2CON1_V2() 
+CM2CON1_V2::~CM2CON1_V2()
 {
+    delete cm_stimulus[2];
+    delete cm_stimulus[3];
 }
 
 double CM2CON1_V2::get_Vpos(unsigned int cm, unsigned int cmxcon0)
@@ -1111,12 +1130,12 @@ double CM2CON1_V2::get_Vpos(unsigned int cm, unsigned int cmxcon0)
 	    (cm == 1 && (cmxcon1 & C2RSEL)))
 	{
 	    Voltage =  m_cmModule->FVR_voltage;
-	    Dprintf(("%s cm%d FVR %.2f\n", __FUNCTION__, cm+1, Voltage));
+	    Dprintf(("%s cm%u FVR %.2f\n", __FUNCTION__, cm+1, Voltage));
 	}
-	else 
+	else
 	{
 	    Voltage =  m_cmModule->DAC_voltage;
-	    Dprintf(("%s cm%d DAC %.2f\n", __FUNCTION__, cm+1, Voltage));
+	    Dprintf(("%s cm%u DAC %.2f\n", __FUNCTION__, cm+1, Voltage));
 	}
     }
     else		// use CM1IN+ or CM2IN+
@@ -1124,7 +1143,7 @@ double CM2CON1_V2::get_Vpos(unsigned int cm, unsigned int cmxcon0)
         if (stimulus_pin[POS+cm*2] != cm_inputPos[cm])
 	    setPinStimulus(cm_inputPos[cm], POS+cm*2);
         Voltage =  cm_inputPos[cm]->getPin().get_nodeVoltage();
-    	Dprintf(("%s cm%d %s %.2f\n", __FUNCTION__, cm+1, cm_inputPos[cm]->getPin().name().c_str(), Voltage));
+    	Dprintf(("%s cm%u %s %.2f\n", __FUNCTION__, cm+1, cm_inputPos[cm]->getPin().name().c_str(), Voltage));
     }
     return Voltage;
 }
@@ -1136,7 +1155,7 @@ double CM2CON1_V2::get_Vneg(unsigned int cm, unsigned int cmxcon0)
 	setPinStimulus(cm_inputNeg[cxNchan], NEG+cm*2);
     if (cm_inputNeg[cxNchan]->getPin().snode)
 	cm_inputNeg[cxNchan]->getPin().snode->update();
-    Dprintf(("%s cm%d pin %d %s %.2f\n", __FUNCTION__, cm+1, cxNchan, cm_inputNeg[cxNchan]->getPin().name().c_str(), cm_inputNeg[cxNchan]->getPin().get_nodeVoltage()));
+    Dprintf(("%s cm%u pin %u %s %.2f\n", __FUNCTION__, cm+1, cxNchan, cm_inputNeg[cxNchan]->getPin().name().c_str(), cm_inputNeg[cxNchan]->getPin().get_nodeVoltage()));
     return cm_inputNeg[cxNchan]->getPin().get_nodeVoltage();
 }
 
@@ -1153,7 +1172,7 @@ bool CM2CON1_V2::hyst_active(unsigned int cm)
 }
 void CM2CON1_V2::tmr_gate(unsigned int cm, bool output)
 {
-   Dprintf(("CM2CON1_V2::tmr_gate cm%d output %d\n", cm+1, output));
+   Dprintf(("CM2CON1_V2::tmr_gate cm%u output %d\n", cm+1, output));
    for (int i=0; i < 3; i++)
    {
 	if (m_cmModule->t1gcon[i])
@@ -1218,12 +1237,12 @@ CMxCON1::CMxCON1(Processor *pCpu, const char *pName, const char *pDesc, unsigned
 CMxCON1::~CMxCON1()
 {
 }
-CMxCON1_base::CMxCON1_base(Processor *pCpu, const char *pName, 
+CMxCON1_base::CMxCON1_base(Processor *pCpu, const char *pName,
 	const char *pDesc, unsigned int _cm, ComparatorModule2 *cmModule)
   : sfr_register(pCpu, pName, pDesc),
   cm(_cm), m_cmModule(cmModule)
 {
-     
+
      assert(m_cmModule->cmxcon0[cm]);
      cm_stimulus[NEG] = new CM_stimulus((CMCON *)m_cmModule->cmxcon0[cm], "cm_stimulus_-", 0, 1e12);
      cm_stimulus[POS] = new CM_stimulus((CMCON *)m_cmModule->cmxcon0[cm], "cm_stimulus_+", 0, 1e12);
@@ -1251,7 +1270,7 @@ double CMxCON1::get_Vneg(unsigned int arg, unsigned int arg2)
 	setPinStimulus(cm_inputNeg[cxNchan], NEG);
     if (cm_inputNeg[cxNchan]->getPin().snode)
 	cm_inputNeg[cxNchan]->getPin().snode->update();
-    Dprintf(("%s pin %d %s %.2f\n", __FUNCTION__, cxNchan, cm_inputNeg[cxNchan]->getPin().name().c_str(), cm_inputNeg[cxNchan]->getPin().get_nodeVoltage()));
+    Dprintf(("%s pin %u %s %.2f\n", __FUNCTION__, cxNchan, cm_inputNeg[cxNchan]->getPin().name().c_str(), cm_inputNeg[cxNchan]->getPin().get_nodeVoltage()));
     return cm_inputNeg[cxNchan]->getPin().get_nodeVoltage();
 }
 
@@ -1455,19 +1474,19 @@ void ComparatorModule2::set_if(unsigned int cm)
 {
     switch(cm)
     {
-    case 0:	
+    case 0:
 	pir_set->set_c1if();
 	break;
 
-    case 1:	
+    case 1:
 	pir_set->set_c2if();
 	break;
 
-    case 2:	
+    case 2:
 	pir_set->set_c3if();
 	break;
 
-    case 3:	
+    case 3:
 	pir_set->set_c4if();
 	break;
 
