@@ -23,6 +23,7 @@ License along with this library; if not, see
 #define __COMPARATOR_H__
 
 #include "14bit-tmrs.h"
+#include <cstdio>
 #include <math.h>
 
 /***************************************************************************
@@ -33,8 +34,8 @@ License along with this library; if not, see
  *
  ***************************************************************************/
 
-#define CFG_MASK 0x7
-#define CFG_SHIFT 3
+#define CFG_MASK 0xf
+#define CFG_SHIFT 4
 
 class CMSignalSource;
 class CMxSignalSource;
@@ -51,8 +52,9 @@ class TMRL;
 	AN4,
 	AN5,
 	VREF = 6,	// use reference voltage
-	NO_IN = 7	// no input port
-   };
+	NO_IN = 7,	// no input port
+  	V06 = 8		// Reference voltage 0.6 
+  };
   enum compare_outputs
    {
 	OUT0 = 0,
@@ -184,6 +186,7 @@ class CMCON1 : public sfr_register
 
  private:
   TMRL *m_tmrl;
+  unsigned int		valid_bits;
 
 };
 
@@ -210,7 +213,7 @@ class CMCON : public sfr_register
   virtual void setOUTpin(int i, PinModule *);
   virtual void assign_pir_set(PIR_SET *new_pir_set);
   virtual unsigned int get();
-  virtual void rename_pins(unsigned int) { cout << "CMCON::rename_pins() should not be called\n";}
+  virtual void rename_pins(unsigned int) { puts("CMCON::rename_pins() should not be called");}
   virtual void put(unsigned int);
   virtual void set_configuration(int comp, int mode, int il1, int ih1, int il2, int ih2, int out);
   virtual double comp_voltage(int ind, int invert);
@@ -241,7 +244,7 @@ protected:
   static const int cMaxConfigurations=8;
   static const int cMaxComparators=2;
 
-  unsigned int m_configuration_bits[cMaxComparators][cMaxConfigurations];
+  guint32 m_configuration_bits[cMaxComparators][cMaxConfigurations];
 
 };
 
@@ -343,7 +346,7 @@ public:
     virtual unsigned int get();
     virtual double get_Vpos(){return 0.;}
     virtual double get_Vneg(){return 0.;}
-    virtual void put(unsigned int)  { cout << "Help\n";}
+    virtual void put(unsigned int)  { puts("Help");}
 //    virtual int get(){return 0;}
     virtual void setBitMask(unsigned int bm) { mValidBits = bm; }
     virtual void setIntSrc(InterruptSource *_IntSrc) { IntSrc = _IntSrc;}
