@@ -1,22 +1,22 @@
 /*
    Copyright (C) 2006 T. Scott Dattalo
 
-This file is part of gpsim.
+This file is part of the libgpsim_dspic library of gpsim
 
-gpsim is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-gpsim is distributed in the hope that it will be useful,
+This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with gpsim; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, see 
+<http://www.gnu.org/licenses/lgpl-2.1.html>.
+*/
 
 #include "dspic-registers.h"
 #include "dspic-processors.h"
@@ -111,7 +111,8 @@ namespace dspic_registers {
   {
 
     dspic::gTrace->raw(trace_other | (value<<1));
-    value = new_address & memory_size_mask;
+    value = new_address;
+    value = (value >= memory_size) ? value - memory_size : value;
 
     m_pcl->value.put(value & 0xffff);
 
@@ -128,7 +129,8 @@ namespace dspic_registers {
     // Use the new_address and the cached pclath (or page select bits
     // for 12 bit cores) to generate the destination address: 
 
-    value = (new_address >>1) & memory_size_mask;
+    value = (new_address >>1);
+    value = (value >= memory_size) ? value - memory_size : value;
 
     // see Update pcl comment in Program_Counter::increment()
     m_pcl->value.put((value<<1) & 0xffff);
@@ -147,7 +149,8 @@ namespace dspic_registers {
 
     dspic::gTrace->raw(trace_other | (value<<1));
 
-    value = new_value & memory_size_mask;
+    value = new_value;
+    value = (value >= memory_size) ? value - memory_size : value;
     m_pcl->value.put(value & 0xff);
     //m_cpu->pclath->value.put((value >> 8) & 0xff);
     m_pcl->update();
@@ -173,7 +176,8 @@ namespace dspic_registers {
 
     // Trace the value of the program counter before it gets changed.
     dspic::gTrace->raw(trace_increment | value);
-    value = (value + 1) & memory_size_mask;
+    value = (value + 1);
+    value = (value >= memory_size) ? value - memory_size : value;
 
     // Update pcl. Note that we don't want to pcl.put() because that 
     // will trigger a break point if there's one set on pcl. (A read/write
