@@ -64,6 +64,7 @@ LiteralBoolean::LiteralBoolean(Boolean* value_)
 
 LiteralBoolean::~LiteralBoolean()
 {
+    delete value;
 }
 
 Value* LiteralBoolean::evaluate()
@@ -102,6 +103,7 @@ Value* LiteralInteger::evaluate()
 
 string LiteralInteger::toString()
 {
+  //return value->toString("0x%x");
   return value->toString();
 }
 
@@ -172,13 +174,22 @@ string LiteralString::toString()
  *
  * The literal symbol is a thin 'literal' wrapper for the symbol class.
  * The command line parser uses LiteralSymbol whenever an expression
- * encounters a symbol. 
+ * encounters a symbol.
  */
 
 LiteralSymbol::LiteralSymbol(gpsimObject *_sym)
 {
   sym = dynamic_cast<Value *>(_sym);
-  assert(sym != 0);
+  if (!sym)
+  {
+      string s;
+      if (_sym)
+        s = "literal symbol '" + _sym->name() + "' does not have a value";
+      else
+        s = "NULL pointer to literal symbol";
+
+      throw new Error(s);
+  }
 }
 
 LiteralSymbol::~LiteralSymbol()
@@ -187,7 +198,10 @@ LiteralSymbol::~LiteralSymbol()
 
 Value* LiteralSymbol::evaluate()
 {
-  return  sym->evaluate();
+  if (sym)
+    return  sym->evaluate();
+
+  return 0;
 }
 
 Value *LiteralSymbol::GetSymbol()
@@ -268,7 +282,7 @@ string IndexedSymbol::toString() {
  *
  * The literal symbol is a thin 'literal' wrapper for the symbol class.
  * The command line parser uses RegisterExpression whenever an expression
- * encounters a symbol. 
+ * encounters a symbol.
  */
 
 RegisterExpression::RegisterExpression(unsigned int uAddress)
@@ -287,9 +301,9 @@ Value* RegisterExpression::evaluate()
     return new Integer(pReg->get_value());
   }
   else {
-    static char sFormat[] = "reg(%d) is not a valid register";
+    static char sFormat[] = "reg(%u) is not a valid register";
     char sBuffer[sizeof(sFormat) + 10];
-    sprintf(sBuffer, sFormat, m_uAddress);
+    snprintf(sBuffer, sizeof(sBuffer), sFormat, m_uAddress);
     throw Error(string(sBuffer));
   }
 }
@@ -297,6 +311,6 @@ Value* RegisterExpression::evaluate()
 string RegisterExpression::toString()
 {
   char sBuffer[10];
-  sprintf(sBuffer, "%d", m_uAddress);
+  snprintf(sBuffer, sizeof(sBuffer), "%u", m_uAddress);
   return string(sBuffer);
 }
