@@ -15,14 +15,14 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
 //
 // fopen-path.cc
 //
-// Modified version of fopen, that searches a path.  
+// Modified version of fopen, that searches a path.
 // Technically this is a c++ file, but it should work in C too.
 // The functions use a C calling convention for compatibility.
 
@@ -31,6 +31,7 @@ License along with this library; if not, see
 #include <string>
 #include <string.h>
 #include <assert.h>
+#include <unistd.h>
 
 #include "../config.h"
 #include "gpsim_def.h"
@@ -41,10 +42,10 @@ License along with this library; if not, see
 static char **searchPath = 0;
 static int searchPathCount = 0;
 
-class CSourceSearchPath : public String 
+class CSourceSearchPath : public String
 {
 public:
-        CSourceSearchPath() : String("SourcePath", NULL, "Search path for source files") 
+        CSourceSearchPath() : String("SourcePath", NULL, "Search path for source files")
         {
         }
   virtual void set(const char *cP,int len=0) {
@@ -72,7 +73,6 @@ public:
 
   char *toString(char *return_str, int len)
   {
-    string sPath;
     for (int iIndex = 0;
         iIndex < searchPathCount && len < 0;
         ++iIndex) {
@@ -88,7 +88,7 @@ public:
 };
 
 ///
-/// InitSourceSearchSymbol 
+/// InitSourceSearchSymbol
 /// Used to initialize the CSourceSearchPath during startup.
 /// The symbol table will delete the CSourceSearchPath
 /// object so CSourceSearchPath cannot be static.
@@ -139,7 +139,7 @@ void set_search_path (const char *path)
     for (cp = path, pathStr = searchPath, ii = 0, tp = strchr (path, ':');
 	       (0 != tp) && (ii < pathLen);
 	       ++ii) {
-	
+
       assert ((0 != cp) && (0 != tp));
       if (tp == cp) {               // treat empty path as "."
 	        *pathStr = strdup (".");  // allocate, in case we free later
@@ -175,6 +175,7 @@ void set_search_path (const char *path)
 //	PATH2/src/pic/foo.inc, PATH2/pic/foo.inc, PATH2/foo.inc
 //	...
 // It also converts back slashes to forward slashes (for MPLAB compatibility)
+
 FILE *fopen_path(const char *filename, const char *perms)
 {
   FILE *fp;
@@ -182,21 +183,40 @@ FILE *fopen_path(const char *filename, const char *perms)
   char **pathStr;			// path pointer
   int	ii;				// loop counter
   char	*cp;				// for string walking
-  char	nameBuff[256];			// where to build new filename
+  char	*nameBuff=0;			// where to build new filename
+  long maxpath;
+
+#if defined(_WIN32)
+  maxpath = MAX_PATH;
+#else
+  if ((maxpath = pathconf(filename, _PC_PATH_MAX)) < 0)
+	return 0;
+#endif
+
+  if (strlen(filename) >= (unsigned int)maxpath)
+  {
+	errno = ENAMETOOLONG;
+        return 0;
+  }
+
+  nameBuff = (char *)malloc(maxpath);
+  strncpy(nameBuff, filename, maxpath);
+  nameBuff[maxpath-1] = 0;
+
 
-  assert (strlen (filename) <= (sizeof (nameBuff) - 1));
-  strcpy (nameBuff, filename);
   for (cp = nameBuff; *cp; ++cp) {	// convert DOS slash to Unix slash
       if ('\\' == *cp) *cp = '/';
   }
 
   fp = fopen (nameBuff, perms);		// first try absolute path
-  if (0 != fp) {
+  if (fp) {
       if (verbose) printf ("Found %s as %s\n", filename, nameBuff);
+      free(nameBuff);
       return fp;
   }
 					// check along each directory
   for (pathStr = searchPath, ii=0;
+
        ii < searchPathCount;
        ++ii, ++pathStr) {
 					// check each subdir in path
@@ -204,20 +224,23 @@ FILE *fopen_path(const char *filename, const char *perms)
            fileStr && *fileStr;
            fileStr = strpbrk (fileStr+1, "/\\")) {
         strcpy (nameBuff, *pathStr);
-        strcat (nameBuff, fileStr);
-        assert (strlen (nameBuff) <= (sizeof (nameBuff) - 1));
-        for (cp = nameBuff; *cp; ++cp) { // convert DOS slash to Unix slash
-          if ('\\' == *cp)
-            *cp = '/';
-        }
-	if (verbose) {
-	  printf ("Trying to open %s\n", nameBuff);
-	}
-        fp = fopen (nameBuff, perms);	// try it
-        if (0 != fp) {
-          if (verbose)
-            printf ("Found %s as %s\n", filename, nameBuff);
-          return fp;
+        if (strlen(nameBuff) + strlen(fileStr) < (unsigned int)maxpath)
+        {
+          strcat (nameBuff, fileStr);
+          for (cp = nameBuff; *cp; ++cp) { // convert DOS slash to Unix slash
+            if ('\\' == *cp)
+              *cp = '/';
+          }
+  	if (verbose) {
+  	  printf ("Trying to open %s\n", nameBuff);
+  	}
+          fp = fopen (nameBuff, perms);	// try it
+          if (0 != fp) {
+            if (verbose)
+              printf ("Found %s as %s\n", filename, nameBuff);
+            free(nameBuff);
+            return fp;
+          }
         }
       }
   }
@@ -231,5 +254,6 @@ FILE *fopen_path(const char *filename, const char *perms)
       printf ("\n");
   }
 
+  free(nameBuff);
   return 0;
 }
