@@ -280,7 +280,11 @@ static int icd_write(const char *s)
   if(icd_fd<0)
     return -1;
 
-  write(icd_fd,s,  strlen(s));   /* Error checking ... */
+  if (write(icd_fd,s,  strlen(s)) < 0)
+  {
+      perror("icd_write: ");
+      return -1;
+  }
 
   return 1;
 }
@@ -367,7 +371,11 @@ static int icd_baudrate_init()
     if(icd_fd<0) return 0;
 
     while(tries) {
-        write(icd_fd,"U",1);
+        if (write(icd_fd,"U",1) != 1)
+        {
+	    perror("icd_baudrate_init() write: ");
+	    return 0;
+        }
 
         if(read(icd_fd,&ch,1) > 0) {
             rts_clear();
@@ -383,7 +391,7 @@ static int icd_baudrate_init()
     return 0;
 }
 
-char *icd_target(void)
+const char *icd_target(void)
 {
     static char return_string[256];
     unsigned int dev_id, type,rev;
@@ -399,25 +407,25 @@ char *icd_target(void)
     } else {
         switch(type) {
         case 0x68:
-            sprintf(return_string,"16F870 rev %d",rev);
+            sprintf(return_string,"16F870 rev %u",rev);
             break;
         case 0x69:
-            sprintf(return_string,"16F871 rev %d",rev);
+            sprintf(return_string,"16F871 rev %u",rev);
             break;
         case 0x47:
-            sprintf(return_string,"16F872 rev %d",rev);
+            sprintf(return_string,"16F872 rev %u",rev);
             break;
         case 0x4B:
-            sprintf(return_string,"16F873 rev %d",rev);
+            sprintf(return_string,"16F873 rev %u",rev);
             break;
         case 0x49:
-            sprintf(return_string,"16F874 rev %d",rev);
+            sprintf(return_string,"16F874 rev %u",rev);
             break;
         case 0x4F:
-            sprintf(return_string,"16F876 rev %d",rev);
+            sprintf(return_string,"16F876 rev %u",rev);
             break;
         case 0x4D:
-            sprintf(return_string,"16F877 rev %d",rev);
+            sprintf(return_string,"16F877 rev %u",rev);
             break;
 
         default:
@@ -482,9 +490,9 @@ void put_dumb_fsr_register(FSR **frp)
 
 static void create_dumb_register_file(void)
 {
-  pic_processor *cpu=dynamic_cast<pic_processor *>(active_cpu);
-  if(!cpu)
-    return;
+    pic_processor *cpu=dynamic_cast<pic_processor *>(active_cpu);
+    if(!cpu)
+      return;
 
     for(unsigned int i=0;i<cpu->register_memory_size();i++)
     {
@@ -591,9 +599,9 @@ static void make_stale(void)
 {
     if(icd_fd<0) return;
 
-  pic_processor *cpu=dynamic_cast<pic_processor *>(active_cpu);
-  if(!cpu)
-    return;
+    pic_processor *cpu=dynamic_cast<pic_processor *>(active_cpu);
+    if(!cpu)
+      return;
 
     for(unsigned int i=0;i<cpu->register_memory_size();i++)
     {
@@ -633,9 +641,9 @@ int icd_reset(void)
 
     make_stale();
 
-  pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
-  if(!pic)
-    return 0;
+    pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
+    if(!pic)
+      return 0;
 
     pic->pc->get_value();
     gi.simulation_has_stopped();
@@ -651,7 +659,7 @@ int icd_detected(void)
     return 0;
 }
 
-char *icd_version(void)
+const char *icd_version(void)
 {
     static char ret[256];
     unsigned int ver1,ver2;
