@@ -1,7 +1,6 @@
 /*
    Copyright (C) 1998-2003 Scott Dattalo
                  2003 Mike Durian
-
 This file is part of the libgpsim library of gpsim
 
 This library is free software; you can redistribute it and/or
@@ -22,12 +21,14 @@ License along with this library; if not, see
 
 #ifndef INTCON_H
 #define INTCON_H
-#include <iostream>		// for cout used in breakpoints.h
 
 #include "gpsim_classes.h"
 #include "registers.h"
 #include "breakpoints.h"
+#include "pic-ioports.h"
 
+class IOCxF;
+class PicPortGRegister;
 //---------------------------------------------------------
 // INTCON - Interrupt control register
 
@@ -35,6 +36,7 @@ class INTCON : public sfr_register
 {
 public:
   unsigned int interrupt_trace;
+  bool in_interrupt;
 
 enum
 {
@@ -51,12 +53,12 @@ enum
 
 
   INTCON(Processor *pCpu, const char *pName, const char *pDesc);
-  inline void set_gie()
+  virtual void set_gie()
   {
     put(value.get() | GIE);
   }
 
-  inline void clear_gie()
+  virtual void clear_gie()
   {
     put(value.get() & ~GIE);
   }
@@ -71,14 +73,7 @@ enum
   */
   virtual void peripheral_interrupt ( bool hi_pri = false );
 
-  inline void set_rbif(bool b)
-  {
-    bool current = (value.get() & RBIF) == RBIF;
-    if (b && !current)
-      put(value.get() | RBIF);
-    if (!b && current)
-      put(value.get() & ~RBIF);
-  }
+  virtual void set_rbif(bool b);
 
   inline void set_intf(bool b)
   {
@@ -87,6 +82,7 @@ enum
       put(value.get() | INTF);
     if (!b && current)
       put(value.get() & ~INTF);
+
  }
 
   inline void set_t0if()
@@ -109,8 +105,16 @@ enum
       put(value.get() | T0IE);
     }
 
+  void set_portGReg(  PicPortGRegister *_portGReg)
+  {
+	portGReg = _portGReg;
+  }
   virtual int check_peripheral_interrupt()=0;
   virtual void put(unsigned int new_value);
+  virtual void put_value(unsigned int new_value);
+  virtual void aocxf_val(IOCxF *, unsigned int val){}
+
+  PicPortGRegister *portGReg;
 
 };
 
@@ -130,7 +134,9 @@ public:
   enum
   {
     RBIP    = 1<<0,
+    INT3IP  = 1<<1,
     TMR0IP  = 1<<2,
+    INTEDG3 = 1<<3,
     INTEDG2 = 1<<4,
     INTEDG1 = 1<<5,
     INTEDG0 = 1<<6,
@@ -146,12 +152,54 @@ class INTCON3 :  public sfr_register
 {
 public:
   INTCON3(Processor *pCpu, const char *pName, const char *pDesc);
+
+  virtual void put_value(unsigned int new_value);
+  virtual void put(unsigned int new_value);
+
+  inline void set_int1f(bool b)
+  {
+    bool current = (value.get() & INT1IF) == INT1IF;
+    if (b && !current)
+      put(value.get() | INT1IF);
+    if (!b && current)
+      put(value.get() & ~INT1IF);
+  }
+  inline void set_int2f(bool b)
+  {
+    bool current = (value.get() & INT2IF) == INT2IF;
+    if (b && !current)
+      put(value.get() | INT2IF);
+    if (!b && current)
+      put(value.get() & ~INT2IF);
+  }
+  inline void set_int3f(bool b)
+  {
+    bool current = (value.get() & INT3IF) == INT3IF;
+    if (b && !current)
+      put(value.get() | INT3IF);
+    if (!b && current)
+      put(value.get() & ~INT3IF);
+  }
+  inline void set_int1e()
+    {
+      put(value.get() | INT1IE);
+    }
+  inline void set_int2e()
+    {
+      put(value.get() | INT2IE);
+    }
+  inline void set_int3e()
+    {
+      put(value.get() | INT3IE);
+    }
   enum
   {
     INT1IF  = 1<<0,
     INT2IF  = 1<<1,
+    INT3IF  = 1<<2,
     INT1IE  = 1<<3,
     INT2IE  = 1<<4,
+    INT3IE  = 1<<5,
     INT1IP  = 1<<6,
     INT2IP  = 1<<7
   };
@@ -167,12 +215,37 @@ public:
 
   INTCON_14_PIR(Processor *pCpu, const char *pName, const char *pDesc);
 
+  virtual void put(unsigned int new_value);
+  virtual void put_value(unsigned int new_value);
   inline void set_pir_set(PIR_SET *p) { pir_set = p; }
 
   virtual int check_peripheral_interrupt();
+  virtual void set_rbif(bool b);
+  virtual void set_gie() { put_value(value.get() | GIE); }
+  virtual void clear_gie() { put_value(value.get() & ~GIE); }
+  virtual void aocxf_val(IOCxF *, unsigned int val);
+
+enum
+{
+  IOCIF = 1<<0,
+  INTF  = 1<<1,
+  T0IF  = 1<<2,
+  IOCIE = 1<<3,
+  INTE  = 1<<4,
+  T0IE  = 1<<5,
+  PEIE  = 1<<6,
+  GIE   = 1<<7
+};
 
   //private:
   PIR_SET *pir_set;
+  unsigned int write_mask;    // Bits that instructions can modify
+  struct aocxf
+  {
+	struct IOCxF *ptr_iocxf;
+	unsigned int val;
+  };
+  vector<aocxf>aocxf_list;
 };
 
 
@@ -187,7 +260,11 @@ public:
 
   enum {
     GIEH = GIE,
-    GIEL = XXIE
+    GIEL = XXIE,
+    TMR0IE = T0IE,
+    INT0IE = INTE,
+    TMR0IF = T0IF,
+    INT0IF = INTF
   };
 #define INTERRUPT_VECTOR_LO       (0x18 >> 1)
 #define INTERRUPT_VECTOR_HI       (0x08 >> 1)
@@ -199,6 +276,7 @@ public:
   inline void set_pir_set(PIR_SET *p) { pir_set = p; }
 
   virtual void put(unsigned int new_value);
+  virtual void put_value(unsigned int new_value);
 
   virtual void peripheral_interrupt ( bool hi_pri = false );
 
