@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -181,7 +181,7 @@ private:
   // Storage for scripts specifically associated with this module.
   class ModuleScript {
   public:
-    ModuleScript(string &name_);
+    explicit ModuleScript(std::string &name_);
     ~ModuleScript();
     void add_command(string &command);
     void run(ICommandHandler *);
@@ -195,6 +195,7 @@ private:
 
 protected:
   double	Vdd;
+  // Derived modules should assign more reasonable values for this.
   char *version;
   SymbolTable_t mSymbolTable;
 };
