@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -27,11 +27,6 @@ License along with this library; if not, see
 #include <stdlib.h>
 #include <string>
 
-#include <iostream>
-#include <iomanip>
-#include <vector>
-#include <algorithm>
-
 #include "../config.h"
 #include "exports.h"
 #include "modules.h"
@@ -112,7 +107,7 @@ bool LIBGPSIM_EXPORT IsFileExtension(const char *pszFile, const char *pFileExt)
   string s(pszFile);
 
   string::size_type i = s.rfind('.') ;
-  
+
   return (i != string::npos) && (s.substr(i+1)==pFileExt);
 
 }
@@ -132,7 +127,6 @@ void SplitPathAndFile(string &sSource, string &sFolder, string &sFile) {
     sFile = sSource;
   }
   else {
-    string sNewFolder;
     sFolder = sSource.substr(0, LastDelimiter + 1);
     sFile = sSource.substr(LastDelimiter + 1);
   }
@@ -194,7 +188,7 @@ void AddModulePathFromFilePath(char *arg) {
       if(pFile) {
         while(fgets(szLine, 1024, pFile) != NULL) {
           char *pChar = &szLine[strlen(szLine) - 1];
-          while((*pChar == '\n' || *pChar == '\n') && pChar != szLine)
+          while((*pChar == '\n') && pChar != szLine)
             *pChar-- = 0;
           if(*pChar != FOLDERDELIMITER) {
             pChar++;
@@ -301,10 +295,10 @@ unsigned long get_error(const char *err_str) {
 #else
   /*
   ** In Linux and likely all Unix like OSs, dlopen leaves errno as 0
-  ** even after failure, If so, look in error string returned by dlerror 
+  ** even after failure, If so, look in error string returned by dlerror
   ** to try to determine if file was not found. RRR
-  */ 
-  unsigned long ret = errno;	// in Linux errno is 0 
+  */
+  unsigned long ret = errno;	// in Linux errno is 0
   if (! ret && err_str && strstr(err_str, "No such file"))
 	ret = OS_E_FILENOTFOUND;
   return ret;
@@ -325,7 +319,7 @@ void * load_library(const char *library_name, const char **pszError)
   // the second time append the os-dependent library extension.
   for (int i=0; i<2; i++) {
 
-    // First, see if we can load the library from where ever the 
+    // First, see if we can load the library from where ever the
     // system thinks libraries are located.
     if( (handle = sLoad(sPath.c_str())) != 0)
       return handle;
@@ -342,7 +336,7 @@ void * load_library(const char *library_name, const char **pszError)
       CFileSearchPath::iterator itSearchPath;
       for (itSearchPath = asDllSearchPath.begin();
 	   itSearchPath != asDllSearchPath.end();
-	   itSearchPath++) {
+	   ++itSearchPath) {
 	sPath = *itSearchPath + sFile;
 
 	handle = sLoad(sPath.c_str());
@@ -365,7 +359,7 @@ void * load_library(const char *library_name, const char **pszError)
 	i--;
 	sFile.append(MODULE_VER);
     }
-#endif 
+#endif
     sPath = sFile;
   }
 
