@@ -55,12 +55,14 @@ class P16F871 : public P16C64   // The 74 has too much RAM and too many CCPs
 
   virtual PROCESSOR_TYPE isa(){return _P16F871_;};
   virtual unsigned int program_memory_size() const { return 0x0800; };
+  virtual unsigned int eeprom_memory_size() const { return 64; };
   virtual void create_symbols();
   void create_sfr_map();
   void create();
   virtual unsigned int register_memory_size () const { return 0x200;};
 
   P16F871(const char *_name=0, const char *desc=0);
+  ~P16F871();
   static Processor *construct(const char *name);
 
   virtual void set_eeprom(EEPROM *ep) {
@@ -73,7 +75,6 @@ class P16F871 : public P16C64   // The 74 has too much RAM and too many CCPs
   }
   virtual EEPROM_WIDE *get_eeprom() { return ((EEPROM_WIDE *)eeprom); }
 
-  virtual bool hasSSP() { return false;}
 
 private:
 
@@ -99,11 +100,13 @@ class P16F873 : public P16C73
   virtual unsigned int register_memory_size () const { return 0x200;};
 
   P16F873(const char *_name=0, const char *desc=0);
+  ~P16F873();
 
   virtual void set_eeprom(EEPROM *ep) {
     // use set_eeprom_wide as P16F873 expect a wide EEPROM
     assert(0);
   }
+  virtual unsigned int eeprom_memory_size() const { return 128; };
   virtual void set_eeprom_wide(EEPROM_WIDE *ep) {
     eeprom = ep;
   }
@@ -125,13 +128,18 @@ public:
   void create();
 
   P16F873A(const char *_name=0, const char *desc=0);
+  ~P16F873A();
   static Processor *construct(const char *name);
 };
 
 
-class P16F876 : public P16F873
+class P16F876 : public P16C73
 {
  public:
+
+  sfr_register adresl;
+  virtual void set_out_of_range_pm(unsigned int address, unsigned int value);
+
   virtual PROCESSOR_TYPE isa(){return _P16F876_;};
   virtual unsigned int program_memory_size() const { return 0x2000; };
   virtual void create_symbols();
@@ -140,7 +148,18 @@ class P16F876 : public P16F873
   virtual unsigned int register_memory_size () const { return 0x200;};
 
   P16F876(const char *_name=0, const char *desc=0);
+  ~P16F876();
   static Processor *construct(const char *name);
+
+  virtual void set_eeprom(EEPROM *ep) {
+    // use set_eeprom_wide as P16F873 expect a wide EEPROM
+    assert(0);
+  }
+  virtual unsigned int eeprom_memory_size() const { return 256; };
+  virtual void set_eeprom_wide(EEPROM_WIDE *ep) {
+    eeprom = ep;
+  }
+  virtual EEPROM_WIDE *get_eeprom() { return ((EEPROM_WIDE *)eeprom); }
 };
 
 class P16F876A : public P16F873A
@@ -149,12 +168,14 @@ class P16F876A : public P16F873A
   ComparatorModule comparator;
   virtual PROCESSOR_TYPE isa(){return _P16F876A_;};
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int eeprom_memory_size() const { return 256; };
 
   void create_sfr_map();
   void create();
   virtual unsigned int register_memory_size () const { return 0x200;};
 
   P16F876A(const char *_name=0, const char *desc=0);
+  ~P16F876A();
   static Processor *construct(const char *name);
 };
 
@@ -176,8 +197,10 @@ public:
   virtual unsigned int register_memory_size () const { return 0x200;};
 
   P16F874(const char *_name=0, const char *desc=0);
+  ~P16F874();
   static Processor *construct(const char *name);
 
+  virtual unsigned int eeprom_memory_size() const { return 128; };
   virtual void set_eeprom(EEPROM *ep) {
     // use set_eeprom_wide as P16F873 expect a wide EEPROM
     assert(0);
@@ -186,6 +209,7 @@ public:
     eeprom = ep;
   }
   virtual EEPROM_WIDE *get_eeprom() { return ((EEPROM_WIDE *)eeprom); }
+  //virtual bool hasSSP() { return true;}
 };
 
 class P16F877 : public P16F874
@@ -193,11 +217,13 @@ class P16F877 : public P16F874
 public:
   virtual PROCESSOR_TYPE isa(){return _P16F877_;};
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int eeprom_memory_size() const { return 256; };
   virtual void create_symbols();
   void create_sfr_map();
   void create();
 
   P16F877(const char *_name=0, const char *desc=0);
+  ~P16F877();
   static Processor *construct(const char *name);
 };
 
@@ -210,12 +236,14 @@ public:
 
   virtual PROCESSOR_TYPE isa(){return _P16F874A_;};
   virtual unsigned int program_memory_size() const { return 0x1000; };
+  virtual unsigned int eeprom_memory_size() const { return 256; };
   virtual void create_symbols();
   void create_sfr_map();
   void create();
   virtual unsigned int register_memory_size () const { return 0x200;};
 
   P16F874A(const char *_name=0, const char *desc=0);
+  ~P16F874A();
   static Processor *construct(const char *name);
 
 };
@@ -226,11 +254,13 @@ public:
   ComparatorModule comparator;
   virtual PROCESSOR_TYPE isa(){return _P16F877A_;};
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int eeprom_memory_size() const { return 256; };
   virtual void create_symbols();
   void create_sfr_map();
   void create();
 
   P16F877A(const char *_name=0, const char *desc=0);
+  ~P16F877A();
   static Processor *construct(const char *name);
 };
 
