@@ -120,7 +120,7 @@ P16F88x::P16F88x(const char *_name, const char *desc)
     ccpr2h(this, "ccpr2h", "Capture Compare 2 High"),
     pcon(this, "pcon", "pcon"),
     ssp(this),
-    osccon(this, "osccon", "OSC Control"),
+    osccon(0),
     osctune(this, "osctune", "OSC Tune"),
     wdtcon(this, "wdtcon", "WDT Control", 1),
     usart(this),
@@ -192,7 +192,7 @@ P16F88x::~P16F88x()
   delete get_eeprom();
 
   remove_sfr_register(&intcon_reg);
-  remove_sfr_register(&osccon);
+  remove_sfr_register(osccon);
   remove_sfr_register(&osctune);
   remove_sfr_register(&usart.rcsta);
   remove_sfr_register(&usart.txsta);
@@ -221,12 +221,10 @@ P16F88x::~P16F88x()
   remove_sfr_register(&pstrcon);
   remove_sfr_register(&eccpas);
   remove_sfr_register(&ssp.sspcon2);
-  if (hasSSP()) {
-    remove_sfr_register(&ssp.sspbuf);
-    remove_sfr_register(&ssp.sspcon);
-    remove_sfr_register(&ssp.sspadd);
-    remove_sfr_register(&ssp.sspstat);
-  }
+  remove_sfr_register(&ssp.sspbuf);
+  remove_sfr_register(&ssp.sspcon);
+  remove_sfr_register(&ssp.sspadd);
+  remove_sfr_register(&ssp.sspstat);
   delete_sfr_register(usart.txreg);
   delete_sfr_register(usart.rcreg);
   remove_sfr_register(comparator.cmxcon0[0]);
@@ -322,11 +320,14 @@ void P16F88x::create_sfr_map()
   add_sfr_register(&pr2,    0x92, RegisterValue(0xff,0));
 
 
+  get_eeprom()->get_reg_eedata()->new_name("eedat");
+  get_eeprom()->get_reg_eedatah()->new_name("eedath");
   add_sfr_register(get_eeprom()->get_reg_eedata(),  0x10c);
   add_sfr_register(get_eeprom()->get_reg_eeadr(),   0x10d);
   add_sfr_register(get_eeprom()->get_reg_eedatah(),  0x10e);
   add_sfr_register(get_eeprom()->get_reg_eeadrh(),   0x10f);
   add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x18c, RegisterValue(0,0));
+  get_eeprom()->get_reg_eecon1()->set_bits(EECON1::EEPGD);
   add_sfr_register(get_eeprom()->get_reg_eecon2(),  0x18d);
 
   alias_file_registers(0x0a,0x0b,0x080);
@@ -338,11 +339,11 @@ void P16F88x::create_sfr_map()
   intcon_reg.set_pir_set(get_pir_set());
 
 
-  add_sfr_register(&osccon, 0x8f, RegisterValue(0x60,0),"osccon");
+  add_sfr_register(osccon, 0x8f, RegisterValue(0x60,0),"osccon");
   add_sfr_register(&osctune, 0x90, RegisterValue(0,0),"osctune");
 
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osctune.set_osccon(osccon);
 
   usart.initialize(pir1,&(*m_portc)[6], &(*m_portc)[7],
 		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
@@ -429,15 +430,16 @@ void P16F88x::create_sfr_map()
   add_sfr_register(&eccpas, 0x9c, RegisterValue(0,0));
   eccpas.setIOpin(0, 0, &(*m_portb)[0]);
   eccpas.link_registers(&pwm1con, &ccp1con);
-  add_sfr_register(&ssp.sspcon2,  0x91, RegisterValue(0,0) ,"sspcon2");
-  if (hasSSP()) {
-    add_sfr_register(&ssp.sspbuf,  0x13, RegisterValue(0,0),"sspbuf");
-    add_sfr_register(&ssp.sspcon,  0x14, RegisterValue(0,0),"sspcon");
-    add_sfr_register(&ssp.sspadd,  0x93, RegisterValue(0,0),"sspadd");
-    add_sfr_register(&ssp.sspstat, 0x94, RegisterValue(0,0),"sspstat");
-    tmr2.ssp_module[0] = &ssp;
+  ssp.sspmsk = new _SSPMSK(this, "ssp1msk");
+  add_sfr_register(&ssp.sspbuf,  0x13, RegisterValue(0,0),"sspbuf");
+  add_sfr_register(&ssp.sspcon,  0x14, RegisterValue(0,0),"sspcon");
+  add_sfr_register(&ssp.sspcon2,  0x91, RegisterValue(0,0),"sspcon2");
+  add_sfr_register(&ssp.sspadd,  0x93, RegisterValue(0,0),"sspadd");
+  add_sfr_register(ssp.sspmsk,  0x93, RegisterValue(0xff,0), "sspmsk", false);
+  add_sfr_register(&ssp.sspstat, 0x94, RegisterValue(0,0),"sspstat");
+  tmr2.ssp_module[0] = &ssp;
 
-    ssp.initialize(
+  ssp.initialize(
 		get_pir_set(),    // PIR
                 &(*m_portc)[3],   // SCK
                 &(*m_porta)[5],   // SS
@@ -446,7 +448,6 @@ void P16F88x::create_sfr_map()
                 m_trisc,          // i2c tris port
 		SSP_TYPE_SSP
 	);
-  }
   tmr1l.tmrh = &tmr1h;
   tmr1l.t1con = &t1con;
   tmr1h.tmrl  = &tmr1l;
@@ -632,6 +633,7 @@ void  P16F88x::create(int eesize)
   create_iopin_map();
 
   _14bit_processor::create();
+  osccon = new OSCCON(this, "osccon", "OSC Control");
 
   EEPROM_WIDE *e;
   e = new EEPROM_WIDE(this,pir2);
@@ -1125,7 +1127,7 @@ P16F631::P16F631(const char *_name, const char *desc)
     osctune(this, "osctune", "OSC Tune"),
     pcon(this, "pcon", "pcon"),
     wdtcon(this, "wdtcon", "WDT Control", 0x1f),
-    osccon(this, "osccon", "OSC Control"),
+    osccon(0),
     vrcon(this, "vrcon", "Voltage Reference Control Register"),
     srcon(this, "srcon", "SR Latch Control Resgister"),
     ansel(this,"ansel", "Analog Select"),
@@ -1194,7 +1196,7 @@ P16F631::~P16F631()
   remove_sfr_register(&t1con);
   remove_sfr_register(&pcon);
   remove_sfr_register(&wdtcon);
-  remove_sfr_register(&osccon);
+  remove_sfr_register(osccon);
   remove_sfr_register(&pie1);
   remove_sfr_register(&pie2);
   remove_sfr_register(&intcon_reg);
@@ -1270,6 +1272,7 @@ void P16F631::create(int eesize)
    create_iopin_map();
 
    _14bit_processor::create();
+   osccon = new OSCCON(this, "osccon", "OSC Control");
 
    e = new EEPROM_WIDE(this,pir2);
    e->initialize(eesize);
@@ -1345,7 +1348,7 @@ void P16F631::create_sfr_map()
   add_sfr_register(&t1con, 0x10, RegisterValue(0,0));
   add_sfr_register(&pcon, 0x8e, RegisterValue(0,0));
   add_sfr_register(&wdtcon, 0x97, RegisterValue(0x08,0));
-  add_sfr_register(&osccon, 0x8f, RegisterValue(0x60,0));
+  add_sfr_register(osccon, 0x8f, RegisterValue(0x60,0));
 
   add_sfr_register(&vrcon, 0x118, RegisterValue(0,0),"vrcon");
   add_sfr_register(comparator.cmxcon0[0], 0x119, RegisterValue(0,0), "cm1con0");
@@ -1400,6 +1403,7 @@ void P16F631::create_sfr_map()
   pie1.setPir(pir1);
   pie2.setPir(pir2);
 
+  get_eeprom()->get_reg_eedata()->new_name("eedat");
   add_sfr_register(get_eeprom()->get_reg_eedata(),  0x10c);
   add_sfr_register(get_eeprom()->get_reg_eeadr(),   0x10d);
   add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x18c, RegisterValue(0,0));
@@ -1410,8 +1414,8 @@ void P16F631::create_sfr_map()
   add_sfr_register(m_iocb, 0x116, RegisterValue(0,0),"iocb");
   add_sfr_register(&osctune, 0x90, RegisterValue(0,0),"osctune");
 
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osctune.set_osccon(osccon);
 
 }
 //-------------------------------------------------------------------
@@ -1441,6 +1445,7 @@ bool P16F631::set_config_word(unsigned int address, unsigned int cfg_word)
     CFG_FOSC2 = 1<<2,
     CFG_WDTE  = 1<<3,
     CFG_MCLRE = 1<<5,
+    CFG_IESO  = 1<<10,
   };
 
   
@@ -1469,7 +1474,16 @@ bool P16F631::set_config_word(unsigned int address, unsigned int cfg_word)
 	//
         (&(*m_porta)[4])->AnalogReq((Register *)this, false, "porta4");
 	valid_pins |= 0x20;
-       	switch(cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) 
+
+        unsigned int fosc = cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2);
+        if (osccon)
+        {
+            osccon->set_config_xosc(fosc < 3);
+            osccon->set_config_irc(fosc == 4 || fosc == 5);
+	    osccon->set_config_ieso(cfg_word & CFG_IESO);
+        }
+
+       	switch(fosc)
 	{
 
        	case 0:  // LP oscillator: low power crystal is on RA4 and RA5
@@ -1491,7 +1505,7 @@ bool P16F631::set_config_word(unsigned int address, unsigned int cfg_word)
 	case 4: // INTOSC
             (m_porta->getPin(5))->newGUIname("porta5");
              set_int_osc(true);
-	    osccon.set_rc_frequency();
+	    osccon->set_rc_frequency();
 	    break;
 
 	case 6: //RC oscillator: I/O on RA4 pin, RC on RA5
@@ -1534,7 +1548,7 @@ P16F684::P16F684(const char *_name, const char *desc)
     osctune(this, "osctune", "OSC Tune"),
     pcon(this, "pcon", "pcon"),
     wdtcon(this, "wdtcon", "WDT Control", 0x1f),
-    osccon(this, "osccon", "OSC Control"),
+    osccon(0),
     ansel(this,"ansel", "Analog Select"),
     adcon0(this,"adcon0", "A2D Control 0"),
     adcon1(this,"adcon1", "A2D Control 1"),
@@ -1551,7 +1565,7 @@ P16F684::P16F684(const char *_name, const char *desc)
   if(verbose)
     cout << "P16F684 constructor, type = " << isa() << '\n';
 
-    pir1_3_reg = new PIR1v3(this,"pir1","Peripheral Interrupt Register",&intcon_reg,&pie1);
+  pir1_3_reg = new PIR1v3(this,"pir1","Peripheral Interrupt Register",&intcon_reg,&pie1);
   pir1 = pir1_3_reg;
   pir1->valid_bits = pir1->writable_bits = 0xff;
 
@@ -1603,7 +1617,7 @@ P16F684::~P16F684()
   remove_sfr_register(&adcon0);
   remove_sfr_register(&pie1);
   remove_sfr_register(&pcon);
-  remove_sfr_register(&osccon);
+  remove_sfr_register(osccon);
   remove_sfr_register(&osctune);
   remove_sfr_register(&ansel);
   remove_sfr_register(&pr2);
@@ -1676,6 +1690,8 @@ void P16F684::create(int eesize)
 
    _14bit_processor::create();
 
+   osccon = new OSCCON(this, "osccon", "OSC Control");
+
    e = new EEPROM_WIDE(this,pir1);
    e->initialize(eesize);
    e->set_intcon(&intcon_reg);
@@ -1747,7 +1763,7 @@ void P16F684::create_sfr_map()
 
   add_sfr_register(&pie1,   0x8c, RegisterValue(0,0));
   add_sfr_register(&pcon, 0x8e, RegisterValue(0,0));
-  add_sfr_register(&osccon, 0x8f, RegisterValue(0x60,0));
+  add_sfr_register(osccon, 0x8f, RegisterValue(0x60,0));
 
   add_sfr_register(&osctune, 0x90, RegisterValue(0,0),"osctune");
   add_sfr_register(&ansel, 0x91, RegisterValue(0xff,0));
@@ -1853,8 +1869,8 @@ void P16F684::create_sfr_map()
   ccpr1l.tmrl   = &tmr1l;
   ccpr1h.ccprl  = &ccpr1l;
 
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osctune.set_osccon(osccon);
 
 }
 //-------------------------------------------------------------------
@@ -1883,11 +1899,19 @@ bool P16F684::set_config_word(unsigned int address, unsigned int cfg_word)
     CFG_FOSC2 = 1<<2,
     CFG_WDTE  = 1<<3,
     CFG_MCLRE = 1<<5,
+    CFG_IESO  = 1<<11,
   };
 
   
    if(address == config_word_address())
     {
+       	config_clock_mode = (cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)); 
+        if (osccon)
+        {
+            osccon->set_config_xosc(config_clock_mode < 3);
+            osccon->set_config_irc(config_clock_mode == 4 || config_clock_mode == 5);
+            osccon->set_config_ieso(cfg_word & CFG_IESO);
+	}
        unsigned int valid_pins = m_porta->getEnableMask();
 
         if ((cfg_word & CFG_MCLRE) == CFG_MCLRE)
@@ -1911,7 +1935,7 @@ bool P16F684::set_config_word(unsigned int address, unsigned int cfg_word)
 	//
         (&(*m_porta)[4])->AnalogReq((Register *)this, false, "porta4");
 	valid_pins |= 0x20;
-       	switch(cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) 
+       	switch(config_clock_mode) 
 	{
 
        	case 0:  // LP oscillator: low power crystal is on RA4 and RA5
@@ -1933,7 +1957,7 @@ bool P16F684::set_config_word(unsigned int address, unsigned int cfg_word)
 	case 4: // INTOSC
             (m_porta->getPin(5))->newGUIname("porta5");
              set_int_osc(true);
-	    osccon.set_rc_frequency();
+	    osccon->set_rc_frequency();
 	    break;
 
 	case 6: //RC oscillator: I/O on RA4 pin, RC on RA5
@@ -2164,9 +2188,13 @@ void P16F685::create_sfr_map()
   P16F677::create_sfr_map();
 
 
-  add_sfr_register(get_eeprom()->get_reg_eedatah(),  0x10e);
+  add_sfr_register(get_eeprom()->get_reg_eedatah(),  0x10e );
   add_sfr_register(get_eeprom()->get_reg_eeadrh(),   0x10f);
 
+  // Enable program memory reads and writes.
+  get_eeprom()->get_reg_eecon1()->set_bits(EECON1::EEPGD);
+
+
   add_sfr_register(&tmr2,   0x11, RegisterValue(0,0));
   add_sfr_register(&t2con,  0x12, RegisterValue(0,0));
   add_sfr_register(&pr2,    0x92, RegisterValue(0xff,0));
