@@ -93,7 +93,7 @@ public:
 P16F8x::P16F8x(const char *_name, const char *desc)
   : P16X6X_processor(_name,desc),
     wdtcon(this, "wdtcon", "WDT Control", 0x1f),
-    osccon(this, "osccon", "OSC Control"),
+    osccon(0),
     osctune(this, "osctune", "OSC Tune"),
     usart(this),
     comparator(this)
@@ -126,7 +126,7 @@ P16F8x::~P16F8x()
   delete_sfr_register(usart.txreg);
   delete_sfr_register(usart.rcreg);
   delete get_eeprom();
-  remove_sfr_register(&osccon);
+  remove_sfr_register(osccon);
   remove_sfr_register(&osctune);
   remove_sfr_register(&pie2);
 }
@@ -213,11 +213,13 @@ void P16F8x::create_sfr_map()
   intcon = &intcon_reg;
   intcon_reg.set_pir_set(get_pir_set());
 
-  add_sfr_register(&osccon, 0x8f, RegisterValue(0,0),"osccon");
+  add_sfr_register(osccon, 0x8f, RegisterValue(0,0),"osccon");
   add_sfr_register(&osctune, 0x90, RegisterValue(0,0),"osctune");
 
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osctune.set_osccon(osccon);
+  osccon->write_mask = 0x73;
+  osccon->has_iofs_bit = true;
 
   usart.initialize(pir1,&(*m_portb)[5], &(*m_portb)[2],
 		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
@@ -280,9 +282,12 @@ bool P16F8x::set_config_word(unsigned int address, unsigned int cfg_word)
     CFG_FOSC1 = 1<<1,
     CFG_FOSC2 = 1<<4,
     CFG_MCLRE = 1<<5,
-    CFG_CCPMX = 1<<12
+    CFG_CCPMX = 1<<12,
+    CFG2_IESO = 1<<1
   };
 
+  unsigned int fosc;
+
   // Let the base class do most of the work:
 
   if (address == 0x2007)
@@ -290,15 +295,21 @@ bool P16F8x::set_config_word(unsigned int address, unsigned int cfg_word)
     pic_processor::set_config_word(address, cfg_word);
 
     if (verbose)
-        cout << "p16f88 0x" << hex << address << " setting config word 0x" << cfg_word << '\n';
+        cout << "p16f8x 0x" << hex << address << " setting config word 0x" << cfg_word << '\n';
 
 
     unsigned int valid_pins = m_porta->getEnableMask();
 
     set_int_osc(false);
     // Careful these bits not adjacent
-    switch(cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) {
-
+    fosc = ((cfg_word & CFG_FOSC2) >> 2) | (cfg_word & (CFG_FOSC0 | CFG_FOSC1));
+    if (osccon) 
+    {
+	osccon->set_config_xosc(fosc < 3);
+	osccon->set_config_irc(fosc == 4 || fosc == 5);
+    }
+    switch(fosc)
+    {
     case 0:  // LP oscillator: low power crystal is on RA6 and RA7
     case 1:     // XT oscillator: crystal/resonator is on RA6 and RA7
     case 2:     // HS oscillator: crystal/resonator is on RA6 and RA7
@@ -306,32 +317,32 @@ bool P16F8x::set_config_word(unsigned int address, unsigned int cfg_word)
 	(m_porta->getPin(7))->newGUIname("OSC1");
 	break;
 
-    case 0x13:  // ER oscillator: RA6 is CLKOUT, resistor (?) on RA7 
-	(m_porta->getPin(6))->newGUIname("CLKOUT");
-	(m_porta->getPin(7))->newGUIname("OSC1");
-	break;
-
     case 3:     // EC:  RA6 is an I/O, RA7 is a CLKIN
-    case 0x12:  // ER oscillator: RA6 is an I/O, RA7 is a CLKIN
+    case 6:  // ER oscillator: RA6 is an I/O, RA7 is a CLKIN
         (m_porta->getPin(6))->newGUIname("porta6");
         (m_porta->getPin(7))->newGUIname("CLKIN");
         valid_pins =  (valid_pins & 0x7f)|0x40;
         break;
 
-    case 0x10:  // INTRC: Internal Oscillator, RA6 and RA7 are I/O's
+    case 4:  // INTRC: Internal Oscillator, RA6 and RA7 are I/O's
         set_int_osc(true);
         (m_porta->getPin(6))->newGUIname("porta6");
         (m_porta->getPin(7))->newGUIname("porta7");
         valid_pins |= 0xc0;
         break;
 
-    case 0x11:  // INTRC: Internal Oscillator, RA7 is an I/O, RA6 is CLKOUT
+    case 5:  // INTRC: Internal Oscillator, RA7 is an I/O, RA6 is CLKOUT
         set_int_osc(true);
 	(m_porta->getPin(6))->newGUIname("CLKOUT");
         (m_porta->getPin(7))->newGUIname("porta7");
         valid_pins = (valid_pins & 0xbf)|0x80;
         break;
 
+    case 7:  // ER oscillator: RA6 is CLKOUT, resistor (?) on RA7 
+	(m_porta->getPin(6))->newGUIname("CLKOUT");
+	(m_porta->getPin(7))->newGUIname("OSC1");
+	break;
+
     }
 
     // If the /MCLRE bit is set then RA5 is the MCLR pin, otherwise it's 
@@ -360,7 +371,9 @@ bool P16F8x::set_config_word(unsigned int address, unsigned int cfg_word)
   }
   else if (address == 0x2008 )
   {
-    cout << "p16f88 0x" << hex << address << " config word 0x" << cfg_word << '\n';
+    cout << "p16f8x 0x" << hex << address << " config word 0x" << cfg_word << '\n';
+    if (osccon) osccon->set_config_ieso(cfg_word & CFG2_IESO);
+    return true;
   }
 
   return false;
@@ -387,9 +400,10 @@ void  P16F8x::create(int eesize)
   set_hasSSP();
   create_iopin_map();
 
-
   _14bit_processor::create();
 
+  osccon = new OSCCON_1(this, "osccon", "OSC Control");
+
   EEPROM_WIDE *e;
   e = new EEPROM_WIDE(this,pir2);
   e->initialize(eesize);
@@ -406,6 +420,25 @@ void  P16F8x::create(int eesize)
   P16F8x::create_sfr_map();
 
 }
+
+void P16F8x::enter_sleep()
+{
+    tmr1l.sleep();
+    osccon->sleep();
+    _14bit_processor::enter_sleep();
+}
+
+
+void P16F8x::exit_sleep()
+{
+    if (m_ActivityState == ePASleeping)
+    {
+        tmr1l.wake();
+        osccon->wake();
+        _14bit_processor::exit_sleep();
+    }
+}
+
 //========================================================================
 
 P16F81x::P16F81x(const char *_name, const char *desc)
@@ -414,7 +447,7 @@ P16F81x::P16F81x(const char *_name, const char *desc)
     adcon1(this,"adcon1", "A2D Control 1"),
     adresh(this,"adresh", "A2D Result High"),
     adresl(this,"adresl", "A2D Result Low"),
-    osccon(this, "osccon", "OSC Control"),
+    osccon(0),
     osctune(this, "osctune", "OSC Tune")
 {
   pir1_2_reg = new PIR1v2(this,"pir1","Peripheral Interrupt Register",&intcon_reg,&pie1);
@@ -427,7 +460,7 @@ P16F81x::P16F81x(const char *_name, const char *desc)
 
 P16F81x::~P16F81x()
 {
-  remove_sfr_register(&osccon);
+  remove_sfr_register(osccon);
   remove_sfr_register(&osctune);
   remove_sfr_register(&adresl);
   remove_sfr_register(&adresh);
@@ -526,11 +559,12 @@ void P16F81x::create_sfr_map()
   intcon_reg.set_pir_set(get_pir_set());
 
 
-  add_sfr_register(&osccon, 0x8f, RegisterValue(0,0),"osccon");
+  add_sfr_register(osccon, 0x8f, RegisterValue(0,0),"osccon");
   add_sfr_register(&osctune, 0x90, RegisterValue(0,0),"osctune");
 
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osccon->write_mask = 0x70;
+  osctune.set_osccon(osccon);
 
   add_sfr_register(&adresl,  0x9e, RegisterValue(0,0));
   add_sfr_register(&adresh,  0x1e, RegisterValue(0,0));
@@ -726,6 +760,7 @@ void  P16F81x::create(int eesize)
 
   _14bit_processor::create();
 
+   osccon = new OSCCON_1(this, "osccon", "OSC Control");
 
   EEPROM_WIDE *e;
   e = new EEPROM_WIDE(this,pir2);
