@@ -31,10 +31,15 @@ License along with this library; if not, see
 #include "stimuli.h"
 #include "symbol.h"
 
-#define MCLRE (1<<7)
+/* Config Word defines */
+#define MCLRE 	(1<<7)
+#define P2BMX 	(1<<5)
+#define T3CMX	(1<<4)
+#define HFOFST	(1<<3)
 #define LPT1OSC (1<<2)
-#define PBADEN (1<<1)
-#define CCP2MX (1<<0)
+#define CCP3MX 	(1<<2)
+#define PBADEN 	(1<<1)
+#define CCP2MX 	(1<<0)
 
 
 /**
@@ -162,7 +167,8 @@ void P18C2x2::create()
 
   create_iopin_map();
   _16bit_compat_adc::create();
-  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x00,0), "osccon");
+  osccon->value = RegisterValue(0x00,0);
+  osccon->por_value = RegisterValue(0x00,0);
   init_pir2(pir2, PIR2v2::TMR3IF);
 
 }
@@ -350,7 +356,8 @@ void P18C4x2::create()
   create_iopin_map();
 
   _16bit_compat_adc::create();
-  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x00,0), "osccon");
+  osccon->value = RegisterValue(0x00,0);
+  osccon->por_value = RegisterValue(0x00,0);
 
 }
 //------------------------------------------------------------------------
@@ -483,6 +490,9 @@ void P18C4x2::create_sfr_map()
 
   RegisterValue porv(0,0);
 
+  // Assume this should follow the old behaviour as it's an old chip
+  osccon->por_value = porv;
+
   add_sfr_register(m_portd,       0xf83,porv);
   add_sfr_register(m_porte,       0xf84,porv);
 
@@ -606,37 +616,19 @@ P18F242::P18F242(const char *_name, const char *desc)
 
 void P18F242::create()
 {
-  EEPROM_PIR *e;
 
   if(verbose)
     cout << " 18f242 create \n";
 
-  e = new EEPROM_PIR(this,pir2);
-
-  // We might want to pass this value in for larger eeproms
-  e->initialize(256);
-  //e->set_pir_set(&pir_set_def);
-  e->set_intcon(&intcon);
-
-  // assign this eeprom to the processor
-  set_eeprom_pir(e);
+  tbl.initialize ( eeprom_memory_size(), 32, 4, CONFIG1L, false);
+  tbl.set_intcon(&intcon);
+  set_eeprom_pir(&tbl);
+  tbl.set_pir(pir2);
+  tbl.eecon1.set_valid_bits(0xbf);
 
   P18C242::create();
 }
 
-/*
-void P18F242::create_sfr_map()
-{
-
-  // Add eeprom
-  add_sfr_register(get_eeprom()->get_reg_eedata(), 0xfa8);
-  add_sfr_register(get_eeprom()->get_reg_eeadr(), 0xfa9);
-  add_sfr_register(get_eeprom()->get_reg_eecon1(), 0xfa6, RegisterValue(0,0));
-  add_sfr_register(get_eeprom()->get_reg_eecon2(), 0xfa7);
-
-}
-*/
-
 
 Processor * P18F242::construct(const char *name)
 {
@@ -711,36 +703,18 @@ P18F442::P18F442(const char *_name, const char *desc)
 
 void P18F442::create()
 {
-  EEPROM_PIR *e;
 
   if(verbose)
     cout << " 18f442 create \n";
 
-  e = new EEPROM_PIR(this,pir2);
-
-  // We might want to pass this value in for larger eeproms
-  e->initialize(256);
-  //e->set_pir_set(get_pir_set());
-  //e->set_pir_set(&pir_set_def);
-  e->set_intcon(&intcon);
-
-  // assign this eeprom to the processor
-  set_eeprom_pir(e);
+  tbl.initialize ( eeprom_memory_size(), 32, 4, CONFIG1L, false);
+  tbl.set_intcon(&intcon);
+  set_eeprom_pir(&tbl);
+  tbl.set_pir(pir2);
+  tbl.eecon1.set_valid_bits(0xbf);
 
   P18C442::create();
 }
-/*
-void P18F442::create_sfr_map()
-{
-
-  // Add eeprom
-  add_sfr_register(get_eeprom()->get_reg_eedata(), 0xfa8);
-  add_sfr_register(get_eeprom()->get_reg_eeadr(), 0xfa9);
-  add_sfr_register(get_eeprom()->get_reg_eecon1(), 0xfa6, RegisterValue(0,0));
-  add_sfr_register(get_eeprom()->get_reg_eecon2(), 0xfa7);
-
-}
-*/
 
 
 Processor * P18F442::construct(const char *name)
@@ -760,7 +734,7 @@ Processor * P18F442::construct(const char *name)
 
 //------------------------------------------------------------------------
 //
-// P18F258
+// P18F248
 // 
 
 P18F248::P18F248(const char *_name, const char *desc)
@@ -798,7 +772,45 @@ Processor * P18F248::construct(const char *name)
 
 //------------------------------------------------------------------------
 //
-// P18F458
+// P18F258
+// 
+
+P18F258::P18F258(const char *_name, const char *desc)
+  : P18F252(_name,desc)
+{
+
+  if(verbose)
+    cout << "18f258 constructor, type = " << isa() << '\n';
+
+}
+
+void P18F258::create()
+{
+
+  if(verbose)
+    cout << " 18f258 create \n";
+
+  P18F252::create();
+}
+
+Processor * P18F258::construct(const char *name)
+{
+
+  P18F258 *p = new P18F258(name);
+
+  if(verbose)
+    cout << " 18F258 construct\n";
+
+  p->create();
+  p->create_invalid_registers();
+  p->create_symbols();
+  return p;
+}
+
+
+//------------------------------------------------------------------------
+//
+// P18F448
 // 
 
 P18F448::P18F448(const char *_name, const char *desc)
@@ -872,6 +884,44 @@ Processor * P18F452::construct(const char *name)
 }
 
 
+//------------------------------------------------------------------------
+//
+// P18F458
+// 
+
+P18F458::P18F458(const char *_name, const char *desc)
+  : P18F452(_name,desc)
+{
+
+  if(verbose)
+    cout << "18f458 constructor, type = " << isa() << '\n';
+
+}
+
+void P18F458::create()
+{
+
+  if(verbose)
+    cout << " 18f458 create \n";
+
+  P18F452::create();
+}
+
+Processor * P18F458::construct(const char *name)
+{
+
+  P18F458 *p = new P18F458(name);
+
+  if(verbose)
+    cout << " 18F458 construct\n";
+
+  p->create();
+  p->create_invalid_registers();
+  p->create_symbols();
+  return p;
+}
+
+
 //------------------------------------------------------------------------
 //
 // P18F2455	- 28 pin
@@ -1489,27 +1539,25 @@ Processor * P18F1220::construct(const char *name)
 
 void P18F1220::create()
 {
-  EEPROM_PIR *e;
 
   if(verbose)
     cout << "P18F1220::create\n";
 
-  e = new EEPROM_PIR(this,pir2);
+  tbl.initialize ( eeprom_memory_size(), 32, 4, CONFIG1L, false);
+  tbl.set_intcon(&intcon);
+  set_eeprom_pir(&tbl);
+  tbl.set_pir(pir2);
+  tbl.eecon1.set_valid_bits(0xbf);
 
-  // We might want to pass this value in for larger eeproms
-  e->initialize(256);
-  //e->set_pir_set(&pir_set_def);
-  e->set_intcon(&intcon);
-
-  // assign this eeprom to the processor
-  set_eeprom_pir(e);
 
   create_iopin_map();
 
 
   _16bit_processor::create();
   _16bit_v2_adc::create(7);
-  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x00,0), "osccon");
+  osccon->value = RegisterValue(0x00,0);
+  osccon->por_value = RegisterValue(0x00,0);
+  osccon->has_iofs_bit = true;
   usart.txsta.setIOpin(&(*m_portb)[1]);
   usart.rcsta.setIOpin(&(*m_portb)[4]);
   adcon1->setIOPin(4, &(*m_portb)[0]);
@@ -1525,8 +1573,8 @@ void P18F1220::create()
   remove_sfr_register(&ssp.sspbuf);
 
   add_sfr_register(&osctune,      0xf9b,RegisterValue(0,0));
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osctune.set_osccon(osccon);
 
 
   set_osc_pin_Number(0,16, &(*m_porta)[7]);
@@ -1608,7 +1656,16 @@ void P18F1220::osc_mode(unsigned int value)
 {
   IOPIN *m_pin;
   unsigned int pin_Number =  get_osc_pin_Number(0);
+  unsigned int fosc = value & (FOSC3 | FOSC2 | FOSC1 | FOSC0);
   
+ if (osccon)
+  {
+      osccon->set_config_irc(fosc >= 8 && fosc <= 9);
+      osccon->set_config_xosc(fosc > 9 || fosc < 3 || fosc == 6);
+      osccon->set_config_ieso(value & IESO);
+  }
+
+  value &= (FOSC3 | FOSC2 | FOSC1 | FOSC0);
   set_int_osc(false);
   if (pin_Number < 253)
   {
@@ -1715,23 +1772,21 @@ Processor * P18F1320::construct(const char *name)
 
 void P18F2x21::create()
 {
-  EEPROM_PIR *e;
 
   if(verbose)
     cout << "P18F2x21::create\n";
 
   delete pir2;
   pir2 = (PIR2v2 *)(new PIR2v4(this, "pir2" , "Peripheral Interrupt Register",0,0  ));
-  e = new EEPROM_PIR(this,pir2);
-  e->initialize ( eeprom_memory_size() );
-  e->set_intcon(&intcon);
-  // assign this eeprom to the processor
-  set_eeprom_pir(e);
 
+  tbl.initialize ( eeprom_memory_size(), 32, 4, CONFIG1L, false);
+  tbl.set_intcon(&intcon);
+  set_eeprom_pir(&tbl);
+  tbl.set_pir(pir2);
+  tbl.eecon1.set_valid_bits(0xbf);
   create_iopin_map();
 
   _16bit_processor::create();
-  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x40,0), "osccon");
 
   m_configMemory->addConfigWord(CONFIG3H-CONFIG1L,new Config3H_2x21(this, CONFIG3H, 0x83));
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H_4bits(this, CONFIG1H, 0x07));
@@ -1835,8 +1890,8 @@ P18F2x21::P18F2x21(const char *_name, const char *desc)
     osctune(this, "osctune", "OSC Tune"),
     comparator(this)
 {
-  if(verbose)
-    cout << "18F2x21 constructor, type = " << isa() << '\n';
+    if(verbose)
+      cout << "18F2x21 constructor, type = " << isa() << '\n';
 
     m_porte = new PicPortRegister(this,"porte","",8,0x08);
     // No TRIS register for port E on 28-pin devices
@@ -1881,15 +1936,16 @@ void P18F2x21::create_sfr_map()
   adcon1->setIOPin(12, &(*m_portb)[0]);
 
   add_sfr_register(&osctune,      0xf9b,porv);
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osccon->has_iofs_bit = true;
+  osctune.set_osccon(osccon);
 
   // rest of configuration in parent class
 
   // Link the comparator and voltage ref to porta
   comparator.initialize(&pir_set_def, &(*m_porta)[2], &(*m_porta)[0], 
-	&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[3],
-	&(*m_porta)[4]);
+	&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[4],
+	&(*m_porta)[5]);
 
   comparator.cmcon.set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
   comparator.cmcon.set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
@@ -1929,7 +1985,16 @@ void P18F2x21::osc_mode(unsigned int value)
 {
   IOPIN *m_pin;
   unsigned int pin_Number =  get_osc_pin_Number(0);
+  unsigned int fosc = value & (FOSC3 | FOSC2 | FOSC1 | FOSC0);
+
+  if (osccon)
+  {
+      osccon->set_config_irc(fosc >= 8 && fosc <= 11);
+      osccon->set_config_xosc(fosc > 11 || fosc < 4);
+      osccon->set_config_ieso(value & IESO);
+  }
   
+  value &= (FOSC3 | FOSC2 | FOSC1 | FOSC0);
   set_int_osc(false);
   if (pin_Number < 253)
   {
@@ -2121,6 +2186,71 @@ Processor * P18F2520::construct(const char *name)
   return p;
 }
 
+//------------------------------------------------------------------------
+//
+// P18F2525
+// 
+
+P18F2525::P18F2525(const char *_name, const char *desc)
+  : P18F2x21(_name,desc)
+{
+
+  if(verbose)
+    cout << "18F2525 constructor, type = " << isa() << '\n';
+
+}
+
+
+Processor * P18F2525::construct(const char *name)
+{
+
+  P18F2525 *p = new P18F2525(name);
+
+  if(verbose)
+    cout << " 18F2525 construct\n";
+
+  p->create();
+  p->create_invalid_registers();
+  p->create_symbols();
+
+  if(verbose&2)
+    cout << " 18F2525 construct completed\n";
+  return p;
+}
+
+
+//------------------------------------------------------------------------
+//
+// P18F2620
+// 
+
+P18F2620::P18F2620(const char *_name, const char *desc)
+  : P18F2x21(_name,desc)
+{
+
+  if(verbose)
+    cout << "18F2620 constructor, type = " << isa() << '\n';
+
+}
+
+
+Processor * P18F2620::construct(const char *name)
+{
+
+  P18F2620 *p = new P18F2620(name);
+
+  if(verbose)
+    cout << " 18F2620 construct\n";
+
+  p->create();
+  p->create_invalid_registers();
+  p->create_symbols();
+
+  if(verbose&2)
+    cout << " 18F2620 construct completed\n";
+  return p;
+}
+
 
 
 //========================================================================
@@ -2130,18 +2260,18 @@ Processor * P18F2520::construct(const char *name)
 
 void P18F4x21::create()
 {
-  EEPROM_PIR *e;
 
   if(verbose)
     cout << "P18F4x21::create\n";
 
   delete pir2;
   pir2 = (PIR2v2 *)(new PIR2v4(this, "pir2" , "Peripheral Interrupt Register",0,0  ));
-  e = new EEPROM_PIR(this,pir2);
-  e->initialize ( eeprom_memory_size() );
-  e->set_intcon(&intcon);
-  // assign this eeprom to the processor
-  set_eeprom_pir(e);
+
+  tbl.initialize ( eeprom_memory_size(), 32, 4, CONFIG1L, false);
+  tbl.set_intcon(&intcon);
+  set_eeprom_pir(&tbl);
+  tbl.set_pir(pir2);
+  tbl.eecon1.set_valid_bits(0xbf);
 
   create_iopin_map();
 
@@ -2155,7 +2285,6 @@ void P18F4x21::create()
 
   add_sfr_register(&pwm1con, 0xfb7, RegisterValue(0,0));
   add_sfr_register(&eccpas, 0xfb6, RegisterValue(0,0));
-  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x40,0), "osccon");
   eccpas.setIOpin(0, 0, &(*m_portb)[0]);
   eccpas.link_registers(&pwm1con, &ccp1con);
   comparator.cmcon.set_eccpas(&eccpas);
@@ -2225,15 +2354,6 @@ void P18F4x21::create_iopin_map()
   package->assign_pin(39, m_portb->addPin(new IO_bi_directional_pu("portb6"),6));
   package->assign_pin(40, m_portb->addPin(new IO_bi_directional_pu("portb7"),7));
 
-/*RRR
-  psp.initialize(&pir_set_def,    // PIR
-                m_portd,           // Parallel port
-                m_trisd,           // Parallel tris
-                m_trise,           // Control tris
-                &(*m_porte)[0],    // NOT RD
-                &(*m_porte)[1],    // NOT WR
-                &(*m_porte)[2]);   // NOT CS
-RRR*/
   tmr1l.setIOpin(&(*m_portc)[0]);
 
   ssp.initialize(&pir_set_def,    // PIR
@@ -2309,8 +2429,8 @@ void P18F4x21::create_sfr_map()
 
 
   add_sfr_register(&osctune,      0xf9b,porv);
-  osccon.set_osctune(&osctune);
-  osctune.set_osccon(&osccon);
+  osccon->set_osctune(&osctune);
+  osctune.set_osccon(osccon);
 
   adcon1->setIOPin(4, &(*m_porta)[5]);
   adcon1->setIOPin(5, &(*m_porte)[0]);
@@ -2335,8 +2455,8 @@ void P18F4x21::create_sfr_map()
 
   // Link the comparator and voltage ref to porta
   comparator.initialize(&pir_set_def, &(*m_porta)[2], &(*m_porta)[0], 
-	&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[3],
-	&(*m_porta)[4]);
+	&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[4],
+	&(*m_porta)[5]);
 
   comparator.cmcon.set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
   comparator.cmcon.set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
@@ -2545,21 +2665,20 @@ Processor * P18F4620::construct(const char *name)
 
 void P18F6x20::create()
 {
-  EEPROM_PIR *e;
 
   if(verbose)
     cout << "P18F6x20::create\n";
 
-  e = new EEPROM_PIR(this,pir2);
-  e->initialize ( eeprom_memory_size() );
-  e->set_intcon(&intcon);
-  // assign this eeprom to the processor
-  set_eeprom_pir(e);
+  tbl.initialize ( eeprom_memory_size(), 32, 4, CONFIG1L, true);
+  tbl.set_intcon(&intcon);
+  set_eeprom_pir(&tbl);
+  tbl.set_pir(pir2);
+  tbl.eecon1.set_valid_bits(0xbf);
+
 
   create_iopin_map();
 
   _16bit_processor::create();
-  add_sfr_register(&osccon, 0xfd3, RegisterValue(0x40,0), "osccon");
 
   m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H_4bits(this, CONFIG1H, 0x27));
   init_pir2(pir2, PIR2v2::TMR3IF);
@@ -2791,6 +2910,8 @@ void P18F6x20::create_sfr_map()
 
   RegisterValue porv(0,0);
 
+  osccon->por_value.put(0x01,0x01);
+
   // cout << "Create extra ports\n";
   add_sfr_register(m_portd,       0xf83,porv);
   add_sfr_register(m_porte,       0xf84,porv);
@@ -2811,6 +2932,8 @@ void P18F6x20::create_sfr_map()
   add_sfr_register(&pir3,	  0xfa4,porv,"pir3");
   add_sfr_register(&ipr3,	  0xfa5,porv,"ipr3");
 
+
+
   add_sfr_register(pspcon,       0xfb0,RegisterValue(0x00,0));
 
   // cout << "Assign ADC pins to " << adcon1 << "\n";
@@ -2832,25 +2955,32 @@ void P18F6x20::create_sfr_map()
 */
 
 
-  // Link the comparator and voltage ref to porta
-  comparator.initialize(&pir_set_def, &(*m_porta)[2], &(*m_porta)[0], 
-	&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[3],
-	&(*m_porta)[4]);
+  // Link the comparator and voltage ref to portf
+  comparator.initialize(&pir_set_def, &(*m_portf)[5], 
+	0, 0, 0, 0,
+	&(*m_portf)[2], &(*m_portf)[1]);
 
-  comparator.cmcon.set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
-  comparator.cmcon.set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
-  comparator.cmcon.set_configuration(1, 1, AN0, AN3, AN0, AN3, OUT0);
+  // set anx for input pins
+  comparator.cmcon.setINpin(0, &(*m_portf)[6], "an11");
+  comparator.cmcon.setINpin(1, &(*m_portf)[5], "an10");
+  comparator.cmcon.setINpin(2, &(*m_portf)[4], "an9");
+  comparator.cmcon.setINpin(3, &(*m_portf)[3], "an8");
+
+
+  comparator.cmcon.set_configuration(1, 0, AN0, AN1, AN0, AN1, ZERO);
+  comparator.cmcon.set_configuration(2, 0, AN2, AN3, AN2, AN3, ZERO);
+  comparator.cmcon.set_configuration(1, 1, AN0, AN1, AN0, AN1, OUT0);
   comparator.cmcon.set_configuration(2, 1, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  comparator.cmcon.set_configuration(1, 2, AN0, AN3, AN0, AN3, NO_OUT);
-  comparator.cmcon.set_configuration(2, 2, AN1, AN2, AN1, AN2, NO_OUT);
-  comparator.cmcon.set_configuration(1, 3, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon.set_configuration(2, 3, AN1, AN2, AN1, AN2, OUT1);
-  comparator.cmcon.set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
-  comparator.cmcon.set_configuration(2, 4, AN1, AN3, AN1, AN3, NO_OUT);
-  comparator.cmcon.set_configuration(1, 5, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon.set_configuration(2, 5, AN1, AN3, AN1, AN3, OUT1);
-  comparator.cmcon.set_configuration(1, 6, AN0, VREF, AN3, VREF, NO_OUT);
-  comparator.cmcon.set_configuration(2, 6, AN1, VREF, AN2, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(1, 2, AN0, AN1, AN0, AN1, NO_OUT);
+  comparator.cmcon.set_configuration(2, 2, AN2, AN3, AN2, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 3, AN0, AN1, AN0, AN1, OUT0);
+  comparator.cmcon.set_configuration(2, 3, AN2, AN3, AN2, AN3, OUT1);
+  comparator.cmcon.set_configuration(1, 4, AN0, AN1, AN0, AN1, NO_OUT);
+  comparator.cmcon.set_configuration(2, 4, AN2, AN1, AN2, AN1, NO_OUT);
+  comparator.cmcon.set_configuration(1, 5, AN0, AN1, AN0, AN1, OUT0);
+  comparator.cmcon.set_configuration(2, 5, AN2, AN1, AN2, AN1, OUT1);
+  comparator.cmcon.set_configuration(1, 6, AN0, VREF, AN1, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(2, 6, AN2, VREF, AN3, VREF, NO_OUT);
   comparator.cmcon.set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
   comparator.cmcon.set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
 
