@@ -1,6 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
-   Copyright (C) 2010 Roy R Rankin
+   Copyright (C) 2010,2015 Roy R Rankin
 
 
 This file is part of the libgpsim library of gpsim
@@ -28,6 +28,10 @@ License along with this library; if not, see
 #include "psp.h"
 #include "pir.h"
 #include "comparator.h"
+#include "spp.h"
+#include "ctmu.h"
+
+#define IESO (1<<12)
 
 class PicPortRegister;
 class PicTrisRegister;
@@ -64,7 +68,7 @@ class P18C242 : public P18C2x2
   void create();
 
   virtual unsigned int program_memory_size() const { return 0x2000; };
-  virtual unsigned int last_actual_register () const { return 0x02FF;};
+  virtual unsigned int last_actual_register () const { return 0x01FF;};
 
 };
 
@@ -129,7 +133,8 @@ class P18C442 : public P18C4x2
   static Processor *construct(const char *name);
   void create();
   virtual unsigned int program_memory_size() const { return 0x2000; };
-  virtual unsigned int last_actual_register () const { return 0x02FF;};
+  virtual unsigned int eeprom_memory_size() const { return 256; };
+  virtual unsigned int last_actual_register () const { return 0x01FF;};
 
 };
 
@@ -154,6 +159,7 @@ class P18F242 : public P18C242
   static Processor *construct(const char *name);
   void create();
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int eeprom_memory_size() const { return 256; };
   virtual unsigned int IdentMemorySize() const { return 4; }
 
   virtual void set_eeprom(EEPROM *ep) {
@@ -173,6 +179,7 @@ class P18F252 : public P18F242
   static Processor *construct(const char *name);
   void create();
   virtual unsigned int program_memory_size() const { return 0x4000; };
+  virtual unsigned int last_actual_register () const { return 0x05FF;};
 
 };
 
@@ -207,6 +214,18 @@ class P18F248 : public P18F242
   void create();
 };
  
+//
+// The P18F258 is the same as the P18F252 except it has CAN, one fewer
+// CCP module and a 5/10 ADC.  For now just assume it is identical.
+class P18F258 : public P18F252
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F258_;};
+  P18F258(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+  void create();
+};
+ 
 //
 // The P18F448 is the same as the P18F442 except it has CAN, one fewer
 // CCP module and a 5/10 ADC.  For now just assume it is identical.
@@ -232,42 +251,52 @@ class P18F452 : public P18F442
   virtual unsigned int last_actual_register () const { return 0x05FF;};
 };
 
-
-
-class P18Fxx20 : public _16bit_v2_adc
+//
+// The P18F458 is the same as the P18F452 except it has CAN and one
+// fewer CCP module. For now just assume it is identical.
+class P18F458 : public P18F452
 {
-public:
-  P18Fxx20(const char *_name=0, const char *desc=0);
-  virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
-  virtual unsigned int last_actual_register () const { return 0x00FF;};
-
-  // Strip down from base class
-  virtual bool HasPortC(void) { return false; };
-  virtual bool HasCCP2(void) { return false; };
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F458_;};
+  P18F458(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+  void create();
 };
 
-class P18F1220 : public P18Fxx20
+
+
+class P18F1220 : public  _16bit_v2_adc
 {
  public:
   OSCTUNE      osctune;
   ECCPAS        eccpas;
   PWM1CON       pwm1con;
 
+  virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
   virtual PROCESSOR_TYPE isa(){return _P18F1220_;};
   P18F1220(const char *_name=0, const char *desc=0);
   ~P18F1220();
+
+
   static Processor *construct(const char *name);
   void create();
   virtual void create_iopin_map();
   virtual unsigned int program_memory_size() const { return 0x1000; };
+  virtual unsigned int eeprom_memory_size() const { return 256; };
   virtual void osc_mode(unsigned int value);
+  virtual unsigned int last_actual_register () const { return 0x00FF;};
 
+  // Strip down from base class
+  virtual bool HasPortC(void) { return false; };
+  virtual bool HasCCP2(void) { return false; };
+  
   virtual void set_eeprom(EEPROM *ep) {
     // Use set_eeprom_pir as the 18Fxxx devices use an EEPROM with PIR
    assert(0);
   }
   virtual void set_eeprom_pir(EEPROM_PIR *ep) { eeprom = ep; }
   virtual EEPROM_PIR *get_eeprom() { return ((EEPROM_PIR *)eeprom); }
+  virtual unsigned int get_device_id() { return (0x07 << 8)|(0x7 <<5); }
 };
 
 
@@ -280,6 +309,7 @@ class P18F1320 : public P18F1220
   void create();
 
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int get_device_id() { return (0x07 << 8)|(0x6 <<5); }
 
 };
 
@@ -338,6 +368,7 @@ class P18F2221 : public P18F2x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x800; };
+  virtual unsigned int get_device_id() { return (0x21 << 8)|(0x3 <<5); }
 };
 
 class P18F2321 : public P18F2x21
@@ -348,6 +379,7 @@ class P18F2321 : public P18F2x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x1000; };
+  virtual unsigned int get_device_id() { return (0x21 << 8)|(0x1 <<5); }
 };
 
 class P18F2420 : public P18F2x21
@@ -358,6 +390,9 @@ class P18F2420 : public P18F2x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int eeprom_memory_size() const { return 256; };
+  virtual unsigned int last_actual_register () const { return 0x02FF;};
+  virtual unsigned int get_device_id() { return (0x0c << 8)|(0x6 <<5); }
 };
 
 class P18F2455 : public P18F2x21
@@ -368,7 +403,6 @@ class P18F2455 : public P18F2x21
                uep6,  uep7,  uep8, uep9, uep10, uep11, uep12,  uep13,
                uep14,  uep15;
 
-  PIR2v4       pir2;
   virtual PROCESSOR_TYPE isa(){return _P18F2455_;};
   P18F2455(const char *_name=0, const char *desc=0);
   ~P18F2455();
@@ -378,6 +412,28 @@ class P18F2455 : public P18F2x21
   virtual unsigned int access_gprs() { return 0x60; };  // USB peripheral moves access split
   virtual unsigned int program_memory_size() const { return 0x3000; };
   virtual unsigned int last_actual_register () const { return 0x07FF;};
+  virtual unsigned int get_device_id() { return (0x12 << 8)|(0x3 <<5); }
+
+};
+
+class P18F2550 : public P18F2x21
+{
+ public:
+  sfr_register ufrml, ufrmh, uir,  uie,  ueir,  ueie,  ustat,  ucon,
+               uaddr, ucfg,  uep0, uep1, uep2,  uep3,  uep4,   uep5,
+               uep6,  uep7,  uep8, uep9, uep10, uep11, uep12,  uep13,
+               uep14,  uep15;
+
+  virtual PROCESSOR_TYPE isa(){return _P18F2550_;};
+  P18F2550(const char *_name=0, const char *desc=0);
+  ~P18F2550();
+  static Processor *construct(const char *name);
+  void create_sfr_map();
+
+  virtual unsigned int access_gprs() { return 0x60; };  // USB peripheral moves access split
+  virtual unsigned int program_memory_size() const { return 16384; };
+  virtual unsigned int last_actual_register () const { return 0x07FF;};
+  virtual unsigned int get_device_id() { return (0x12 << 8)|(0x2 <<5); }
 
 };
 
@@ -389,9 +445,42 @@ class P18F2520 : public P18F2x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x4000; };
+  virtual unsigned int last_actual_register () const { return 0x05FF;};
+  virtual unsigned int get_device_id() { return (0x0c << 8)|(0x4 <<5); }
 };
 
+class P18F2525 : public P18F2x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F2525_;};
+  P18F2525(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
 
+  virtual unsigned int program_memory_size() const { return 24576; };
+  virtual unsigned int last_actual_register () const { return 0x05FF;};
+  virtual unsigned int get_device_id() { return (0x0c << 8)|(0x6 <<5); }
+};
+
+class P18F2620 : public P18F2x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F2620_;};
+  P18F2620(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+
+  virtual unsigned int program_memory_size() const { return 0x8000; };
+  virtual unsigned int last_actual_register () const { return 0x0F7F;};
+  virtual unsigned int get_device_id() { return (0x0c << 8)|(0x4 <<5); }
+};
+
+class RegZero : public Register
+{
+public:
+   RegZero(Module *_cpu, const char *_name=0, const char *desc=0): 
+	Register(_cpu, _name, desc) {}
+   virtual void put(unsigned int new_value) { value.put(0);}
+   virtual void put_value(unsigned int new_value) { value.put(0);}
+};
 
 class P18F4x21 : public P18F2x21
 {
@@ -401,7 +490,6 @@ class P18F4x21 : public P18F2x21
   PicTrisRegister  *m_trisd;
   PicLatchRegister *m_latd;
 
-  PSP               psp;
 
   P18F4x21(const char *_name=0, const char *desc=0);
   ~P18F4x21();
@@ -424,6 +512,7 @@ class P18F4221 : public P18F4x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x800; };
+  virtual unsigned int get_device_id() { return (0x21 << 8)|(0x2 <<5); }
 };
 
 class P18F4321 : public P18F4x21
@@ -434,6 +523,7 @@ class P18F4321 : public P18F4x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x1000; };
+  virtual unsigned int get_device_id() { return (0x21 << 8)|(0x0 <<5); }
 };
 
 class P18F4420 : public P18F4x21
@@ -444,6 +534,7 @@ class P18F4420 : public P18F4x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int get_device_id() { return (0x0c << 8)|(0x2 <<5); }
 };
 
 class P18F4455 : public P18F4x21
@@ -454,7 +545,11 @@ class P18F4455 : public P18F4x21
                uep6,  uep7,  uep8, uep9, uep10, uep11, uep12,  uep13,
                uep14,  uep15;
 
-  PIR2v4       pir2;
+  SPP		spp;
+  SPPCON 	sppcon;
+  SPPCFG 	sppcfg;
+  SPPEPS 	sppeps;
+  SPPDATA 	sppdata;
 
   virtual PROCESSOR_TYPE isa(){return _P18F4455_;};
   P18F4455(const char *_name=0, const char *desc=0);
@@ -465,6 +560,34 @@ class P18F4455 : public P18F4x21
   virtual unsigned int access_gprs() { return 0x60; };  // USB peripheral moves access split
   virtual unsigned int program_memory_size() const { return 0x3000; };
   virtual unsigned int last_actual_register () const { return 0x07FF;};
+  virtual unsigned int get_device_id() { return (0x12 << 8)|(0x1 <<5); }
+
+};
+
+class P18F4550 : public P18F4x21
+{
+ public:
+  sfr_register ufrml, ufrmh, uir,  uie,  ueir,  ueie,  ustat,  ucon,
+               uaddr, ucfg,  uep0, uep1, uep2,  uep3,  uep4,   uep5,
+               uep6,  uep7,  uep8, uep9, uep10, uep11, uep12,  uep13,
+               uep14,  uep15;
+
+  SPP		spp;
+  SPPCON 	sppcon;
+  SPPCFG 	sppcfg;
+  SPPEPS 	sppeps;
+  SPPDATA 	sppdata;
+
+  virtual PROCESSOR_TYPE isa(){return _P18F4550_;};
+  P18F4550(const char *_name=0, const char *desc=0);
+  ~P18F4550();
+  static Processor *construct(const char *name);
+  void create();
+
+  virtual unsigned int access_gprs() { return 0x60; };  // USB peripheral moves access split
+  virtual unsigned int program_memory_size() const { return 16384; };
+  virtual unsigned int last_actual_register () const { return 0x07FF;};
+  virtual unsigned int get_device_id() { return (0x12 << 8)|(0x0 <<5); }
 
 };
 
@@ -476,6 +599,8 @@ class P18F4520 : public P18F4x21
   static Processor *construct(const char *name);
 
   virtual unsigned int program_memory_size() const { return 0x4000; };
+  virtual unsigned int last_actual_register () const { return 0x05FF;};
+  virtual unsigned int get_device_id() { return (0x0c << 8)|(0x0 <<5); }
 };
 
 
@@ -498,6 +623,7 @@ class P18F4620 : public P18F4x21
   virtual unsigned int program_memory_size() const { return 0x8000; };
   virtual unsigned int eeprom_memory_size() const { return 1024; };
   virtual unsigned int last_actual_register () const { return 0x0F7F;};
+  virtual unsigned int get_device_id() { return (0x0c << 8)|(0x4 <<5); }
 };
 
 
@@ -511,7 +637,6 @@ class P18F6x20 : public _16bit_v2_adc
   PicLatchRegister *m_latd;
 
   PicPortRegister  *m_porte;
-  //RRRPicPSP_TrisRegister  *m_trise;
   PicTrisRegister  *m_trise;
   PicLatchRegister *m_late;
 
@@ -587,6 +712,7 @@ class P18F6520 : public P18F6x20
 
 //  virtual unsigned int program_memory_size() const { return 0x4000; };
   virtual unsigned int bugs() { return BUG_DAW; };
+  virtual unsigned int get_device_id() { return (0x0b << 8)|(0x1 <<5); }
 };
 
 #endif
