@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -279,7 +279,7 @@ PIR3v178x(Processor *pCpu, const char *pName, const char *pDesc,INTCON *_intcon,
 
 
 P12F1822::P12F1822(const char *_name, const char *desc)
-  : _14bit_e_processor(_name,desc), 
+  : _14bit_e_processor(_name,desc),
     comparator(this),
     pie1(this,"PIE1", "Peripheral Interrupt Enable"),
     pie2(this,"PIE2", "Peripheral Interrupt Enable"),
@@ -299,7 +299,7 @@ P12F1822::P12F1822(const char *_name, const char *desc)
     adcon1(this,"adcon1", "A2D Control 1"),
     adresh(this,"adresh", "A2D Result High"),
     adresl(this,"adresl", "A2D Result Low"),
-    osccon(this, "osccon", "Oscillator Control Register"),
+    osccon(0),
     osctune(this, "osctune", "Oscillator Tunning Register"),
     oscstat(this, "oscstat", "Oscillator Status Register"),
     wdtcon(this, "wdtcon", "Watch dog timer control", 0x3f),
@@ -311,7 +311,7 @@ P12F1822::P12F1822(const char *_name, const char *desc)
     pstr1con(this, "pstr1con", "Pulse Sterring Control Register"),
     cpscon0(this, "cpscon0", " Capacitive Sensing Control Register 0"),
     cpscon1(this, "cpscon1", " Capacitive Sensing Control Register 1"),
-    sr_module(this)
+    sr_module(this), dsm_module(this)
 
 
 {
@@ -374,7 +374,7 @@ P12F1822::~P12F1822()
     remove_sfr_register(&cpscon1);
     remove_sfr_register(&ssp.sspbuf);
     remove_sfr_register(&ssp.sspadd);
-    remove_sfr_register(&ssp.ssp1msk);
+    remove_sfr_register(ssp.sspmsk);
     remove_sfr_register(&ssp.sspstat);
     remove_sfr_register(&ssp.sspcon);
     remove_sfr_register(&ssp.sspcon2);
@@ -410,7 +410,7 @@ P12F1822::~P12F1822()
     remove_sfr_register(&usart.baudcon);
     remove_sfr_register(&ssp.sspbuf);
     remove_sfr_register(&ssp.sspadd);
-    remove_sfr_register(&ssp.ssp1msk);
+    remove_sfr_register(ssp.sspmsk);
     remove_sfr_register(&ssp.sspstat);
     remove_sfr_register(&ssp.sspcon);
     remove_sfr_register(&ssp.sspcon2);
@@ -423,18 +423,23 @@ P12F1822::~P12F1822()
     remove_sfr_register(&pstr1con);
     remove_sfr_register(&osctune);
     remove_sfr_register(&option_reg);
-    remove_sfr_register(&osccon);
+    remove_sfr_register(osccon);
     remove_sfr_register(&oscstat);
 
-    remove_sfr_register(comparator.cmxcon0[0]); 
-    remove_sfr_register(comparator.cmxcon1[0]); 
-    remove_sfr_register(comparator.cmout); 
+    remove_sfr_register(comparator.cmxcon0[0]);
+    remove_sfr_register(comparator.cmxcon1[0]);
+    remove_sfr_register(comparator.cmout);
     delete_sfr_register(usart.rcreg);
     delete_sfr_register(usart.txreg);
     delete_sfr_register(pir1);
     delete_sfr_register(pir2);
+    remove_sfr_register(&dsm_module.mdcon);
+    remove_sfr_register(&dsm_module.mdsrc);
+    remove_sfr_register(&dsm_module.mdcarl);
+    remove_sfr_register(&dsm_module.mdcarh);
     delete e;
     delete m_cpu_temp;
+    delete osccon;
 }
 Processor * P12F1822::construct(const char *name)
 {
@@ -487,7 +492,7 @@ void P12F1822::create_sfr_map()
   pcon.valid_bits = 0xcf;
   add_sfr_register(&option_reg, 0x95, RegisterValue(0xff,0));
   add_sfr_register(&osctune,    0x98, RegisterValue(0,0));
-  add_sfr_register(&osccon,     0x99, RegisterValue(0x38,0));
+  add_sfr_register(osccon,     0x99, RegisterValue(0x38,0));
   add_sfr_register(&oscstat,    0x9a, RegisterValue(0,0));
 
   intcon_reg.set_pir_set(get_pir_set());
@@ -496,13 +501,13 @@ void P12F1822::create_sfr_map()
   tmr1l.tmrh = &tmr1h;
   tmr1l.t1con = &t1con_g;
   tmr1l.setInterruptSource(new InterruptSource(pir1, PIR1v1::TMR1IF));
-  
+
   tmr1h.tmrl  = &tmr1l;
   t1con_g.tmrl  = &tmr1l;
   t1con_g.t1gcon.set_tmrl(&tmr1l);
   t1con_g.t1gcon.setInterruptSource(new InterruptSource(pir1, PIR1v1822::TMR1IF));
 
-  
+
 
   tmr1l.setIOpin(&(*m_porta)[5]);
   t1con_g.t1gcon.setGatepin(&(*m_porta)[3]);
@@ -518,15 +523,15 @@ void P12F1822::create_sfr_map()
   usart.initialize(pir1,
 	&(*m_porta)[0], // TX pin
 	&(*m_porta)[1], // RX pin
-	new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
+	new _TXREG(this,"txreg", "USART Transmit Register", &usart),
         new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
   usart.set_eusart(true);
 
   add_sfr_register(m_lata,    0x10c);
-  add_sfr_register(comparator.cmxcon0[0], 0x111, RegisterValue(0x04,0)); 
-  add_sfr_register(comparator.cmxcon1[0], 0x112, RegisterValue(0x00,0)); 
-  add_sfr_register(comparator.cmout,      0x115, RegisterValue(0x00,0)); 
+  add_sfr_register(comparator.cmxcon0[0], 0x111, RegisterValue(0x04,0));
+  add_sfr_register(comparator.cmxcon1[0], 0x112, RegisterValue(0x00,0));
+  add_sfr_register(comparator.cmout,      0x115, RegisterValue(0x00,0));
   add_sfr_register(&borcon,   0x116, RegisterValue(0x80,0));
   add_sfr_register(&fvrcon,   0x117, RegisterValue(0x00,0));
   add_sfr_register(m_daccon0, 0x118, RegisterValue(0x00,0));
@@ -537,6 +542,8 @@ void P12F1822::create_sfr_map()
   add_sfr_register(&ansela,   0x18c, RegisterValue(0x17,0));
   add_sfr_register(get_eeprom()->get_reg_eeadr(),   0x191);
   add_sfr_register(get_eeprom()->get_reg_eeadrh(),   0x192);
+  get_eeprom()->get_reg_eedata()->new_name("eedatl");
+  get_eeprom()->get_reg_eedatah()->new_name("eedath");
   add_sfr_register(get_eeprom()->get_reg_eedata(),  0x193);
   add_sfr_register(get_eeprom()->get_reg_eedatah(),  0x194);
   add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x195, RegisterValue(0,0));
@@ -552,7 +559,7 @@ void P12F1822::create_sfr_map()
   add_sfr_register(m_wpua,       0x20c, RegisterValue(0x3f,0),"wpua");
   add_sfr_register(&ssp.sspbuf,  0x211, RegisterValue(0,0),"ssp1buf");
   add_sfr_register(&ssp.sspadd,  0x212, RegisterValue(0,0),"ssp1add");
-  add_sfr_register(&ssp.ssp1msk, 0x213, RegisterValue(0xff,0),"ssp1msk");
+  add_sfr_register(ssp.sspmsk, 0x213, RegisterValue(0xff,0),"ssp1msk");
   add_sfr_register(&ssp.sspstat, 0x214, RegisterValue(0,0),"ssp1stat");
   add_sfr_register(&ssp.sspcon,  0x215, RegisterValue(0,0),"ssp1con");
   add_sfr_register(&ssp.sspcon2, 0x216, RegisterValue(0,0),"ssp1con2");
@@ -568,6 +575,10 @@ void P12F1822::create_sfr_map()
   add_sfr_register(m_iocan, 0x392, RegisterValue(0,0),"iocan");
   add_sfr_register(m_iocaf, 0x393, RegisterValue(0,0),"iocaf");
   m_iocaf->set_intcon(intcon);
+  add_sfr_register(&dsm_module.mdcon, 0x39c, RegisterValue(0x20,0));
+  add_sfr_register(&dsm_module.mdsrc, 0x39d, RegisterValue(0x00,0));
+  add_sfr_register(&dsm_module.mdcarl, 0x39e, RegisterValue(0x00,0));
+  add_sfr_register(&dsm_module.mdcarh, 0x39f, RegisterValue(0x00,0));
 
   tmr2.ssp_module[0] = &ssp;
 
@@ -630,7 +641,7 @@ void P12F1822::create_sfr_map()
     adcon0.setChannel_shift(2);
     adcon0.setGo(1);
 
-    adcon1.setAdcon0(&adcon0); 
+    adcon1.setAdcon0(&adcon0);
     adcon1.setNumberOfChannels(32); // not all channels are used
     adcon1.setIOPin(0, &(*m_porta)[0]);
     adcon1.setIOPin(1, &(*m_porta)[1]);
@@ -670,9 +681,11 @@ void P12F1822::create_sfr_map()
 
     sr_module.setPins(&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[5]);
 
-    osccon.set_osctune(&osctune);
-    osccon.set_oscstat(&oscstat);
-    osctune.set_osccon((OSCCON *)&osccon);
+    osccon->set_osctune(&osctune);
+    osccon->set_oscstat(&oscstat);
+    osctune.set_osccon((OSCCON *)osccon);
+    osccon->write_mask = 0xfb;
+    dsm_module.usart_mod = &usart;
 }
 
 //-------------------------------------------------------------------
@@ -715,8 +728,9 @@ void  P12F1822::create(int ram_top, int eeprom_size)
   e = new EEPROM_EXTND(this, pir2);
   set_eeprom(e);
 
-  pic_processor::create();
+  osccon = new OSCCON_2(this, "osccon", "Oscillator Control Register");
 
+  pic_processor::create();
 
   e->initialize(eeprom_size, 16, 16, 0x8000);
   e->set_intcon(intcon);
@@ -724,7 +738,11 @@ void  P12F1822::create(int ram_top, int eeprom_size)
 
   add_file_registers(0x20, ram_top, 0x00);
   _14bit_e_processor::create_sfr_map();
-  P12F1822::create_sfr_map();
+  create_sfr_map();
+  dsm_module.setOUTpin(&(*m_porta)[0]);
+  dsm_module.setMINpin(&(*m_porta)[1]);
+  dsm_module.setCIN1pin(&(*m_porta)[2]);
+  dsm_module.setCIN2pin(&(*m_porta)[4]);
   // Set DeviceID
   if (m_configMemory && m_configMemory->getConfigWord(6))
       m_configMemory->getConfigWord(6)->set(0x2700);
@@ -734,6 +752,7 @@ void  P12F1822::create(int ram_top, int eeprom_size)
 void P12F1822::enter_sleep()
 {
     tmr1l.sleep();
+    osccon->sleep();
     _14bit_e_processor::enter_sleep();
 }
 
@@ -743,7 +762,7 @@ void P12F1822::exit_sleep()
     if (m_ActivityState == ePASleeping)
     {
 	tmr1l.wake();
-        osccon.wake();
+        osccon->wake();
 	_14bit_e_processor::exit_sleep();
     }
 }
@@ -751,20 +770,24 @@ void P12F1822::exit_sleep()
 void P12F1822::option_new_bits_6_7(unsigned int bits)
 {
 	Dprintf(("P12F1822::option_new_bits_6_7 bits=%x\n", bits));
-    m_porta->setIntEdge ( (bits & OPTION_REG::BIT6) == OPTION_REG::BIT6); 
-    m_wpua->set_wpu_pu ( (bits & OPTION_REG::BIT7) != OPTION_REG::BIT7); 
+    m_porta->setIntEdge ( (bits & OPTION_REG::BIT6) == OPTION_REG::BIT6);
+    m_wpua->set_wpu_pu ( (bits & OPTION_REG::BIT7) != OPTION_REG::BIT7);
 }
 
 void P12F1822::oscillator_select(unsigned int cfg_word1, bool clkout)
 {
     unsigned int mask = 0x1f;
 
-    osccon.set_config(cfg_word1 & (FOSC0|FOSC1|FOSC2), cfg_word1 & IESO);
+    unsigned int fosc = cfg_word1 & (FOSC0|FOSC1|FOSC2);
+
+    osccon->set_config_irc(fosc == 4);
+    osccon->set_config_xosc(fosc < 3);
+    osccon->set_config_ieso(cfg_word1 & IESO);
     set_int_osc(false);
-    switch(cfg_word1 & (FOSC0|FOSC1|FOSC2))
+    switch(fosc)
     {
     case 0:	//LP oscillator: low power crystal
-    case 1:	//XT oscillator: Crystal/resonator 
+    case 1:	//XT oscillator: Crystal/resonator
     case 2:	//HS oscillator: High-speed crystal/resonator
         (m_porta->getPin(4))->newGUIname("OSC2");
         (m_porta->getPin(5))->newGUIname("OSC1");
@@ -775,7 +798,7 @@ void P12F1822::oscillator_select(unsigned int cfg_word1, bool clkout)
     case 3:	//EXTRC oscillator External RC circuit connected to CLKIN pin
         (m_porta->getPin(5))->newGUIname("CLKIN");
 	mask = 0x1f;
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(4))->newGUIname("CLKOUT");
 	    mask = 0x0f;
@@ -785,7 +808,7 @@ void P12F1822::oscillator_select(unsigned int cfg_word1, bool clkout)
     case 4:	//INTOSC oscillator: I/O function on CLKIN pin
         set_int_osc(true);
 	mask = 0x3f;
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(4))->newGUIname("CLKOUT");
 	    mask = 0x2f;
@@ -795,7 +818,7 @@ void P12F1822::oscillator_select(unsigned int cfg_word1, bool clkout)
 
     case 5:	//ECL: External Clock, Low-Power mode (0-0.5 MHz): on CLKIN pin
 	mask = 0x1f;
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(4))->newGUIname("CLKOUT");
 	    mask = 0x0f;
@@ -805,7 +828,7 @@ void P12F1822::oscillator_select(unsigned int cfg_word1, bool clkout)
 
     case 6:	//ECM: External Clock, Medium-Power mode (0.5-4 MHz): on CLKIN pin
 	mask = 0x1f;
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(4))->newGUIname("CLKOUT");
 	    mask = 0x0f;
@@ -815,7 +838,7 @@ void P12F1822::oscillator_select(unsigned int cfg_word1, bool clkout)
 
     case 7:	//ECH: External Clock, High-Power mode (4-32 MHz): on CLKIN pin
 	mask = 0x1f;
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(4))->newGUIname("CLKOUT");
 	    mask = 0x0f;
@@ -847,16 +870,50 @@ void P12F1822::program_memory_wp(unsigned int mode)
 	    break;
 
 	default:
-	    printf("%s unexpected mode %d\n", __FUNCTION__, mode);
+	    printf("%s unexpected mode %u\n", __FUNCTION__, mode);
 	    break;
 	}
 
 }
 //========================================================================
+Processor * P12F1840::construct(const char *name)
+{
+
+  P12F1840 *p = new P12F1840(name);
+
+  p->create(0x7f, 256);
+  p->create_invalid_registers ();
+  p->create_symbols();
+  return p;
+
+}
+P12F1840::P12F1840(const char *_name, const char *desc) :
+	P12F1822(_name, desc)
+{
+}
+P12F1840::~P12F1840()
+{
+  delete_file_registers(0xc0, 0xef, 0x00);
+  delete_file_registers(0x120, 0x16f, 0x00);
+  delete_sfr_register(vrefcon);
+}
+void  P12F1840::create(int ram_top, int eeprom_size)
+{
+  P12F1822::create(ram_top, eeprom_size);
+  add_file_registers(0xc0, 0xef, 0x00);
+  add_file_registers(0x120, 0x16f, 0x00);
+  // Set DeviceID
+  if (m_configMemory && m_configMemory->getConfigWord(6))
+      m_configMemory->getConfigWord(6)->set(0x1b80);
+  vrefcon = new sfr_register(this, "vrefcon",
+		"Voltage Regulator Control Register");
+  add_sfr_register(vrefcon, 0x197, RegisterValue(0x01,0));
+}
+//========================================================================
 
 
 P16F178x::P16F178x(const char *_name, const char *desc)
-  : _14bit_e_processor(_name,desc), 
+  : _14bit_e_processor(_name,desc),
     comparator(this),
     pie1(this,"pie1", "Peripheral Interrupt Enable"),
     pie2(this,"pie2", "Peripheral Interrupt Enable"),
@@ -880,7 +937,7 @@ P16F178x::P16F178x(const char *_name, const char *desc)
     adcon2(this,"adcon2", "A2D Control 2"),
     adresh(this,"adresh", "A2D Result High"),
     adresl(this,"adresl", "A2D Result Low"),
-    osccon(this, "osccon", "Oscillator Control Register"),
+    osccon(0),
     osctune(this, "osctune", "Oscillator Tunning Register"),
     oscstat(this, "oscstat", "Oscillator Status Register"),
     wdtcon(this, "wdtcon", "Watch dog timer control", 0x3f),
@@ -1020,7 +1077,7 @@ P16F178x::~P16F178x()
     remove_sfr_register(&t2con);
     remove_sfr_register(&ssp.sspbuf);
     remove_sfr_register(&ssp.sspadd);
-    remove_sfr_register(&ssp.ssp1msk);
+    remove_sfr_register(ssp.sspmsk);
     remove_sfr_register(&ssp.sspstat);
     remove_sfr_register(&ssp.sspcon);
     remove_sfr_register(&ssp.sspcon2);
@@ -1059,7 +1116,7 @@ P16F178x::~P16F178x()
     remove_sfr_register(&usart.baudcon);
     remove_sfr_register(&ssp.sspbuf);
     remove_sfr_register(&ssp.sspadd);
-    remove_sfr_register(&ssp.ssp1msk);
+    remove_sfr_register(ssp.sspmsk);
     remove_sfr_register(&ssp.sspstat);
     remove_sfr_register(&ssp.sspcon);
     remove_sfr_register(&ssp.sspcon2);
@@ -1072,16 +1129,16 @@ P16F178x::~P16F178x()
     remove_sfr_register(&pstr1con);
     remove_sfr_register(&osctune);
     remove_sfr_register(&option_reg);
-    remove_sfr_register(&osccon);
+    remove_sfr_register(osccon);
     remove_sfr_register(&oscstat);
 
-    remove_sfr_register(comparator.cmxcon0[0]); 
-    remove_sfr_register(comparator.cmxcon1[0]); 
-    remove_sfr_register(comparator.cmout); 
-    remove_sfr_register(comparator.cmxcon0[1]); 
-    remove_sfr_register(comparator.cmxcon1[1]); 
-    remove_sfr_register(comparator.cmxcon0[2]); 
-    remove_sfr_register(comparator.cmxcon1[2]); 
+    remove_sfr_register(comparator.cmxcon0[0]);
+    remove_sfr_register(comparator.cmxcon1[0]);
+    remove_sfr_register(comparator.cmout);
+    remove_sfr_register(comparator.cmxcon0[1]);
+    remove_sfr_register(comparator.cmxcon1[1]);
+    remove_sfr_register(comparator.cmxcon0[2]);
+    remove_sfr_register(comparator.cmxcon1[2]);
     delete_sfr_register(usart.rcreg);
     delete_sfr_register(usart.txreg);
     delete_sfr_register(pir1);
@@ -1145,7 +1202,7 @@ void P16F178x::create_sfr_map()
   pcon.valid_bits = 0xcf;
   add_sfr_register(&option_reg, 0x95, RegisterValue(0xff,0));
   add_sfr_register(&osctune,    0x98, RegisterValue(0,0));
-  add_sfr_register(&osccon,     0x99, RegisterValue(0x38,0));
+  add_sfr_register(osccon,     0x99, RegisterValue(0x38,0));
   add_sfr_register(&oscstat,    0x9a, RegisterValue(0,0));
 
   intcon_reg.set_pir_set(get_pir_set());
@@ -1154,7 +1211,7 @@ void P16F178x::create_sfr_map()
   tmr1l.tmrh = &tmr1h;
   tmr1l.t1con = &t1con_g;
   tmr1l.setInterruptSource(new InterruptSource(pir1, PIR1v1::TMR1IF));
-  
+
   tmr1h.tmrl  = &tmr1l;
   t1con_g.tmrl  = &tmr1l;
   t1con_g.t1gcon.set_tmrl(&tmr1l);
@@ -1176,7 +1233,7 @@ void P16F178x::create_sfr_map()
   usart.initialize(pir1,
 	&(*m_porta)[0], // TX pin
 	&(*m_porta)[1], // RX pin
-	new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
+	new _TXREG(this,"txreg", "USART Transmit Register", &usart),
         new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
   usart.set_eusart(true);
@@ -1184,22 +1241,24 @@ void P16F178x::create_sfr_map()
     add_sfr_register(m_lata,    0x10c);
     add_sfr_register(m_latb, 0x10d);
     add_sfr_register(m_latc, 0x10e);
-    add_sfr_register(comparator.cmxcon0[0], 0x111, RegisterValue(0x04,0)); 
-    add_sfr_register(comparator.cmxcon1[0], 0x112, RegisterValue(0x00,0)); 
-    add_sfr_register(comparator.cmxcon0[1], 0x113, RegisterValue(0x04,0)); 
-    add_sfr_register(comparator.cmxcon1[1], 0x114, RegisterValue(0x00,0)); 
-    add_sfr_register(comparator.cmout,      0x115, RegisterValue(0x00,0)); 
+    add_sfr_register(comparator.cmxcon0[0], 0x111, RegisterValue(0x04,0));
+    add_sfr_register(comparator.cmxcon1[0], 0x112, RegisterValue(0x00,0));
+    add_sfr_register(comparator.cmxcon0[1], 0x113, RegisterValue(0x04,0));
+    add_sfr_register(comparator.cmxcon1[1], 0x114, RegisterValue(0x00,0));
+    add_sfr_register(comparator.cmout,      0x115, RegisterValue(0x00,0));
     add_sfr_register(&borcon,   0x116, RegisterValue(0x80,0));
     add_sfr_register(&fvrcon,   0x117, RegisterValue(0x00,0));
     add_sfr_register(m_daccon0, 0x118, RegisterValue(0x00,0));
     add_sfr_register(m_daccon1, 0x119, RegisterValue(0x00,0));
     add_sfr_register(&apfcon2 ,  0x11c, RegisterValue(0x00,0));
     add_sfr_register(&apfcon1 ,  0x11d, RegisterValue(0x00,0));
-    add_sfr_register(comparator.cmxcon0[2], 0x11e, RegisterValue(0x04,0)); 
-    add_sfr_register(comparator.cmxcon1[2], 0x11f, RegisterValue(0x00,0)); 
+    add_sfr_register(comparator.cmxcon0[2], 0x11e, RegisterValue(0x04,0));
+    add_sfr_register(comparator.cmxcon1[2], 0x11f, RegisterValue(0x00,0));
     add_sfr_register(&ansela,   0x18c, RegisterValue(0x17,0));
     add_sfr_register(&anselb,   0x18d, RegisterValue(0x7f,0));
     add_sfr_register(&anselc,   0x18e, RegisterValue(0xff,0));
+  get_eeprom()->get_reg_eedata()->new_name("eedatl");
+  get_eeprom()->get_reg_eedatah()->new_name("eedath");
   add_sfr_register(get_eeprom()->get_reg_eeadr(),   0x191);
   add_sfr_register(get_eeprom()->get_reg_eeadrh(),   0x192);
   add_sfr_register(get_eeprom()->get_reg_eedata(),  0x193);
@@ -1221,7 +1280,7 @@ void P16F178x::create_sfr_map()
 
   add_sfr_register(&ssp.sspbuf,  0x211, RegisterValue(0,0),"ssp1buf");
   add_sfr_register(&ssp.sspadd,  0x212, RegisterValue(0,0),"ssp1add");
-  add_sfr_register(&ssp.ssp1msk, 0x213, RegisterValue(0xff,0),"ssp1msk");
+  add_sfr_register(ssp.sspmsk, 0x213, RegisterValue(0xff,0),"ssp1msk");
   add_sfr_register(&ssp.sspstat, 0x214, RegisterValue(0,0),"ssp1stat");
   add_sfr_register(&ssp.sspcon,  0x215, RegisterValue(0,0),"ssp1con");
   add_sfr_register(&ssp.sspcon2, 0x216, RegisterValue(0,0),"ssp1con2");
@@ -1327,7 +1386,7 @@ void P16F178x::create_sfr_map()
     adcon0.setChannel_shift(2);
     adcon0.setGo(1);
 
-    adcon1.setAdcon0(&adcon0); 
+    adcon1.setAdcon0(&adcon0);
     adcon1.setNumberOfChannels(32); // not all channels are used
     adcon1.setIOPin(0, &(*m_porta)[0]);
     adcon1.setIOPin(1, &(*m_porta)[1]);
@@ -1387,9 +1446,10 @@ void P16F178x::create_sfr_map()
     m_dac4con0->setDACOUT(&(*m_porta)[4], &(*m_portb)[7]);
 
 
-    osccon.set_osctune(&osctune);
-    osccon.set_oscstat(&oscstat);
-    osctune.set_osccon((OSCCON *)&osccon);
+    osccon->set_osctune(&osctune);
+    osccon->set_oscstat(&oscstat);
+    osctune.set_osccon((OSCCON *)osccon);
+    osccon->write_mask = 0xfb;
 }
 
 //-------------------------------------------------------------------
@@ -1410,8 +1470,9 @@ void  P16F178x::create(int ram_top, int eeprom_size)
   e = new EEPROM_EXTND(this, pir2);
   set_eeprom(e);
 
-  pic_processor::create();
+  osccon = new OSCCON_2(this, "osccon", "Oscillator Control Register");
 
+  pic_processor::create();
 
   e->initialize(eeprom_size, 16, 16, 0x8000);
   e->set_intcon(intcon);
@@ -1425,6 +1486,7 @@ void  P16F178x::create(int ram_top, int eeprom_size)
 void P16F178x::enter_sleep()
 {
     tmr1l.sleep();
+    osccon->sleep();
     _14bit_e_processor::enter_sleep();
 }
 
@@ -1434,7 +1496,7 @@ void P16F178x::exit_sleep()
     if (m_ActivityState == ePASleeping)
     {
 	tmr1l.wake();
-        osccon.wake();
+        osccon->wake();
 	_14bit_e_processor::exit_sleep();
     }
 }
@@ -1443,20 +1505,23 @@ void P16F178x::exit_sleep()
 void P16F178x::option_new_bits_6_7(unsigned int bits)
 {
 	Dprintf(("P16F178x::option_new_bits_6_7 bits=%x\n", bits));
-    m_porta->setIntEdge ( (bits & OPTION_REG::BIT6) == OPTION_REG::BIT6); 
-    m_wpua->set_wpu_pu ( (bits & OPTION_REG::BIT7) != OPTION_REG::BIT7); 
+    m_porta->setIntEdge ( (bits & OPTION_REG::BIT6) == OPTION_REG::BIT6);
+    m_wpua->set_wpu_pu ( (bits & OPTION_REG::BIT7) != OPTION_REG::BIT7);
 }
 
 void P16F178x::oscillator_select(unsigned int cfg_word1, bool clkout)
 {
     unsigned int mask = m_porta->getEnableMask();
+    unsigned int fosc = cfg_word1 & (FOSC0|FOSC1|FOSC2);
 
-    osccon.set_config(cfg_word1 & (FOSC0|FOSC1|FOSC2), cfg_word1 & IESO);
+    osccon->set_config_irc(fosc == 4);
+    osccon->set_config_xosc(fosc < 3);
+    osccon->set_config_ieso(cfg_word1 & IESO);
     set_int_osc(false);
-    switch(cfg_word1 & (FOSC0|FOSC1|FOSC2))
+    switch(fosc)
     {
     case 0:	//LP oscillator: low power crystal
-    case 1:	//XT oscillator: Crystal/resonator 
+    case 1:	//XT oscillator: Crystal/resonator
     case 2:	//HS oscillator: High-speed crystal/resonator
         (m_porta->getPin(6))->newGUIname("OSC2");
         (m_porta->getPin(7))->newGUIname("OSC1");
@@ -1467,7 +1532,7 @@ void P16F178x::oscillator_select(unsigned int cfg_word1, bool clkout)
     case 3:	//EXTRC oscillator External RC circuit connected to CLKIN pin
         (m_porta->getPin(7))->newGUIname("CLKIN");
 	mask &= 0x7f;
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(6))->newGUIname("CLKOUT");
 	    mask &= 0xbf;
@@ -1481,7 +1546,7 @@ void P16F178x::oscillator_select(unsigned int cfg_word1, bool clkout)
 
     case 4:	//INTOSC oscillator: I/O function on CLKIN pin
         set_int_osc(true);
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(6))->newGUIname("CLKOUT");
 	    mask &= 0xbf;
@@ -1496,7 +1561,7 @@ void P16F178x::oscillator_select(unsigned int cfg_word1, bool clkout)
 	break;
 
     case 5:	//ECL: External Clock, Low-Power mode (0-0.5 MHz): on CLKIN pin
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(6))->newGUIname("CLKOUT");
 	    mask &= 0xbf;
@@ -1511,7 +1576,7 @@ void P16F178x::oscillator_select(unsigned int cfg_word1, bool clkout)
 	break;
 
     case 6:	//ECM: External Clock, Medium-Power mode (0.5-4 MHz): on CLKIN pin
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(6))->newGUIname("CLKOUT");
 	    mask &= 0xbf;
@@ -1526,7 +1591,7 @@ void P16F178x::oscillator_select(unsigned int cfg_word1, bool clkout)
 	break;
 
     case 7:	//ECH: External Clock, High-Power mode (4-32 MHz): on CLKIN pin
-	if(clkout) 
+	if(clkout)
 	{
 	    (m_porta->getPin(6))->newGUIname("CLKOUT");
 	    mask &= 0xbf;
@@ -1564,7 +1629,7 @@ void P16F178x::program_memory_wp(unsigned int mode)
 	    break;
 
 	default:
-	    printf("%s unexpected mode %d\n", __FUNCTION__, mode);
+	    printf("%s unexpected mode %u\n", __FUNCTION__, mode);
 	    break;
 	}
 
@@ -1573,16 +1638,16 @@ void P16F178x::program_memory_wp(unsigned int mode)
 
 
 P16F1788::P16F1788(const char *_name, const char *desc)
-  : P16F178x(_name,desc) 
+  : P16F178x(_name,desc)
 {
   comparator.cmxcon0[3] = new CMxCON0(this, "cm4con0", " Comparator C4 Control Register 0", 3, &comparator);
   comparator.cmxcon1[3] = new CMxCON1(this, "cm4con1", " Comparator C4 Control Register 1", 3, &comparator);
-   
+
 }
 P16F1788::~P16F1788()
 {
-    remove_sfr_register(comparator.cmxcon0[3]); 
-    remove_sfr_register(comparator.cmxcon1[3]); 
+    remove_sfr_register(comparator.cmxcon0[3]);
+    remove_sfr_register(comparator.cmxcon1[3]);
 }
 void P16F1788::create_iopin_map()
 {
@@ -1660,8 +1725,8 @@ void P16F1788::create_sfr_map()
 
 
 
-    add_sfr_register(comparator.cmxcon0[3], 0x11a, RegisterValue(0x04,0)); 
-    add_sfr_register(comparator.cmxcon1[3], 0x11b, RegisterValue(0x00,0)); 
+    add_sfr_register(comparator.cmxcon0[3], 0x11a, RegisterValue(0x04,0));
+    add_sfr_register(comparator.cmxcon1[3], 0x11b, RegisterValue(0x00,0));
 
     adcon1.setIOPin(12, &(*m_portb)[0]);
     adcon1.setIOPin(10, &(*m_portb)[1]);
@@ -1702,10 +1767,10 @@ void P16F1788::create_sfr_map()
 
 
 P16F1823::P16F1823(const char *_name, const char *desc)
-  : P12F1822(_name,desc), 
+  : P12F1822(_name,desc),
     anselc(this, "anselc", "Analog Select port c")
 {
-   
+
   m_portc = new PicPortBRegister(this,"portc","", intcon, 8,0x3f);
   m_trisc = new PicTrisRegister(this,"trisc","", m_portc, false, 0x3f);
   m_latc  = new PicLatchRegister(this,"latc","",m_portc, 0x3f);
@@ -1721,8 +1786,8 @@ P16F1823::~P16F1823()
     delete_sfr_register(m_portc);
     delete_sfr_register(m_trisc);
     delete_sfr_register(m_latc);
-    remove_sfr_register(comparator.cmxcon0[1]); 
-    remove_sfr_register(comparator.cmxcon1[1]); 
+    remove_sfr_register(comparator.cmxcon0[1]);
+    remove_sfr_register(comparator.cmxcon1[1]);
     delete_sfr_register(m_wpuc);
     remove_sfr_register(&anselc);
 }
@@ -1775,8 +1840,9 @@ void  P16F1823::create(int ram_top, int eeprom_size)
   e = new EEPROM_EXTND(this, pir2);
   set_eeprom(e);
 
-  pic_processor::create();
+  osccon = new OSCCON_2(this, "osccon", "Oscillator Control Register");
 
+  pic_processor::create();
 
   e->initialize(eeprom_size, 16, 16, 0x8000);
   e->set_intcon(intcon);
@@ -1785,7 +1851,11 @@ void  P16F1823::create(int ram_top, int eeprom_size)
   add_file_registers(0x20, ram_top, 0x00);
   _14bit_e_processor::create_sfr_map();
   P12F1822::create_sfr_map();
-  P16F1823::create_sfr_map();
+  create_sfr_map();
+  dsm_module.setOUTpin(&(*m_portc)[4]);
+  dsm_module.setMINpin(&(*m_portc)[3]);
+  dsm_module.setCIN1pin(&(*m_portc)[2]);
+  dsm_module.setCIN2pin(&(*m_portc)[5]);
   // Set DeviceID
   if (m_configMemory && m_configMemory->getConfigWord(6))
       m_configMemory->getConfigWord(6)->set(0x2720);
@@ -1796,8 +1866,8 @@ void P16F1823::create_sfr_map()
     add_sfr_register(m_portc, 0x0e);
     add_sfr_register(m_trisc, 0x8e, RegisterValue(0x3f,0));
     add_sfr_register(m_latc, 0x10e);
-    add_sfr_register(comparator.cmxcon0[1], 0x113, RegisterValue(0x04,0)); 
-    add_sfr_register(comparator.cmxcon1[1], 0x114, RegisterValue(0x00,0)); 
+    add_sfr_register(comparator.cmxcon0[1], 0x113, RegisterValue(0x04,0));
+    add_sfr_register(comparator.cmxcon1[1], 0x114, RegisterValue(0x00,0));
     add_sfr_register(&anselc, 0x18e, RegisterValue(0x0f,0));
     add_sfr_register(m_wpuc, 0x20e, RegisterValue(0x3f,0),"wpuc");
 
@@ -1851,3 +1921,50 @@ void P16F1823::create_sfr_map()
     sr_module.srcon1.set_ValidBits(0xff);
     sr_module.setPins(&(*m_porta)[1], &(*m_porta)[2], &(*m_portc)[4]);
 }
+//========================================================================
+Processor * P16F1825::construct(const char *name)
+{
+
+  P16F1825 *p = new P16F1825(name);
+
+  p->create(0x7f, 256);
+  p->create_invalid_registers ();
+  p->create_symbols();
+  return p;
+
+}
+P16F1825::P16F1825(const char *_name, const char *desc) :
+	P16F1823(_name, desc)
+{
+}
+P16F1825::~P16F1825()
+{
+  delete_file_registers(0xc0, 0xef);
+  delete_file_registers(0x120, 0x16f);
+  delete_file_registers(0x1a0, 0x1ef);
+  delete_file_registers(0x220, 0x26f);
+  delete_file_registers(0x2a0, 0x2ef);
+  delete_file_registers(0x320, 0x32f);
+  delete_file_registers(0x420, 0x46f);
+  delete_file_registers(0x4a0, 0x4ef);
+  delete_file_registers(0x520, 0x56f);
+  delete_file_registers(0x5a0, 0x5ef);
+}
+void  P16F1825::create(int ram_top, int eeprom_size)
+{
+  P16F1823::create(ram_top, eeprom_size);
+  add_file_registers(0xc0, 0xef, 0x00);
+  add_file_registers(0x120, 0x16f, 0x00);
+  add_file_registers(0x1a0, 0x1ef, 0x00);
+  add_file_registers(0x220, 0x26f, 0x00);
+  add_file_registers(0x2a0, 0x2ef, 0x00);
+  add_file_registers(0x320, 0x32f, 0x00);
+  add_file_registers(0x420, 0x46f, 0x00);
+  add_file_registers(0x4a0, 0x4ef, 0x00);
+  add_file_registers(0x520, 0x56f, 0x00);
+  add_file_registers(0x5a0, 0x5ef, 0x00);
+
+  // Set DeviceID
+  if (m_configMemory && m_configMemory->getConfigWord(6))
+      m_configMemory->getConfigWord(6)->set(0x2760);
+}
