@@ -94,9 +94,11 @@ protected:
   INTCON  *m_pIntcon;
   INTCON2  *m_pIntcon2;
   INTCON3  *m_pIntcon3;
+  RegisterValue lastDrivenValue;
 };
 
 class IOC;
+class IOCxF;
 //	Like PicPortBRegister, PicPortGRegister allows wakeup from sleep
 //	and interrupt on pin level change. However, PicPortGRegister
 //	uses IOC to determine which of any of the bits will do this.
@@ -109,29 +111,28 @@ class PicPortGRegister : public PicPortBRegister
  
 public:
   INTCON *m_pIntcon;
-  IOC	*m_pIoc;
+  IOC	 *m_pIoc;
+  unsigned int intf_bit;	// port bit that can trigger intf interrupt
 
   PicPortGRegister(Processor *pCpu, const char *pName, const char *pDesc,
                    INTCON *pIntcon, IOC *pIoc,
-                   unsigned int numIopins, unsigned int enableMask=0x3f)
-	: PicPortBRegister(pCpu, pName, pDesc, pIntcon, numIopins, enableMask),
-	m_pIntcon(pIntcon), m_pIoc(pIoc)
-  {
-  }
+                   unsigned int numIopins, unsigned int enableMask=0x3f);
 
   virtual void setbit(unsigned int bit_number, char new3State);
+  virtual void setIOCif();
+
 };
 class PicPortIOCRegister : public PicPortBRegister
 {
  
 public:
   INTCON *m_pIntcon;
-  IOC	*m_Iocap;
-  IOC	*m_Iocan;
-  IOC	*m_Iocaf;
+  IOC    *m_Iocap;	// pins which can cause interrupts on positive edge
+  IOC    *m_Iocan;	// pins which can cause interrupts on negative edge
+  IOCxF  *m_Iocaf;	// which pins triggered interrupt
 
   PicPortIOCRegister(Processor *pCpu, const char *pName, const char *pDesc,
-                   INTCON *pIntcon, IOC *pIocap, IOC *pIocan, IOC *pIocaf,
+                   INTCON *pIntcon, IOC *pIocap, IOC *pIocan, IOCxF *pIocaf,
                    unsigned int numIopins, unsigned int enableMask=0x3f)
 	: PicPortBRegister(pCpu, pName, pDesc, pIntcon, numIopins, enableMask),
 	m_pIntcon(pIntcon), m_Iocap(pIocap), m_Iocan(pIocan), m_Iocaf(pIocaf)
