@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -33,7 +33,6 @@ License along with this library; if not, see
 #include <iostream>
 #include <iomanip>
 #include <string>
-#include <map>
 
 #include "../config.h"
 
@@ -52,6 +51,7 @@ License along with this library; if not, see
 #include "p16f87x.h"
 #include "p16x6x.h"
 #include "p16x7x.h"
+#include "p16f91x.h"
 #include "p12x.h"
 #include "p12f6xx.h"
 #include "p1xf1xxx.h"
@@ -59,6 +59,7 @@ License along with this library; if not, see
 #include "p17c75x.h"
 #endif
 #include "p18x.h"
+#include "p18fk.h"
 #include "icd.h"
 
 #include "fopen-path.h"
@@ -70,99 +71,6 @@ guint64 simulation_start_cycle;
 
 #include "clock_phase.h"
 
-class phaseCaptureInterrupt : public ProcessorPhase
-{
-public:
-  phaseCaptureInterrupt(Processor *pcpu);
-  ~phaseCaptureInterrupt();
-  virtual ClockPhase *advance();
-  void firstHalf();
-protected:
-  ClockPhase *m_pCurrentPhase;
-  ClockPhase *m_pNextNextPhase;
-};
-
-
-ClockPhase *mCurrentPhase=0;
-phaseExecute1Cycle *mExecute1Cycle=0;
-phaseExecute2ndHalf *mExecute2ndHalf=0;     // misnomer - should be 2-cycle
-phaseCaptureInterrupt *mCaptureInterrupt=0;
-phaseIdle *mIdle=0;
-
-void setCurrentPhase(ClockPhase *pPhase)
-{
-  mCurrentPhase = pPhase;
-}
-
-const char* phaseDesc(ClockPhase *pPhase)
-{
-  if (pPhase == mIdle)
-    return ("mIdle");
-  if (pPhase == mExecute1Cycle)
-    return ("mExecute1Cycle");
-  if (pPhase == mExecute2ndHalf)
-    return ("mExecute2ndHalf");
-  if (pPhase == mCaptureInterrupt)
-    return ("mCaptureInterrupt");
-  return "unknown phase";
-}
-
-phaseCaptureInterrupt::phaseCaptureInterrupt(Processor *pcpu)
-  :  ProcessorPhase(pcpu), m_pCurrentPhase(0),m_pNextNextPhase(0)
-{
-}
-phaseCaptureInterrupt::~phaseCaptureInterrupt()
-{}
-#define Rprintf(arg) {printf("0x%06X %s() ",cycles.get(),__FUNCTION__); printf arg; }
-ClockPhase *phaseCaptureInterrupt::advance()
-{
-
-  //Rprintf (("phaseCaptureInterrupt\n"));
-  if (m_pNextPhase == mExecute2ndHalf)
-    m_pNextPhase->advance();
-
-  if (m_pCurrentPhase == mIdle) { // Interrupted sleep
-
-    // complete sleeping phase
-    m_pNextPhase = m_pNextNextPhase->advance();
-    
-    if (m_pNextPhase == mIdle)
-    {
-        m_pNextPhase = mExecute1Cycle;
-	do 
-	{
-	    m_pNextPhase = mExecute1Cycle->advance();
-	}while (m_pNextPhase != mExecute1Cycle);
-    }
-    mCurrentPhase = this;
-    if (bp.global_break)
-	m_pNextNextPhase = m_pNextPhase;
-    else
-      m_pCurrentPhase = NULL;
-    
-    ((pic_processor *)m_pcpu)->exit_sleep();
-    return this;
-  }
-
-  m_pcpu->interrupt();
-
-  return m_pNextPhase;
-}
-
-void phaseCaptureInterrupt::firstHalf()
-{
-  m_pCurrentPhase = mCurrentPhase;
-
-  m_pNextPhase = this;
-  m_pNextNextPhase = mCurrentPhase->getNextPhase();
-  mCurrentPhase->setNextPhase(this);
-  mCurrentPhase = this;
-}
-
-
-//================================================================================
-// Global Declarations
-//  FIXME -  move these global references somewhere else
 
 
 //================================================================================
@@ -206,6 +114,8 @@ ProcessorConstructor pP12F683(P12F683::construct ,
                               "__12F683",   "pic12f683",    "p12f683",  "12f683");
 ProcessorConstructor pP12F1822(P12F1822::construct ,
                               "__12F1822", "pic12f1822", "p12f1822", "12f1822");
+ProcessorConstructor pP12F1840(P12F1840::construct ,
+                              "__12F1840", "pic12f1840", "p12f1840", "12f1840");
 ProcessorConstructor pP16C54(P16C54::construct ,
                              "__16C54",     "pic16c54",     "p16c54",   "16c54");
 ProcessorConstructor pP16C55(P16C55::construct ,
@@ -252,6 +162,8 @@ ProcessorConstructor pP16F73(P16F73::construct ,
                              "__16F73",     "pic16f73",     "p16f73",   "16f73");
 ProcessorConstructor pP16F74(P16F74::construct ,
                              "__16F74",     "pic16f74",     "p16f74",   "16f74");
+ProcessorConstructor pP16F716(P16F716::construct ,
+                              "__16F716",   "pic16f716",    "p16f716",  "16f716");
 ProcessorConstructor pP16F83(P16F83::construct ,
                              "__16F83",     "pic16f83",     "p16f83",   "16f83");
 ProcessorConstructor pP16F84(P16F84::construct ,
@@ -271,61 +183,71 @@ ProcessorConstructor pP16F886(P16F886::construct ,
 ProcessorConstructor pP16F887(P16F887::construct ,
                      "__16F887",    "pic16f887",     "p16f887",   "16f887");
 ProcessorConstructor pP16F627(P16F627::construct ,
-                              "__16F627",   "pic16f627",    "p16f627",  "16f627");
+                       "__16F627",   "pic16f627",    "p16f627",  "16f627");
 ProcessorConstructor pP16F627A(P16F627::construct ,
-                              "__16F627A",  "pic16f627a",   "p16f627a", "16f627a");
+                       "__16F627A",  "pic16f627a",   "p16f627a", "16f627a");
 ProcessorConstructor pP16F628(P16F628::construct ,
-                              "__16F628",   "pic16f628",    "p16f628",  "16f628");
+                       "__16F628",   "pic16f628",    "p16f628",  "16f628");
 ProcessorConstructor pP16F628A(P16F628::construct ,
-                              "__16F628A",  "pic16f628a",   "p16f628a", "16f628a");
+                       "__16F628A",  "pic16f628a",   "p16f628a", "16f628a");
 ProcessorConstructor pP16F630(P16F630::construct ,
-                              "__16F630",   "pic16f630",    "p16f630",  "16f630");
+                       "__16F630",   "pic16f630",    "p16f630",  "16f630");
 ProcessorConstructor pP16F631(P16F631::construct ,
-                              "__16F631",   "pic16f631",    "p16f631",  "16f631");
+                       "__16F631",   "pic16f631",    "p16f631",  "16f631");
 ProcessorConstructor pP16F648(P16F648::construct ,
-                              "__16F648",   "pic16f648",    "p16f648",  "16f648");
+                       "__16F648",   "pic16f648",    "p16f648",  "16f648");
 ProcessorConstructor pP16F648A(P16F648::construct ,
-                              "__16F648A",  "pic16f648a",   "p16f648a", "16f648a");
+                       "__16F648A",  "pic16f648a",   "p16f648a", "16f648a");
 ProcessorConstructor pP16F676(P16F676::construct ,
-                              "__16F676",   "pic16f676",    "p16f676",  "16f676");
+                       "__16F676",   "pic16f676",    "p16f676",  "16f676");
 ProcessorConstructor pP16F677(P16F677::construct ,
-                              "__16F677",   "pic16f677",    "p16f677",  "16f677");
+                       "__16F677",   "pic16f677",    "p16f677",  "16f677");
 ProcessorConstructor pP16F684(P16F684::construct ,
-                              "__16F684",   "pic16f684",    "p16f684",  "16f684");
+                       "__16F684",   "pic16f684",    "p16f684",  "16f684");
 ProcessorConstructor pP16F685(P16F685::construct ,
-                              "__16F685",   "pic16f685",    "p16f685",  "16f685");
+                       "__16F685",   "pic16f685",    "p16f685",  "16f685");
 ProcessorConstructor pP16F687(P16F687::construct ,
-                              "__16F687",   "pic16f687",    "p16f687",  "16f687");
+                       "__16F687",   "pic16f687",    "p16f687",  "16f687");
 ProcessorConstructor pP16F689(P16F689::construct ,
-                              "__16F689",   "pic16f689",    "p16f689",  "16f689");
+                       "__16F689",   "pic16f689",    "p16f689",  "16f689");
 ProcessorConstructor pP16F690(P16F690::construct ,
-                              "__16F690",   "pic16f690",    "p16f690",  "16f690");
+                       "__16F690",   "pic16f690",    "p16f690",  "16f690");
 ProcessorConstructor pP16F818(P16F818::construct ,
-                              "__16F818",   "pic16f818",    "p16f818",  "16f818");
+                       "__16F818",   "pic16f818",    "p16f818",  "16f818");
 ProcessorConstructor pP16F819(P16F819::construct ,
-                              "__16F819",   "pic16f819",    "p16f819",  "16f819");
+                       "__16F819",   "pic16f819",    "p16f819",  "16f819");
 ProcessorConstructor pP16F871(P16F871::construct ,
-                              "__16F871",   "pic16f871",    "p16f871",  "16f871");
+                       "__16F871",   "pic16f871",    "p16f871",  "16f871");
 ProcessorConstructor pP16F873(P16F873::construct ,
-                              "__16F873",   "pic16f873",    "p16f873",  "16f873");
+                       "__16F873",   "pic16f873",    "p16f873",  "16f873");
 ProcessorConstructor pP16F874(P16F874::construct ,
-                              "__16F874",   "pic16f874",    "p16f874",  "16f874");
+                       "__16F874",   "pic16f874",    "p16f874",  "16f874");
 ProcessorConstructor pP16F876(P16F876::construct ,
-                              "__16F876",   "pic16f876",    "p16f876",  "16f876");
+                       "__16F876",   "pic16f876",    "p16f876",  "16f876");
 ProcessorConstructor pP16F877(P16F877::construct ,
-                              "__16F877",   "pic16f877",    "p16f877",  "16f877");
+                       "__16F877",   "pic16f877",    "p16f877",  "16f877");
 ProcessorConstructor pP16F873A(P16F873A::construct ,
-                              "__16F873a",  "pic16f873a",   "p16f873a", "16f873a");
+                       "__16F873a",  "pic16f873a",   "p16f873a", "16f873a");
 ProcessorConstructor pP16F874A(P16F874A::construct ,
-                              "__16F874a",  "pic16f874a",   "p16f874a", "16f874a");
+                       "__16F874a",  "pic16f874a",   "p16f874a", "16f874a");
 ProcessorConstructor pP16F876A(P16F876A::construct ,
-                              "__16F876a",  "pic16f876a",   "p16f876a", "16f876a");
+                       "__16F876a",  "pic16f876a",   "p16f876a", "16f876a");
 ProcessorConstructor pP16F877A(P16F877A::construct ,
-                              "__16F877a",  "pic16f877a",   "p16f877a", "16f877a");
+                       "__16F877a",  "pic16f877a",   "p16f877a", "16f877a");
+ProcessorConstructor pP16F913(P16F913::construct ,
+                       "__16F913",  "pic16f913",   "p16f913", "16f913");
+ProcessorConstructor pP16F914(P16F914::construct ,
+                       "__16F914",  "pic16f914",   "p16f914", "16f914");
+ProcessorConstructor pP16F916(P16F916::construct ,
+                       "__16F916",  "pic16f916",   "p16f916", "16f916");
+ProcessorConstructor pP16F917(P16F917::construct ,
+                       "__16F917",  "pic16f917",   "p16f917", "16f917");
 ProcessorConstructor pP16F1788(P16F1788::construct ,
-                              "__16F1788", "pic16f1788", "p16f1788", "16f1788");
+                       "__16F1788", "pic16f1788", "p16f1788", "16f1788");
 ProcessorConstructor pP16F1823(P16F1823::construct ,
-                              "__16F1823", "pic16f1823", "p16f1823", "16f1823");
+                       "__16F1823", "pic16f1823", "p16f1823", "16f1823");
+ProcessorConstructor pP16F1825(P16F1825::construct ,
+                       "__16F1825", "pic16f1825", "p16f1825", "16f1825");
 #ifdef P17C7XX  // code no longer works
 ProcessorConstructor pP17C7xx(P17C7xx::construct ,
                               "__17C7xx", "pic17c7xx",  "p17c7xx", "17c7xx");
@@ -354,18 +276,24 @@ ProcessorConstructor pP18F242(P18F242::construct ,
                               "__18F242",   "pic18f242",    "p18f242",  "18f242");
 ProcessorConstructor pP18F248(P18F248::construct ,
                               "__18F248",   "pic18f248",    "p18f248",  "18f248");
+ProcessorConstructor pP18F258(P18F258::construct ,
+                              "__18F258",   "pic18f258",    "p18f258",  "18f258");
 ProcessorConstructor pP18F252(P18F252::construct ,
                               "__18F252",   "pic18f252",    "p18f252",  "18f252");
 ProcessorConstructor pP18F442(P18F442::construct ,
                               "__18F442",   "pic18f442",    "p18f442",  "18f442");
 ProcessorConstructor pP18F448(P18F448::construct ,
                               "__18F448",   "pic18f448",    "p18f448",  "18f448");
+ProcessorConstructor pP18F458(P18F458::construct ,
+                              "__18F458",   "pic18f458",    "p18f458",  "18f458");
 ProcessorConstructor pP18F452(P18F452::construct,
                               "__18F452",   "pic18f452",    "p18f452",  "18f452");
 ProcessorConstructor pP18F1220(P18F1220::construct,
                               "__18F1220",  "pic18f1220",   "p18f1220", "18f1220");
 ProcessorConstructor pP18F1320(P18F1320::construct,
                               "__18F1320",  "pic18f1320",   "p18f1320", "18f1320");
+ProcessorConstructor pP18F14K22(P18F14K22::construct,
+                              "__18F14K22", "pic18f14k22",  "p18f14k22", "18f14k22");
 ProcessorConstructor pP18F2221(P18F2221::construct,
                               "__18F2221",  "pic18f2221",   "p18f2221", "18f2221");
 ProcessorConstructor pP18F2321(P18F2321::construct,
@@ -376,12 +304,14 @@ ProcessorConstructor pP18F2455(P18F2455::construct,
                               "__18F2455",  "pic18f2455",   "p18f2455", "18f2455");
 ProcessorConstructor pP18F2520(P18F2520::construct,
                               "__18F2520",  "pic18f2520",   "p18f2520", "18f2520");
+ProcessorConstructor pP18F2525(P18F2525::construct,
+                              "__18F2525",  "pic18f2525",   "p18f2525", "18f2525");
 ProcessorConstructor pP18F2550(P18F2550::construct,
                               "__18F2550",  "pic18f2550",   "p18f2550", "18f2550");
 ProcessorConstructor pP18F2620(P18F2620::construct,
                               "__18F2620",  "pic18f2620",   "p18f2620", "18f2620");
 ProcessorConstructor pP18F26K22(P18F26K22::construct,
-                              "__18F26K22",  "pic18f26k22",   "p18f26k22", "18f26k22");
+                              "__18F26K22", "pic18f26k22",  "p18f26k22", "18f26k22");
 ProcessorConstructor pP18F4221(P18F4221::construct,
                               "__18F4221",  "pic18f4221",   "p18f4221", "18f4221");
 ProcessorConstructor pP18F4321(P18F4321::construct,
@@ -674,9 +604,9 @@ public:
     // We just hit the break point. A few moments ago we
     // grabbed a snap shot of the system time and the simulated
     // pic's time. Now we're going to compare the two deltas and
-    // see how well they've tracked. 
+    // see how well they've tracked.
     //
-    // If the host is running faster than the PIC, we'll put the 
+    // If the host is running faster than the PIC, we'll put the
     // host to sleep briefly.
     //
     // If the host is running slower than the PIC, lengthen the
@@ -785,6 +715,7 @@ void pic_processor::save_state()
 // run  -- Begin simulating and don't stop until there is a break.
 //
 
+
 void pic_processor::run (bool refresh)
 {
   if(simulation_mode != eSM_STOPPED) {
@@ -799,7 +730,7 @@ void pic_processor::run (bool refresh)
   // then ignore it.
 
   if(realtime_mode)
-    realtime_cbp.start(active_cpu);
+    realtime_cbp.start(this);
 
   simulation_start_cycle = get_cycles().get();
   bp.clear_global();
@@ -814,22 +745,6 @@ void pic_processor::run (bool refresh)
   if(realtime_mode)
     realtime_cbp.stop();
 
-    /* FIXME
-    if(bp.have_pm_write())
-      pm_write();
-    */
-
-    /*
-    if(bp.have_socket_break()) {
-      cout << " socket break point \n";
-      Interface *i = gi.get_socket_interface();
-      if (i)
-        i->Update(0);
-      bp.clear_socket_break();
-    }
-    */
-
-    //} while(!bp.global_break);
 
 
   bp.clear_global();
@@ -1232,36 +1147,37 @@ void pic_processor::create ()
 // FIXME parent's constructor.
 
 void pic_processor::add_sfr_register(Register *reg, unsigned int addr,
-                                     RegisterValue por_value, const char *new_name)
+                                     RegisterValue por_value,
+				     const char *new_name,
+				     bool warn_dup)
 {
 
   reg->set_cpu(this);
   if(addr < register_memory_size())
     {
-      if (registers[addr]) 
+      if (registers[addr])
       {
 	if (registers[addr]->isa() == Register::INVALID_REGISTER)
+	{
 	  delete registers[addr];
-	else
-	    printf("%s 0x%x Already register %s\n", __FUNCTION__, addr, registers[addr]->name().c_str());
+          registers[addr] = reg;
+	}
+	else if (warn_dup)
+	    printf("%s %s 0x%x Already register %s\n", __FUNCTION__, name().c_str(), addr, registers[addr]->name().c_str());
       }
-#ifdef DEBUG
-      if (addr == 0xfa1)
-      {
-	printf("DEBUG trace 0x%x\n", addr);
-      }
-#endif
-      registers[addr] = reg;
-      registers[addr]->address = addr;
-      registers[addr]->alias_mask = 0;
+      else
+          registers[addr] = reg;
+
+      reg->address = addr;
+      reg->alias_mask = 0;
 
       if(new_name)
-        registers[addr]->new_name(new_name);
+        reg->new_name(new_name);
 
       RegisterValue rv = getWriteTT(addr);
-      registers[addr]->set_write_trace(rv);
+      reg->set_write_trace(rv);
       rv = getReadTT(addr);
-      registers[addr]->set_read_trace(rv);
+      reg->set_read_trace(rv);
     }
 
   reg->value       = por_value;
@@ -1279,7 +1195,6 @@ void pic_processor::delete_sfr_register(Register *pReg)
 {
 
   if (pReg) {
-
     unsigned int a = pReg->getAddress();
 
     if (0)
@@ -1289,10 +1204,7 @@ void pic_processor::delete_sfr_register(Register *pReg)
       delete_file_registers(a,a);
     else
       delete pReg;
-
-    pReg = 0;
   }
-
 }
 
 //-------------------------------------------------------------------
@@ -1376,6 +1288,7 @@ bool pic_processor::set_config_word(unsigned int address,unsigned int cfg_word)
 
   int i = get_config_index(address);
 
+
   if ( i >= 0)
   {
       m_configMemory->getConfigWord(i)->set((int)cfg_word);
@@ -1405,7 +1318,7 @@ unsigned int pic_processor::get_config_word(unsigned int address)
 
 int pic_processor::get_config_index(unsigned int address)
 {
-  
+
   if (m_configMemory)
   {
 
@@ -1420,7 +1333,7 @@ int pic_processor::get_config_index(unsigned int address)
 	}
      }
   }
- 
+
   return -1;
 }
 
@@ -1838,6 +1751,7 @@ void pic_processor::createMCLRPin(int pkgPinNumber)
   if(package) {
     m_MCLR = new IO_open_collector("MCLR");
     package->assign_pin(pkgPinNumber,m_MCLR);
+    addSymbol(m_MCLR);
 
     m_MCLRMonitor = new MCLRPinMonitor(this);
     m_MCLR->setMonitor(m_MCLRMonitor);
@@ -1858,6 +1772,7 @@ void pic_processor::assignMCLRPin(int pkgPinNumber)
     {
 	m_MCLR_pin = pkgPinNumber;
     	m_MCLR = new IO_open_collector("MCLR");
+	addSymbol(m_MCLR);
 	m_MCLR_Save = package->get_pin(pkgPinNumber);
     	package->assign_pin(pkgPinNumber,m_MCLR, false);
 
@@ -1868,8 +1783,8 @@ void pic_processor::assignMCLRPin(int pkgPinNumber)
     else if (m_MCLR != package->get_pin(pkgPinNumber))
     {
 
-        cout << "BUG?: assigning multiple MCLR pins: " 
-	<< dec << pkgPinNumber << " " << __FILE__ <<  " " 
+        cout << "BUG?: assigning multiple MCLR pins: "
+	<< dec << pkgPinNumber << " " << __FILE__ <<  " "
 	<< __LINE__ << endl;
     }
   }
@@ -1878,16 +1793,20 @@ void pic_processor::assignMCLRPin(int pkgPinNumber)
 // This function sets the pin currently set as MCLR back to its original function
 void pic_processor::unassignMCLRPin()
 {
-  
+
     if (package && m_MCLR_Save)
     {
-
+        size_t l = m_MCLR_Save->name().find_first_of('.');
 	package->assign_pin(m_MCLR_pin,m_MCLR_Save, false);
-        m_MCLR_Save->newGUIname(m_MCLR_Save->name().c_str());
+
+	if (l == string::npos)
+            m_MCLR_Save->newGUIname(m_MCLR_Save->name().c_str());
+	else
+            m_MCLR_Save->newGUIname(m_MCLR_Save->name().substr(l+1).c_str());
 	if (m_MCLR)
 	{
 	    m_MCLR->setMonitor(0);
-	    delete m_MCLR;
+	    deleteSymbol(m_MCLR);
 	    m_MCLR = NULL;
 	    if (m_MCLRMonitor)
 	    {
@@ -1914,8 +1833,8 @@ private:
 // This function sets a label on a pin and if PinMod is defined
 // removes its control from it's port register
 //
-void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number, 
-		PinModule *PinMod, 
+void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
+		PinModule *PinMod,
 		const char * name,
 		bool in,
                 PicPortRegister *m_port,
@@ -1951,7 +1870,7 @@ void pic_processor::set_clk_pin(unsigned int pkg_Pin_Number,
   }
 }
 // This function reverses the effects of the previous function
-void pic_processor::clr_clk_pin(unsigned int pkg_Pin_Number, 
+void pic_processor::clr_clk_pin(unsigned int pkg_Pin_Number,
 		PinModule *PinMod,
 		PicPortRegister *m_port,
 		PicTrisRegister *m_tris,
@@ -1981,7 +1900,7 @@ void pic_processor::osc_mode(unsigned int value)
 {
   IOPIN *m_pin;
   unsigned int pin_Number =  get_osc_pin_Number(0);
-  
+
   if (pin_Number < 253)
   {
 	m_pin = package->get_pin(pin_Number);
@@ -2004,7 +1923,7 @@ void pic_processor::osc_mode(unsigned int value)
 	    clr_clk_pin(pin_Number, m_osc_Monitor[1]);
 	}
   }
-  
+
 }
 void pic_processor::Wput(unsigned int value)
 {
