@@ -46,6 +46,7 @@ public:
       INTCON *, PIE *, int _valid_bits);
   // The PIR base class supports no PIR bits directly
   virtual void clear_sspif(){}
+  virtual void clear_sppif(){}
   virtual void clear_rcif(){}
   virtual void clear_txif(){}
   virtual void set_adif(){}
@@ -56,6 +57,7 @@ public:
   virtual void set_c2if(){fprintf(stderr, "RRR set_c2if FIX\n");}
   virtual void set_c3if(){fprintf(stderr, "RRR set_c3if FIX\n");}
   virtual void set_c4if(){fprintf(stderr, "RRR set_c4if FIX\n");}
+  virtual void set_c5if(){fprintf(stderr, "RRR set_c5if FIX\n");}
   virtual void set_eccp1if(){}
   virtual void set_eeif(){}
   virtual void set_errif(){}
@@ -66,10 +68,11 @@ public:
   virtual void set_rxb0if(){}
   virtual void set_rxb1if(){}
   virtual void set_sspif(){}
+  virtual void set_sppif(){fprintf(stderr, "set_sppif FIX\n");}
   virtual void set_tmr1if(){}
   virtual void set_tmr1gif(){}
   virtual void set_tmr2if(){}
-  virtual void set_tmr3if(){}
+  virtual void set_tmr3if(){ fprintf(stderr, "set_tmr3if FIX\n");}
   virtual void set_txb0if(){}
   virtual void set_txb1if(){}
   virtual void set_txb2if(){}
@@ -80,6 +83,7 @@ public:
   virtual unsigned int get_txif() { return 0;}
   virtual unsigned int get_rcif() { return 0;}
   virtual unsigned int get_sspif() { return 0;}
+  virtual unsigned int get_sppif() { return 0;}
 
   /// A generic method to set an interrupt bit by mask
   virtual void set(int mask)
@@ -100,14 +104,23 @@ public:
   void set_ipr(sfr_register *);
 
 };
-//---------------------------------------------------------
+/*---------------------------------------------------------
 // InterruptSource
 
+  this is an alternative to the set_xxif method which
+  a> is becoming unwieldly
+  b> removes hard coding to allow multiple instantiation of classes
+     with different interrupts
+
+*/
+
 class InterruptSource
 {
 public:
   InterruptSource(PIR *_pir, unsigned int bitMask);
   void Trigger();
+  void Clear();
+  unsigned int Get() { return m_pir->value.get() & m_bitMask;} 
   void release(); // called when source is no longer needed.
 private:
   PIR *m_pir;
@@ -205,7 +218,8 @@ public:
     TXIF    = 1<<4,
     RCIF    = 1<<5,
     ADIF    = 1<<6,     // 18cxxx
-    PSPIF   = 1<<7
+    PSPIF   = 1<<7,
+    SPPIF   = 1<<7
   };
  
   virtual void set_tmr1if()
@@ -239,8 +253,15 @@ public:
     put(get() | ADIF);
   }
 
+  unsigned int get_sppif()
+  {
+    return value.get() & SPPIF;
+  }
+  virtual void set_sppif();
+
   virtual void set_pspif();
 
+
   virtual unsigned int get_txif()
   {
     return value.get() & TXIF;
@@ -293,6 +314,44 @@ public:
          INTCON *, PIE *);
 };
 
+//---------------------------------------------------------
+// PIR1 Peripheral Interrupt register # 1
+//
+// This is version 4 of the PIR1 register - as seen on the p16f91x devices
+
+class PIR1v4 : public PIR
+{
+public:
+
+  enum {
+    TMR1IF  = 1<<0,
+    TMR2IF  = 1<<1,
+    CCP1IF  = 1<<2,
+    SSPIF   = 1<<3,
+    TXIF    = 1<<4,
+    RCIF    = 1<<5,
+    ADIF    = 1<<6,
+    EEIF    = 1<<7
+  };
+ 
+  virtual void set_tmr1if(){ put(get() | TMR1IF); }
+  virtual void set_tmr2if(){ put(get() | TMR2IF); }
+  virtual void set_ccpif() { put(get() | CCP1IF); }
+  virtual void set_sspif() { put(get() | SSPIF); }
+  virtual void set_txif();
+  virtual void set_rcif();
+  virtual void clear_txif();
+  virtual void clear_rcif();
+  virtual unsigned int get_txif() { return value.get() & TXIF; }
+  virtual unsigned int get_rcif() { return value.get() & RCIF; }
+  virtual void set_adif() { put(get() | ADIF); }
+  virtual void set_eeif() { put(get() | EEIF); }
+
+ 
+  PIR1v4(Processor *pCpu, const char *pName, const char *pDesc,
+         INTCON *, PIE *);
+};
+
 
 
 //---------------------------------------------------------
@@ -334,9 +393,13 @@ enum
     ECCP1IF = 1<<0,             /* only on the PIC18F4xx devices */
     TMR3IF  = 1<<1,
     LVDIF   = 1<<2,
+    HLVDIF  = 1<<2,		/* 18f26k22 */ 
     BCLIF   = 1<<3,
     EEIF    = 1<<4,
-    CMIF    = 1<<6              /* PIC16F87xA, PIC18F4xx devices */
+    C2IF    = 1<<5,		/* 18f14k22 */
+    C1IF    = 1<<6,		/* 18f14k22 */
+    CMIF    = 1<<6,              /* PIC16F87xA, PIC18F4xx devices */
+    OSCFIF  = 1<<7		/* 18f14k22 */
 };
 
   virtual void set_eccp1if()
@@ -450,9 +513,38 @@ enum
          INTCON *, PIE *);
 };
 
+//---------------------------------------------------------
+// PIR2 Peripheral Interrupt register 
+//
+// This is version 5 of the PIR2 register - as seen on the p16f91x devices
+
+class PIR2v5 : public PIR
+{
+public:
+
+  enum {
+    
+    CCP2IF  = 1<<0,
+    LVDIF   = 1<<2,
+    LCDIF   = 1<<4,
+    C1IF    = 1<<5,
+    C2IF    = 1<<6,
+    OSFIF   = 1<<7
+  };
+ 
+  virtual void set_ccp2if() { put(get() | CCP2IF); }
+  virtual void set_lvdif() { put(get() | LVDIF); }
+  virtual void set_lcdif() { put(get() | LCDIF); }
+  virtual void set_c1if() { put(get() | C1IF); }
+  virtual void set_c2if() { put(get() | C2IF); }
+  virtual void set_osfif() { put(get() | OSFIF); }
+
+  PIR2v5(Processor *pCpu, const char *pName, const char *pDesc,
+         INTCON *, PIE *);
+};
 
 //---------------------------------------------------------
-// PIR2 Peripheral Interrupt register # 3
+// PIR3 Peripheral Interrupt register # 3
 //
 // This is version 1 of the PIR3 register - as seen on the 18F6520 devices
 
@@ -498,7 +590,7 @@ enum
 };
 
 //---------------------------------------------------------
-// PIR2 Peripheral Interrupt register # 3
+// PIR3 Peripheral Interrupt register # 3
 //
 // This is version 2 of the PIR3 register - as seen on the 18F248 devices
 // Perhaps other devices too - it contains bits for the CAN device
@@ -563,7 +655,82 @@ enum
          INTCON *, PIE *);
 };
 
+class PIR3v3 : public PIR
+{
+public:
 
+enum
+{
+    TMR1GIF  = 1<<0,
+    TMR3GIF  = 1<<1,
+    TMR5GIF  = 1<<2,
+    CTMUIF   = 1<<3,
+    TX2IF    = 1<<4,
+    RC2IF    = 1<<5,
+    BCL2IF   = 1<<6,
+    SSP2IF   = 1<<7
+};
+  PIR3v3(Processor *pCpu, const char *pName, const char *pDesc,
+         INTCON *, PIE *);
+};
+
+
+/*---------------------------------------------------------
+ PIR2 Peripheral Interrupt register # 2
+
+ This is version 1 of the PIR4 register - as seen on the 18f26k22
+*/
+
+class PIR4v1: public PIR
+{
+public:
+
+enum
+{
+    CCP3IF  = 1<<0,
+    CCP4IF  = 1<<1,
+    CCP5IF  = 1<<2,
+};
+  virtual void set_ccp3if()
+    {
+      put(get() | CCP3IF);
+    }
+  virtual void set_ccp4if()
+    {
+      put(get() | CCP4IF);
+    }
+  virtual void set_ccp5if()
+    {
+      put(get() | CCP5IF);
+    }
+  PIR4v1(Processor *pCpu, const char *pName, const char *pDesc,
+         INTCON *, PIE *);
+};
+class PIR5v1: public PIR
+{
+public:
+
+enum
+{
+    TMR4IF  = 1<<0,
+    TMR5IF  = 1<<1,
+    TMR6IF  = 1<<2,
+};
+  virtual void set_tmr4if()
+    {
+      put(get() | TMR4IF);
+    }
+  virtual void set_tmr5if()
+    {
+      put(get() | TMR5IF);
+    }
+  virtual void set_tmr6if()
+    {
+      put(get() | TMR6IF);
+    }
+  PIR5v1(Processor *pCpu, const char *pName, const char *pDesc,
+         INTCON *, PIE *);
+};
 /*
  * PIR_SET defines an interface to some common interrupt capabilities.
  * PIR_SET is a pure virtual class - you must instantiate a more specific
@@ -611,6 +778,7 @@ public:
   virtual void set_bclif() {}
  
 
+  virtual void set_sppif() {}
   virtual void set_pspif() {}
   virtual void set_cmif() {}
   virtual void set_c1if() {}
@@ -638,7 +806,7 @@ public:
 class PIR_SET_1 : public PIR_SET
 {
  public:
-  PIR_SET_1() { pir1 = 0; pir2 = 0; }
+  PIR_SET_1() { pir1 = 0; pir2 = 0;}
 
   virtual ~PIR_SET_1()
   {
@@ -696,6 +864,7 @@ class PIR_SET_1 : public PIR_SET
     pir1->clear_sspif();
   }
 
+
   // eeprom stuff
   virtual void set_eeif() {
     assert(pir1 != 0);
@@ -753,12 +922,12 @@ private:
 };
 
 
-// Supports 1, 2 or 3 version 2 Pir registers
+// Supports 1, 2 ,3, 4 or 5 Pir registers
 
 class PIR_SET_2 : public PIR_SET
 {
  public:
-  PIR_SET_2() { pir1 = 0; pir2 = 0; pir3 = 0; }
+  PIR_SET_2() { pir1 = 0; pir2 = 0; pir3 = 0; pir4 = 0; pir5 = 0;}
 
   virtual ~PIR_SET_2()
   {
@@ -767,6 +936,8 @@ class PIR_SET_2 : public PIR_SET
   void set_pir1(PIR *p1) { pir1 = p1; }
   void set_pir2(PIR *p2) { pir2 = p2; }
   void set_pir3(PIR *p3) { pir3 = p3; }
+  void set_pir4(PIR *p4) { pir4 = p4; }
+  void set_pir5(PIR *p5) { pir5 = p5; }
 
   virtual int interrupt_status() {
     assert(pir1 != 0);
@@ -775,6 +946,10 @@ class PIR_SET_2 : public PIR_SET
         result |= pir2->interrupt_status();
     if ( pir3 != 0 )
         result |= pir3->interrupt_status();
+    if ( pir4 != 0 )
+        result |= pir4->interrupt_status();
+    if ( pir5 != 0 )
+        result |= pir5->interrupt_status();
     return result; 
   }
 
@@ -818,6 +993,19 @@ class PIR_SET_2 : public PIR_SET
     pir1->clear_sspif();
   }
 
+  // spp stuff 
+  virtual bool get_sppif() {
+    assert(pir1 != 0);
+    return (pir1->get_sppif() != 0);
+  }
+  virtual void set_sppif() {
+    assert(pir1 != 0);
+    pir1->set_sppif();
+  }
+  virtual void clear_sppif() {
+    assert(pir1 != 0);
+    pir1->clear_sppif();
+  }
 
   // eeprom stuff
   virtual void set_eeif() {
@@ -891,6 +1079,8 @@ private:
   PIR   *pir1;
   PIR   *pir2;
   PIR   *pir3;
+  PIR   *pir4;
+  PIR   *pir5;
 };
 
 
