@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -29,13 +29,12 @@ License along with this library; if not, see
 #include <stdio.h>
 #ifdef _WIN32
 #include "uxtime.h"
-#else
-#include <sys/time.h>
 #endif
-#include <time.h>
+
 #include <iostream>
 #include <iomanip>
 #include <string>
+#include <sstream>
 #include <cstdio>
 
 #include "../config.h"
@@ -84,20 +83,43 @@ public:
   CPU_Freq(Processor * _cpu, double freq); //const char *_name, double newValue, const char *desc);
 
   virtual void set(double d);
+  void set_rc_freq(double d);
+  virtual void get(double &);
+  void set_rc_active(bool _use_rc_freq) { use_rc_freq = _use_rc_freq;}
+  
 
 private:
   Processor * cpu;
+  double 	RCfreq;
+  bool		use_rc_freq;
 };
 
 CPU_Freq::CPU_Freq(Processor * _cpu, double freq)
   : Float("frequency",freq, " oscillator frequency."),
-    cpu(_cpu)
+    cpu(_cpu), use_rc_freq(false)
 {
 }
 
+void CPU_Freq::set_rc_freq(double d)
+{
+   RCfreq = d;
+}
+void CPU_Freq::get(double &d)
+{
+    if (use_rc_freq)
+	d = RCfreq;
+    else
+    {
+        double x;
+        Float::get(x);
+        d = x;
+    }
+}
+       
 void CPU_Freq::set(double d)
 {
   pic_processor *pCpu = dynamic_cast<pic_processor *>(cpu);
+
   Float::set ( d );
   if ( cpu )
     cpu->update_cps();
@@ -136,6 +158,7 @@ Processor::Processor(const char *_name, const char *_desc)
 
   m_pConstructorObject = 0;
   m_Capabilities = 0;
+  m_ProgramMemoryAllocationSize = 0;
   if(verbose)
     cout << "processor constructor\n";
 
@@ -180,7 +203,6 @@ Processor::Processor(const char *_name, const char *_desc)
 }
 
 
-static unsigned int  m_ProgramMemoryAllocationSize = 0;
 
 //-------------------------------------------------------------------
 Processor::~Processor()
@@ -194,6 +216,7 @@ Processor::~Processor()
   deleteSymbol(m_pUnknownMode);
   deleteSymbol(m_pBreakOnReset);
   deleteSymbol(mFrequency);
+  deleteSymbol(m_vdd);
 
   delete interface;
 
@@ -206,9 +229,14 @@ Processor::~Processor()
 
   destroyProgramMemoryAccess(pma);
 
+
   for (unsigned int i = 0; i < m_ProgramMemoryAllocationSize; i++)
+  {
     if (program_memory[i] != &bad_instruction)
+    {
       delete program_memory[i];
+    }
+  }
 
   delete []program_memory;
 
@@ -240,19 +268,31 @@ void Processor::setBreakOnReset(bool newBreakOnReset)
 //------------------------------------------------------------------------
 // Attributes
 
+
+void Processor::set_RCfreq_active(bool state)
+{
+    if (mFrequency)
+        mFrequency->set_rc_active(state);
+    update_cps();
+}
 void Processor::set_frequency(double f)
 {
   if(mFrequency)
     mFrequency->set(f);
   update_cps();
 }
+void Processor::set_frequency_rc(double f)
+{
+  if(mFrequency)
+    mFrequency->set_rc_freq(f);
+  update_cps();
+}
 double Processor::get_frequency()
 {
   double d=0.0;
 
   if(mFrequency)
     mFrequency->get(d);
-
   return d;
 }
 
@@ -375,17 +415,17 @@ void Processor::delete_invalid_registers ()
 
   for (i = 0; i < rma.get_size(); i++) {
    // cout << __FUNCTION__ << "  reg: 0x"<<hex << i << " ptr:" << registers[i] << endl;
-	
+
     InvalidRegister *pReg = dynamic_cast<InvalidRegister *> (registers[i]);
     if (pReg) {
       delete registers[i];
       registers[i]= 0;
-      
+
     }
     else if (registers[i])
    {
       char reg_name[11];
-      cout << __FUNCTION__ << "  reg: 0x"<<hex << i << " ptr:" << 
+      cout << __FUNCTION__ << "  reg: 0x"<<hex << i << " ptr:" <<
 	registers[i] ;
       cout.flush();
       strncpy(reg_name, registers[i]->name().c_str(), 10);
@@ -423,8 +463,8 @@ void Processor::add_file_registers(unsigned int start_address, unsigned int end_
     if (registers[j] && (registers[j]->isa() == Register::INVALID_REGISTER))
 	delete registers[j];
     else if (registers[j])
-	cout << __FUNCTION__ << " Already register " << registers[j]->name() 
-		<< " at 0x" << hex << j <<endl; 
+	cout << __FUNCTION__ << " Already register " << registers[j]->name()
+		<< " at 0x" << hex << j <<endl;
 
     //The default register name is simply its address
     snprintf (str, sizeof(str), "REG%03X", j);
@@ -555,7 +595,7 @@ void Processor::alias_file_registers(unsigned int start_address, unsigned int en
 //
 // The purpose of this member function is to allocate memory for the
 // pic's code space. The 'memory_size' parameter tells how much memory
-// is to be allocated 
+// is to be allocated
 //
 //  The following is not correct for 18f2455 and 18f4455 processors
 //  so test has been disabled (RRR)
@@ -650,7 +690,7 @@ void Processor::init_program_memory(unsigned int address, unsigned int value)
 //erase_program_memory(unsigned int address)
 //
 //	Checks if a program memory location contains an instruction
-//	and deletes it if it does. 
+//	and deletes it if it does.
 //
 void Processor::erase_program_memory(unsigned int address)
 {
@@ -674,7 +714,7 @@ void Processor::erase_program_memory(unsigned int address)
     cout << "Erase Program memory\n";
     cout << "Warning::Out of range address " << hex << address << endl;
     cout << "Max allowed address is 0x" << hex << (program_address_limit()-1) << '\n';
-  
+
   }
 
 
@@ -823,8 +863,8 @@ void Processor::read_src_files(void)
 
     while(fc->gets(buf,sizeof(buf))) {
 
-      int address;
-      int opcode;
+      unsigned int address;
+      unsigned int opcode;
 
       if (sscanf(buf,"%x   %x",&address, &opcode) == 2) {
         unsigned int uIndex = map_pm_address2index(address);
@@ -1609,6 +1649,7 @@ public:
   virtual unsigned int GetLowerBound();
   virtual unsigned int GetUpperBound();
   virtual bool bIsIndexInRange(unsigned int uIndex);
+  virtual void get(char *return_str, int len);
 private:
   Processor *   m_pProcessor;
   ProgramMemoryAccess   *m_pPma;
@@ -1633,6 +1674,10 @@ ProgramMemoryCollection::~ProgramMemoryCollection()
     m_pProcessor->removeSymbol(this);
 }
 
+void ProgramMemoryCollection::get(char *return_str, int len)
+{
+    if (return_str) strncpy(return_str, "", len);
+}
 unsigned int ProgramMemoryCollection::GetSize()
 {
   return m_pProcessor->program_memory_size();
@@ -1661,24 +1706,22 @@ void ProgramMemoryCollection::SetAt(unsigned int uAddress, Value *pValue)
 }
 
 void ProgramMemoryCollection::ConsolidateValues(int &iColumnWidth,
-                                           vector<string> &aList,
-                                           vector<string> &aValue)
+                                           std::vector<string> &aList,
+                                           std::vector<string> &aValue)
 {
   unsigned int  uFirstAddress = 0;
   unsigned int  uAddress;
   //Register *    pReg = m_ppRegisters[0];
   //Integer       uLastValue(pReg->getRV_notrace().data);
   Integer uLastValue(m_pPma->get_opcode(0));
-  uLastValue.setBitmask((1<<(m_pProcessor->opcode_size()*8)) - 1);
+  uLastValue.setBitmask((1 << (m_pProcessor->opcode_size() * 8)) - 1);
 
   unsigned int uSize = GetSize();
 
-  for(uAddress = 0; uAddress < uSize; uAddress++) {
-    ostringstream sIndex;
-
+  for (uAddress = 0; uAddress < uSize; uAddress++) {
     unsigned int ui_opcode = m_pPma->get_opcode(uAddress);
 
-    if((unsigned int)uLastValue != ui_opcode) {
+    if ((unsigned int)uLastValue != ui_opcode) {
       PushValue(uFirstAddress, uAddress,
                 &uLastValue, aList, aValue);
       iColumnWidth = max(iColumnWidth, (int)aList.back().size());
@@ -1688,7 +1731,7 @@ void ProgramMemoryCollection::ConsolidateValues(int &iColumnWidth,
   }
   uAddress--;
   // Record the last set of elements
-  if(uFirstAddress <= uAddress) {
+  if (uFirstAddress <= uAddress) {
     PushValue(uFirstAddress, uAddress,
               &uLastValue, aList, aValue);
     iColumnWidth = max(iColumnWidth, (int)aList.back().size());
@@ -1997,7 +2040,6 @@ void ProgramMemoryAccess::put_opcode(unsigned int addr, unsigned int new_opcode)
                                old_inst->get_hll_src_line(),
                                old_inst->get_hll_file_id());
 
-  //new_inst->xref = old_inst->xref;
 
   if(b)
     b->setReplaced(new_inst);
@@ -2006,7 +2048,6 @@ void ProgramMemoryAccess::put_opcode(unsigned int addr, unsigned int new_opcode)
 
   cpu->program_memory[uIndex]->setModified(true);
 
-  //if(cpu->program_memory[addr]->xref)
   cpu->program_memory[uIndex]->update();
 
   delete(old_inst);
@@ -2014,12 +2055,17 @@ void ProgramMemoryAccess::put_opcode(unsigned int addr, unsigned int new_opcode)
 
 //--------------------------------------------------------------------------
 
-void  ProgramMemoryAccess::assign_xref(unsigned int address, gpointer xref)
+void  ProgramMemoryAccess::assign_xref(unsigned int address, XrefObject * xref)
 {
 
   instruction &q = *getFromAddress(address);
   if(q.isa()==instruction::INVALID_INSTRUCTION)
+  {
+    delete (int *)xref->data;
+    delete xref;
     return;
+   }
+
 
   q.add_xref(xref);
 }
@@ -2382,7 +2428,7 @@ string ProcessorConstructorList::DisplayString(void)
 
   for (processor_iterator = pl->begin();
        processor_iterator != pl->end();
-       processor_iterator++) {
+       ++processor_iterator) {
 
     p = *processor_iterator;
 
@@ -2423,8 +2469,8 @@ string ProcessorConstructorList::DisplayString(void)
 //------------------------------------------------------------------------
 
 FileContext::FileContext(string &new_name)
+  : name_str(new_name)
 {
-  name_str = new_name;
   fptr = NULL;
   m_uiMaxLine = 0;
   m_bIsList = false;
@@ -2432,8 +2478,8 @@ FileContext::FileContext(string &new_name)
 }
 
 FileContext::FileContext(const char *new_name)
+  : name_str(new_name)
 {
-  name_str = string(new_name);
   fptr = NULL;
   m_uiMaxLine = 0;
   m_bIsList = false;
@@ -2584,7 +2630,7 @@ FileContextList::~FileContextList(void)
 {
   FileContextList::iterator it;
   FileContextList::iterator itEnd = end();
-  for (it = begin(); it != itEnd; it++)
+  for (it = begin(); it != itEnd; ++it)
     it->close();
 }
 
@@ -2679,9 +2725,10 @@ extern void EnsureTrailingFolderDelimiter(string &sPath);
 extern void SplitPathAndFile(string &sSource, string &sFolder, string &sFile);
 
 //----------------------------------------
-void FileContextList::SetSourcePath(const char *pPath) {
-  string sPath(pPath);
-  string sFolder, sFile;
+void FileContextList::SetSourcePath(const char *pPath)
+{
+  std::string sPath(pPath);
+  std::string sFile;
   SplitPathAndFile(sPath, sSourcePath, sFile);
   EnsureTrailingFolderDelimiter(sSourcePath);
 }
