@@ -215,10 +215,12 @@ int CSimulationContext::LoadProgram(const char *filename,
     char cw[_MAX_PATH];
 
     perror((string("failed to open program file ") + filename).c_str());
-    getcwd(cw, sizeof(cw));
-    cerr << "current working directory is ";
-    cerr << cw;
-    cerr << endl;
+    if (!getcwd(cw, sizeof(cw)))
+        perror("getcwd failed: ");
+    else
+    {
+        cerr << "current working directory is " << cw << endl;
+    }
     return false;
   }
   if(pProcessorType != NULL) {
@@ -267,7 +269,7 @@ void CSimulationContext::dump_processor_list(void)
   CProcessorList::iterator processor_iterator; 
   for (processor_iterator = processor_list.begin();
        processor_iterator != processor_list.end(); 
-       processor_iterator++) {
+       ++processor_iterator) {
       CProcessorList::value_type vt = *processor_iterator;
       Processor *p = vt.second;
       cout << p->name() << '\n';
@@ -356,7 +358,7 @@ CSimulationContext::CProcessorList::findByType(const key_type& _Keyval) {
   // we found above.
   iterator it;
   iterator itEnd = end();
-  for(it = begin(); it != itEnd; it++) {
+  for(it = begin(); it != itEnd; ++it) {
     if(it->second->m_pConstructorObject == pc) {
       return it;
     }
