@@ -15,7 +15,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -33,10 +33,22 @@ License along with this library; if not, see
 
 //#define DEBUG
 #if defined(DEBUG)
-#define Dprintf(arg) {printf("%s:%d-%s() ",__FILE__,__LINE__,__FUNCTION__); printf arg; }
+#define Dprintf(arg) {printf("%s:%d-%s() %s ",__FILE__,__LINE__,__FUNCTION__,cpu->name().c_str()); printf arg; }
 #else
 #define Dprintf(arg) {}
 #endif
+//#define I2C_PROTO
+#if defined(I2C_PROTO)
+#define I2Cproto(arg) {printf("I2C %s %d ",cpu->name().c_str(), __LINE__); printf arg;}
+#else
+#define I2Cproto(arg) {}
+#endif
+//#define SPI_PROTO
+#if defined(SPI_PROTO)
+#define SPIproto(arg) {printf("SPI %s %d ", cpu->name().c_str(), __LINE__); printf arg;}
+#else
+#define SPIproto(arg) {}
+#endif
 
 //#warning only supports SPI mode.
 //-----------------------------------------------------------
@@ -95,7 +107,7 @@ public:
 
   virtual void toggle()
   {
-    switch (m_cState) 
+    switch (m_cState)
     {
     case '1':
     case 'W':
@@ -316,7 +328,7 @@ void _SSPCON::setWCOL()
 	return true if a I2C mode is enabled in SSPCON
 */
 
-bool _SSPCON::isI2CActive(unsigned int val) 
+bool _SSPCON::isI2CActive(unsigned int val)
 {
     if ( (val & SSPEN) != SSPEN)
 	return(false);
@@ -333,7 +345,7 @@ bool _SSPCON::isI2CActive(unsigned int val)
     }
     return(false);
 }
-bool _SSPCON::isI2CMaster(unsigned int val) 
+bool _SSPCON::isI2CMaster(unsigned int val)
 {
     if ( (val & SSPEN) != SSPEN)
 	return(false);
@@ -349,7 +361,7 @@ bool _SSPCON::isI2CMaster(unsigned int val)
 /*
 	return true if an I2C slave mode is enabled in SSPCON
 */
-bool _SSPCON::isI2CSlave(unsigned int val) 
+bool _SSPCON::isI2CSlave(unsigned int val)
 {
     if ( (val & SSPEN) != SSPEN)
 	return(false);
@@ -410,13 +422,27 @@ unsigned int _SSPBUF::get()
 
   return value.get();
 }
-  
+
 unsigned int _SSPBUF::get_value()
 {
   return value.get();
 }
 
 
+//-----------------------------------------------------------
+// SSPMSK - Synchronous Serial Port Address mask(for I2C)
+//-----------------------------------------------------------
+_SSPMSK::_SSPMSK(Processor *pCpu, const char *_name)
+  : sfr_register(pCpu, _name,"Synchronous I2C Address mask")
+{
+   put_value(0xff);
+}
+void _SSPMSK::put(unsigned int new_value)
+{
+  // trace cannot handle 2 registers at same address RRR
+  trace.raw(write_trace.get() | value.get());
+  put_value(new_value);
+}
 //-----------------------------------------------------------
 // SSPADD - Synchronous Serial Port Address (for I2C)
 //-----------------------------------------------------------
@@ -426,27 +452,64 @@ _SSPADD::_SSPADD(Processor *pCpu, SSP_MODULE *pSSP)
 {
 }
 
+/*
+   On some processors SSPMSK is accessed through SSPADD
+   lt is assummed that SSPM_LoadMaskFunction bits of SSPCON
+   are reserved on those processors which have seperate addresses
+   for SSPMSK and SSPADD
+
+*/
 void _SSPADD::put(unsigned int new_value)
 {
+
+  if (m_sspmod && m_sspmod->sspmsk &&
+	((m_sspmod->sspcon.value.get() & _SSPCON::SSPM_mask) ==
+		_SSPCON::SSPM_LoadMaskFunction))
+  {
+	m_sspmod->sspmsk->put(new_value);
+	return;
+  }
   trace.raw(write_trace.get() | value.get());
   put_value(new_value);
 
   if( m_sspmod )
+  {
+     if (m_sspmod->sspmsk)
+     {
+	m_sspmod->newSSPADD(m_sspmod->sspmsk->value.get() &new_value);
+     }
+     else
 	m_sspmod->newSSPADD(new_value);
-
+  }
 }
 
 void _SSPADD::put_value(unsigned int new_value)
 {
   value.put(new_value & 0xff);
 }
+unsigned int  _SSPADD::get()
+{
+    unsigned int val = value.get();
+    if (m_sspmod->sspmsk)
+    {
+        unsigned int con_val = m_sspmod->sspcon.value.get() & _SSPCON::SSPM_mask;
+	if (con_val == _SSPCON::SSPM_LoadMaskFunction)
+	    return m_sspmod->sspmsk->value.get();
+    }
+
+    return val;
+}
+
+
+
 SPI::SPI(SSP_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat, _SSPBUF *_sspbuf)
+  : m_state(eIDLE)
 {
     m_sspmod = _ssp_mod;
     m_sspcon = _sspcon;
     m_sspstat = _sspstat;
     m_sspbuf = _sspbuf;
-    m_state = eIDLE;
+    cpu = m_sspmod->cpu;
 }
 
 void SPI::clock( bool ClockState )
@@ -468,7 +531,7 @@ void SPI::clock( bool ClockState )
       onbeat = true;
     else
       onbeat = false;
-  } 
+  }
   else // falling edge
   {
     if( ( !(sspcon_val & _SSPCON::CKP) && !(sspstat_val & _SSPSTAT::CKE) )
@@ -479,29 +542,31 @@ void SPI::clock( bool ClockState )
   }
 
   if( m_state == eIDLE ){
-    if( sspstat_val & _SSPSTAT::CKE ) 
+    SPIproto(("Idle clock %d CKE %d CKP %d onbeat %d\n", ClockState, (sspstat_val & _SSPSTAT::CKE), (sspcon_val & _SSPCON::CKP), onbeat));
+    if( sspstat_val & _SSPSTAT::CKE )
     {
       // FIX: I have NOT verified that PICs actually behave like this.
       cout << "SSP: I can't handle a non-started transfer with CKE = 1." << endl;
       return;
-    } 
-    else if( onbeat ) 
+    }
+    else if( onbeat )
     {
       // FIX: I have NOT verified that PICs actually behave like this.
-      cout << "SSP: Ignoring clock transition to neutral in state IDLE." << endl;
+      cout << "SSP: " << cpu->name() << " Ignoring clock transition to neutral in state IDLE." << endl;
       return;
     }
-    else 
+    else
     {
         // RP: This is only relevant in slave mode? I think clock is never called
         // while idle in master mode.
       if (verbose)
       	cout << "SPI clock called start_transfer\n";
+      SPIproto(("Clock called start transfer\n"));
       start_transfer();
     }
 
   }
- 
+
   if (!m_sspmod)
     return;
 
@@ -512,7 +577,8 @@ void SPI::clock( bool ClockState )
       if (m_sspmod->get_SDI_State())
 	m_SSPsr |= 1;
       if (verbose)
-      	cout << "SSP: SPI Received bit = " << (m_SSPsr & 1) << ". (SMP=0)" << endl;
+      	cout << "SSP: SPI Received bit = " << (m_SSPsr & 1) << ". onbeat(SMP=0)" << endl;
+      SPIproto(("In Bit %u byte count=%d onbeat m_SSPsr=0x%02x\n", (m_SSPsr & 1), bits_transfered+1, m_SSPsr));
     }
   } else {
     // off beat: data is shifted out, data is read in if SMP = 1
@@ -522,25 +588,27 @@ void SPI::clock( bool ClockState )
       if (m_sspmod->get_SDI_State())
 	m_SSPsr |= 1;
       if (verbose)
-      	cout << "SSP: SPI Received bit = " << (m_SSPsr & 1) << ". (SMP=1)" << endl;
+      	cout << "SSP: SPI Received bit = " << (m_SSPsr & 1) << ". offbeat(SMP=1)" << endl;
+      SPIproto(("In Bit %u byte count=%d offbeat\n", (m_SSPsr & 1), bits_transfered+1));
     }
-	
+
     char nextSDO = (m_SSPsr&(1<<7)) ? '1' : '0';
     m_sspmod->putStateSDO(nextSDO);
     if (verbose)
       cout << "SSP: SPI Sent bit = " << nextSDO << "."  << endl;
+    SPIproto(("\tSent bit %c m_SSPsr 0x%x\n", nextSDO, m_SSPsr));
   }
 
   bool bSSPCONValue = (sspcon_val & _SSPCON::CKP) ? true : false;
   if(bSSPCONValue == ClockState) {
     bits_transfered++;
-    if( bits_transfered == 8 ) 
+    if( bits_transfered == 8 )
     {
-      if( (sspstat_val & _SSPSTAT::SMP) && !(sspstat_val & _SSPSTAT::CKE) ) 
+      if( (sspstat_val & _SSPSTAT::SMP) && !(sspstat_val & _SSPSTAT::CKE) )
       {
 	m_state = eWAITING_FOR_LAST_SMP;
-      } 
-      else 
+      }
+      else
       {
 	stop_transfer();
         allDone = true;
@@ -562,7 +630,7 @@ void SPI::set_halfclock_break()
 
   switch( sspcon_val & _SSPCON::SSPM_mask ) {
   // Simulation requires Fosc/4 to be run at Fosc/8
-  case _SSPCON::SSPM_SPImaster4: 
+  case _SSPCON::SSPM_SPImaster4:
     clock_in_cycles = 1;
     break;
   case _SSPCON::SSPM_SPImaster16:
@@ -574,7 +642,7 @@ void SPI::set_halfclock_break()
   case _SSPCON::SSPM_SPImasterTMR2:
     break;
   }
-  
+
   get_cycles().set_break(get_cycles().get() + clock_in_cycles, this);
 }
 void SPI::callback()
@@ -585,6 +653,8 @@ void SPI::callback()
   if (verbose)
     cout << "SPI callback m_state=" << m_state << endl;
 
+  SPIproto(("callback m_state=%d\n", m_state));
+
   switch( m_state ) {
   case eIDLE:
     break;
@@ -601,12 +671,12 @@ void SPI::callback()
       if (verbose)
       	cout << "SSP: Received bit = " << (m_SSPsr & 1) << ". (SMP=1)" << endl;
     }
-	
+
     m_state = eACTIVE;
     stop_transfer();
     break;
   }
-	  
+
 }
 //-----------------------------------------------------------
 
@@ -616,7 +686,7 @@ void SPI::startSPI()
     bits_transfered = 0;
 }
 
-SPI_1::SPI_1(SSP1_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat, 
+SPI_1::SPI_1(SSP1_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat,
 	_SSPBUF *_sspbuf, _SSP1CON3 *_ssp1con3, _SSPADD *_sspadd) :
 	SPI(_ssp_mod, _sspcon, _sspstat, _sspbuf)
 {
@@ -634,7 +704,7 @@ void SPI_1::set_halfclock_break()
 
   switch( sspcon_val & _SSPCON::SSPM_mask ) {
   // Simulation requires Fosc/4 to be run at Fosc/8
-  case _SSPCON::SSPM_SPImaster4: 
+  case _SSPCON::SSPM_SPImaster4:
     clock_in_cycles = 1;
     break;
   case _SSPCON::SSPM_SPImaster16:
@@ -645,7 +715,7 @@ void SPI_1::set_halfclock_break()
     break;
   case _SSPCON::SSPM_SPImasterAdd:
     // Note, this will be low by 1 cycle/clock when sspadd is even
-    clock_in_cycles = (m_sspadd->value.get() + 1)>>1;
+    clock_in_cycles = (m_sspadd->get() + 1)>>1;
     if (clock_in_cycles < 2)
     {
 	cout << "WARNING for SPI sspadd must be >= 3\n";
@@ -655,7 +725,7 @@ void SPI_1::set_halfclock_break()
   case _SSPCON::SSPM_SPImasterTMR2:
     break;
   }
-  
+
   get_cycles().set_break(get_cycles().get() + clock_in_cycles, this);
 }
 void SPI_1::stop_transfer()
@@ -697,7 +767,7 @@ void SPI_1::stop_transfer()
   }
 
   m_state = eIDLE;
-  
+
 }
 
 SSP_MODULE::SSP_MODULE(Processor *pCpu)
@@ -706,6 +776,8 @@ SSP_MODULE::SSP_MODULE(Processor *pCpu)
     sspstat(pCpu,this),
     sspcon2(pCpu,this),
     sspadd(pCpu,this),
+    sspmsk(0),
+    cpu(pCpu),
     m_ssp_if(0),
     m_bcl_if(0),
     m_pirset(0),
@@ -749,7 +821,7 @@ SSP_MODULE::~SSP_MODULE()
     if (m_sdo_active && m_sdo) m_sdo->setSource(0);
     if (m_SdoSource) delete m_SdoSource;
 
-    
+
     if (m_sck_active && m_sck)
 	m_sck->setSource(0);
     if (m_SckSource) delete m_SckSource;
@@ -762,32 +834,40 @@ SSP_MODULE::~SSP_MODULE()
     if (m_bcl_if) delete m_bcl_if;
 }
 
-SSP1_MODULE::SSP1_MODULE(Processor *pCpu) : 
+/*
+    SSP1_MODULE adds SSPCON3 and SSPMSK to SSP_MODULE
+*/
+SSP1_MODULE::SSP1_MODULE(Processor *pCpu) :
     SSP_MODULE(pCpu),
-    ssp1con3(pCpu, this),
-    ssp1msk(pCpu, this)
+    ssp1con3(pCpu, this)
 {
+    sspmsk = new _SSPMSK(pCpu, "ssp1msk");
 }
+SSP1_MODULE::~SSP1_MODULE()
+{
+   delete sspmsk;
+}
+
 void SSP1_MODULE::set_sckPin(PinModule *_sckPin)
 {
    if (m_sck == _sckPin) return;	// No change, do nothing
    m_sck = _sckPin;
-   if(m_SckSource) delete m_SckSource; 
+   if(m_SckSource) delete m_SckSource;
    m_SckSource = new SCK_SignalSource(this, m_sck);
 }
 void SSP1_MODULE::set_sdoPin(PinModule *_sdoPin)
 {
    if (m_sdo == _sdoPin) return;	// No change, do nothing
    m_sdo = _sdoPin;
- 
-   if(m_SdoSource) delete m_SdoSource; 
+
+   if(m_SdoSource) delete m_SdoSource;
    m_SdoSource = new SDO_SignalSource(this, m_sdo);
 }
 void SSP1_MODULE::set_sdiPin(PinModule *_sdiPin)
 {
    if (m_sdi == _sdiPin) return;	// No change, do nothing
    m_sdi = _sdiPin;
-   if(m_SdiSource) delete m_SdiSource; 
+   if(m_SdiSource) delete m_SdiSource;
    m_SdiSource = new SDI_SignalSource(this, m_sdi);
 }
 void SSP1_MODULE::set_ssPin(PinModule *_ssPin)
@@ -820,7 +900,7 @@ void SSP1_MODULE::initialize(
   if (! m_spi)
   {
     m_spi = new SPI_1(this, &sspcon, &sspstat, &sspbuf, &ssp1con3, &sspadd);
-    m_i2c = new I2C_1(this, &sspcon, &sspstat, &sspbuf, &sspcon2, &sspadd, &ssp1con3, &ssp1msk);
+    m_i2c = new I2C_1(this, &sspcon, &sspstat, &sspbuf, &sspcon2, &sspadd, &ssp1con3);
     m_SDI_Sink = new SDI_SignalSink(this);
     m_SCL_Sink = new SCL_SignalSink(this);
     m_SS_Sink = new SS_SignalSink(this);
@@ -832,14 +912,20 @@ void SPI::newSSPBUF(unsigned int newTxByte)
 {
   Dprintf(("enabled %d state %d\n", m_sspcon->isSSPEnabled(), m_state));
   if (m_sspcon->isSSPEnabled()) {
-    if (m_state == eIDLE) {
-	m_SSPsr = newTxByte;
+    if (m_state == eIDLE || bits_transfered == 0) {
+        m_SSPsr = newTxByte;
+	SPIproto(("newSSPBUF send 0x%02x\n", m_SSPsr));
 	start_transfer();
     } else {
       // Collision
+      SPIproto(("newSSPBUF 0x%02x collision\n", m_SSPsr));
       m_sspcon->setWCOL();
     }
-  } 
+  }
+  else
+  {
+	SPIproto(("newSSPBUF !SSPenabled m_SSPsr 0x%02x\n", m_SSPsr));
+  }
 }
 void SPI::start_transfer()
 {
@@ -865,7 +951,7 @@ void SPI::start_transfer()
     // Setup callbacks for clocks
     set_halfclock_break();
     break;
-  case _SSPCON::SSPM_SPImasterTMR2: 
+  case _SSPCON::SSPM_SPImasterTMR2:
       m_sspmod->putStateSDO((m_SSPsr &(1<<7)) ? '1' : '0');
 	break;
   case _SSPCON::SSPM_SPIslaveSS:
@@ -875,12 +961,15 @@ void SPI::start_transfer()
     break;
   case _SSPCON::SSPM_SPIslave:
     // I don't do any thing until first clock edge
+    SPIproto(("SSPM_SPIslave start_transfer 0x%02x\n", m_SSPsr));
+    if( sspstat_val & _SSPSTAT::CKE )
+      m_sspmod->putStateSDO((m_SSPsr &(1<<7)) ? '1' : '0');
     break;
   default:
     cout << "start_transfer: The selected SPI mode is unimplemented. mode=" << hex
 	<<(sspcon_val & _SSPCON::SSPM_mask) << endl;
   }
-  
+
 }
 void SPI::stop_transfer()
 {
@@ -890,15 +979,18 @@ void SPI::stop_transfer()
   if( m_state == eACTIVE ) {
     if( bits_transfered == 8 && !m_sspbuf->isFull() )
     {
+        m_SSPsr &= 0xff;
         if (verbose)
-            cout << "SPI: Stoping transfer. Normal finish. Setting sspif and BF\n";
+            cout << "SPI: Stoping transfer. Normal finish. Setting sspif and BF got=" << (m_SSPsr & 0xff) << endl;
+        SPIproto(("Stoping transfer. Normal finish. Setting sspif and BF got=0x%02x\n" , (m_SSPsr & 0xff)));
 	m_sspbuf->put_value(m_SSPsr & 0xff);
 	m_sspbuf->setFullFlag(true);
         m_sspmod->set_sspif();
 	m_sspstat->put_value(m_sspstat->value.get() | _SSPSTAT::BF);
     } else if( bits_transfered == 8 && m_sspbuf->isFull() ) {
-      if (verbose)
-          cout << "SPI: Stopping transfer. SSPBUF Overflow setting SSPOV." << endl;
+        if (verbose)
+            cout << "SPI: Stopping transfer. SSPBUF Overflow setting SSPOV." << endl;
+        SPIproto(("Stopping transfer. SSPBUF Overflow setting SSPOV.\n"));
         m_sspcon->setSSPOV();
         m_sspmod->set_sspif();      // The real PIC sets sspif even with overflow
     } else {
@@ -908,14 +1000,16 @@ void SPI::stop_transfer()
   } else {
     if (verbose)
       cout << "SSP: Stopping transfer. State != ACTIVE." << endl;
+    SPIproto(("Stopping transfer. State != ACTIVE."));
   }
 
   m_state = eIDLE;
-  
+
 }
 
-I2C::I2C(SSP_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat, 
-	_SSPBUF *_sspbuf, _SSPCON2 *_sspcon2, _SSPADD *_sspadd) 
+I2C::I2C(SSP_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat,
+	_SSPBUF *_sspbuf, _SSPCON2 *_sspcon2, _SSPADD *_sspadd)
+  : i2c_state(eIDLE)
 {
     m_sspmod = _ssp_mod;
     m_sspcon = _sspcon;
@@ -924,29 +1018,30 @@ I2C::I2C(SSP_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat,
     m_sspcon2 = _sspcon2;
     m_sspadd = _sspadd;
     future_cycle = 0;
-    i2c_state = eIDLE;
+
+    cpu = m_sspmod->cpu;
 }
 
-I2C_1::I2C_1(SSP_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat, 
-	_SSPBUF *_sspbuf, _SSPCON2 *_sspcon2, _SSPADD *_sspadd, 
-	_SSP1CON3 *_ssp1con3, _SSP1MSK *_ssp1msk) :
+I2C_1::I2C_1(SSP_MODULE *_ssp_mod, _SSPCON *_sspcon, _SSPSTAT *_sspstat,
+	_SSPBUF *_sspbuf, _SSPCON2 *_sspcon2, _SSPADD *_sspadd,
+	_SSP1CON3 *_ssp1con3) :
    	I2C(_ssp_mod, _sspcon, _sspstat, _sspbuf, _sspcon2, _sspadd)
 {
     m_sspmod = _ssp_mod;
     m_sspcon3 = _ssp1con3;
-    m_ssp1msk = _ssp1msk;
 }
 
 void I2C::set_idle()
 {
     i2c_state = eIDLE;
+    I2Cproto(("%s i2c_state = eIDLE\n", __FUNCTION__));
 }
 bool I2C::isIdle()
 {
-    return(i2c_state == eIDLE); //RRR
+    return(i2c_state == eIDLE);
     return(
 	(m_sspstat->value.get() & _SSPSTAT::RW) == 0 &&
-	(m_sspcon2->value.get() & 
+	(m_sspcon2->value.get() &
 	   (
 	     _SSPCON2::ACKEN ||
 	     _SSPCON2::RCEN ||
@@ -980,7 +1075,7 @@ void I2C::callback()
         cout << "I2C::callback i2c_state " << i2c_state << " phase=" << phase <<endl;
     if (future_cycle != get_cycles().get())
     {
-	cout << "I2C program error future_cycle=" << future_cycle << " now=" 
+	cout << "I2C program error future_cycle=" << future_cycle << " now="
 		<< get_cycles().get() << " i2c_state=" << i2c_state << endl;
     }
 
@@ -1013,7 +1108,7 @@ void I2C::callback()
 	}
 	else
 	{
-	    m_sspcon2->value.put(m_sspcon2->value.get() & 
+	    m_sspcon2->value.put(m_sspcon2->value.get() &
 		~(_SSPCON2::SEN | _SSPCON2::RSEN));
 	    m_sspmod->setSCL(false);
 	    m_sspmod->set_sspif();
@@ -1065,7 +1160,7 @@ void I2C::callback()
 
 
     case CLK_TX_BYTE:
-	if(m_sspmod->get_SCL_State()) 
+	if(m_sspmod->get_SCL_State())
 	{
 	    bool n_ack = m_sspmod->get_SDI_State();
 	    bits_transfered++;
@@ -1075,7 +1170,7 @@ void I2C::callback()
 	        m_sspmod->setSCL(false);
 	        m_sspmod->setSDA((m_SSPsr & 0x80) == 0x80);
 	    }
-	    else if(bits_transfered == 8) 
+	    else if(bits_transfered == 8)
 	    {
 	        m_sspmod->setSCL(false);
 	        m_sspmod->setSDA(true);
@@ -1100,17 +1195,17 @@ void I2C::callback()
 	        m_sspmod->setSCL(false);
 	    }
 	}
-	else 
+	else
 		m_sspmod->setSCL(true);
 	break;
 
     case CLK_RX_BYTE:
-	if(m_sspmod->get_SCL_State()) 
+	if(m_sspmod->get_SCL_State())
 	{
 	    rx_byte();
 	    m_sspmod->setSCL(false);
 	}
-	else 
+	else
 		m_sspmod->setSCL(true);
 	break;
 
@@ -1118,7 +1213,7 @@ void I2C::callback()
     default:
 	cout << "I2C::callback unxpected i2c_state=" << dec << i2c_state << endl;
 	break;
-	
+
     }
 }
 void I2C::clock(bool clock_state)
@@ -1126,8 +1221,8 @@ void I2C::clock(bool clock_state)
   unsigned int sspcon_val = m_sspcon->value.get();
   unsigned int sspstat_val = m_sspstat->value.get();
     if (verbose & 2)
-        cout << "I2C::clock  SCL=" << clock_state << " SDI=" << 
-	    m_sspmod->get_SDI_State() << " i2c_state=" << i2c_state << 
+        cout << "I2C::clock  SCL=" << clock_state << " SDI=" <<
+	    m_sspmod->get_SDI_State() << " i2c_state=" << i2c_state <<
 	    " phase=" << phase << endl;
     if (clock_state)	// Do read on clock high transition
     {
@@ -1167,7 +1262,7 @@ void I2C::clock(bool clock_state)
 		setBRG();
 	    }
 	    break;
-	    
+
 
 	  case RX_CMD:
 	  case RX_CMD2:
@@ -1215,7 +1310,7 @@ void I2C::clock(bool clock_state)
 	    }
 	    else if (phase == 1)
 	    {
-        	m_sspcon2->value.put(m_sspcon2->value.get() & 
+        	m_sspcon2->value.put(m_sspcon2->value.get() &
 		    ~(_SSPCON2::SEN | _SSPCON2::RSEN));
 	    }
 	    break;
@@ -1229,18 +1324,23 @@ void I2C::clock(bool clock_state)
 	case RX_CMD2:
 	    if (bits_transfered == 8)
 	    {
-    		if ( !( m_SSPsr == 0 && 
+    		if ( ( m_SSPsr == 0 &&
 			(m_sspcon2->value.get() & _SSPCON2::GCEN)
 		      )
-		      &&
-     		      (m_SSPsr & 0xfe) != m_sspadd->value.get() )
+		      || match_address(m_SSPsr))
     		{
+		    I2Cproto(("got address sspsr=0x%2x\n", m_SSPsr));
+		}
+		else
+		{
+		    I2Cproto(("address not a match sspsr=0x%02x\n",m_SSPsr));
       		    set_idle();
 		    return;
     		}
 	    }
 	    else if (bits_transfered == 9)
 	    {
+		I2Cproto(("9 bits RXCMD\n"));
 		if(end_ack())
 		{
 		    m_sspstat->put_value(sspstat_val & ~_SSPSTAT::DA);
@@ -1252,6 +1352,7 @@ void I2C::clock(bool clock_state)
 	case RX_DATA:
 	    if (bits_transfered == 8)
 	    {
+		I2Cproto(("RX_DATA 0x%02x\n", m_SSPsr&0xff));
 		if (verbose)
 		    cout << "I2C::clock RX_DATA or CMD m_SSPsr=" << hex << (m_SSPsr & 0xff) << endl;
 		if (m_sspmod->SaveSSPsr(m_SSPsr & 0xff) ) // ACK ?
@@ -1288,14 +1389,14 @@ void I2C::clock(bool clock_state)
 	        m_SSPsr <<= 1;
 	        m_sspmod->setSDA((m_SSPsr & 0x80) == 0x80);
 	    }
-	    else if(bits_transfered == 8) 
+	    else if(bits_transfered == 8)
 	    {
 	        m_sspmod->setSDA(true);
 	    	m_sspstat->put_value(sspstat_val & ~_SSPSTAT::BF);
 		if (verbose)
 		    cout << "I2C::clock TX_DATA  sent byte\n";
 	    }
-	    else if(bits_transfered == 9) 
+	    else if(bits_transfered == 9)
 	    {
 	        m_sspmod->set_sspif();
 	    	if (m_sspmod->get_SDI_State())	// NACK
@@ -1323,13 +1424,41 @@ void I2C::clock(bool clock_state)
 	}
     }
 }
+
+bool I2C::match_address(unsigned int sspsr)
+{
+    unsigned int mask = 0xfe;
+    unsigned int sspm = m_sspcon->value.get() & _SSPCON::SSPM_mask;
+    bool slave_10 = (sspm == _SSPCON::SSPM_I2Cslave_10bitaddr) ||
+			(sspm == _SSPCON::SSPM_I2Cslave_10bitaddr_ints);
+
+    if (slave_10)
+    {
+	unsigned int ret = (sspsr & 0xff) ^ m_sspadd->get();
+        if ((sspsr & 0xf9) == 0xf0)	// 1st byte 10 bit address
+        {
+	    mask = 0x6;
+        }
+	else
+	{
+	    mask =  (m_sspmod->sspmsk) ? m_sspmod->sspmsk->value.get() : 0xff;
+	}
+
+	ret &= mask;
+	return !(bool)ret;
+    }
+    if (m_sspmod->sspmsk)
+	mask &= m_sspmod->sspmsk->value.get();
+
+    return !((sspsr ^ m_sspadd->get()) & mask);
+}
 void I2C_1::clock(bool clock_state)
 {
   unsigned int sspcon_val = m_sspcon->value.get();
   unsigned int sspstat_val = m_sspstat->value.get();
   if (verbose & 2)
-        cout << "I2c_1::clock  SCL=" << clock_state << " SDI=" << 
-	    m_sspmod->get_SDI_State() << " i2c_state=" << i2c_state << 
+        cout << "I2c_1::clock  SCL=" << clock_state << " SDI=" <<
+	    m_sspmod->get_SDI_State() << " i2c_state=" << i2c_state <<
 	    " phase=" << phase << endl;
     if (clock_state)	// Do read on clock high transition
     {
@@ -1369,7 +1498,7 @@ void I2C_1::clock(bool clock_state)
 		setBRG();
 	    }
 	    break;
-	    
+
 
 	  case RX_CMD:
 	  case RX_CMD2:
@@ -1379,7 +1508,7 @@ void I2C_1::clock(bool clock_state)
 	  	    m_SSPsr = (m_SSPsr << 1) | (m_sspmod->get_SDI_State()?1:0);
 		    bits_transfered++;
 		}
-		else if (bits_transfered == 9 && 
+		else if (bits_transfered == 9 &&
 		    m_sspcon3->value.get() & (_SSP1CON3::AHEN | _SSP1CON3::DHEN))
 		{
 		    m_sspcon3->put(m_sspcon3->value.get() & ~_SSP1CON3::ACKTIM);
@@ -1388,7 +1517,7 @@ void I2C_1::clock(bool clock_state)
 	    break;
 
 	    case TX_DATA:
-		if (bits_transfered == 9 && 
+		if (bits_transfered == 9 &&
 		    m_sspcon3->value.get() & (_SSP1CON3::AHEN | _SSP1CON3::DHEN))
 		{
 		    m_sspcon3->put(m_sspcon3->value.get() & ~_SSP1CON3::ACKTIM);
@@ -1433,7 +1562,7 @@ void I2C_1::clock(bool clock_state)
 	    }
 	    else if (phase == 1)
 	    {
-        	m_sspcon2->value.put(m_sspcon2->value.get() & 
+        	m_sspcon2->value.put(m_sspcon2->value.get() &
 		    ~(_SSPCON2::SEN | _SSPCON2::RSEN));
 	    }
 	    break;
@@ -1447,14 +1576,16 @@ void I2C_1::clock(bool clock_state)
 	case RX_CMD2:
 	    if (bits_transfered == 8)
 	    {
-    		if ( !( m_SSPsr == 0 && 
+    		if ( ( m_SSPsr == 0 &&
 			(m_sspcon2->value.get() & _SSPCON2::GCEN)
 		      )
-		      &&
-     		      (m_SSPsr & 0xfe) != m_sspadd->value.get() )
+		      || match_address(m_SSPsr))
+		{
+		}
+		else
     		{
       		    cout << "READ_CMD address missmatch " << hex << m_SSPsr <<
-			" != " << m_sspadd->value.get() << endl;
+			" != " << m_sspadd->get() << endl;
       		    set_idle();
 		    return;
     		}
@@ -1484,7 +1615,7 @@ void I2C_1::clock(bool clock_state)
 		{
 		    unsigned int sspcon3_val = m_sspcon3->value.get();
 
-	
+
 		    m_sspmod->SaveSSPsr(m_SSPsr & 0xff);
 		    m_sspcon->value.put(m_sspcon->value.get() & ~_SSPCON::CKP);
 		    m_sspcon3->value.put(sspcon3_val | _SSP1CON3::ACKTIM);
@@ -1508,8 +1639,8 @@ void I2C_1::clock(bool clock_state)
 	    else if (bits_transfered == 9)
 	    {
 		m_sspstat->put_value(sspstat_val | _SSPSTAT::DA);
-		if(end_ack() 
-		   && m_sspmod->isI2CSlave() 
+		if(end_ack()
+		   && m_sspmod->isI2CSlave()
 		   && (m_sspcon2->value.get() & _SSPCON2::SEN))
 		{
 			m_sspcon->put(m_sspcon->value.get() & ~_SSPCON::CKP);
@@ -1530,7 +1661,7 @@ void I2C_1::clock(bool clock_state)
 	        m_SSPsr <<= 1;
 	        m_sspmod->setSDA((m_SSPsr & 0x80) == 0x80);
 	    }
-	    else if(bits_transfered == 8) 
+	    else if(bits_transfered == 8)
 	    {
 	        m_sspmod->setSDA(true);
 	    	m_sspstat->put_value(sspstat_val & ~_SSPSTAT::BF);
@@ -1541,14 +1672,14 @@ void I2C_1::clock(bool clock_state)
 		    m_sspcon3->value.put(m_sspcon3->value.get() | _SSP1CON3::ACKTIM);
 		}
 	    }
-	    else if(bits_transfered == 9) 
+	    else if(bits_transfered == 9)
 	    {
 	        m_sspmod->set_sspif();
 	    	if (m_sspmod->get_SDI_State())	// NACK
 	    	{
 		    if (verbose)
 		    	cout << "I2c_1::clock TX_DATA  got NACK\n";
-		    m_sspcon2->put(m_sspcon2->value.get() | _SSPCON2::ACKSTAT);	
+		    m_sspcon2->put(m_sspcon2->value.get() | _SSPCON2::ACKSTAT);
 		    m_sspstat->put_value(sspstat_val & _SSPSTAT::BF);
 		    set_idle();
 		    return;
@@ -1556,7 +1687,7 @@ void I2C_1::clock(bool clock_state)
 		m_sspstat->put_value(sspstat_val | _SSPSTAT::DA);
  	     	if (sspstat_val & _SSPSTAT::RW)
 	    	{
-		    m_sspcon2->put(m_sspcon2->value.get() & ~_SSPCON2::ACKSTAT);	
+		    m_sspcon2->put(m_sspcon2->value.get() & ~_SSPCON2::ACKSTAT);
 		    sspcon_val &= ~ _SSPCON::CKP;
 		    m_sspcon->put_value(sspcon_val);
 		    if (verbose)
@@ -1582,12 +1713,13 @@ void I2C::slave_command()
     if ( m_SSPsr == 0 && (m_sspcon2->value.get() & _SSPCON2::GCEN))
     {
 	i2c_state = RX_DATA;
+	I2Cproto(("slave_command i2c_state = RX_DATA\n"));
     }
     else
     {
 	if (verbose)
 	    cout << "I2c::slave_command i2c_state=" << i2c_state << " sspcon=" << sspcon_val << endl;
-    	switch( sspcon_val & _SSPCON::SSPM_mask ) 
+    	switch( sspcon_val & _SSPCON::SSPM_mask )
 	{
       	case _SSPCON::SSPM_I2Cslave_10bitaddr_ints:
       	case _SSPCON::SSPM_I2Cslave_10bitaddr:
@@ -1595,6 +1727,7 @@ void I2C::slave_command()
 		{
 		    sspstat_val |= _SSPSTAT::RW;
 		    i2c_state = TX_DATA;
+		    I2Cproto(("slave_command i2c_state = TX_DATA\n"));
 	    	    m_sspmod->setSCL(false);  // clock low
 	    	    sspcon_val &= ~ _SSPCON::CKP;
 	    	    m_sspcon->put_value(sspcon_val);
@@ -1602,8 +1735,9 @@ void I2C::slave_command()
 		else
 		{
 	       	    sspstat_val |= _SSPSTAT::UA;
-		    i2c_state = (i2c_state == RX_CMD2) ? 
+		    i2c_state = (i2c_state == RX_CMD2) ?
 			RX_DATA : RX_CMD2;
+		    I2Cproto(("slave_command i2c_state = %s\n", i2c_state ==RX_DATA?"RX_DATA":"RX_CMD2"));
 		}
 		break;
 
@@ -1614,6 +1748,7 @@ void I2C::slave_command()
 		    sspstat_val |= _SSPSTAT::RW;
 	            sspstat_val &= ~_SSPSTAT::BF;
 	    	    i2c_state = TX_DATA;
+		    I2Cproto(("slave_command i2c_state = TX_DATA\n"));
 		    sspcon_val &= ~ _SSPCON::CKP;
 		    m_sspcon->put_value(sspcon_val);
 		    m_sspmod->setSCL(false);  // clock low
@@ -1621,6 +1756,7 @@ void I2C::slave_command()
 		else
 		{
 		    i2c_state = RX_DATA;
+		    I2Cproto(("slave_command i2c_state = RX_DATA\n"));
 		}
 		break;
 	}
@@ -1634,6 +1770,7 @@ bool I2C::end_ack()
 	bits_transfered = 0;
  	if (m_sspmod->get_SDI_State())      // NACK
         {
+	    I2Cproto(("end_ack NACK\n"));
 	    if (verbose & 2)
 	      cout << "I2C::end_ack NACK\n";
             set_idle();
@@ -1642,6 +1779,7 @@ bool I2C::end_ack()
 	else
 	{
 	    m_sspmod->setSDA(true);
+	    I2Cproto(("end_ack ACK\n"));
 	    if (verbose & 2)
 	      cout << "I2C::end_ack ACK\n";
 	    return(true);
@@ -1684,8 +1822,8 @@ void I2C::setBRG()
 {
     if (future_cycle)
 	cout << "ERROR I2C::setBRG called with future_cycle=" << future_cycle << endl;
-      future_cycle = get_cycles().get() + 
-	  	((m_sspadd->value.get() &0x7f)/ 2) + 1;
+      future_cycle = get_cycles().get() +
+	  	((m_sspadd->get() &0x7f)/ 2) + 1;
       get_cycles().set_break(future_cycle, this);
 }
 
@@ -1717,11 +1855,12 @@ void I2C::newSSPBUF(unsigned int newTxByte)
 	    m_sspmod->setSDA((m_SSPsr & 0x80) == 0x80);
 	    bits_transfered = 0;
 	    i2c_state = CLK_TX_BYTE;
+	    I2Cproto(("%s i2c_state = CLK_TX_BYTE data %x\n", __FUNCTION__, newTxByte));
 	    setBRG();
 	}
 	else
 	{
-	    cout << "I2C::newSSPBUF I2C not idle on write data=" << hex << 
+	    cout << "I2C::newSSPBUF I2C not idle on write data=" << hex <<
 		newTxByte << endl;
       	    // Collision
       	    m_sspcon->setWCOL();
@@ -1732,21 +1871,22 @@ void I2C::newSSPBUF(unsigned int newTxByte)
 	if (sspstat_val & _SSPSTAT::RW)
 	{
       	    if (!(sspstat_val & _SSPSTAT::BF))
-	    {    
+	    {
 		if (verbose)
 		    cout << "I2C::newSSPBUF send " << hex << newTxByte << endl;
 	        m_SSPsr = newTxByte;
 	        m_sspstat->put_value(sspstat_val | _SSPSTAT::BF);
 	        m_sspmod->setSDA((m_SSPsr & 0x80) == 0x80);
 	        bits_transfered = 0;
-    	    } 
+		I2Cproto(("%s TX 0x%x\n", __FUNCTION__, newTxByte));
+    	    }
 	    else // Collision
 	    {
-	        cout << "I2C::newSSPBUF I2C not idle on write data=" << hex << 
+	        cout << "I2C::newSSPBUF I2C not idle on write data=" << hex <<
 	 	    newTxByte << endl;
       	        m_sspcon->setWCOL();
 	    }
-	} 
+	}
 	else
 	   cout << "I2C::newSSPBUF write SSPSTAT::RW not set\n";
     }
@@ -1775,9 +1915,9 @@ void I2C::sda(bool data_val)
 
 	    case CLK_START:
 		if (phase == 0)
-		{	
-		    guint64 fc = get_cycles().get() + 
-			((m_sspadd->value.get() &0x7f)/ 2) + 1;
+		{
+		    guint64 fc = get_cycles().get() +
+			((m_sspadd->get() &0x7f)/ 2) + 1;
 
 		    if (future_cycle)
 		    {
@@ -1797,6 +1937,7 @@ void I2C::sda(bool data_val)
 
 	    default:
 		i2c_state = RX_CMD;
+		I2Cproto(("%s i2c_state = RX_CMD\n", __FUNCTION__));
 		break;
 	    }
     	    stat_val = (stat_val & _SSPSTAT::BF) | _SSPSTAT::S;
@@ -1807,7 +1948,7 @@ void I2C::sda(bool data_val)
 	}
 	m_sspstat->put_value(stat_val);
 
-	// interrupt ? 
+	// interrupt ?
 	if (sspm == _SSPCON::SSPM_I2Cslave_7bitaddr_ints ||
 	    sspm == _SSPCON::SSPM_I2Cslave_10bitaddr_ints)
 	{
@@ -1838,7 +1979,7 @@ void I2C_1::sda(bool data_val)
 	    if (! future_cycle)
 	    	set_idle();
 
-	    if(con3_val & _SSP1CON3::PCIE && 
+	    if(con3_val & _SSP1CON3::PCIE &&
 		(sspm == _SSPCON::SSPM_I2Cslave_7bitaddr ||
 	         sspm == _SSPCON::SSPM_I2Cslave_10bitaddr))
 	    {
@@ -1861,9 +2002,9 @@ void I2C_1::sda(bool data_val)
 
 	    case CLK_START:
 		if (phase == 0)
-		{	
-		    guint64 fc = get_cycles().get() + 
-			((m_sspadd->value.get() &0x7f)/ 2) + 1;
+		{
+		    guint64 fc = get_cycles().get() +
+			((m_sspadd->get() &0x7f)/ 2) + 1;
 
 		    if (future_cycle)
 		    {
@@ -1883,12 +2024,13 @@ void I2C_1::sda(bool data_val)
 
 	    default:
 		i2c_state = RX_CMD;
+		I2Cproto(("%s i2c_state = RX_CMD\n", __FUNCTION__));
 		break;
 	    }
     	    stat_val = (stat_val & _SSPSTAT::BF) | _SSPSTAT::S;
 	    bits_transfered = 0;
 	    m_SSPsr = 0;
-	    if(con3_val & _SSP1CON3::SCIE && 
+	    if(con3_val & _SSP1CON3::SCIE &&
 		(sspm == _SSPCON::SSPM_I2Cslave_7bitaddr ||
 	         sspm == _SSPCON::SSPM_I2Cslave_10bitaddr))
 	    {
@@ -1899,7 +2041,7 @@ void I2C_1::sda(bool data_val)
 	}
 	m_sspstat->put_value(stat_val);
 
-	// interrupt ? 
+	// interrupt ?
 	if (sspm == _SSPCON::SSPM_I2Cslave_7bitaddr_ints ||
 	    sspm == _SSPCON::SSPM_I2Cslave_10bitaddr_ints)
 	{
@@ -1941,6 +2083,7 @@ void I2C::start_bit()
     {
 
 	i2c_state = CLK_START;
+	I2Cproto(("%s i2c_state = CLK_START\n", __FUNCTION__));
 	phase = 0;
 	setBRG();
 
@@ -1957,14 +2100,15 @@ void I2C::start_bit()
 /*
 	Master mode, begin rstart sequence
 		bring SDA and SCL high, then SDA low with SCL high (start)
-*/ 
+*/
 void I2C::rstart_bit()
 {
     if (verbose)
-        cout << "I2C::rstart_bit SCL=" << m_sspmod->get_SCL_State() << 
+        cout << "I2C::rstart_bit SCL=" << m_sspmod->get_SCL_State() <<
 	    " SDI=" << m_sspmod->get_SDI_State() << endl;
 
     i2c_state = CLK_RSTART;
+    I2Cproto(("%s i2c_state = CLK_RSTART\n", __FUNCTION__));
     phase = 0;
     m_sspmod->setSCL(false);
 
@@ -1976,17 +2120,18 @@ void I2C::rstart_bit()
     else
 	bus_collide();
 
-    
+
 }
 /*
 	master, begin stop sequence
 		drop SDA (mught cause start if SCL high)
 		when SCL high, raise SDA (stop condition)
-		
+
 */
 void I2C::stop_bit()
 {
     i2c_state = CLK_STOP;
+    I2Cproto(("%s i2c_state = CLK_STOP\n", __FUNCTION__));
     phase = 0;
     m_sspmod->setSDA(false);
 
@@ -2000,7 +2145,7 @@ void I2C::stop_bit()
 }
 
 /*
-	master, begin ack sequence 
+	master, begin ack sequence
 		clock SCL low, set SDA as per ACKDT,
 		clock SCL high
 */
@@ -2010,6 +2155,7 @@ void I2C::ack_bit()
         cout << "I2C::ack_bit ACKDT="
 	    << (m_sspcon2->value.get() & _SSPCON2::ACKDT) << endl;
     i2c_state = CLK_ACKEN;
+    I2Cproto(("%s i2c_state = CLK_ACKEN\n", __FUNCTION__));
     phase = 0;
     m_sspmod->setSCL(false);
     if (!m_sspmod->get_SCL_State())
@@ -2021,7 +2167,7 @@ void I2C::ack_bit()
     else
 	bus_collide();
 
-    
+
 }
 void SSP_MODULE::initialize(
 		        PIR_SET *ps,
@@ -2070,7 +2216,7 @@ void SSP_MODULE::ckpSPI(unsigned int value)
 {
     if(m_spi && !m_spi->isIdle())
       cout << "SPI: You just changed CKP in the middle of a transfer." << endl;
-	
+
     switch( value & _SSPCON::SSPM_mask ) {
     case _SSPCON::SSPM_SPImaster4:
     case _SSPCON::SSPM_SPImaster16:
@@ -2160,19 +2306,21 @@ void SSP_MODULE::putStateSCK(char _state)
 void SSP_MODULE::startSSP(unsigned int value)
 {
     if (verbose)
-      cout << "SSP: SPI turned on" << endl;
+      cout << "SSP: SPI turning on 0x" <<hex << value << endl;
     Dprintf(("SSP_MODULE cmd %x\n", value &  _SSPCON::SSPM_mask ));
+    SPIproto(("SSP_MODULE cmd %x\n", value &  _SSPCON::SSPM_mask ));
     sspbuf.setFullFlag(false);
     if (! m_sink_set)
     {
-	if (m_sdi) 
+	if (m_sdi)
 	{
 	   m_sdi->addSink(m_SDI_Sink);
 	   m_SDI_State = m_sdi->getPin().getState();
 	}
-	if (m_sck) 
+	if (m_sck)
 	{
 	   m_sck->addSink(m_SCL_Sink);
+	   m_SCL_State = m_sck->getPin().getState();
 	}
         if (m_ss)
 	{
@@ -2185,7 +2333,7 @@ void SSP_MODULE::startSSP(unsigned int value)
     {
 	if ((value & _SSPCON::SSPM_mask) ==  _SSPCON::SSPM_SPIslaveSS)
 		m_ss->getPin().newGUIname("SS");
-	else
+	else if (m_ss->getPin().GUIname() == string("SS"))
 		m_ss->getPin().newGUIname(m_ss->getPin().name().c_str());
     }
     switch( value & _SSPCON::SSPM_mask ) {
@@ -2195,7 +2343,7 @@ void SSP_MODULE::startSSP(unsigned int value)
     case _SSPCON::SSPM_SPImaster64:
     case _SSPCON::SSPM_SPImasterAdd:
     Dprintf(("SSP_MODULE case cmd %x\n", value &  _SSPCON::SSPM_mask ));
-  	if (m_sck) 
+  	if (m_sck)
 	{
 	    m_sck->setSource(m_SckSource);
 	    m_sck_active = true;
@@ -2228,6 +2376,7 @@ void SSP_MODULE::startSSP(unsigned int value)
 	     m_sck->getPin().newGUIname("SCK");
 
 	if (m_SdoSource) m_SdoSource->putState('0'); // BUG, required to put SDO in know state
+	newSSPBUF(sspbuf.get_value());
 	break;
 
       case _SSPCON::SSPM_I2Cslave_7bitaddr:
@@ -2247,15 +2396,18 @@ void SSP_MODULE::startSSP(unsigned int value)
 	m_sdi_active = true;
 	m_sck->refreshPinOnUpdate(true);
 	m_sdi->refreshPinOnUpdate(true);
-	m_SdiSource->putState('0'); 
-	m_SckSource->putState('0'); 
+	m_SdiSource->putState('0');
+	m_SckSource->putState('0');
 	m_sck->refreshPinOnUpdate(false);
 	m_sdi->refreshPinOnUpdate(false);
 	break;
 
+      case _SSPCON::SSPM_LoadMaskFunction:
+	break;
+
     default:
-	cout << "SSP: start, unexpected SSPM select bits SSPCON="
-		<< hex << value << endl;;
+	cout << "SSP: start, unexpected SSPM select bits SSPM="
+		<< hex << (value & _SSPCON::SSPM_mask) << endl;;
 	break;
     }
 }
@@ -2363,11 +2515,11 @@ void SSP1_MODULE::changeSSP(unsigned int new_value, unsigned int old_value)
 
 //------------------------------------------------------------
 // Called whenever the SDI/SDA input changes states.
-// 
+//
 void SSP_MODULE::SDI_SinkState(char new3State)
 {
   bool new_SDI_State = (new3State == '1' || new3State == 'W');
- 
+
   if (new_SDI_State == m_SDI_State)
 	return;
 
@@ -2383,7 +2535,9 @@ void SSP_MODULE::SDI_SinkState(char new3State)
 void SSP_MODULE::SCL_SinkState(char new3State)
 {
   bool new_SCL_State = (new3State == '1' || new3State == 'W');
- 
+
+  SPIproto(("SCL_SinkState new %d old %d enabled %d m_SS_State %d\n", new_SCL_State, m_SCL_State, sspcon.isSSPEnabled(), m_SS_State));
+
   if (new_SCL_State == m_SCL_State)
 	return;
 
@@ -2392,17 +2546,17 @@ void SSP_MODULE::SCL_SinkState(char new3State)
   if (!sspcon.isSSPEnabled() )
      return;
 
-  switch( sspcon.value.get() & _SSPCON::SSPM_mask ) 
+  switch( sspcon.value.get() & _SSPCON::SSPM_mask )
   {
       case _SSPCON::SSPM_SPIslaveSS:
       /*
      	SS high during transfer for BSSP, suspends transfers which
 	continues when SS goes low.
 
-	None BSSP interfaces handled when SS goes high  
+	None BSSP interfaces handled when SS goes high
       */
 	if (m_SS_State)
-	    return;	// suspend transfer 
+	    return;	// suspend transfer
 			// Fall through
       case _SSPCON::SSPM_SPIslave:
 	if (m_spi) m_spi->clock(m_SCL_State);
@@ -2463,8 +2617,16 @@ void SSP_MODULE::SS_SinkState(char new3State)
   // If SS goes high in the middle of an SPI transfer while in slave_SS mode,
   // transfer is aborted unless BSSP which streches the clocking
 
+#ifdef SPI_PROTO
 
-  if (!sspcon.isSSPEnabled() || 
+  if (sspcon.isSSPEnabled() &&
+	((sspcon.value.get() & _SSPCON::SSPM_mask) == _SSPCON::SSPM_SPIslaveSS)
+     )
+  {
+	SPIproto(("SS State change to %d\n", m_SS_State));
+  }
+#endif
+  if (!sspcon.isSSPEnabled() ||
 	! m_SS_State ||
 	(sspcon.value.get() & _SSPCON::SSPM_mask) != _SSPCON::SSPM_SPIslaveSS ||
 	! m_spi->isIdle() ||
@@ -2477,7 +2639,7 @@ void SSP_MODULE::SS_SinkState(char new3State)
 void SSP_MODULE::tmr2_clock()
 {
   unsigned int sspcon_val = sspcon.value.get();
-  if (! (sspcon_val & _SSPCON::SSPEN) || 
+  if (! (sspcon_val & _SSPCON::SSPEN) ||
      ((sspcon_val & _SSPCON::SSPM_mask) != _SSPCON::SSPM_SPImasterTMR2) ||
      (m_spi && m_spi->isIdle()))
 	return;
@@ -2590,7 +2752,7 @@ void  _SSPCON2::put_value(unsigned int new_value)
 
 }
 /*
-    If a command is currently active, 
+    If a command is currently active,
 	lower 5 bits of register cannot be changed
     if no command is currently active,
 	activate command and write data
@@ -2607,7 +2769,7 @@ void _SSPCON2::put(unsigned int new_value)
 	mask &= ~SEN;
 
   old_active = old_value & mask;
-  
+
   if (verbose & 2)
       cout << "_SSPCON2::put " << hex << new_value << endl;
 
@@ -2618,10 +2780,10 @@ void _SSPCON2::put(unsigned int new_value)
   // if I2C not idle, do not change bits in mask
   if (!m_sspmod->isI2CIdle() && (diff & mask))
   {
-	cout << "Warrning SSPCON::put I2C not idle and new value " 
+	cout << "Warrning SSPCON::put I2C not idle and new value "
 		<< hex << new_value << " changes one of following bits "
 		<< mask << endl;
- 
+
 	put_value((new_value & ~mask) | old_active);
   }
   // Master and only a new command bit to process
@@ -2676,33 +2838,7 @@ void _SSP1CON3::put(unsigned int new_value)
       cout << "_SSP1CON3::put " << hex << new_value << endl;
 
    put_value(new_value & ~ACKTIM); // ACKTIM not writable by user
-	
-}
-
-//-------------------------------------------------------------------
-_SSP1MSK::_SSP1MSK(Processor *pCpu, SSP1_MODULE *pSSP)
-  : sfr_register(pCpu, "ssp1msk","SSP1 mask register"),
-    m_sspmod(pSSP)
-{
-}
-
-/*
-	write to SSP1MSK without processing data
-*/
-void  _SSP1MSK::put_value(unsigned int new_value)
-{
-
-  trace.raw(write_trace.get() | value.get());
-  value.put(new_value);
 
 }
 
-void _SSP1MSK::put(unsigned int new_value)
-{
-  if (verbose & 2)
-      cout << "_SSP1MSK::put " << hex << new_value << endl;
-
-   put_value(new_value); // ACKTIM not writable by user
-	
-}
 
