@@ -22,7 +22,7 @@ License along with this library; if not, see
 
 #include <stdio.h>
 #include <ctype.h>
-#include <errno.h>
+#include <iostream>
 #include <stdlib.h>
 #include <string>
 #include <list>
@@ -48,7 +48,7 @@ License along with this library; if not, see
 #endif
 
 static char num_nodes = 'a';
-static char num_stimuli = 'a';
+static int num_stimuli = 1;
 void  gpsim_set_break_delta(guint64 delta, TriggerObject *f=0);
 
 
@@ -385,7 +385,7 @@ void Stimulus_Node::refresh()
     }
 
     current_time_constant = Cth * Zth;
-    Dprintf(("%s DCVoltage %.3f voltage %.3f Cth=%.2e Zth=%2e time_constant %fsec or %"PRINTF_GINT64_MODIFIER"d cycles now=%"PRINTF_GINT64_MODIFIER"d \n",name().c_str(), DCVoltage, voltage, Cth, Zth, current_time_constant, (guint64)(current_time_constant*get_cycles().instruction_cps()), get_cycles().get()));
+    Dprintf(("%s DCVoltage %.3f voltage %.3f Cth=%.2e Zth=%2e time_constant %fsec or %" PRINTF_GINT64_MODIFIER "d cycles now=%" PRINTF_GINT64_MODIFIER "d \n",name().c_str(), DCVoltage, voltage, Cth, Zth, current_time_constant, (guint64)(current_time_constant*get_cycles().instruction_cps()), get_cycles().get()));
     if (((guint64)(current_time_constant*get_cycles().instruction_cps()) < 5) ||
       (fabs(DCVoltage - voltage) < minThreshold))
     {
@@ -453,7 +453,7 @@ guint64 Stimulus_Node::calc_settlingTimeStep()
 	TimeStep = (guint64) (0.125 * dt_dv);
 	TimeStep = (TimeStep) ? TimeStep : 1;
 
-	Dprintf(("%s dt_dv = %.2f TimeStep 0x%"PRINTF_GINT64_MODIFIER"x now 0x%"PRINTF_GINT64_MODIFIER"x\n", __FUNCTION__, dt_dv, TimeStep, get_cycles().get()));
+	Dprintf(("%s dt_dv = %.2f TimeStep 0x%" PRINTF_GINT64_MODIFIER "x now 0x%" PRINTF_GINT64_MODIFIER "x\n", __FUNCTION__, dt_dv, TimeStep, get_cycles().get()));
 
 	return(TimeStep);
 }
@@ -526,7 +526,7 @@ void Stimulus_Node::callback()
           cout << "\t" << name() << 
 	    " Final voltage " << DCVoltage << " reached at " 
 	    << get_cycles().get() << " cycles\n";
-          Dprintf(("%s DC Voltage %.2f reached at 0x%"PRINTF_GINT64_MODIFIER"x cycles\n", name().c_str(), DCVoltage, get_cycles().get()));
+          Dprintf(("%s DC Voltage %.2f reached at 0x%" PRINTF_GINT64_MODIFIER "x cycles\n", name().c_str(), DCVoltage, get_cycles().get()));
       } 
       else if(get_cycles().get() >= future_cycle) // got here via break
       {
@@ -573,11 +573,6 @@ stimulus::stimulus(const char *cPname,double _Vth, double _Zth)
     Cth(0.0), // Farads
     nodeVoltage(0.0) // volts
 {
-  /*
-  if(cPname && *cPname)
-    new_name(cPname,false);
-  */
-  globalSymbolTable().addSymbol(this);
 }
 void stimulus::new_name(const char *cPname, bool bClearableSymbol)
 {
@@ -965,6 +960,16 @@ IOPIN::~IOPIN()
 }
 
 
+void IOPIN::get(char *return_str, int len)
+{
+    if (return_str)
+    {
+        if (get_direction() == DIR_OUTPUT)
+            strncpy(return_str, IOPIN::getDrivingState()?"1": "0", len);
+        else
+            strncpy(return_str, IOPIN::getState()?"1": "0", len);
+    }
+}
 void IOPIN::attach(Stimulus_Node *s)
 {
   snode = s;
@@ -1510,7 +1515,9 @@ ValueStimulus::ValueStimulus(const char *n)
   current = 0;
 
   if(n)
+  {
     new_name(n,false);
+  }
   else
     {
       char name_str[100];
@@ -1520,6 +1527,14 @@ ValueStimulus::ValueStimulus(const char *n)
     }
 
 }
+void ValueStimulus::get(char *return_str, int len)
+{
+    if (return_str)
+    {
+	double  d = get_Vth();
+	snprintf(return_str,len,"%.2fV",d);
+    }
+}
 
 ValueStimulus::~ValueStimulus()
 {
