@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -30,7 +30,7 @@ License along with this library; if not, see
 #include "trace_orb.h"
 #include "xref.h"
 
-#define NEWTRACE 
+#define NEWTRACE
 
 //#define DEBUG
 #if defined(DEBUG)
@@ -340,12 +340,11 @@ void Trace::deleteTraceFrame()
 {
   if (!current_frame)
     return;
-  list <TraceFrame *> :: iterator tfIter;
-  list <TraceObject *> :: reverse_iterator toIter;
+  std::list <TraceFrame *> :: iterator tfIter;
 
   for(tfIter = traceFrames.begin();
       tfIter != traceFrames.end();
-      ++tfIter) 
+      ++tfIter)
   {
       TraceFrame *tf = *tfIter;
       delete tf;
@@ -1030,7 +1029,7 @@ int Trace::is_cycle_trace(unsigned int index, guint64 *cvt_cycle)
 
     }
 
-  Dprintf(("trace index %d does not point to lower part (0x%lx)\n", j, get(j)));
+  Dprintf(("trace index %d does not point to lower part (0x%x)\n", j, get(j)));
   return 1;
 }
 
@@ -1627,7 +1626,7 @@ void TraceLog::register_read(Register *pReg, guint64 cc)
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-      Dprintf(("cycle=%llx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
 #ifdef NEWTRACE
       bp.set_logging();
       return;
@@ -1640,7 +1639,7 @@ void TraceLog::register_read(Register *pReg, guint64 cc)
       break;
 
     case TRACE_FILE_FORMAT_LXT:
-      Dprintf(("LXT cycle=%llx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("LXT cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
@@ -1655,7 +1654,7 @@ void TraceLog::register_write(Register *pReg, guint64 cc)
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-      Dprintf(("cycle=%llx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
 #ifdef NEWTRACE
       bp.set_logging();
       return;
@@ -1668,7 +1667,7 @@ void TraceLog::register_write(Register *pReg, guint64 cc)
       break;
 
     case TRACE_FILE_FORMAT_LXT:
-      Dprintf(("LXT cycle=%llx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("LXT cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
@@ -1682,7 +1681,7 @@ void TraceLog::register_read_value(Register *pReg, guint64 cc)
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-      Dprintf(("cycle=%llx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
 #ifdef NEWTRACE
       bp.set_logging();
       return;
@@ -1695,7 +1694,7 @@ void TraceLog::register_read_value(Register *pReg, guint64 cc)
       break;
 
     case TRACE_FILE_FORMAT_LXT:
-      Dprintf(("LXT cycle=%llx %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("LXT cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%02x) value 0x%02x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
@@ -1710,7 +1709,7 @@ void TraceLog::register_write_value(Register *pReg, guint64 cc)
   switch(file_format)
     {
     case TRACE_FILE_FORMAT_ASCII:
-      Dprintf(("cycle=%llx %s(0x%x) value %x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%x) value %x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
 #ifdef NEWTRACE
       bp.set_logging();
       return;
@@ -1723,7 +1722,7 @@ void TraceLog::register_write_value(Register *pReg, guint64 cc)
       break;
 
     case TRACE_FILE_FORMAT_LXT:
-      Dprintf(("LXT cycle=%llx %s(0x%x) value %x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
+      Dprintf(("LXT cycle=%" PRINTF_GINT64_MODIFIER "x %s(0x%x) value %x\n", cc, pReg->name().c_str(), pReg->getAddress(), pReg->get_value()));
       lxt_trace(pReg->getAddress(), pReg->get_value(), cc);
       break;
     }
