@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -80,7 +80,7 @@ char * UnquoteString(char * pBuffer) {
 string &toupper(string & sStr) {
   string::iterator it;
   string::iterator itEnd = sStr.end();
-  for(it = sStr.begin(); it != itEnd; it++) {
+  for(it = sStr.begin(); it != itEnd; ++it) {
     if(isalpha(*it)) {
       *it = toupper((int)*it);
     }
@@ -131,9 +131,8 @@ Value::~Value()
 }
 void Value::update()
 {
-  if (!_xref)
-     _xref = new XrefObject();
-  _xref->_update();
+  if (_xref)
+      _xref->_update();
 }
 
 void Value::add_xref(void *an_xref)
@@ -433,9 +432,8 @@ string AbstractRange::toString()
 {
   char buff[256];
 
-  string str = "";
+  snprintf(buff, sizeof(buff), "%u:%u", left, right);
 
-  snprintf(buff,sizeof(buff),"%d:%d",left,right);
   return (string(buff));
 }
 
@@ -443,7 +441,7 @@ string AbstractRange::toString(const char* format)
 {
   char cvtBuf[1024];
 
-  sprintf(cvtBuf, format, left, right);
+  snprintf(cvtBuf, sizeof(cvtBuf), format, left, right);
   return (string(&cvtBuf[0]));
 }
 
@@ -451,7 +449,7 @@ char *AbstractRange::toString(char *return_str, int len)
 {
   if(return_str) {
 
-    snprintf(return_str,len,"%d:%d",left,right);
+    snprintf(return_str, len, "%u:%u", left, right);
   }
 
   return return_str;
@@ -1242,107 +1240,80 @@ bool Float::operator<(Value *rv)
  */
 String::String(const char *newValue)
 {
-  if(newValue)
-    value = strdup(newValue);
-  else
-    value = 0;
+  if (newValue)
+    value = newValue;
 }
 
 String::String(const char *newValue, size_t len)
 {
-  if (newValue) {
-    value = (char *)malloc(len + 1);
-    strncpy(value, newValue, len);
-    value[len] = '\0';
-  }
-  else
-    value = 0;
+  if (newValue)
+    value.assign(newValue, len);
 }
 
-String::String(const char *_name, const char *newValue,const char *_desc)
-  : Value(_name,_desc)
+String::String(const char *_name, const char *newValue, const char *_desc)
+  : Value(_name, _desc)
 {
-  if(newValue)
-    value = strdup(newValue);
-  else
-    value = 0;
+  if (newValue)
+    value = newValue;
 }
 
-
 String::~String()
 {
-  if(value)
-    free(value);
 }
 
-string String::toString()
+std::string String::toString()
 {
-  if(value)
-    return string(value);
-  else
-    return string("");
+  return value;
 }
 
 char *String::toString(char *return_str, int len)
 {
-  if(return_str) {
-
-    if(value)
-      snprintf(return_str,len,"%s",value);
-    else
-      *return_str = 0;
-  }
+  if (return_str)
+    snprintf(return_str, len, "%s", value.c_str());
 
   return return_str;
 }
 
 void String::set(Value *v)
 {
-  char buf[1024];
-
-  if(v) {
-    v->get(buf, sizeof(buf));
-    set(buf);
+  if (v) {
+    std::string buf = v->toString();
+    set(buf.c_str());
   }
 }
 
+// TODO: is this meant to do something
 void String::set(Packet &p)
 {
   cout << " fixme String::set(Packet &) is not implemented\n";
 }
 
-void String::set(const char *s,int len)
+// TODO: was len meant to do anything
+void String::set(const char *s, int len)
 {
-  if(value)
-    free(value);
-  if(s)
-    value = strdup(s);
-  else
-    value = 0;
+  if (s)
+    value = s;
 }
+
 void String::get(char *buf, int len)
 {
-  if(buf && value) {
-    strncpy(buf,value,len);
-  }
-  else if(buf) {
-    buf[0] = 0;
-  }
+  if (buf)
+    snprintf(buf, len, "%s", value.c_str());
 }
 
 void String::get(Packet &p)
 {
-  p.EncodeString(value);
+  p.EncodeString(value.c_str());
 }
 
 const char *String::getVal()
 {
-  return value;
+  return value.c_str();
 }
 
 Value *String::copy()
 {
-  return new String(value);
+  return new String(value.c_str());
 }
 
 //------------------------------------------------------------------------
