@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, see 
+License along with this library; if not, see
 <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
 
@@ -30,15 +30,16 @@ class gpsimObject;
 
 class XrefObject {
  private:
-  gpsimObject *data;
   list<void*> xrefs;
  public:
+  gpsimObject *data;
   XrefObject();
-  XrefObject(gpsimObject *value);
+  explicit XrefObject(gpsimObject *value);
   virtual ~XrefObject();
-    
+
   virtual void _add(void *xref);
   virtual void clear(void *xref);
+  virtual void *first_xref();
   virtual void _update();
   virtual int get_val(void);
   virtual void assign_data(gpsimObject *);
